@@ -73,7 +73,7 @@ def GenerateHmacKey(file_name):
   @param file_name: Path to output file
 
   """
-  utils.WriteFile(file_name, data="%s\n" % utils.GenerateSecret(), mode=0400,
+  utils.WriteFile(file_name, data="%s\n" % utils.GenerateSecret(), mode=0o400,
                   backup=True)
 
 
@@ -177,7 +177,7 @@ def GenerateClusterCrypto(new_cluster_cert, new_rapi_cert, new_spice_cert,
     # certificate
     logging.debug("Writing the public certificate to %s",
                   spicecert_file)
-    utils.io.WriteFile(spicecacert_file, mode=0400, data=cert_pem)
+    utils.io.WriteFile(spicecacert_file, mode=0o400, data=cert_pem)
 
   # Cluster domain secret
   if cds:
@@ -311,8 +311,8 @@ def _InitFileStorageDir(file_storage_dir):
 
   if not os.path.exists(file_storage_dir):
     try:
-      os.makedirs(file_storage_dir, 0750)
-    except OSError, err:
+      os.makedirs(file_storage_dir, 0o750)
+    except OSError as err:
       raise errors.OpPrereqError("Cannot create file storage directory"
                                  " '%s': %s" % (file_storage_dir, err),
                                  errors.ECODE_ENVIRON)
@@ -470,7 +470,7 @@ def _InitCheckDrbdHelper(drbd_helper, drbd_enabled):
   if drbd_helper is not None:
     try:
       curr_helper = drbd.DRBD8.GetUsermodeHelper()
-    except errors.BlockDeviceError, err:
+    except errors.BlockDeviceError as err:
       raise errors.OpPrereqError("Error while checking drbd helper"
                                  " (disable drbd with --enabled-disk-templates"
                                  " if you are not using drbd): %s" % str(err),
@@ -485,16 +485,17 @@ def _InitCheckDrbdHelper(drbd_helper, drbd_enabled):
 def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
                 master_netmask, master_netdev, file_storage_dir,
                 shared_file_storage_dir, gluster_storage_dir,
-                candidate_pool_size, secondary_ip=None,
-                vg_name=None, beparams=None, nicparams=None, ndparams=None,
-                hvparams=None, diskparams=None, enabled_hypervisors=None,
-                modify_etc_hosts=True, modify_ssh_setup=True,
-                maintain_node_health=False, drbd_helper=None, uid_pool=None,
-                default_iallocator=None, default_iallocator_params=None,
-                primary_ip_version=None, ipolicy=None,
-                prealloc_wipe_disks=False, use_external_mip_script=False,
-                hv_state=None, disk_state=None, enabled_disk_templates=None,
-                install_image=None, zeroing_image=None, compression_tools=None,
+                candidate_pool_size, ssh_key_type, ssh_key_bits,
+                secondary_ip=None, vg_name=None, beparams=None, nicparams=None,
+                ndparams=None, hvparams=None, diskparams=None,
+                enabled_hypervisors=None, modify_etc_hosts=True,
+                modify_ssh_setup=True, maintain_node_health=False,
+                drbd_helper=None, uid_pool=None, default_iallocator=None,
+                default_iallocator_params=None, primary_ip_version=None,
+                ipolicy=None, prealloc_wipe_disks=False,
+                use_external_mip_script=False, hv_state=None, disk_state=None,
+                enabled_disk_templates=None, install_image=None,
+                zeroing_image=None, compression_tools=None,
                 enabled_user_shutdown=False):
   """Initialise the cluster.
 
@@ -669,7 +670,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
         ds_data[ds_name] = objects.Cluster.SimpleFillDiskState(state)
 
   # hvparams is a mapping of hypervisor->hvparams dict
-  for hv_name, hv_params in hvparams.iteritems():
+  for hv_name, hv_params in hvparams.items():
     utils.ForceDictType(hv_params, constants.HVS_PARAMETER_TYPES)
     hv_class = hypervisor.GetHypervisor(hv_name)
     hv_class.CheckParameterSyntax(hv_params)
@@ -678,7 +679,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
   for template, dt_params in diskparams.items():
     param_keys = set(dt_params.keys())
     default_param_keys = set(constants.DISK_DT_DEFAULTS[template].keys())
-    if not (param_keys <= default_param_keys):
+    if param_keys > default_param_keys:
       unknown_params = param_keys - default_param_keys
       raise errors.OpPrereqError("Invalid parameters for disk template %s:"
                                  " %s" % (template,
@@ -692,7 +693,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
 
   try:
     utils.VerifyDictOptions(diskparams, constants.DISK_DT_DEFAULTS)
-  except errors.OpPrereqError, err:
+  except errors.OpPrereqError as err:
     raise errors.OpPrereqError("While verify diskparam options: %s" % err,
                                errors.ECODE_INVAL)
 
@@ -713,7 +714,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
     utils.AddHostToEtcHosts(hostname.name, hostname.ip)
 
   if modify_ssh_setup:
-    ssh.InitSSHSetup()
+    ssh.InitSSHSetup(ssh_key_type, ssh_key_bits)
 
   if default_iallocator is not None:
     alloc_script = utils.FindFile(default_iallocator,
@@ -733,7 +734,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
   # check if we have all the users we need
   try:
     runtime.GetEnts()
-  except errors.ConfigurationError, err:
+  except errors.ConfigurationError as err:
     raise errors.OpPrereqError("Required system user/group missing: %s" %
                                err, errors.ECODE_ENVIRON)
 
@@ -797,6 +798,8 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
     zeroing_image=zeroing_image,
     compression_tools=compression_tools,
     enabled_user_shutdown=enabled_user_shutdown,
+    ssh_key_type=ssh_key_type,
+    ssh_key_bits=ssh_key_bits,
     )
   master_node_config = objects.Node(name=hostname.name,
                                     primary_ip=hostname.ip,
@@ -814,7 +817,7 @@ def InitCluster(cluster_name, mac_prefix, # pylint: disable=R0913, R0914
 
   master_uuid = cfg.GetMasterNode()
   if modify_ssh_setup:
-    ssh.InitPubKeyFile(master_uuid)
+    ssh.InitPubKeyFile(master_uuid, ssh_key_type)
   # set up the inter-node password and certificate
   _InitGanetiServerSetup(hostname.name, cfg)
 
@@ -876,7 +879,7 @@ def InitConfig(version, cluster_config, master_node_config,
                                    ctime=now, mtime=now)
   utils.WriteFile(cfg_file,
                   data=serializer.Dump(config_data.ToDict()),
-                  mode=0600)
+                  mode=0o600)
 
 
 def FinalizeClusterDestroy(master_uuid):
@@ -931,6 +934,8 @@ def SetupNodeDaemon(opts, cluster_name, node, ssh_port):
     constants.NDS_CLUSTER_NAME: cluster_name,
     constants.NDS_NODE_DAEMON_CERTIFICATE:
       utils.ReadFile(pathutils.NODED_CERT_FILE),
+    constants.NDS_HMAC:
+      utils.ReadFile(pathutils.CONFD_HMAC_KEY),
     constants.NDS_SSCONF: ssconf.SimpleStore().ReadAll(),
     constants.NDS_START_NODE_DAEMON: True,
     constants.NDS_NODE_NAME: node,
@@ -954,6 +959,11 @@ def MasterFailover(no_voting=False):
   current master to cease being master, and the non-master to become
   new master.
 
+  Note: The call to MasterFailover from lib/client/gnt_cluster.py checks that
+  a majority of nodes are healthy and responding before calling this. If this
+  function is called from somewhere else, the caller should also verify that a
+  majority of nodes are healthy.
+
   @type no_voting: boolean
   @param no_voting: force the operation without remote nodes agreement
                       (dangerous)
@@ -982,14 +992,10 @@ def MasterFailover(no_voting=False):
                                errors.ECODE_STATE)
 
   if not no_voting:
-    vote_list = GatherMasterVotes(node_names)
-
+    vote_list = _GatherMasterVotes(node_names)
     if vote_list:
       voted_master = vote_list[0][0]
-      if voted_master is None:
-        raise errors.OpPrereqError("Cluster is inconsistent, most nodes did"
-                                   " not respond.", errors.ECODE_ENVIRON)
-      elif voted_master != old_master:
+      if voted_master != old_master:
         raise errors.OpPrereqError("I have a wrong configuration, I believe"
                                    " the master is %s but the other nodes"
                                    " voted %s. Please resync the configuration"
@@ -1064,7 +1070,7 @@ def MasterFailover(no_voting=False):
                  " %s, please disable manually: %s" % (old_master, msg))
       logging.error("%s", warning)
       warnings.append(warning)
-  except errors.ConfigurationError, err:
+  except errors.ConfigurationError as err:
     logging.error("Error while trying to set the new master: %s",
                   str(err))
     return 1, warnings
@@ -1145,19 +1151,18 @@ def GetMaster():
   return old_master
 
 
-def GatherMasterVotes(node_names):
+def _GatherMasterVotes(node_names):
   """Check the agreement on who is the master.
 
   This function will return a list of (node, number of votes), ordered
-  by the number of votes. Errors will be denoted by the key 'None'.
+  by the number of votes.
 
   Note that the sum of votes is the number of nodes this machine
   knows, whereas the number of entries in the list could be different
   (if some nodes vote for another master).
 
   @type node_names: list
-  @param node_names: the list of nodes to query for master info; the current
-      node will be removed if it is in the list
+  @param node_names: the list of nodes to query for master info
   @rtype: list
   @return: list of (node, votes)
 
@@ -1170,26 +1175,62 @@ def GatherMasterVotes(node_names):
     # this should not happen (unless internal error in rpc)
     logging.critical("Can't complete rpc call, aborting master startup")
     return [(None, len(node_names))]
-  votes = {}
-  for node_name in results:
-    nres = results[node_name]
-    msg = nres.fail_msg
 
+  votes = {}
+  for (node_name, nres) in results.items():
+    msg = nres.fail_msg
     if msg:
       logging.warning("Error contacting node %s: %s", node_name, msg)
-      node = None
-    else:
-      node = nres.payload
-
+      continue
+    node = nres.payload
+    if not node:
+      logging.warning(('Expected a Node, encountered a None. Skipping this'
+                       ' voting result.'))
     if node not in votes:
       votes[node] = 1
     else:
       votes[node] += 1
 
-  vote_list = [v for v in votes.items()]
-  # sort first on number of votes then on name, since we want None
-  # sorted later if we have the half of the nodes not responding, and
-  # half voting all for the same master
-  vote_list.sort(key=lambda x: (x[1], x[0]), reverse=True)
-
+  vote_list = list(votes.items())
+  vote_list.sort(key=lambda x: x[1], reverse=True)
   return vote_list
+
+
+def MajorityHealthy(ignore_offline_nodes=False):
+  """Check if the majority of nodes is healthy
+
+  Gather master votes from all nodes known to this node;
+  return True if a strict majority of nodes is reachable and
+  has some opinion on which node is master. Note that this will
+  not guarantee any node to win an election but it ensures that
+  a standard master-failover is still possible.
+
+  @return: tuple of (boolean, [str]); the first is if a majority of nodes are
+    healthy, the second is a list of the node names that are not considered
+    healthy.
+  """
+  if ignore_offline_nodes:
+    node_names = ssconf.SimpleStore().GetOnlineNodeList()
+  else:
+    node_names = ssconf.SimpleStore().GetNodeList()
+
+  node_count = len(node_names)
+  vote_list = _GatherMasterVotes(node_names)
+
+  if not vote_list:
+    logging.warning(('Voting list was None; cannot determine if a majority of '
+                     'nodes are healthy'))
+    return (False, node_names)
+
+  total_votes = sum([count for (node, count) in vote_list if node is not None])
+  majority_healthy = 2 * total_votes > node_count
+
+  # The list of nodes that did not vote is calculated to provide useful
+  # debugging information to the client.
+  voting_nodes = [node for (node, _) in vote_list]
+  nonvoting_nodes = [node for node in node_names if node not in voting_nodes]
+
+  logging.info("Total %d nodes, %d votes: %s", node_count, total_votes,
+               vote_list)
+
+  return (majority_healthy, nonvoting_nodes)
