@@ -31,6 +31,9 @@
 """Module containing Ganeti's command line parsing options"""
 
 import re
+
+from optparse import (Option, OptionValueError)
+
 import simplejson
 
 from ganeti import utils
@@ -40,8 +43,6 @@ from ganeti import compat
 from ganeti import pathutils
 from ganeti import serializer
 
-from optparse import (Option, OptionValueError)
-
 
 __all__ = [
   "ABSOLUTE_OPT",
@@ -238,6 +239,8 @@ __all__ = [
   "SPLIT_ISPECS_OPTS",
   "SRC_DIR_OPT",
   "SRC_NODE_OPT",
+  "SSH_KEY_BITS_OPT",
+  "SSH_KEY_TYPE_OPT",
   "STARTUP_PAUSED_OPT",
   "STATIC_OPT",
   "SUBMIT_OPT",
@@ -287,7 +290,7 @@ def check_unit(option, opt, value): # pylint: disable=W0613
   """
   try:
     return utils.ParseUnit(value)
-  except errors.UnitParseError, err:
+  except errors.UnitParseError as err:
     raise OptionValueError("option %s: %s" % (opt, err))
 
 
@@ -551,7 +554,7 @@ class CliOption(Option):
 
 
 # optparse.py sets make_option, so we do it for our own option class, too
-cli_option = CliOption
+cli_option = CliOption # pylint: disable=C0103
 
 
 _YORNO = "yes|no"
@@ -846,12 +849,13 @@ IGNORE_CONSIST_OPT = cli_option("--ignore-consistency",
                                 dest="ignore_consistency",
                                 action="store_true", default=False,
                                 help="Ignore the consistency of the disks on"
-                                " the secondary")
+                                " the secondary. The source node must be "
+                                "marked offline first for this to succeed.")
 
 IGNORE_HVVERSIONS_OPT = cli_option("--ignore-hvversions",
                                    dest="ignore_hvversions",
                                    action="store_true", default=False,
-                                   help="Ignore imcompatible hypervisor"
+                                   help="Ignore incompatible hypervisor"
                                    " versions between source and target")
 
 ALLOW_FAILOVER_OPT = cli_option("--allow-failover",
@@ -1200,7 +1204,7 @@ NOSHUTDOWN_OPT = cli_option("--noshutdown", dest="shutdown",
 
 TIMEOUT_OPT = cli_option("--timeout", dest="timeout", type="int",
                          default=constants.DEFAULT_SHUTDOWN_TIMEOUT,
-                         help="Maximum time to wait")
+                         help="Maximum time (in minutes) to wait")
 
 COMPRESS_OPT = cli_option("--compress", dest="compress",
                           type="string", default=constants.IEC_NONE,
@@ -1214,8 +1218,8 @@ TRANSPORT_COMPRESSION_OPT = \
 SHUTDOWN_TIMEOUT_OPT = cli_option("--shutdown-timeout",
                                   dest="shutdown_timeout", type="int",
                                   default=constants.DEFAULT_SHUTDOWN_TIMEOUT,
-                                  help="Maximum time to wait for instance"
-                                  " shutdown")
+                                  help="Maximum time (in minutes) to wait for"
+                                  " instance shutdown")
 
 INTERVAL_OPT = cli_option("--interval", dest="interval", type="int",
                           default=None,
@@ -1366,7 +1370,7 @@ def _PriorityOptionCb(option, _, value, parser):
 
 PRIORITY_OPT = cli_option("--priority", default=None, dest="priority",
                           metavar="|".join(name for name, _ in _PRIORITY_NAMES),
-                          choices=_PRIONAME_TO_VALUE.keys(),
+                          choices=list(_PRIONAME_TO_VALUE),
                           action="callback", type="choice",
                           callback=_PriorityOptionCb,
                           help="Priority for opcode processing")
@@ -1594,6 +1598,17 @@ LONG_SLEEP_OPT = cli_option(
     "--long-sleep", default=False, dest="long_sleep",
     help="Allow long shutdowns when backing up instances", action="store_true")
 
+SSH_KEY_TYPE_OPT = \
+    cli_option("--ssh-key-type", default=None,
+               choices=list(constants.SSHK_ALL), dest="ssh_key_type",
+               help="Type of SSH key deployed by Ganeti for cluster actions")
+
+SSH_KEY_BITS_OPT = \
+    cli_option("--ssh-key-bits", default=None,
+               type="int", dest="ssh_key_bits",
+               help="Length of SSH keys generated by Ganeti, in bits")
+
+
 #: Options provided by all commands
 COMMON_OPTS = [DEBUG_OPT, REASON_OPT]
 
