@@ -35,12 +35,14 @@
 # W0614: Unused import %s from wildcard import (since we need cli)
 # C0103: Invalid name gnt-cluster
 
-from cStringIO import StringIO
+import itertools
 import os
 import time
-import OpenSSL
 import tempfile
-import itertools
+
+from io import StringIO
+
+import OpenSSL
 
 from ganeti.cli import *
 from ganeti import bootstrap
@@ -53,6 +55,7 @@ from ganeti import objects
 from ganeti import opcodes
 from ganeti import pathutils
 from ganeti import qlang
+from ganeti.rpc.node import RunWithRPC
 from ganeti import serializer
 from ganeti import ssconf
 from ganeti import ssh
@@ -73,6 +76,10 @@ FORCE_FAILOVER = cli_option("--yes-do-it", dest="yes_do_it",
                             help="Override interactive check for --no-voting",
                             default=False, action="store_true")
 
+IGNORE_OFFLINE_NODES_FAILOVER = cli_option(
+    "--ignore-offline-nodes", dest="ignore_offline_nodes",
+    help="Ignores offline nodes for master failover voting", default=True)
+
 FORCE_DISTRIBUTION = cli_option("--yes-do-it", dest="yes_do_it",
                                 help="Unconditionally distribute the"
                                 " configuration, even if the queue"
@@ -126,15 +133,15 @@ def _InitVgName(opts, enabled_disk_templates):
   return vg_name
 
 
-def _InitDrbdHelper(opts, enabled_disk_templates):
+def _InitDrbdHelper(opts, enabled_disk_templates, feedback_fn=ToStdout):
   """Initialize the DRBD usermode helper.
 
   """
   drbd_enabled = constants.DT_DRBD8 in enabled_disk_templates
 
   if not drbd_enabled and opts.drbd_helper is not None:
-    ToStdout("Note: You specified a DRBD usermode helper, while DRBD storage"
-             " is not enabled.")
+    feedback_fn("Note: You specified a DRBD usermode helper, while DRBD storage"
+                " is not enabled.")
 
   if drbd_enabled:
     if opts.drbd_helper is None:
@@ -147,7 +154,7 @@ def _InitDrbdHelper(opts, enabled_disk_templates):
   return opts.drbd_helper
 
 
-@UsesRPC
+@RunWithRPC
 def InitCluster(opts, args):
   """Initialize the cluster.
 
@@ -164,7 +171,7 @@ def InitCluster(opts, args):
   try:
     vg_name = _InitVgName(opts, enabled_disk_templates)
     drbd_helper = _InitDrbdHelper(opts, enabled_disk_templates)
-  except errors.OpPrereqError, e:
+  except errors.OpPrereqError as e:
     ToStderr(str(e))
     return 1
 
@@ -193,7 +200,7 @@ def InitCluster(opts, args):
   # check the disk template types here, as we cannot rely on the type check done
   # by the opcode parameter types
   diskparams_keys = set(diskparams.keys())
-  if not (diskparams_keys <= constants.DISK_TEMPLATES):
+  if diskparams_keys > constants.DISK_TEMPLATES:
     unknown = utils.NiceSort(diskparams_keys - constants.DISK_TEMPLATES)
     ToStderr("Disk templates unknown: %s" % utils.CommaJoin(unknown))
     return 1
@@ -261,7 +268,7 @@ def InitCluster(opts, args):
 
   try:
     primary_ip_version = int(opts.primary_ip_version)
-  except (ValueError, TypeError), err:
+  except (ValueError, TypeError) as err:
     ToStderr("Invalid primary ip version value: %s" % str(err))
     return 1
 
@@ -269,7 +276,7 @@ def InitCluster(opts, args):
   try:
     if master_netmask is not None:
       master_netmask = int(master_netmask)
-  except (ValueError, TypeError), err:
+  except (ValueError, TypeError) as err:
     ToStderr("Invalid master netmask value: %s" % str(err))
     return 1
 
@@ -294,10 +301,15 @@ def InitCluster(opts, args):
 
   default_ialloc_params = opts.default_iallocator_params
 
-  if opts.enabled_user_shutdown:
-    enabled_user_shutdown = True
+  enabled_user_shutdown = bool(opts.enabled_user_shutdown)
+
+  if opts.ssh_key_type:
+    ssh_key_type = opts.ssh_key_type
   else:
-    enabled_user_shutdown = False
+    ssh_key_type = constants.SSH_DEFAULT_KEY_TYPE
+
+  ssh_key_bits = ssh.DetermineKeyBits(ssh_key_type, opts.ssh_key_bits, None,
+                                      None)
 
   bootstrap.InitCluster(cluster_name=args[0],
                         secondary_ip=opts.secondary_ip,
@@ -333,13 +345,15 @@ def InitCluster(opts, args):
                         zeroing_image=zeroing_image,
                         compression_tools=compression_tools,
                         enabled_user_shutdown=enabled_user_shutdown,
+                        ssh_key_type=ssh_key_type,
+                        ssh_key_bits=ssh_key_bits,
                         )
   op = opcodes.OpClusterPostInit()
   SubmitOpCode(op, opts=opts)
   return 0
 
 
-@UsesRPC
+@RunWithRPC
 def DestroyCluster(opts, args):
   """Destroy the cluster.
 
@@ -539,7 +553,7 @@ def ShowClusterConfig(opts, args):
     reserved_lvs = "(none)"
 
   enabled_hv = result["enabled_hypervisors"]
-  hvparams = dict((k, v) for k, v in result["hvparams"].iteritems()
+  hvparams = dict((k, v) for k, v in result["hvparams"].items()
                   if k in enabled_hv)
 
   info = [
@@ -612,6 +626,9 @@ def ShowClusterConfig(opts, args):
       ("zeroing image", result["zeroing_image"]),
       ("compression tools", result["compression_tools"]),
       ("enabled user shutdown", result["enabled_user_shutdown"]),
+      ("modify ssh setup", result["modify_ssh_setup"]),
+      ("ssh_key_type", result["ssh_key_type"]),
+      ("ssh_key_bits", result["ssh_key_bits"]),
       ]),
 
     ("Default node parameters",
@@ -755,15 +772,8 @@ def VerifyCluster(opts, args):
 
   results = jex.GetResults()
 
-  (bad_jobs, bad_results) = \
-    map(len,
-        # Convert iterators to lists
-        map(list,
-            # Count errors
-            map(compat.partial(itertools.ifilterfalse, bool),
-                # Convert result to booleans in a tuple
-                zip(*((job_success, len(op_results) == 1 and op_results[0])
-                      for (job_success, op_results) in results)))))
+  bad_jobs = sum(1 for (job_success, _) in results if not job_success)
+  bad_results = sum(1 for (_, op_res) in results if not (op_res and op_res[0]))
 
   if bad_jobs == 0 and bad_results == 0:
     rcode = constants.EXIT_SUCCESS
@@ -787,7 +797,7 @@ def VerifyDisks(opts, args):
   """
   cl = GetClient()
 
-  op = opcodes.OpClusterVerifyDisks()
+  op = opcodes.OpClusterVerifyDisks(group_name=opts.nodegroup)
 
   result = SubmitOpCode(op, cl=cl, opts=opts)
 
@@ -819,25 +829,26 @@ def VerifyDisks(opts, args):
       try:
         ToStdout("Activating disks for instance '%s'", iname)
         SubmitOpCode(op, opts=opts, cl=cl)
-      except errors.GenericError, err:
+      except errors.GenericError as err:
         nret, msg = FormatError(err)
         retcode |= nret
         ToStderr("Error activating disks for instance %s: %s", iname, msg)
 
     if missing:
-      for iname, ival in missing.iteritems():
+      for iname, ival in missing.items():
         all_missing = compat.all(x[0] in bad_nodes for x in ival)
         if all_missing:
           ToStdout("Instance %s cannot be verified as it lives on"
                    " broken nodes", iname)
-        else:
-          ToStdout("Instance %s has missing logical volumes:", iname)
-          ival.sort()
-          for node, vol in ival:
-            if node in bad_nodes:
-              ToStdout("\tbroken node %s /dev/%s", node, vol)
-            else:
-              ToStdout("\t%s /dev/%s", node, vol)
+          continue
+
+        ToStdout("Instance %s has missing logical volumes:", iname)
+        ival.sort()
+        for node, vol in ival:
+          if node in bad_nodes:
+            ToStdout("\tbroken node %s /dev/%s", node, vol)
+          else:
+            ToStdout("\t%s /dev/%s", node, vol)
 
       ToStdout("You need to replace or recreate disks for all the above"
                " instances if this message persists after fixing broken nodes.")
@@ -862,7 +873,7 @@ def RepairDiskSizes(opts, args):
   SubmitOpCode(op, opts=opts)
 
 
-@UsesRPC
+@RunWithRPC
 def MasterFailover(opts, args):
   """Failover the master node.
 
@@ -877,18 +888,31 @@ def MasterFailover(opts, args):
   @return: the desired exit code
 
   """
-  if opts.no_voting and not opts.yes_do_it:
-    usertext = ("This will perform the failover even if most other nodes"
-                " are down, or if this node is outdated. This is dangerous"
-                " as it can lead to a non-consistent cluster. Check the"
-                " gnt-cluster(8) man page before proceeding. Continue?")
-    if not AskUser(usertext):
+  if opts.no_voting:
+    # Don't ask for confirmation if the user provides the confirmation flag.
+    if not opts.yes_do_it:
+      usertext = ("This will perform the failover even if most other nodes"
+                  " are down, or if this node is outdated. This is dangerous"
+                  " as it can lead to a non-consistent cluster. Check the"
+                  " gnt-cluster(8) man page before proceeding. Continue?")
+      if not AskUser(usertext):
+        return 1
+  else:
+    # Verify that a majority of nodes are still healthy
+    (majority_healthy, unhealthy_nodes) = bootstrap.MajorityHealthy(
+          opts.ignore_offline_nodes)
+    if not majority_healthy:
+      ToStderr("Master-failover with voting is only possible if the majority"
+               " of nodes are still healthy; use the --no-voting option after"
+               " ensuring by other means that you won't end up in a dual-master"
+               " scenario. Unhealthy nodes: %s" % unhealthy_nodes)
       return 1
 
-  rvlaue, msgs = bootstrap.MasterFailover(no_voting=opts.no_voting)
+  rvalue, msgs = bootstrap.MasterFailover(no_voting=opts.no_voting)
   for msg in msgs:
     ToStderr(msg)
-  return rvlaue
+
+  return rvalue
 
 
 def MasterPing(opts, args):
@@ -944,31 +968,32 @@ def _ReadAndVerifyCert(cert_filename, verify_private_key=False):
   """
   try:
     pem = utils.ReadFile(cert_filename)
-  except IOError, err:
+  except IOError as err:
     raise errors.X509CertError(cert_filename,
                                "Unable to read certificate: %s" % str(err))
 
   try:
     OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, pem)
-  except Exception, err:
+  except Exception as err:
     raise errors.X509CertError(cert_filename,
                                "Unable to load certificate: %s" % str(err))
 
   if verify_private_key:
     try:
       OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, pem)
-    except Exception, err:
+    except Exception as err:
       raise errors.X509CertError(cert_filename,
                                  "Unable to load private key: %s" % str(err))
 
   return pem
 
 
+# pylint: disable=R0913
 def _RenewCrypto(new_cluster_cert, new_rapi_cert, # pylint: disable=R0911
                  rapi_cert_filename, new_spice_cert, spice_cert_filename,
                  spice_cacert_filename, new_confd_hmac_key, new_cds,
                  cds_filename, force, new_node_cert, new_ssh_keys,
-                 verbose, debug):
+                 ssh_key_type, ssh_key_bits, verbose, debug):
   """Renews cluster certificates, keys and secrets.
 
   @type new_cluster_cert: bool
@@ -996,10 +1021,14 @@ def _RenewCrypto(new_cluster_cert, new_rapi_cert, # pylint: disable=R0911
   @param new_node_cert: Whether to generate new node certificates
   @type new_ssh_keys: bool
   @param new_ssh_keys: Whether to generate new node SSH keys
+  @type ssh_key_type: One of L{constants.SSHK_ALL}
+  @param ssh_key_type: The type of SSH key to be generated
+  @type ssh_key_bits: int
+  @param ssh_key_bits: The length of the key to be generated
   @type verbose: boolean
-  @param verbose: show verbose output
+  @param verbose: Show verbose output
   @type debug: boolean
-  @param debug: show debug output
+  @param debug: Show debug output
 
   """
   ToStdout("Updating certificates now. Running \"gnt-cluster verify\" "
@@ -1033,14 +1062,15 @@ def _RenewCrypto(new_cluster_cert, new_rapi_cert, # pylint: disable=R0911
     if spice_cert_filename:
       spice_cert_pem = _ReadAndVerifyCert(spice_cert_filename, True)
       spice_cacert_pem = _ReadAndVerifyCert(spice_cacert_filename)
-  except errors.X509CertError, err:
-    ToStderr("Unable to load X509 certificate from %s: %s", err[0], err[1])
+  except errors.X509CertError as err:
+    ToStderr("Unable to load X509 certificate from %s: %s",
+             err.args[0], err.args[1])
     return 1
 
   if cds_filename:
     try:
       cds = utils.ReadFile(cds_filename)
-    except Exception, err: # pylint: disable=W0703
+    except Exception as err: # pylint: disable=W0703
       ToStderr("Can't load new cluster domain secret from %s: %s" %
                (cds_filename, str(err)))
       return 1
@@ -1164,7 +1194,7 @@ def _RenewCrypto(new_cluster_cert, new_rapi_cert, # pylint: disable=R0911
   if new_rapi_cert or new_spice_cert or new_confd_hmac_key or new_cds:
     RunWhileClusterStopped(ToStdout, _RenewCryptoInner)
 
-  # If only node certficates are recreated, call _RenewClientCerts only.
+  # If only node certificates are recreated, call _RenewClientCerts only.
   if new_node_cert and not new_cluster_cert:
     RunWhileDaemonsStopped(ToStdout, [constants.NODED, constants.WCONFD],
                            _RenewClientCerts, verbose=verbose, debug=debug)
@@ -1180,7 +1210,9 @@ def _RenewCrypto(new_cluster_cert, new_rapi_cert, # pylint: disable=R0911
     cl = GetClient()
     renew_op = opcodes.OpClusterRenewCrypto(
         node_certificates=new_node_cert or new_cluster_cert,
-        ssh_keys=new_ssh_keys)
+        renew_ssh_keys=new_ssh_keys,
+        ssh_key_type=ssh_key_type,
+        ssh_key_bits=ssh_key_bits)
     SubmitOpCode(renew_op, cl=cl)
 
   ToStdout("All requested certificates and keys have been replaced."
@@ -1197,18 +1229,25 @@ def _BuildGanetiPubKeys(options, pub_key_file=pathutils.SSH_PUB_KEYS, cl=None,
   """Recreates the 'ganeti_pub_key' file by polling all nodes.
 
   """
+
+  if not cl:
+    cl = GetClient()
+
+  (cluster_name, master_node, modify_ssh_setup, ssh_key_type) = \
+    cl.QueryConfigValues(["cluster_name", "master_node", "modify_ssh_setup",
+                          "ssh_key_type"])
+
+  # In case Ganeti is not supposed to modify the SSH setup, simply exit and do
+  # not update this file.
+  if not modify_ssh_setup:
+    return
+
   if os.path.exists(pub_key_file):
     utils.CreateBackup(pub_key_file)
     utils.RemoveFile(pub_key_file)
 
   ssh.ClearPubKeyFile(pub_key_file)
 
-  if not cl:
-    cl = GetClient()
-
-  (cluster_name, master_node) = \
-    cl.QueryConfigValues(["cluster_name", "master_node"])
-
   online_nodes = get_online_nodes_fn([], cl=cl)
   ssh_ports = get_nodes_ssh_ports_fn(online_nodes + [master_node], cl)
   ssh_port_map = dict(zip(online_nodes + [master_node], ssh_ports))
@@ -1221,7 +1260,7 @@ def _BuildGanetiPubKeys(options, pub_key_file=pathutils.SSH_PUB_KEYS, cl=None,
 
   _, pub_key_filename, _ = \
     ssh.GetUserFiles(constants.SSH_LOGIN_USER, mkdir=False, dircheck=False,
-                     kind=constants.SSHK_DSA, _homedir_fn=homedir_fn)
+                     kind=ssh_key_type, _homedir_fn=homedir_fn)
 
   # get the key file of the master node
   pub_key = utils.ReadFile(pub_key_filename)
@@ -1255,6 +1294,8 @@ def RenewCrypto(opts, args):
                       opts.force,
                       opts.new_node_cert,
                       opts.new_ssh_keys,
+                      opts.ssh_key_type,
+                      opts.ssh_key_bits,
                       opts.verbose,
                       opts.debug > 0)
 
@@ -1370,7 +1411,7 @@ def SetClusterParams(opts, args):
 
   try:
     drbd_helper = _GetDrbdHelper(opts, enabled_disk_templates)
-  except errors.OpPrereqError, e:
+  except errors.OpPrereqError as e:
     ToStderr(str(e))
     return 1
 
@@ -1449,7 +1490,7 @@ def SetClusterParams(opts, args):
       for k, v in opts.enabled_data_collectors.items())
 
   unrecognized_data_collectors = [
-      k for k in enabled_data_collectors.keys()
+      k for k in enabled_data_collectors
       if k not in constants.DATA_COLLECTOR_NAMES]
   if unrecognized_data_collectors:
     ToStderr("Data collector names not recognized: %s" %
@@ -1457,7 +1498,7 @@ def SetClusterParams(opts, args):
 
   try:
     data_collector_interval = dict(
-        (k, long(1e6 * float(v)))
+        (k, int(1e6 * float(v)))
         for (k, v) in opts.data_collector_interval.items())
   except ValueError:
     ToStderr("Can't transform all values to integers: {}".format(
@@ -1815,7 +1856,7 @@ def _EpoOff(opts, node_list, inst_map):
   @return: The desired exit status
 
   """
-  if not _InstanceStart(opts, inst_map.keys(), False, no_remember=True):
+  if not _InstanceStart(opts, list(inst_map), False, no_remember=True):
     ToStderr("Please investigate and stop instances manually before continuing")
     return constants.EXIT_FAILURE
 
@@ -1861,7 +1902,7 @@ def Epo(opts, args, qcl=None, _on_fn=_EpoOn, _off_fn=_EpoOff,
                                             "sinst_list", "powered", "offline"],
                           False)
 
-  all_nodes = map(compat.fst, result)
+  all_nodes = [r[0] for r in result]
   node_list = []
   inst_map = {}
   for (node, master, pinsts, sinsts, powered, offline) in result:
@@ -1944,8 +1985,7 @@ def _VerifyCommand(cmd):
   As this function is intended to run during upgrades, it
   is implemented in such a way that it still works, if all Ganeti
   daemons are down.
-
-  @param cmd: the command to execute
+  @param cmd: a list of unquoted shell arguments
   @type cmd: list
   @rtype: list
   @return: the list of node names that are online where
@@ -1953,6 +1993,23 @@ def _VerifyCommand(cmd):
 
   """
   command = utils.text.ShellQuoteArgs([str(val) for val in cmd])
+  return _VerifyCommandRaw(command)
+
+
+def _VerifyCommandRaw(command):
+  """Verify that a given command succeeds on all online nodes.
+
+  As this function is intended to run during upgrades, it
+  is implemented in such a way that it still works, if all Ganeti
+  daemons are down.
+  @param cmd: a bare string to pass to SSH. The caller must do their
+              own shell/ssh escaping.
+  @type cmd: string
+  @rtype: list
+  @return: the list of node names that are online where
+      the command failed.
+
+  """
 
   nodes = ssconf.SimpleStore().GetOnlineNodeList()
   master_node = ssconf.SimpleStore().GetMasterNode()
@@ -2009,35 +2066,51 @@ def _GetRunning():
   return len(cl.Query(constants.QR_JOB, [], qfilter).data)
 
 
-def _SetGanetiVersion(versionstring):
-  """Set the active version of ganeti to the given versionstring
+def _SetGanetiVersionAndEnsure(versionstring):
+  """Symlink the active version of ganeti to the given versionstring,
+  and run the ensure-dirs script.
 
   @type versionstring: string
   @rtype: list
   @return: the list of nodes where the version change failed
 
   """
-  failed = []
+
+  # Update symlinks to point at the new version.
   if constants.HAS_GNU_LN:
-    failed.extend(_VerifyCommand(
-        ["ln", "-s", "-f", "-T",
-         os.path.join(pathutils.PKGLIBDIR, versionstring),
-         os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]))
-    failed.extend(_VerifyCommand(
-        ["ln", "-s", "-f", "-T",
-         os.path.join(pathutils.SHAREDIR, versionstring),
-         os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]))
+    link_lib_cmd = [
+        "ln", "-s", "-f", "-T",
+        os.path.join(pathutils.PKGLIBDIR, versionstring),
+        os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]
+    link_share_cmd = [
+        "ln", "-s", "-f", "-T",
+        os.path.join(pathutils.SHAREDIR, versionstring),
+        os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]
+    cmds = [link_lib_cmd, link_share_cmd]
   else:
-    failed.extend(_VerifyCommand(
-        ["rm", "-f", os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]))
-    failed.extend(_VerifyCommand(
-        ["ln", "-s", "-f", os.path.join(pathutils.PKGLIBDIR, versionstring),
-         os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]))
-    failed.extend(_VerifyCommand(
-        ["rm", "-f", os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]))
-    failed.extend(_VerifyCommand(
-        ["ln", "-s", "-f", os.path.join(pathutils.SHAREDIR, versionstring),
-         os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]))
+    rm_lib_cmd = [
+        "rm", "-f", os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]
+    link_lib_cmd = [
+        "ln", "-s", "-f", os.path.join(pathutils.PKGLIBDIR, versionstring),
+        os.path.join(pathutils.SYSCONFDIR, "ganeti/lib")]
+    rm_share_cmd = [
+        "rm", "-f", os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]
+    ln_share_cmd = [
+        "ln", "-s", "-f", os.path.join(pathutils.SHAREDIR, versionstring),
+        os.path.join(pathutils.SYSCONFDIR, "ganeti/share")]
+    cmds = [rm_lib_cmd, link_lib_cmd, rm_share_cmd, ln_share_cmd]
+
+  # Run the ensure-dirs script to verify the new version is OK.
+  cmds.append([pathutils.ENSURE_DIRS])
+
+  # Submit all commands to ssh, exiting on the first failure.
+  # The command string is a single argument that's given to ssh to submit to
+  # the remote shell, so it only needs enough escaping to satisfy the remote
+  # shell, rather than the 2 levels of escaping usually required when using
+  # ssh from the commandline.
+  quoted_cmds = [utils.text.ShellQuoteArgs(cmd) for cmd in cmds]
+  cmd = " && ".join(quoted_cmds)
+  failed = _VerifyCommandRaw(cmd)
   return list(set(failed))
 
 
@@ -2112,6 +2185,7 @@ def _UpgradeBeforeConfigurationChange(versionstring):
   """
   rollback = []
 
+  ToStdoutAndLoginfo("Verifying %s present on all nodes", versionstring)
   if not _VerifyVersionInstalled(versionstring):
     return (False, rollback)
 
@@ -2119,7 +2193,7 @@ def _UpgradeBeforeConfigurationChange(versionstring):
   rollback.append(
     lambda: utils.RunCmd(["rm", "-f", pathutils.INTENT_TO_UPGRADE]))
 
-  ToStdout("Draining queue")
+  ToStdoutAndLoginfo("Draining queue")
   client = GetClient()
   client.SetQueueDrainFlag(True)
 
@@ -2131,19 +2205,15 @@ def _UpgradeBeforeConfigurationChange(versionstring):
     ToStderr("Failed to completely empty the queue.")
     return (False, rollback)
 
-  ToStdout("Pausing the watcher for one hour.")
+  ToStdoutAndLoginfo("Pausing the watcher for one hour.")
   rollback.append(lambda: GetClient().SetWatcherPause(None))
   GetClient().SetWatcherPause(time.time() + 60 * 60)
 
-  ToStdout("Stopping daemons on master node.")
+  ToStdoutAndLoginfo("Stopping daemons on master node.")
   if not _RunCommandAndReport([pathutils.DAEMON_UTIL, "stop-all"]):
     return (False, rollback)
 
-  if not _VerifyVersionInstalled(versionstring):
-    utils.RunCmd([pathutils.DAEMON_UTIL, "start-all"])
-    return (False, rollback)
-
-  ToStdout("Stopping daemons everywhere.")
+  ToStdoutAndLoginfo("Stopping daemons everywhere.")
   rollback.append(lambda: _VerifyCommand([pathutils.DAEMON_UTIL, "start-all"]))
   badnodes = _VerifyCommand([pathutils.DAEMON_UTIL, "stop-all"])
   if badnodes:
@@ -2151,16 +2221,26 @@ def _UpgradeBeforeConfigurationChange(versionstring):
     return (False, rollback)
 
   backuptar = os.path.join(pathutils.BACKUP_DIR, "ganeti%d.tar" % time.time())
-  ToStdout("Backing up configuration as %s" % backuptar)
+  ToStdoutAndLoginfo("Backing up configuration as %s", backuptar)
   if not _RunCommandAndReport(["mkdir", "-p", pathutils.BACKUP_DIR]):
     return (False, rollback)
 
   # Create the archive in a safe manner, as it contains sensitive
   # information.
   (_, tmp_name) = tempfile.mkstemp(prefix=backuptar, dir=pathutils.BACKUP_DIR)
-  if not _RunCommandAndReport(["tar", "-cf", tmp_name,
-                               "--exclude=queue/archive",
-                               pathutils.DATA_DIR]):
+  tar_cmd = ["tar", "-cf", tmp_name, "--exclude=queue/archive"]
+
+  # Some distributions (e.g. Debian) may set EXPORT_DIR to a subdirectory of
+  # DATA_DIR. Avoid backing up the EXPORT_DIR, as it might contain significant
+  # amounts of data.
+  if utils.IsBelowDir(pathutils.DATA_DIR, pathutils.EXPORT_DIR):
+    tar_cmd.append("--exclude=%s" %
+                   os.path.relpath(pathutils.EXPORT_DIR,
+                                   pathutils.DATA_DIR))
+
+  tar_cmd.append(pathutils.DATA_DIR)
+
+  if not _RunCommandAndReport(tar_cmd):
     return (False, rollback)
 
   os.rename(tmp_name, backuptar)
@@ -2179,7 +2259,7 @@ def _VersionSpecificDowngrade():
 
   @return: True upon success
   """
-  ToStdout("Performing version-specific downgrade tasks.")
+  ToStdoutAndLoginfo("Performing version-specific downgrade tasks.")
 
   return True
 
@@ -2200,7 +2280,7 @@ def _SwitchVersionAndConfig(versionstring, downgrade):
   """
   rollback = []
   if downgrade:
-    ToStdout("Downgrading configuration")
+    ToStdoutAndLoginfo("Downgrading configuration")
     if not _RunCommandAndReport([pathutils.CFGUPGRADE, "--downgrade", "-f"]):
       return (False, rollback)
     # Note: version specific downgrades need to be done before switching
@@ -2212,9 +2292,9 @@ def _SwitchVersionAndConfig(versionstring, downgrade):
   # Configuration change is the point of no return. From then onwards, it is
   # safer to push through the up/dowgrade than to try to roll it back.
 
-  ToStdout("Switching to version %s on all nodes" % versionstring)
-  rollback.append(lambda: _SetGanetiVersion(constants.DIR_VERSION))
-  badnodes = _SetGanetiVersion(versionstring)
+  ToStdoutAndLoginfo("Switching to version %s on all nodes", versionstring)
+  rollback.append(lambda: _SetGanetiVersionAndEnsure(constants.DIR_VERSION))
+  badnodes = _SetGanetiVersionAndEnsure(versionstring)
   if badnodes:
     ToStderr("Failed to switch to Ganeti version %s on nodes %s"
              % (versionstring, ", ".join(badnodes)))
@@ -2227,7 +2307,7 @@ def _SwitchVersionAndConfig(versionstring, downgrade):
   # commands using their canonical (version independent) path.
 
   if not downgrade:
-    ToStdout("Upgrading configuration")
+    ToStdoutAndLoginfo("Upgrading configuration")
     if not _RunCommandAndReport([pathutils.CFGUPGRADE, "-f"]):
       return (False, rollback)
 
@@ -2242,7 +2322,7 @@ def _UpgradeAfterConfigurationChange(oldversion):
   As this part is run at a time where the new version of Ganeti is already
   running, no communication should happen via luxi, as this is not a stable
   interface. Also, as the configuration change is the point of no return,
-  all actions are pushed trough, even if some of them fail.
+  all actions are pushed through, even if some of them fail.
 
   @param oldversion: the version the upgrade started from
   @type oldversion: string
@@ -2252,24 +2332,17 @@ def _UpgradeAfterConfigurationChange(oldversion):
   """
   returnvalue = 0
 
-  ToStdout("Ensuring directories everywhere.")
-  badnodes = _VerifyCommand([pathutils.ENSURE_DIRS])
-  if badnodes:
-    ToStderr("Warning: failed to ensure directories on %s." %
-             (", ".join(badnodes)))
-    returnvalue = 1
-
-  ToStdout("Starting daemons everywhere.")
+  ToStdoutAndLoginfo("Starting daemons everywhere.")
   badnodes = _VerifyCommand([pathutils.DAEMON_UTIL, "start-all"])
   if badnodes:
     ToStderr("Warning: failed to start daemons on %s." % (", ".join(badnodes),))
     returnvalue = 1
 
-  ToStdout("Redistributing the configuration.")
+  ToStdoutAndLoginfo("Redistributing the configuration.")
   if not _RunCommandAndReport(["gnt-cluster", "redist-conf", "--yes-do-it"]):
     returnvalue = 1
 
-  ToStdout("Restarting daemons everywhere.")
+  ToStdoutAndLoginfo("Restarting daemons everywhere.")
   badnodes = _VerifyCommand([pathutils.DAEMON_UTIL, "stop-all"])
   badnodes.extend(_VerifyCommand([pathutils.DAEMON_UTIL, "start-all"]))
   if badnodes:
@@ -2277,21 +2350,21 @@ def _UpgradeAfterConfigurationChange(oldversion):
              (", ".join(list(set(badnodes))),))
     returnvalue = 1
 
-  ToStdout("Undraining the queue.")
+  ToStdoutAndLoginfo("Undraining the queue.")
   if not _RunCommandAndReport(["gnt-cluster", "queue", "undrain"]):
     returnvalue = 1
 
   _RunCommandAndReport(["rm", "-f", pathutils.INTENT_TO_UPGRADE])
 
-  ToStdout("Running post-upgrade hooks")
+  ToStdoutAndLoginfo("Running post-upgrade hooks")
   if not _RunCommandAndReport([pathutils.POST_UPGRADE, oldversion]):
     returnvalue = 1
 
-  ToStdout("Unpausing the watcher.")
+  ToStdoutAndLoginfo("Unpausing the watcher.")
   if not _RunCommandAndReport(["gnt-cluster", "watcher", "continue"]):
     returnvalue = 1
 
-  ToStdout("Verifying cluster.")
+  ToStdoutAndLoginfo("Verifying cluster.")
   if not _RunCommandAndReport(["gnt-cluster", "verify"]):
     returnvalue = 1
 
@@ -2330,6 +2403,8 @@ def UpgradeGanetiCommand(opts, args):
                  " finish it first" % (oldversion, versionstring))
         return 1
 
+  utils.SetupLogging(pathutils.LOG_COMMANDS, 'gnt-cluster upgrade', debug=1)
+
   oldversion = constants.RELEASE_VERSION
 
   if opts.resume:
@@ -2405,17 +2480,17 @@ commands = {
      HV_STATE_OPT, DISK_STATE_OPT, ENABLED_DISK_TEMPLATES_OPT,
      IPOLICY_STD_SPECS_OPT, GLOBAL_GLUSTER_FILEDIR_OPT, INSTALL_IMAGE_OPT,
      ZEROING_IMAGE_OPT, COMPRESSION_TOOLS_OPT,
-     ENABLED_USER_SHUTDOWN_OPT,
+     ENABLED_USER_SHUTDOWN_OPT, SSH_KEY_BITS_OPT, SSH_KEY_TYPE_OPT,
      ]
      + INSTANCE_POLICY_OPTS + SPLIT_ISPECS_OPTS,
-    "[opts...] <cluster_name>", "Initialises a new cluster configuration"),
+    "[<opts>...] <cluster-name>", "Initialises a new cluster configuration"),
   "destroy": (
     DestroyCluster, ARGS_NONE, [YES_DOIT_OPT],
     "", "Destroy cluster"),
   "rename": (
     RenameCluster, [ArgHost(min=1, max=1)],
     [FORCE_OPT, DRY_RUN_OPT],
-    "<new_name>",
+    "<new-name>",
     "Renames the cluster"),
   "redist-conf": (
     RedistributeConfig, ARGS_NONE, SUBMIT_OPTS +
@@ -2429,13 +2504,14 @@ commands = {
      VERIFY_CLUTTER_OPT],
     "", "Does a check on the cluster configuration"),
   "verify-disks": (
-    VerifyDisks, ARGS_NONE, [PRIORITY_OPT],
+    VerifyDisks, ARGS_NONE, [PRIORITY_OPT, NODEGROUP_OPT],
     "", "Does a check on the cluster disk status"),
   "repair-disk-sizes": (
     RepairDiskSizes, ARGS_MANY_INSTANCES, [DRY_RUN_OPT, PRIORITY_OPT],
-    "[instance...]", "Updates mismatches in recorded disk sizes"),
+    "[<instance-name>...]", "Updates mismatches in recorded disk sizes"),
   "master-failover": (
-    MasterFailover, ARGS_NONE, [NOVOTING_OPT, FORCE_FAILOVER],
+    MasterFailover, ARGS_NONE,
+    [NOVOTING_OPT, FORCE_FAILOVER, IGNORE_OFFLINE_NODES_FAILOVER],
     "", "Makes the current node the master"),
   "master-ping": (
     MasterPing, ARGS_NONE, [],
@@ -2449,11 +2525,13 @@ commands = {
   "copyfile": (
     ClusterCopyFile, [ArgFile(min=1, max=1)],
     [NODE_LIST_OPT, USE_REPL_NET_OPT, NODEGROUP_OPT],
-    "[-n node...] <filename>", "Copies a file to all (or only some) nodes"),
+    "[-n <node-name>...] <filename>",
+    "Copies a file to all (or only some) nodes"),
   "command": (
     RunClusterCommand, [ArgCommand(min=1)],
     [NODE_LIST_OPT, NODEGROUP_OPT, SHOW_MACHINE_OPT, FAILURE_ONLY_OPT],
-    "[-n node...] <command>", "Runs a command on all (or only some) nodes"),
+    "[-n <node-name>...] <command>",
+    "Runs a command on all (or only some) nodes"),
   "info": (
     ShowClusterConfig, ARGS_NONE, [ROMAN_OPT],
     "[--roman]", "Show cluster configuration"),
@@ -2461,10 +2539,10 @@ commands = {
     ListTags, ARGS_NONE, [], "", "List the tags of the cluster"),
   "add-tags": (
     AddTags, [ArgUnknown()], [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "tag...", "Add tags to the cluster"),
+    "<tag>...", "Add tags to the cluster"),
   "remove-tags": (
     RemoveTags, [ArgUnknown()], [TAG_SRC_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "tag...", "Remove tags from the cluster"),
+    "<tag>...", "Remove tags from the cluster"),
   "search-tags": (
     SearchTags, [ArgUnknown(min=1, max=1)], [PRIORITY_OPT], "",
     "Searches the tags on all objects on"
@@ -2496,7 +2574,7 @@ commands = {
      [GLOBAL_FILEDIR_OPT, GLOBAL_SHARED_FILEDIR_OPT, ZEROING_IMAGE_OPT,
       COMPRESSION_TOOLS_OPT] +
      [ENABLED_DATA_COLLECTORS_OPT, DATA_COLLECTOR_INTERVAL_OPT],
-    "[opts...]",
+    "[<opts>...]",
     "Alters the parameters of the cluster"),
   "renew-crypto": (
     RenewCrypto, ARGS_NONE,
@@ -2505,14 +2583,14 @@ commands = {
      NEW_CLUSTER_DOMAIN_SECRET_OPT, CLUSTER_DOMAIN_SECRET_OPT,
      NEW_SPICE_CERT_OPT, SPICE_CERT_OPT, SPICE_CACERT_OPT,
      NEW_NODE_CERT_OPT, NEW_SSH_KEY_OPT, NOSSH_KEYCHECK_OPT,
-     VERBOSE_OPT],
-    "[opts...]",
+     VERBOSE_OPT, SSH_KEY_BITS_OPT, SSH_KEY_TYPE_OPT],
+    "[<opts>...]",
     "Renews cluster certificates, keys and secrets"),
   "epo": (
     Epo, [ArgUnknown()],
     [FORCE_OPT, ON_OPT, GROUPS_OPT, ALL_OPT, OOB_TIMEOUT_OPT,
      SHUTDOWN_TIMEOUT_OPT, POWER_DELAY_OPT],
-    "[opts...] [args]",
+    "[<opts>...] [args]",
     "Performs an emergency power-off on given args"),
   "activate-master-ip": (
     ActivateMasterIp, ARGS_NONE, [], "", "Activates the master IP"),
