@@ -34,10 +34,11 @@
 # W0614: Unused import %s from wildcard import (since we need cli)
 # C0103: Invalid name gnt-backup
 
-import simplejson
-import time
-import socket
 import logging
+import socket
+import time
+
+import simplejson
 
 from ganeti.cli import *
 from ganeti import cli
@@ -103,7 +104,6 @@ def GenericOpCodes(opts, args):
     ToStdout("Loading...")
   for job_idx in range(opts.rep_job):
     for fname in args:
-      # pylint: disable=W0142
       op_data = simplejson.loads(utils.ReadFile(fname))
       op_list = [opcodes.OpCode.LoadOpCode(val) for val in op_data]
       op_list = op_list * opts.rep_op
@@ -147,7 +147,7 @@ def TestAllocator(opts, args):
       constants.IDISK_SIZE: utils.ParseUnit(val),
       constants.IDISK_MODE: constants.DISK_RDWR,
       } for val in opts.disks.split(",")]
-  except errors.UnitParseError, err:
+  except errors.UnitParseError as err:
     ToStderr("Invalid disks parameter '%s': %s", opts.disks, err)
     return 1
 
@@ -204,7 +204,7 @@ def _TestJobDependency(opts):
   try:
     cl = cli.GetClient()
     SubmitOpCode(opcodes.OpTestDelay(duration=0, depends=[(-1, None)]), cl=cl)
-  except errors.GenericError, err:
+  except errors.GenericError as err:
     if opts.debug:
       ToStdout("Ignoring error for 'wrong dependencies' test: %s", err)
   else:
@@ -309,7 +309,7 @@ def _TestJobSubmission(opts):
     try:
       cl = cli.GetClient()
       cl.SubmitJob(ops)
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       if opts.debug:
         ToStdout("Ignoring error for 'wrong priority' test: %s", err)
     else:
@@ -326,7 +326,7 @@ def _TestJobSubmission(opts):
     try:
       cl = cli.GetClient()
       cl.SubmitManyJobs(jobs)
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       if opts.debug:
         ToStdout("Ignoring error for 'wrong priority' test: %s", err)
     else:
@@ -527,7 +527,7 @@ def TestJobqueue(opts, _):
 
     try:
       results = cli.PollJob(job_id, cl=cl, reporter=reporter)
-    except errors.OpExecError, err:
+    except errors.OpExecError as err:
       if not fail:
         raise
       ToStdout("Ignoring error for 'job fail' test: %s", err)
@@ -583,6 +583,16 @@ def TestJobqueue(opts, _):
   return 0
 
 
+def TestOsParams(opts, _):
+  """Set secret os parameters.
+
+  """
+  op = opcodes.OpTestOsParams(osparams_secret=opts.osparams_secret)
+  SubmitOrSend(op, opts)
+
+  return 0
+
+
 def ListLocks(opts, args): # pylint: disable=W0613
   """List all locks.
 
@@ -648,7 +658,7 @@ def Metad(opts, args): # pylint: disable=W0613
       ToStderr("Command 'echo' takes only precisely argument.")
       return 1
     result = metad.Client().Echo(args[1])
-    print "Answer: %s" % (result,)
+    print("Answer: %s" % (result,))
   else:
     ToStderr("Command '%s' not supported", args[0])
     return 1
@@ -671,13 +681,13 @@ def Wconfd(opts, args): # pylint: disable=W0613
       ToStderr("Command 'echo' takes only precisely argument.")
       return 1
     result = wconfd.Client().Echo(args[1])
-    print "Answer: %s" % (result,)
+    print("Answer: %s" % (result,))
   elif args[0] == "cleanuplocks":
     if len(args) != 1:
       ToStderr("Command 'cleanuplocks' takes no arguments.")
       return 1
     wconfd.Client().CleanupLocks()
-    print "Stale locks cleaned up."
+    print("Stale locks cleaned up.")
   elif args[0] == "listlocks":
     if len(args) != 2:
       ToStderr("Command 'listlocks' takes precisely one argument.")
@@ -685,25 +695,25 @@ def Wconfd(opts, args): # pylint: disable=W0613
     wconfdcontext = (int(args[1]),
                      utils.livelock.GuessLockfileFor("masterd_1"))
     result = wconfd.Client().ListLocks(wconfdcontext)
-    print "Answer: %s" % (result,)
+    print("Answer: %s" % (result,))
   elif args[0] == "listalllocks":
     if len(args) != 1:
       ToStderr("Command 'listalllocks' takes no arguments.")
       return 1
     result = wconfd.Client().ListAllLocks()
-    print "Answer: %s" % (result,)
+    print("Answer: %s" % (result,))
   elif args[0] == "listalllocksowners":
     if len(args) != 1:
       ToStderr("Command 'listalllocks' takes no arguments.")
       return 1
     result = wconfd.Client().ListAllLocksOwners()
-    print "Answer: %s" % (result,)
+    print("Answer: %s" % (result,))
   elif args[0] == "flushconfig":
     if len(args) != 1:
       ToStderr("Command 'flushconfig' takes no arguments.")
       return 1
     wconfd.Client().FlushConfig()
-    print "Configuration flushed."
+    print("Configuration flushed.")
   else:
     ToStderr("Command '%s' not supported", args[0])
     return 1
@@ -728,7 +738,7 @@ commands = {
                 action="store_true",
                 help="Don't take locks while performing the delay"),
      DRY_RUN_OPT, PRIORITY_OPT] + SUBMIT_OPTS,
-    "[opts...] <duration>", "Executes a TestDelay OpCode"),
+    "[<opts>...] <duration>", "Executes a TestDelay OpCode"),
   "submit-job": (
     GenericOpCodes, [ArgFile(min=1)],
     [VERBOSE_OPT,
@@ -742,7 +752,7 @@ commands = {
                 help="Submit each job separately"),
      DRY_RUN_OPT, PRIORITY_OPT,
      ],
-    "<op_list_file...>", "Submits jobs built from json files"
+    "<op-list-file>...", "Submits jobs built from json files"
     " containing a list of serialized opcodes"),
   "iallocator": (
     TestAllocator, [ArgUnknown(min=1)],
@@ -781,10 +791,14 @@ commands = {
                 default=2, type="int"),
      DRY_RUN_OPT, PRIORITY_OPT,
      ],
-    "{opts...} <instance>", "Executes a TestAllocator OpCode"),
+    "{<opts>...} <instance-name>", "Executes a TestAllocator OpCode"),
   "test-jobqueue": (
     TestJobqueue, ARGS_NONE, [PRIORITY_OPT],
     "", "Test a few aspects of the job queue"),
+  "test-osparams": (
+    TestOsParams, ARGS_NONE, [OSPARAMS_SECRET_OPT] + SUBMIT_OPTS,
+    "[--os-parameters-secret <params>]",
+    "Test secret os parameter transmission"),
   "locks": (
     ListLocks, ARGS_NONE,
     [NOHDR_OPT, SEP_OPT, FIELDS_OPT, INTERVAL_OPT, VERBOSE_OPT],
