@@ -281,7 +281,7 @@ def BatchCreate(opts, args):
 
   try:
     instance_data = simplejson.loads(utils.ReadFile(json_filename))
-  except Exception, err: # pylint: disable=W0703
+  except Exception as err: # pylint: disable=W0703
     ToStderr("Can't parse the instance definition file: %s" % str(err))
     return 1
 
@@ -547,7 +547,7 @@ def RecreateDisks(opts, args):
         try:
           ddict[constants.IDISK_SIZE] = \
             utils.ParseUnit(ddict[constants.IDISK_SIZE])
-        except ValueError, err:
+        except ValueError as err:
           raise errors.OpPrereqError("Invalid disk size for disk %d: %s" %
                                      (didx, err), errors.ECODE_INVAL)
 
@@ -555,7 +555,7 @@ def RecreateDisks(opts, args):
         try:
           ddict[constants.IDISK_SPINDLES] = \
               int(ddict[constants.IDISK_SPINDLES])
-        except ValueError, err:
+        except ValueError as err:
           raise errors.OpPrereqError("Invalid spindles for disk %d: %s" %
                                      (didx, err), errors.ECODE_INVAL)
 
@@ -598,7 +598,7 @@ def GrowDisk(opts, args):
   disk = args[1]
   try:
     disk = int(disk)
-  except (TypeError, ValueError), err:
+  except (TypeError, ValueError) as err:
     raise errors.OpPrereqError("Invalid disk index: %s" % str(err),
                                errors.ECODE_INVAL)
   try:
@@ -692,7 +692,7 @@ def ReplaceDisks(opts, args):
   else:
     try:
       disks = [int(i) for i in opts.disks.split(",")]
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       raise errors.OpPrereqError("Invalid disk index passed: %s" % str(err),
                                  errors.ECODE_INVAL)
   cnt = [opts.on_primary, opts.on_secondary, opts.auto,
@@ -757,7 +757,7 @@ def FailoverInstance(opts, args):
 
   if ignore_consistency:
     usertext = ("To failover instance %s, the source node must be marked"
-                " offline first. Is this aready the case?") % instance_name
+                " offline first. Is this already the case?") % instance_name
     if not AskUser(usertext):
       return 1
 
@@ -925,7 +925,7 @@ def _DoConsole(console, show_command, cluster_name, feedback_fn=ToStdout,
                 console.host, console.port)
   elif console.kind == constants.CONS_SSH:
     # Convert to string if not already one
-    if isinstance(console.command, basestring):
+    if isinstance(console.command, str):
       cmd = console.command
     else:
       cmd = utils.ShellQuoteArgs(console.command)
@@ -1328,7 +1328,7 @@ def _ParseExtStorageParams(params):
                                  " converting to an ExtStorage disk template" %
                                  constants.IDISK_PROVIDER, errors.ECODE_INVAL)
     else:
-      for param in params.keys():
+      for param in params:
         if (param != constants.IDISK_PROVIDER and
             param in constants.IDISK_PARAMS):
           raise errors.OpPrereqError("Invalid parameter '%s' when converting"
@@ -1377,7 +1377,7 @@ def SetInstanceParams(opts, args):
     return 1
 
   for param in opts.beparams:
-    if isinstance(opts.beparams[param], basestring):
+    if isinstance(opts.beparams[param], str):
       if opts.beparams[param].lower() == "default":
         opts.beparams[param] = constants.VALUE_DEFAULT
 
@@ -1385,7 +1385,7 @@ def SetInstanceParams(opts, args):
                       allowed_values=[constants.VALUE_DEFAULT])
 
   for param in opts.hvparams:
-    if isinstance(opts.hvparams[param], basestring):
+    if isinstance(opts.hvparams[param], str):
       if opts.hvparams[param].lower() == "default":
         opts.hvparams[param] = constants.VALUE_DEFAULT
 
