@@ -106,17 +106,17 @@ _USER_STORAGE_TYPE = {
 _STORAGE_TYPE_OPT = \
   cli_option("-t", "--storage-type",
              dest="user_storage_type",
-             choices=_USER_STORAGE_TYPE.keys(),
+             choices=list(_USER_STORAGE_TYPE),
              default=None,
              metavar="STORAGE_TYPE",
              help=("Storage type (%s)" %
-                   utils.CommaJoin(_USER_STORAGE_TYPE.keys())))
+                   utils.CommaJoin(list(_USER_STORAGE_TYPE))))
 
 _REPAIRABLE_STORAGE_TYPES = \
-  [st for st, so in constants.VALID_STORAGE_OPERATIONS.iteritems()
+  [st for st, so in constants.VALID_STORAGE_OPERATIONS.items()
    if constants.SO_FIX_CONSISTENCY in so]
 
-_MODIFIABLE_STORAGE_TYPES = constants.MODIFIABLE_STORAGE_FIELDS.keys()
+_MODIFIABLE_STORAGE_TYPES = list(constants.MODIFIABLE_STORAGE_FIELDS)
 
 _OOB_COMMAND_ASK = compat.UniqueFrozenset([
   constants.OOB_POWER_OFF,
@@ -160,7 +160,7 @@ def _TryReadFile(path):
   """
   try:
     return utils.ReadFile(path)
-  except EnvironmentError, err:
+  except EnvironmentError as err:
     if err.errno == errno.ENOENT:
       return None
     else:
@@ -1126,7 +1126,7 @@ def ListDrbd(opts, args):
 
   def DoConfdRequestReply(req):
     counting_callback.RegisterQuery(req.rsalt)
-    cf_client.SendRequest(req, async=False)
+    cf_client.SendRequest(req, async_=False)
     while not counting_callback.AllAnswered():
       if not cf_client.ReceiveReply():
         ToStderr("Did not receive all expected confd replies")
