@@ -93,43 +93,68 @@ class _VerifyErrors(object):
 
   """
 
-  ETYPE_FIELD = "code"
   ETYPE_ERROR = constants.CV_ERROR
   ETYPE_WARNING = constants.CV_WARNING
 
-  def _Error(self, ecode, item, msg, *args, **kwargs):
-    """Format an error message.
+  def _ErrorMsgList(self, error_descriptor, object_name, message_list,
+                    log_type=ETYPE_ERROR):
+    """Format multiple error messages.
 
     Based on the opcode's error_codes parameter, either format a
     parseable error code, or a simpler error string.
 
     This must be called only from Exec and functions called from Exec.
 
+
+    @type error_descriptor: tuple (string, string, string)
+    @param error_descriptor: triplet describing the error (object_type,
+        code, description)
+    @type object_name: string
+    @param object_name: name of object (instance, node ..) the error relates to
+    @type message_list: list of strings
+    @param message_list: body of error messages
+    @type log_type: string
+    @param log_type: log message type (WARNING, ERROR ..)
     """
-    ltype = kwargs.get(self.ETYPE_FIELD, self.ETYPE_ERROR)
-    itype, etxt, _ = ecode
+    # Called with empty list - nothing to do
+    if not message_list:
+      return
+
+    object_type, error_code, _ = error_descriptor
     # If the error code is in the list of ignored errors, demote the error to a
     # warning
-    if etxt in self.op.ignore_errors:     # pylint: disable=E1101
-      ltype = self.ETYPE_WARNING
-    # first complete the msg
-    if args:
-      msg = msg % args
-    # then format the whole message
+    if error_code in self.op.ignore_errors:     # pylint: disable=E1101
+      log_type = self.ETYPE_WARNING
+
+    prefixed_list = []
     if self.op.error_codes: # This is a mix-in. pylint: disable=E1101
-      msg = "%s:%s:%s:%s:%s" % (ltype, etxt, itype, item, msg)
+      for msg in message_list:
+        prefixed_list.append("  - %s:%s:%s:%s:%s" % (
+            log_type, error_code, object_type, object_name, msg))
     else:
-      if item:
-        item = " " + item
-      else:
-        item = ""
-      msg = "%s: %s%s: %s" % (ltype, itype, item, msg)
-    # and finally report it via the feedback_fn
-    self._feedback_fn("  - %s" % msg) # Mix-in. pylint: disable=E1101
+      if not object_name:
+        object_name = ""
+      for msg in message_list:
+        prefixed_list.append("  - %s: %s %s: %s" % (
+            log_type, object_type, object_name, msg))
+
+    # Report messages via the feedback_fn
+    # pylint: disable=E1101
+    self._feedback_fn(constants.ELOG_MESSAGE_LIST, prefixed_list)
+
     # do not mark the operation as failed for WARN cases only
-    if ltype == self.ETYPE_ERROR:
+    if log_type == self.ETYPE_ERROR:
       self.bad = True
 
+  def _ErrorMsg(self, error_descriptor, object_name, message,
+                log_type=ETYPE_ERROR):
+    """Log a single error message.
+
+    """
+    self._ErrorMsgList(error_descriptor, object_name, [message], log_type)
+
+  # TODO: Replace this method with a cleaner interface, get rid of the if
+  # condition as it only rarely saves lines, but makes things less readable.
   def _ErrorIf(self, cond, *args, **kwargs):
     """Log an error message if the passed condition is True.
 
@@ -138,6 +163,17 @@ class _VerifyErrors(object):
         or self.op.debug_simulate_errors): # pylint: disable=E1101
       self._Error(*args, **kwargs)
 
+  # TODO: Replace this method with a cleaner interface
+  def _Error(self, ecode, item, message, *args, **kwargs):
+    """Log an error message if the passed condition is True.
+
+    """
+    #TODO: Remove 'code' argument in favour of using log_type
+    log_type = kwargs.get('code', self.ETYPE_ERROR)
+    if args:
+      message = message % args
+    self._ErrorMsgList(ecode, item, [message], log_type=log_type)
+
 
 class LUClusterVerify(NoHooksLU):
   """Submits all jobs necessary to verify the cluster.
@@ -173,7 +209,7 @@ class LUClusterVerify(NoHooksLU):
       for group in groups)
 
     # Fix up all parameters
-    for op in itertools.chain(*jobs): # pylint: disable=W0142
+    for op in itertools.chain(*jobs):
       op.debug_simulate_errors = self.op.debug_simulate_errors
       op.verbose = self.op.verbose
       op.error_codes = self.op.error_codes
@@ -193,16 +229,37 @@ class LUClusterVerifyDisks(NoHooksLU):
 
   def ExpandNames(self):
     self.share_locks = ShareAll()
-    self.needed_locks = {
-      locking.LEVEL_NODEGROUP: locking.ALL_SET,
-      }
+    if self.op.group_name:
+      self.needed_locks = {
+        locking.LEVEL_NODEGROUP: [self.cfg.LookupNodeGroup(self.op.group_name)]
+        }
+    else:
+      self.needed_locks = {
+        locking.LEVEL_NODEGROUP: locking.ALL_SET,
+        }
 
   def Exec(self, feedback_fn):
     group_names = self.owned_locks(locking.LEVEL_NODEGROUP)
+    instances = self.cfg.GetInstanceList()
 
-    # Submit one instance of L{opcodes.OpGroupVerifyDisks} per node group
-    return ResultWithJobs([[opcodes.OpGroupVerifyDisks(group_name=group)]
-                           for group in group_names])
+    only_ext = compat.all(
+        self.cfg.GetInstanceDiskTemplate(i) == constants.DT_EXT
+        for i in instances)
+
+    # We skip current NodeGroup verification if there are only external storage
+    # devices. Currently we provide an interface for external storage provider
+    # for disk verification implementations, however current ExtStorageDevice
+    # does not provide an API for this yet.
+    #
+    # This check needs to be revisited if ES_ACTION_VERIFY on ExtStorageDevice
+    # is implemented.
+    if only_ext:
+      logging.info("All instances have ext storage, skipping verify disks.")
+      return ResultWithJobs([])
+    else:
+      # Submit one instance of L{opcodes.OpGroupVerifyDisks} per node group
+      return ResultWithJobs([[opcodes.OpGroupVerifyDisks(group_name=group)]
+                             for group in group_names])
 
 
 class LUClusterVerifyConfig(NoHooksLU, _VerifyErrors):
@@ -222,7 +279,7 @@ class LUClusterVerifyConfig(NoHooksLU, _VerifyErrors):
         hv_class = hypervisor.GetHypervisorClass(hv_name)
         utils.ForceDictType(hv_params, constants.HVS_PARAMETER_TYPES)
         hv_class.CheckParameterSyntax(hv_params)
-      except errors.GenericError, err:
+      except errors.GenericError as err:
         self._ErrorIf(True, constants.CV_ECLUSTERCFG, None, msg % str(err))
 
   def ExpandNames(self):
@@ -247,8 +304,8 @@ class LUClusterVerifyConfig(NoHooksLU, _VerifyErrors):
 
     feedback_fn("* Verifying cluster config")
 
-    for msg in self.cfg.VerifyConfig():
-      self._ErrorIf(True, constants.CV_ECLUSTERCFG, None, msg)
+    msg_list = self.cfg.VerifyConfig()
+    self._ErrorMsgList(constants.CV_ECLUSTERCFG, None, msg_list)
 
     feedback_fn("* Verifying cluster certificate files")
 
@@ -265,8 +322,9 @@ class LUClusterVerifyConfig(NoHooksLU, _VerifyErrors):
 
     feedback_fn("* Verifying hypervisor parameters")
 
+    instances_info = list(self.all_inst_info.values())
     self._VerifyHVP(_GetAllHypervisorParameters(self.cfg.GetClusterInfo(),
-                                                self.all_inst_info.values()))
+                                                instances_info))
 
     feedback_fn("* Verifying all nodes belong to an existing group")
 
@@ -450,7 +508,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
 
     # We detect here the nodes that will need the extra RPC calls for verifying
     # split LV volumes; they should be locked.
-    extra_lv_nodes = set()
+    extra_lv_nodes = {}
 
     for inst in self.my_inst_info.values():
       disks = self.cfg.GetInstanceDisks(inst.uuid)
@@ -458,16 +516,23 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
         inst_nodes = self.cfg.GetInstanceNodes(inst.uuid)
         for nuuid in inst_nodes:
           if self.all_node_info[nuuid].group != self.group_uuid:
-            extra_lv_nodes.add(nuuid)
+            if nuuid in extra_lv_nodes:
+              extra_lv_nodes[nuuid].append(inst.name)
+            else:
+              extra_lv_nodes[nuuid] = [inst.name]
 
+    extra_lv_nodes_set = set(extra_lv_nodes)
     unlocked_lv_nodes = \
-        extra_lv_nodes.difference(self.owned_locks(locking.LEVEL_NODE))
+        extra_lv_nodes_set.difference(self.owned_locks(locking.LEVEL_NODE))
 
     if unlocked_lv_nodes:
+      node_strings = ['%s: [%s]' % (
+          self.cfg.GetNodeName(node), utils.CommaJoin(extra_lv_nodes[node]))
+            for node in unlocked_lv_nodes]
       raise errors.OpPrereqError("Missing node locks for LV check: %s" %
-                                 utils.CommaJoin(unlocked_lv_nodes),
+                                 utils.CommaJoin(node_strings),
                                  errors.ECODE_STATE)
-    self.extra_lv_nodes = list(extra_lv_nodes)
+    self.extra_lv_nodes = list(extra_lv_nodes_set)
 
   def _VerifyNode(self, ninfo, nresult):
     """Perform some basic validation on data returned from a node.
@@ -520,7 +585,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
 
     hyp_result = nresult.get(constants.NV_HYPERVISOR, None)
     if ninfo.vm_capable and isinstance(hyp_result, dict):
-      for hv_name, hv_result in hyp_result.iteritems():
+      for hv_name, hv_result in hyp_result.items():
         test = hv_result is not None
         self._ErrorIf(test, constants.CV_ENODEHV, ninfo.name,
                       "hypervisor %s verify failure: '%s'", hv_name, hv_result)
@@ -641,7 +706,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     # exclusive_storage wants all PVs to have the same size (approximately),
     # if the smallest and the biggest ones are okay, everything is fine.
     # pv_min is None iff pv_max is None
-    vals = filter((lambda ni: ni.pv_min is not None), node_image.values())
+    vals = [ni for ni in node_image.values() if ni.pv_min is not None]
     if not vals:
       return
     (pvmin, minnode_uuid) = min((ni.pv_min, ni.uuid) for ni in vals)
@@ -708,27 +773,26 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
           self._ErrorIf(True, constants.CV_ENODESSH, ninfo.name,
                         "ssh communication with node '%s': %s", a_node, a_msg)
 
-    test = constants.NV_NODENETTEST not in nresult
-    self._ErrorIf(test, constants.CV_ENODENET, ninfo.name,
-                  "node hasn't returned node tcp connectivity data")
-    if not test:
-      if nresult[constants.NV_NODENETTEST]:
-        nlist = utils.NiceSort(nresult[constants.NV_NODENETTEST].keys())
-        for anode in nlist:
-          self._ErrorIf(True, constants.CV_ENODENET, ninfo.name,
-                        "tcp communication with node '%s': %s",
-                        anode, nresult[constants.NV_NODENETTEST][anode])
+    if constants.NV_NODENETTEST not in nresult:
+      self._ErrorMsg(constants.CV_ENODENET, ninfo.name,
+                     "node hasn't returned node tcp connectivity data")
+    elif nresult[constants.NV_NODENETTEST]:
+      nlist = utils.NiceSort(nresult[constants.NV_NODENETTEST])
+      msglist = []
+      for node in nlist:
+        msglist.append("tcp communication with node '%s': %s" %
+                       (node, nresult[constants.NV_NODENETTEST][node]))
+      self._ErrorMsgList(constants.CV_ENODENET, ninfo.name, msglist)
 
-    test = constants.NV_MASTERIP not in nresult
-    self._ErrorIf(test, constants.CV_ENODENET, ninfo.name,
-                  "node hasn't returned node master IP reachability data")
-    if not test:
-      if not nresult[constants.NV_MASTERIP]:
-        if ninfo.uuid == self.master_node:
-          msg = "the master node cannot reach the master IP (not configured?)"
-        else:
-          msg = "cannot reach the master IP"
-        self._ErrorIf(True, constants.CV_ENODENET, ninfo.name, msg)
+    if constants.NV_MASTERIP not in nresult:
+      self._ErrorMsg(constants.CV_ENODENET, ninfo.name,
+                     "node hasn't returned node master IP reachability data")
+    elif nresult[constants.NV_MASTERIP] is False:  # be explicit, could be None
+      if ninfo.uuid == self.master_node:
+        msg = "the master node cannot reach the master IP (not configured?)"
+      else:
+        msg = "cannot reach the master IP"
+      self._ErrorMsg(constants.CV_ENODENET, ninfo.name, msg)
 
   def _VerifyInstance(self, instance, node_image, diskstatus):
     """Verify an instance.
@@ -851,7 +915,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
                            groupinfo[group].name)
         # Sort so that we always list the primary node first.
         for group, nodes in sorted(instance_groups.items(),
-                                   key=lambda (_, nodes): pnode_uuid in nodes,
+                                   key=lambda g_n: pnode_uuid in g_n[1],
                                    reverse=True)]
 
       self._ErrorIf(len(instance_groups) > 1,
@@ -894,6 +958,10 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
 
     @type vg_name: string
     @param vg_name: the name of the Ganeti-administered volume group
+    @type node_vol_should: dict
+    @param node_vol_should: mapping of node UUIDs to expected LVs on each node
+    @type node_image: dict
+    @param node_image: mapping of node UUIDs to L{NodeImage} objects
     @type reserved: L{ganeti.utils.FieldSet}
     @param reserved: a FieldSet of reserved volume names
 
@@ -954,6 +1022,11 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
                       " should node %s fail (%dMiB needed, %dMiB available)",
                       self.cfg.GetNodeName(prinode), needed_mem, n_img.mfree)
 
+  def _CertError(self, *args):
+    """Helper function for _VerifyClientCertificates."""
+    self._Error(constants.CV_ECLUSTERCLIENTCERT, None, *args)
+    self._cert_error_found = True
+
   def _VerifyClientCertificates(self, nodes, all_nvinfo):
     """Verifies the consistency of the client certificates.
 
@@ -968,20 +1041,25 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
       all nodes
 
     """
-    candidate_certs = self.cfg.GetClusterInfo().candidate_certs
-    if candidate_certs is None or len(candidate_certs) == 0:
-      self._ErrorIf(
-        True, constants.CV_ECLUSTERCLIENTCERT, None,
-        "The cluster's list of master candidate certificates is empty."
-        " If you just updated the cluster, please run"
+
+    rebuild_certs_msg = (
+        "To rebuild node certificates, please run"
         " 'gnt-cluster renew-crypto --new-node-certificates'.")
+
+    self._cert_error_found = False
+
+    candidate_certs = self.cfg.GetClusterInfo().candidate_certs
+    if not candidate_certs:
+      self._CertError(
+        "The cluster's list of master candidate certificates is empty."
+        " This may be because you just updated the cluster. " +
+        rebuild_certs_msg)
       return
 
-    self._ErrorIf(
-      len(candidate_certs) != len(set(candidate_certs.values())),
-      constants.CV_ECLUSTERCLIENTCERT, None,
-      "There are at least two master candidates configured to use the same"
-      " certificate.")
+    if len(candidate_certs) != len(set(candidate_certs.values())):
+      self._CertError(
+        "There are at least two master candidates configured to use the same"
+        " certificate.")
 
     # collect the client certificate
     for node in nodes:
@@ -994,45 +1072,42 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
 
       (errcode, msg) = nresult.payload.get(constants.NV_CLIENT_CERT, None)
 
-      self._ErrorIf(
-        errcode is not None, constants.CV_ECLUSTERCLIENTCERT, None,
-        "Client certificate of node '%s' failed validation: %s (code '%s')",
-        node.uuid, msg, errcode)
-
+      if errcode is not None:
+        self._CertError(
+          "Client certificate of node '%s' failed validation: %s (code '%s')",
+          node.uuid, msg, errcode)
       if not errcode:
         digest = msg
         if node.master_candidate:
           if node.uuid in candidate_certs:
-            self._ErrorIf(
-              digest != candidate_certs[node.uuid],
-              constants.CV_ECLUSTERCLIENTCERT, None,
-              "Client certificate digest of master candidate '%s' does not"
-              " match its entry in the cluster's map of master candidate"
-              " certificates. Expected: %s Got: %s", node.uuid,
-              digest, candidate_certs[node.uuid])
+            if digest != candidate_certs[node.uuid]:
+              self._CertError(
+                "Client certificate digest of master candidate '%s' does not"
+                " match its entry in the cluster's map of master candidate"
+                " certificates. Expected: %s Got: %s", node.uuid,
+                digest, candidate_certs[node.uuid])
           else:
-            self._ErrorIf(
-              True, constants.CV_ECLUSTERCLIENTCERT, None,
+            self._CertError(
               "The master candidate '%s' does not have an entry in the"
               " map of candidate certificates.", node.uuid)
-            self._ErrorIf(
-              digest in candidate_certs.values(),
-              constants.CV_ECLUSTERCLIENTCERT, None,
-              "Master candidate '%s' is using a certificate of another node.",
-              node.uuid)
+            if digest in candidate_certs.values():
+              self._CertError(
+                "Master candidate '%s' is using a certificate of another node.",
+                node.uuid)
         else:
-          self._ErrorIf(
-            node.uuid in candidate_certs,
-            constants.CV_ECLUSTERCLIENTCERT, None,
-            "Node '%s' is not a master candidate, but still listed in the"
-            " map of master candidate certificates.", node.uuid)
-          self._ErrorIf(
-            (node.uuid not in candidate_certs) and
-              (digest in candidate_certs.values()),
-            constants.CV_ECLUSTERCLIENTCERT, None,
-            "Node '%s' is not a master candidate and is incorrectly using a"
-            " certificate of another node which is master candidate.",
-            node.uuid)
+          if node.uuid in candidate_certs:
+            self._CertError(
+              "Node '%s' is not a master candidate, but still listed in the"
+              " map of master candidate certificates.", node.uuid)
+          if (node.uuid not in candidate_certs and
+              digest in candidate_certs.values()):
+            self._CertError(
+              "Node '%s' is not a master candidate and is incorrectly using a"
+              " certificate of another node which is master candidate.",
+              node.uuid)
+
+    if self._cert_error_found:
+      self._CertError(rebuild_certs_msg)
 
   def _VerifySshSetup(self, nodes, all_nvinfo):
     """Evaluates the verification results of the SSH setup and clutter test.
@@ -1056,8 +1131,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
           self._ErrorIf(result,
                         constants.CV_ENODESSH, None, error_msg)
 
-  def _VerifyFiles(self, nodes, master_node_uuid, all_nvinfo,
-                   (files_all, files_opt, files_mc, files_vm)):
+  def _VerifyFiles(self, nodes, master_node_uuid, all_nvinfo, filemap):
     """Verifies file checksums collected from all nodes.
 
     @param nodes: List of L{objects.Node} objects
@@ -1065,7 +1139,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     @param all_nvinfo: RPC results
 
     """
-    # Define functions determining which nodes to consider for a file
+    (files_all, files_opt, files_mc, files_vm) = filemap
     files2nodefn = [
       (files_all, None),
       (files_mc, lambda node: (node.master_candidate or
@@ -1079,9 +1153,8 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
       if fn is None:
         filenodes = nodes
       else:
-        filenodes = filter(fn, nodes)
-      nodefiles.update((filename,
-                        frozenset(map(operator.attrgetter("uuid"), filenodes)))
+        filenodes = [n for n in nodes if fn(n)]
+      nodefiles.update((filename, frozenset(fn.uuid for fn in filenodes))
                        for filename in files)
 
     assert set(nodefiles) == (files_all | files_mc | files_vm)
@@ -1136,23 +1209,22 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
                       "File %s is optional, but it must exist on all or no"
                       " nodes (not found on %s)",
                       filename,
-                      utils.CommaJoin(
-                        utils.NiceSort(
-                          map(self.cfg.GetNodeName, missing_file))))
+                      utils.CommaJoin(utils.NiceSort(
+                        self.cfg.GetNodeName(n) for n in missing_file)))
       else:
         self._ErrorIf(missing_file, constants.CV_ECLUSTERFILECHECK, None,
                       "File %s is missing from node(s) %s", filename,
-                      utils.CommaJoin(
-                        utils.NiceSort(
-                          map(self.cfg.GetNodeName, missing_file))))
+                      utils.CommaJoin(utils.NiceSort(
+                        self.cfg.GetNodeName(n) for n in missing_file)))
 
         # Warn if a node has a file it shouldn't
         unexpected = with_file - expected_nodes
         self._ErrorIf(unexpected,
                       constants.CV_ECLUSTERFILECHECK, None,
                       "File %s should not exist on node(s) %s",
-                      filename, utils.CommaJoin(
-                        utils.NiceSort(map(self.cfg.GetNodeName, unexpected))))
+                      filename,
+                      utils.CommaJoin(utils.NiceSort(
+                        self.cfg.GetNodeName(n) for n in unexpected)))
 
       # See if there are multiple versions of the file
       test = len(checksums) > 1
@@ -1160,7 +1232,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
         variants = ["variant %s on %s" %
                     (idx + 1,
                      utils.CommaJoin(utils.NiceSort(
-                       map(self.cfg.GetNodeName, node_uuids))))
+                       self.cfg.GetNodeName(n) for n in node_uuids)))
                     for (idx, (checksum, node_uuids)) in
                       enumerate(sorted(checksums.items()))]
       else:
@@ -1358,7 +1430,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
                       self.cfg.GetNodeName(base.uuid), a, b)
 
     # check any missing OSes
-    missing = set(base.oslist.keys()).difference(nimg.oslist.keys())
+    missing = set(base.oslist).difference(nimg.oslist)
     self._ErrorIf(missing, constants.CV_ENODEOS, ninfo.name,
                   "OSes present on reference node %s"
                   " but missing on this node: %s",
@@ -1489,7 +1561,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     lvdata = nresult.get(constants.NV_LVLIST, "Missing LV data")
     if vg_name is None:
       pass
-    elif isinstance(lvdata, basestring):
+    elif isinstance(lvdata, str):
       self._ErrorIf(True, constants.CV_ENODELVM, ninfo.name,
                     "LVM problem on node: %s", utils.SafeEncode(lvdata))
     elif not isinstance(lvdata, dict):
@@ -1609,7 +1681,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
 
     # Collect data from all nodes with disks
     result = self.rpc.call_blockdev_getmirrorstatus_multi(
-               node_disks.keys(), node_disks_dev_inst_only)
+               list(node_disks), node_disks_dev_inst_only)
 
     assert len(result) == len(node_disks)
 
@@ -1678,10 +1750,10 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
                  not node.offline)]
     keyfunc = operator.attrgetter("group")
 
-    return map(itertools.cycle,
-               [sorted(map(operator.attrgetter("name"), names))
-                for _, names in itertools.groupby(sorted(nodes, key=keyfunc),
-                                                  keyfunc)])
+    nodes_by_group = itertools.groupby(sorted(nodes, key=keyfunc), keyfunc)
+    node_names = (sorted(n.name for n in names) for _, names in nodes_by_group)
+
+    return [itertools.cycle(ns) for ns in node_names]
 
   @classmethod
   def _SelectSshCheckNodes(cls, group_nodes, group_uuid, all_nodes):
@@ -1707,7 +1779,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     sel = cls._SshNodeSelector(group_uuid, all_nodes)
 
     return (online_nodes,
-            dict((name, sorted([i.next() for i in sel]))
+            dict((name, sorted([next(i) for i in sel]))
                  for name in online_nodes),
             online_mcs)
 
@@ -1743,7 +1815,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     """Build hooks nodes.
 
     """
-    return ([], list(self.my_node_info.keys()))
+    return ([], list(self.my_node_info))
 
   @staticmethod
   def _VerifyOtherNotes(feedback_fn, i_non_redundant, i_non_a_balanced,
@@ -1766,6 +1838,36 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     if n_drained:
       feedback_fn("  - NOTICE: %d drained node(s) found." % n_drained)
 
+  def _VerifyExclusionTags(self, nodename, pinst, ctags):
+    """Verify that all instances have different exclusion tags.
+
+    @type nodename: string
+    @param nodename: the name of the node for which the check is done
+    @type pinst: list of string
+    @param pinst: list of UUIDs of those instances having the given node
+        as primary node
+    @type ctags: list of string
+    @param ctags: tags of the cluster
+
+    """
+    exclusion_prefixes = utils.GetExclusionPrefixes(ctags)
+    tags_seen = set([])
+    conflicting_tags = set([])
+    for iuuid in pinst:
+      allitags = self.my_inst_info[iuuid].tags
+      if allitags is None:
+        allitags = []
+      itags = set([tag for tag in allitags
+                   if utils.IsGoodTag(exclusion_prefixes, tag)])
+      conflicts = itags.intersection(tags_seen)
+      if len(conflicts) > 0:
+        conflicting_tags = conflicting_tags.union(conflicts)
+      tags_seen = tags_seen.union(itags)
+
+    self._ErrorIf(len(conflicting_tags) > 0, constants.CV_EEXTAGS, nodename,
+                  "Tags where there is more than one instance: %s",
+                  list(conflicting_tags), code=constants.CV_WARNING)
+
   def Exec(self, feedback_fn): # pylint: disable=R0915
     """Verify integrity of the node group, performing various test on nodes.
 
@@ -1786,7 +1888,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     drbd_helper = self.cfg.GetDRBDHelper()
     cluster = self.cfg.GetClusterInfo()
     hypervisors = cluster.enabled_hypervisors
-    node_data_list = self.my_node_info.values()
+    node_data_list = list(self.my_node_info.values())
 
     i_non_redundant = [] # Non redundant instances
     i_non_a_balanced = [] # Non auto-balanced instances
@@ -1804,33 +1906,40 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     master_node_uuid = self.master_node = self.cfg.GetMasterNode()
     master_ip = self.cfg.GetMasterIP()
 
+    online_master_candidates = sorted(
+        node.name for node in node_data_list
+        if (node.master_candidate and not node.offline))
+
     feedback_fn("* Gathering data (%d nodes)" % len(self.my_node_uuids))
 
     user_scripts = []
     if self.cfg.GetUseExternalMipScript():
       user_scripts.append(pathutils.EXTERNAL_MASTER_SETUP_SCRIPT)
 
+    online_nodes = [(node.name, node.primary_ip, node.secondary_ip)
+                    for node in node_data_list if not node.offline]
+    node_nettest_params = (online_nodes, online_master_candidates)
+
     node_verify_param = {
       constants.NV_FILELIST:
-        map(vcluster.MakeVirtualPath,
-            utils.UniqueSequence(filename
-                                 for files in filemap
-                                 for filename in files)),
+        [vcluster.MakeVirtualPath(f)
+         for f in utils.UniqueSequence(filename
+                                       for files in filemap
+                                       for filename in files)],
       constants.NV_NODELIST:
         self._SelectSshCheckNodes(node_data_list, self.group_uuid,
-                                  self.all_node_info.values()),
+                                  list(self.all_node_info.values())),
       constants.NV_HYPERVISOR: hypervisors,
       constants.NV_HVPARAMS:
-        _GetAllHypervisorParameters(cluster, self.all_inst_info.values()),
-      constants.NV_NODENETTEST: [(node.name, node.primary_ip, node.secondary_ip)
-                                 for node in node_data_list
-                                 if not node.offline],
+        _GetAllHypervisorParameters(cluster, list(self.all_inst_info.values())),
+      constants.NV_NODENETTEST: node_nettest_params,
       constants.NV_INSTANCELIST: hypervisors,
       constants.NV_VERSION: None,
       constants.NV_HVINFO: self.cfg.GetHypervisorType(),
       constants.NV_NODESETUP: None,
       constants.NV_TIME: None,
-      constants.NV_MASTERIP: (self.cfg.GetMasterNodeName(), master_ip),
+      constants.NV_MASTERIP: (self.cfg.GetMasterNodeName(), master_ip,
+                              online_master_candidates),
       constants.NV_OSLIST: None,
       constants.NV_NONVMNODES: self.cfg.GetNonVmCapableNodeNameList(),
       constants.NV_USERSCRIPTS: user_scripts,
@@ -1838,7 +1947,8 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
       }
 
     if self.cfg.GetClusterInfo().modify_ssh_setup:
-      node_verify_param[constants.NV_SSH_SETUP] = self._PrepareSshSetupCheck()
+      node_verify_param[constants.NV_SSH_SETUP] = \
+        (self._PrepareSshSetupCheck(), self.cfg.GetClusterInfo().ssh_key_type)
       if self.op.verify_clutter:
         node_verify_param[constants.NV_SSH_CLUTTER] = True
 
@@ -1921,17 +2031,13 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
         nimg.sbp[pnode].append(instance.uuid)
 
     es_flags = rpc.GetExclusiveStorageForNodes(self.cfg,
-                                               self.my_node_info.keys())
+                                               list(self.my_node_info))
     # The value of exclusive_storage should be the same across the group, so if
     # it's True for at least a node, we act as if it were set for all the nodes
     self._exclusive_storage = compat.any(es_flags.values())
     if self._exclusive_storage:
       node_verify_param[constants.NV_EXCLUSIVEPVS] = True
 
-    node_group_uuids = dict(map(lambda n: (n.name, n.group),
-                                self.cfg.GetAllNodesInfo().values()))
-    groups_config = self.cfg.GetAllNodeGroupsInfoDict()
-
     # At this point, we have the in-memory data structures complete,
     # except for the runtime information, which we'll gather next
 
@@ -1949,7 +2055,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
       feedback_fn("* Gathering information about nodes (%s nodes)" %
                   len(self.my_node_uuids))
       # Force the configuration to be fully distributed before doing any tests
-      self.cfg.FlushConfig()
+      self.cfg.FlushConfigGroup(self.group_uuid)
       # Due to the way our RPC system works, exact response times cannot be
       # guaranteed (e.g. a broken node could run into a timeout). By keeping
       # the time before and after executing the request, we can at least have
@@ -1962,12 +2068,11 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
       # locking the configuration for something but very fast, pure operations.
       cluster_name = self.cfg.GetClusterName()
       hvparams = self.cfg.GetClusterInfo().hvparams
+
       all_nvinfo = self.rpc.call_node_verify(self.my_node_uuids,
                                              node_verify_param,
                                              cluster_name,
-                                             hvparams,
-                                             node_group_uuids,
-                                             groups_config)
+                                             hvparams)
       nvinfo_endtime = time.time()
 
       if self.extra_lv_nodes and vg_name is not None:
@@ -1977,9 +2082,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
             self.rpc.call_node_verify(self.extra_lv_nodes,
                                       {constants.NV_LVLIST: vg_name},
                                       self.cfg.GetClusterName(),
-                                      self.cfg.GetClusterInfo().hvparams,
-                                      node_group_uuids,
-                                      groups_config)
+                                      self.cfg.GetClusterInfo().hvparams)
       else:
         extra_lv_nvinfo = {}
 
@@ -2007,25 +2110,23 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
         feedback_fn("* Gathering information about the master node")
         vf_nvinfo.update(self.rpc.call_node_verify(
            additional_node_uuids, {key: node_verify_param[key]},
-           self.cfg.GetClusterName(), self.cfg.GetClusterInfo().hvparams,
-           node_group_uuids,
-           groups_config))
+           self.cfg.GetClusterName(), self.cfg.GetClusterInfo().hvparams))
       else:
         vf_nvinfo = all_nvinfo
-        vf_node_info = self.my_node_info.values()
+        vf_node_info = list(self.my_node_info.values())
 
     all_drbd_map = self.cfg.ComputeDRBDMap()
 
     feedback_fn("* Gathering disk information (%s nodes)" %
                 len(self.my_node_uuids))
-    instdisk = self._CollectDiskInfo(self.my_node_info.keys(), node_image,
+    instdisk = self._CollectDiskInfo(list(self.my_node_info), node_image,
                                      self.my_inst_info)
 
     feedback_fn("* Verifying configuration file consistency")
 
-    self._VerifyClientCertificates(self.my_node_info.values(), all_nvinfo)
+    self._VerifyClientCertificates(list(self.my_node_info.values()), all_nvinfo)
     if self.cfg.GetClusterInfo().modify_ssh_setup:
-      self._VerifySshSetup(self.my_node_info.values(), all_nvinfo)
+      self._VerifySshSetup(list(self.my_node_info.values()), all_nvinfo)
     self._VerifyFiles(vf_node_info, master_node_uuid, vf_nvinfo, filemap)
 
     feedback_fn("* Verifying node status")
@@ -2105,6 +2206,8 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
           self._ErrorIf(not test, constants.CV_ENODEORPHANINSTANCE, node_i.name,
                         "node is running unknown instance %s", inst_uuid)
 
+        self._VerifyExclusionTags(node_i.name, nimg.pinst, cluster.tags)
+
     self._VerifyGroupDRBDVersion(all_nvinfo)
     self._VerifyGroupLVM(node_image, vg_name)
 
@@ -2137,7 +2240,7 @@ class LUClusterVerifyGroup(LogicalUnit, _VerifyErrors):
     for instance in self.all_inst_info.values():
       for secondary in self.cfg.GetInstanceSecondaryNodes(instance.uuid):
         if (secondary in self.my_node_info
-            and instance.name not in self.my_inst_info):
+            and instance.uuid not in self.my_inst_info):
           self.cfg.GetInstanceLVsByNode(instance.uuid, lvmap=node_vol_should)
           break
 
