@@ -77,7 +77,7 @@ class LUGroupAdd(LogicalUnit):
       full_ipolicy = cluster.SimpleFillIPolicy(self.op.ipolicy)
       try:
         objects.InstancePolicy.CheckParameterSyntax(full_ipolicy, False)
-      except errors.ConfigurationError, err:
+      except errors.ConfigurationError as err:
         raise errors.OpPrereqError("Invalid instance policy: %s" % err,
                                    errors.ECODE_INVAL)
       CheckIpolicyVsDiskTemplates(full_ipolicy,
@@ -121,7 +121,7 @@ class LUGroupAdd(LogicalUnit):
       self.new_diskparams = self.op.diskparams
       try:
         utils.VerifyDictOptions(self.new_diskparams, constants.DISK_DT_DEFAULTS)
-      except errors.OpPrereqError, err:
+      except errors.OpPrereqError as err:
         raise errors.OpPrereqError("While verify diskparams options: %s" % err,
                                    errors.ECODE_INVAL)
     else:
@@ -410,7 +410,7 @@ class LUGroupSetParams(LogicalUnit):
       violations = \
           ComputeNewInstanceViolations(gmi.CalculateGroupIPolicy(cluster,
                                                                  self.group),
-                                       new_ipolicy, instances.values(),
+                                       new_ipolicy, list(instances.values()),
                                        self.cfg)
 
       if violations:
@@ -456,7 +456,7 @@ class LUGroupSetParams(LogicalUnit):
         utils.VerifyDictOptions(self.new_diskparams, constants.DISK_DT_DEFAULTS)
         CheckDiskAccessModeConsistency(self.new_diskparams, self.cfg,
                                        group=self.group)
-      except errors.OpPrereqError, err:
+      except errors.OpPrereqError as err:
         raise errors.OpPrereqError("While verify diskparams options: %s" % err,
                                    errors.ECODE_INVAL)
 
@@ -660,8 +660,8 @@ class LUGroupEvacuate(LogicalUnit):
     self.group_uuid = self.cfg.LookupNodeGroup(self.op.group_name)
 
     if self.op.target_groups:
-      self.req_target_uuids = map(self.cfg.LookupNodeGroup,
-                                  self.op.target_groups)
+      self.req_target_uuids = [self.cfg.LookupNodeGroup(g)
+                               for g in self.op.target_groups]
     else:
       self.req_target_uuids = []
 
@@ -932,7 +932,7 @@ class LUGroupVerifyDisks(NoHooksLU):
 
       # any leftover items in nv_dict are missing LVs, let's arrange the data
       # better
-      for key, inst in node_lv_to_inst.iteritems():
+      for key, inst in node_lv_to_inst.items():
         missing_disks.setdefault(inst.name, []).append(list(key))
 
   def _VerifyDrbdStates(self, node_errors, offline_disk_instance_names):
