@@ -32,7 +32,6 @@
 
 import itertools
 
-from ganeti import compat
 from ganeti import constants
 from ganeti import locking
 from ganeti import utils
@@ -90,7 +89,7 @@ class LUInstanceQueryData(NoHooksLU):
         # via the node before it's locked, requiring verification later on
         self.needed_locks[locking.LEVEL_NODEGROUP] = \
           frozenset(group_uuid
-                    for instance_uuid in owned_instances.keys()
+                    for instance_uuid in owned_instances
                     for group_uuid in
                     self.cfg.GetInstanceNodeGroups(instance_uuid))
 
@@ -128,7 +127,7 @@ class LUInstanceQueryData(NoHooksLU):
       assert not (owned_instances or owned_groups or
                   owned_node_uuids or owned_networks)
 
-    self.wanted_instances = instances.values()
+    self.wanted_instances = list(instances.values())
 
   def _ComputeBlockdevStatus(self, node_uuid, instance, dev):
     """Returns the status of a block device
@@ -194,10 +193,10 @@ class LUInstanceQueryData(NoHooksLU):
     dev_sstatus = self._ComputeBlockdevStatus(snode_uuid, instance, dev)
 
     if dev.children:
-      dev_children = map(compat.partial(self._ComputeDiskStatusInner,
-                                        instance, snode_uuid,
-                                        node_uuid2name_fn),
-                         dev.children)
+      dev_children = [
+        self._ComputeDiskStatusInner(instance, snode_uuid, node_uuid2name_fn, d)
+        for d in dev.children
+      ]
     else:
       dev_children = []
 
@@ -274,9 +273,8 @@ class LUInstanceQueryData(NoHooksLU):
       node_uuid2name_fn = lambda uuid: nodes[uuid].name
 
       disk_objects = self.cfg.GetInstanceDisks(instance.uuid)
-      output_disks = map(compat.partial(self._ComputeDiskStatus, instance,
-                                        node_uuid2name_fn),
-                         disk_objects)
+      output_disks = [self._ComputeDiskStatus(instance, node_uuid2name_fn, d)
+                      for d in disk_objects]
 
       secondary_nodes = self.cfg.GetInstanceSecondaryNodes(instance.uuid)
       snodes_group_uuids = [nodes[snode_uuid].group
@@ -289,9 +287,9 @@ class LUInstanceQueryData(NoHooksLU):
         "pnode": pnode.name,
         "pnode_group_uuid": pnode.group,
         "pnode_group_name": group2name_fn(pnode.group),
-        "snodes": map(node_uuid2name_fn, secondary_nodes),
+        "snodes": [node_uuid2name_fn(n) for n in secondary_nodes],
         "snodes_group_uuids": snodes_group_uuids,
-        "snodes_group_names": map(group2name_fn, snodes_group_uuids),
+        "snodes_group_names": [group2name_fn(u) for u in snodes_group_uuids],
         "os": instance.os,
         # this happens to be the same format used for hooks
         "nics": NICListToTuple(self, instance.nics),
