@@ -39,6 +39,7 @@ from ganeti import errors
 from ganeti import ht
 from ganeti import hypervisor
 from ganeti import locking
+from ganeti.masterd import iallocator
 from ganeti import netutils
 from ganeti import objects
 from ganeti import utils
@@ -51,7 +52,8 @@ from ganeti.cmdlib.common import INSTANCE_DOWN, \
   CheckParamsNotGlobal, \
   IsExclusiveStorageEnabledNode, CheckHVParams, CheckOSParams, \
   GetUpdatedParams, CheckInstanceState, ExpandNodeUuidAndName, \
-  IsValidDiskAccessModeCombination, AnnotateDiskParams
+  IsValidDiskAccessModeCombination, AnnotateDiskParams, \
+  CheckIAllocatorOrNode
 from ganeti.cmdlib.instance_storage import CalculateFileStorageDir, \
   CheckDiskExtProvider, CheckNodesFreeDiskPerVG, CheckRADOSFreeSpace, \
   CheckSpindlesExclusiveStorage, ComputeDiskSizePerVG, ComputeDisksInfo, \
@@ -162,16 +164,18 @@ class LUInstanceSetParams(LogicalUnit):
         utils.ForceDictType(params, (key_types if op != constants.DDM_ATTACH
                                      else key_types_attach))
 
+      if op not in (constants.DDM_ADD, constants.DDM_ATTACH,
+                    constants.DDM_MODIFY, constants.DDM_REMOVE,
+                    constants.DDM_DETACH):
+        raise errors.ProgrammerError("Unhandled operation '%s'" % op)
+
       if op in (constants.DDM_REMOVE, constants.DDM_DETACH):
         if params:
           raise errors.OpPrereqError("No settings should be passed when"
                                      " removing or detaching a %s" % kind,
                                      errors.ECODE_INVAL)
-      elif op in (constants.DDM_ADD, constants.DDM_ATTACH,
-                  constants.DDM_MODIFY):
-        item_fn(op, params)
-      else:
-        raise errors.ProgrammerError("Unhandled operation '%s'" % op)
+
+      item_fn(op, params)
 
   def _VerifyDiskModification(self, op, params, excl_stor, group_access_types):
     """Verifies a disk modification.
@@ -263,6 +267,11 @@ class LUInstanceSetParams(LogicalUnit):
       if name is not None and name.lower() == constants.VALUE_NONE:
         params[constants.IDISK_NAME] = None
 
+    if op == constants.DDM_REMOVE and not self.op.hotplug:
+      CheckInstanceState(self, self.instance, INSTANCE_NOT_RUNNING,
+                         msg="can't remove volume from a running instance"
+                             " without using hotplug")
+
   @staticmethod
   def _VerifyNicModification(op, params):
     """Verifies a network interface modification.
@@ -357,10 +366,7 @@ class LUInstanceSetParams(LogicalUnit):
 
       # mirrored template node checks
       if self.op.disk_template in constants.DTS_INT_MIRROR:
-        if not self.op.remote_node:
-          raise errors.OpPrereqError("Changing the disk template to a mirrored"
-                                     " one requires specifying a secondary"
-                                     " node", errors.ECODE_INVAL)
+        CheckIAllocatorOrNode(self, "iallocator", "remote_node")
       elif self.op.remote_node:
         self.LogWarning("Changing the disk template to a non-mirrored one,"
                         " the secondary node will be ignored")
@@ -399,11 +405,22 @@ class LUInstanceSetParams(LogicalUnit):
                                  errors.ECODE_INVAL)
 
     instance_nodes = self.cfg.GetInstanceNodes(self.instance.uuid)
-    if not set(instance_nodes).issubset(set(disk.nodes)):
+    # Make sure we do not attach disks to instances on wrong nodes. If the
+    # instance is diskless, that instance is associated only to the primary
+    # node, whereas the disk can be associated to two nodes in the case of DRBD,
+    # hence, we have a subset check here.
+    if disk.nodes and not set(instance_nodes).issubset(set(disk.nodes)):
       raise errors.OpPrereqError("Disk nodes are %s while the instance's nodes"
                                  " are %s" %
                                  (disk.nodes, instance_nodes),
                                  errors.ECODE_INVAL)
+    # Make sure a DRBD disk has the same primary node as the instance where it
+    # will be attached to.
+    disk_primary = disk.GetPrimaryNode(self.instance.primary_node)
+    if self.instance.primary_node != disk_primary:
+      raise errors.OpExecError("The disks' primary node is %s whereas the "
+                               "instance's primary node is %s."
+                               % (disk_primary, self.instance.primary_node))
 
   def ExpandNames(self):
     self._ExpandAndLockInstance()
@@ -433,6 +450,12 @@ class LUInstanceSetParams(LogicalUnit):
           ExpandNodeUuidAndName(self.cfg, self.op.remote_node_uuid,
                                 self.op.remote_node)
         self.needed_locks[locking.LEVEL_NODE].append(self.op.remote_node_uuid)
+      elif self.op.disk_template in constants.DTS_INT_MIRROR:
+        # If we have to find the secondary node for a conversion to DRBD,
+        # close node locks to the whole node group.
+        self.needed_locks[locking.LEVEL_NODE] = \
+          list(self.cfg.GetNodeGroupMembersByNodes(
+            self.needed_locks[locking.LEVEL_NODE]))
     elif level == locking.LEVEL_NODE_RES and self.op.disk_template:
       # Copy node locks
       self.needed_locks[locking.LEVEL_NODE_RES] = \
@@ -674,7 +697,8 @@ class LUInstanceSetParams(LogicalUnit):
                                        default_vg, self.op.ext_params)
 
     # mirror node verification
-    if self.op.disk_template in constants.DTS_INT_MIRROR:
+    if self.op.disk_template in constants.DTS_INT_MIRROR \
+        and self.op.remote_node_uuid:
       if self.op.remote_node_uuid == pnode_uuid:
         raise errors.OpPrereqError("Given new secondary node %s is the same"
                                    " as the primary node of the instance" %
@@ -709,7 +733,8 @@ class LUInstanceSetParams(LogicalUnit):
     if not self.op.disk_template in constants.DTS_EXCL_STORAGE:
       # Make sure none of the nodes require exclusive storage
       nodes = [pnode_info]
-      if self.op.disk_template in constants.DTS_INT_MIRROR:
+      if self.op.disk_template in constants.DTS_INT_MIRROR \
+          and self.op.remote_node_uuid:
         assert snode_info
         nodes.append(snode_info)
       has_es = lambda n: IsExclusiveStorageEnabledNode(self.cfg, n)
@@ -731,8 +756,9 @@ class LUInstanceSetParams(LogicalUnit):
           utils.AllDiskOfType(inst_disks, [constants.DT_PLAIN])):
       # for conversions from the 'plain' to the 'drbd' disk template, check
       # only the remote node's capacity
-      req_sizes = ComputeDiskSizePerVG(self.op.disk_template, self.disks_info)
-      CheckNodesFreeDiskPerVG(self, [self.op.remote_node_uuid], req_sizes)
+      if self.op.remote_node_uuid:
+        req_sizes = ComputeDiskSizePerVG(self.op.disk_template, self.disks_info)
+        CheckNodesFreeDiskPerVG(self, [self.op.remote_node_uuid], req_sizes)
     elif self.op.disk_template in constants.DTS_LVM:
       # rest lvm-based capacity checks
       node_uuids = [pnode_uuid]
@@ -761,7 +787,7 @@ class LUInstanceSetParams(LogicalUnit):
 
     inst_nodes = self.cfg.GetInstanceNodes(self.instance.uuid)
     excl_stor = compat.any(
-      rpc.GetExclusiveStorageForNodes(self.cfg, inst_nodes).values()
+      list(rpc.GetExclusiveStorageForNodes(self.cfg, inst_nodes).values())
       )
 
     # Get the group access type
@@ -1249,7 +1275,7 @@ class LUInstanceSetParams(LogicalUnit):
       res_min = ComputeIPolicyInstanceSpecViolation(ipolicy, ispec_min,
                                                     new_disk_types)
 
-      if (res_max or res_min):
+      if res_max or res_min:
         # FIXME: Improve error message by including information about whether
         # the upper or lower limit of the parameter fails the ipolicy.
         msg = ("Instance allocation to group %s (%s) violates policy: %s" %
@@ -1395,6 +1421,24 @@ class LUInstanceSetParams(LogicalUnit):
     """
     feedback_fn("Converting disk template from 'plain' to 'drbd'")
 
+    if not self.op.remote_node_uuid:
+      feedback_fn("Using %s to choose new secondary" % self.op.iallocator)
+
+      req = iallocator.IAReqInstanceAllocateSecondary(
+        name=self.op.instance_name)
+      ial = iallocator.IAllocator(self.cfg, self.rpc, req)
+      ial.Run(self.op.iallocator)
+
+      if not ial.success:
+        raise errors.OpPrereqError("Can's find secondary node using"
+                                   " iallocator %s: %s" %
+                                   (self.op.iallocator, ial.info),
+                                   errors.ECODE_NORES)
+      feedback_fn("%s choose %s as new secondary"
+                  % (self.op.iallocator, ial.result))
+      self.op.remote_node = ial.result
+      self.op.remote_node_uuid = self.cfg.GetNodeInfoByName(ial.result).uuid
+
     pnode_uuid = self.instance.primary_node
     snode_uuid = self.op.remote_node_uuid
     old_disks = self.cfg.GetInstanceDisks(self.instance.uuid)
@@ -1436,7 +1480,7 @@ class LUInstanceSetParams(LogicalUnit):
           f_create = node_uuid == pnode_uuid
           CreateSingleBlockDev(self, node_uuid, self.instance, disk, info,
                                f_create, excl_stor)
-    except errors.GenericError, e:
+    except errors.GenericError as e:
       feedback_fn("Initializing of DRBD devices failed;"
                   " renaming back original volumes...")
       rename_back_list = [(n.children[0], o.logical_id)
@@ -1617,7 +1661,7 @@ class LUInstanceSetParams(LogicalUnit):
     disk = self.GenericGetDiskInfo(uuid, name)
 
     # Rename disk before attaching (if disk is filebased)
-    if disk.dev_type in (constants.DTS_INSTANCE_DEPENDENT_PATH):
+    if disk.dev_type in constants.DTS_INSTANCE_DEPENDENT_PATH:
       # Add disk size/mode, else GenerateDiskTemplate will not work.
       params[constants.IDISK_SIZE] = disk.size
       params[constants.IDISK_MODE] = str(disk.mode)
@@ -1670,7 +1714,7 @@ class LUInstanceSetParams(LogicalUnit):
 
     # Modify arbitrary params in case instance template is ext
 
-    for key, value in params.iteritems():
+    for key, value in params.items():
       if (key not in constants.MODIFIABLE_IDISK_PARAMS and
           disk.dev_type == constants.DT_EXT):
         # stolen from GetUpdatedParams: default means reset/delete
@@ -1904,13 +1948,13 @@ class LUInstanceSetParams(LogicalUnit):
     # hvparams changes
     if self.op.hvparams:
       self.instance.hvparams = self.hv_inst
-      for key, val in self.op.hvparams.iteritems():
+      for key, val in self.op.hvparams.items():
         result.append(("hv/%s" % key, val))
 
     # beparams changes
     if self.op.beparams:
       self.instance.beparams = self.be_inst
-      for key, val in self.op.beparams.iteritems():
+      for key, val in self.op.beparams.items():
         result.append(("be/%s" % key, val))
 
     # OS change
@@ -1920,12 +1964,12 @@ class LUInstanceSetParams(LogicalUnit):
     # osparams changes
     if self.op.osparams:
       self.instance.osparams = self.os_inst
-      for key, val in self.op.osparams.iteritems():
+      for key, val in self.op.osparams.items():
         result.append(("os/%s" % key, val))
 
     if self.op.osparams_private:
       self.instance.osparams_private = self.os_inst_private
-      for key, val in self.op.osparams_private.iteritems():
+      for key, val in self.op.osparams_private.items():
         # Show the Private(...) blurb.
         result.append(("os_private/%s" % key, repr(val)))
 
