@@ -38,14 +38,13 @@ from ganeti import errors
 from ganeti import ht
 from ganeti import locking
 from ganeti.masterd import iallocator
-from ganeti import network
 from ganeti import netutils
 from ganeti import objects
 from ganeti import pathutils
 from ganeti import utils
 from ganeti.cmdlib.common import AnnotateDiskParams, \
   ComputeIPolicyInstanceViolation, CheckDiskTemplateEnabled, \
-  CheckDiskTemplateEnabled, ComputeIPolicySpecViolation
+  ComputeIPolicySpecViolation
 
 
 #: Type description for changes as returned by L{ApplyContainerMods}'s
@@ -131,7 +130,7 @@ def BuildInstanceHookEnv(name, primary_node_name, secondary_node_names, os_type,
       if netinfo:
         nobj = objects.Network.FromDict(netinfo)
         env.update(nobj.HooksDict("INSTANCE_NIC%d_" % idx))
-      elif network:
+      elif net:
         # FIXME: broken network reference: the instance NIC specifies a
         # network, but the relevant network entry was not in the config. This
         # should be made impossible.
@@ -214,7 +213,7 @@ def BuildInstanceHookEnvByObject(lu, instance, secondary_nodes=None,
   }
   if override:
     args.update(override)
-  return BuildInstanceHookEnv(**args) # pylint: disable=W0142
+  return BuildInstanceHookEnv(**args)
 
 
 def GetClusterDomainSecret():
@@ -336,6 +335,11 @@ def RemoveDisks(lu, instance, disks=None,
     disks = all_disks
 
   anno_disks = AnnotateDiskParams(instance, disks, lu.cfg)
+
+  uuid_idx_map = {}
+  for (idx, device) in enumerate(all_disks):
+    uuid_idx_map[device.uuid] = idx
+
   for (idx, device) in enumerate(anno_disks):
     if target_node_uuid:
       edata = [(target_node_uuid, device)]
@@ -345,7 +349,7 @@ def RemoveDisks(lu, instance, disks=None,
       result = lu.rpc.call_blockdev_remove(node_uuid, (disk, instance))
       if result.fail_msg:
         lu.LogWarning("Could not remove disk %s on node %s,"
-                      " continuing anyway: %s", idx,
+                      " continuing anyway: %s", uuid_idx_map.get(device.uuid),
                       lu.cfg.GetNodeName(node_uuid), result.fail_msg)
         if not (result.offline and node_uuid != instance.primary_node):
           all_result = False
@@ -1197,7 +1201,7 @@ def ComputeFullBeParams(op, cluster):
 
   """
   default_beparams = cluster.beparams[constants.PP_DEFAULT]
-  for param, value in op.beparams.iteritems():
+  for param, value in op.beparams.items():
     if value == constants.VALUE_AUTO:
       op.beparams[param] = default_beparams[param]
   objects.UpgradeBeParams(op.beparams)
