@@ -30,6 +30,7 @@
 
 """Module with helper classes and functions for daemons"""
 
+from __future__  import print_function
 
 import asyncore
 import asynchat
@@ -142,58 +143,6 @@ class GanetiBaseAsyncoreDispatcher(asyncore.dispatcher):
     return False
 
 
-class AsyncStreamServer(GanetiBaseAsyncoreDispatcher):
-  """A stream server to use with asyncore.
-
-  Each request is accepted, and then dispatched to a separate asyncore
-  dispatcher to handle.
-
-  """
-
-  _REQUEST_QUEUE_SIZE = 5
-
-  def __init__(self, family, address):
-    """Constructor for AsyncStreamServer
-
-    @type family: integer
-    @param family: socket family (one of socket.AF_*)
-    @type address: address family dependent
-    @param address: address to bind the socket to
-
-    """
-    GanetiBaseAsyncoreDispatcher.__init__(self)
-    self.family = family
-    self.create_socket(self.family, socket.SOCK_STREAM)
-    self.set_reuse_addr()
-    self.bind(address)
-    self.listen(self._REQUEST_QUEUE_SIZE)
-
-  # this method is overriding an asyncore.dispatcher method
-  def handle_accept(self):
-    """Accept a new client connection.
-
-    Creates a new instance of the handler class, which will use asyncore to
-    serve the client.
-
-    """
-    accept_result = utils.IgnoreSignals(self.accept)
-    if accept_result is not None:
-      connected_socket, client_address = accept_result
-      if self.family == socket.AF_UNIX:
-        # override the client address, as for unix sockets nothing meaningful
-        # is passed in from accept anyway
-        client_address = netutils.GetSocketCredentials(connected_socket)
-      logging.info("Accepted connection from %s",
-                   netutils.FormatAddress(client_address, family=self.family))
-      self.handle_connection(connected_socket, client_address)
-
-  def handle_connection(self, connected_socket, client_address):
-    """Handle an already accepted connection.
-
-    """
-    raise NotImplementedError
-
-
 class AsyncTerminatedMessageStream(asynchat.async_chat):
   """A terminator separated message stream asyncore module.
 
@@ -353,7 +302,7 @@ class AsyncUDPSocket(GanetiBaseAsyncoreDispatcher):
 
   # this method is overriding an asyncore.dispatcher method
   def handle_read(self):
-    recv_result = utils.IgnoreSignals(self.recvfrom,
+    recv_result = utils.IgnoreSignals(self.socket.recvfrom,
                                       constants.MAX_UDP_DATA_SIZE)
     if recv_result is not None:
       payload, address = recv_result
@@ -383,13 +332,15 @@ class AsyncUDPSocket(GanetiBaseAsyncoreDispatcher):
       logging.error("handle_write called with empty output queue")
       return
     (ip, port, payload) = self._out_queue[0]
-    utils.IgnoreSignals(self.sendto, payload, 0, (ip, port))
+    utils.IgnoreSignals(self.socket.sendto, payload, 0, (ip, port))
     self._out_queue.pop(0)
 
   def enqueue_send(self, ip, port, payload):
     """Enqueue a datagram to be sent when possible
 
     """
+    if isinstance(payload, str):
+      payload = payload.encode("utf-8")
     if len(payload) > constants.MAX_UDP_DATA_SIZE:
       raise errors.UdpDataSizeError("Packet too big: %s > %s" % (len(payload),
                                     constants.MAX_UDP_DATA_SIZE))
@@ -404,7 +355,7 @@ class AsyncUDPSocket(GanetiBaseAsyncoreDispatcher):
     @return: True if some data has been handled, False otherwise
 
     """
-    result = utils.WaitForFdCondition(self, select.POLLIN, timeout)
+    result = utils.WaitForFdCondition(self.socket, select.POLLIN, timeout)
     if result is not None and result & select.POLLIN:
       self.handle_read()
       return True
@@ -463,7 +414,7 @@ class AsyncAwaker(GanetiBaseAsyncoreDispatcher):
     # sending more than one wakeup token, which doesn't harm at all.
     if self.need_signal:
       self.need_signal = False
-      self.out_socket.send(chr(0))
+      self.out_socket.send(b"\x00")
 
 
 class _ShutdownCheck(object):
@@ -514,7 +465,7 @@ class Mainloop(object):
     timed events
 
   """
-  _SHUTDOWN_TIMEOUT_PRIORITY = -(sys.maxint - 1)
+  _SHUTDOWN_TIMEOUT_PRIORITY = -(sys.maxsize - 1)
 
   def __init__(self):
     """Constructs a new Mainloop instance.
@@ -522,6 +473,7 @@ class Mainloop(object):
     """
     self._signal_wait = []
     self.scheduler = AsyncoreScheduler(time.time)
+    self.awaker = AsyncAwaker()
 
     # Resolve uid/gids used
     runtime.GetEnts()
@@ -544,6 +496,11 @@ class Mainloop(object):
            len(signal_handlers) > 0, \
            "Broken SignalHandled decorator"
 
+    _sig_notify = lambda _, __: self.awaker.signal()
+
+    for handler in signal_handlers.values():
+      handler.SetHandlerFn(_sig_notify)
+
     # Counter for received signals
     shutdown_signals = 0
 
@@ -773,7 +730,7 @@ def GenericMain(daemon_name, optionparser,
     if options.bind_address != default_bind_address:
       msg = ("Can't specify both, bind address (%s) and bind interface (%s)" %
              (options.bind_address, options.bind_interface))
-      print >> sys.stderr, msg
+      print(msg, file=sys.stderr)
       sys.exit(constants.EXIT_FAILURE)
     interface_ip_addresses = \
       netutils.GetInterfaceIpAddresses(options.bind_interface)
@@ -783,7 +740,7 @@ def GenericMain(daemon_name, optionparser,
       if_addresses = interface_ip_addresses[constants.IP4_VERSION]
     if len(if_addresses) < 1:
       msg = "Failed to find IP for interface %s" % options.bind_interace
-      print >> sys.stderr, msg
+      print(msg, file=sys.stderr)
       sys.exit(constants.EXIT_FAILURE)
     options.bind_address = if_addresses[0]
 
@@ -793,9 +750,9 @@ def GenericMain(daemon_name, optionparser,
       "key": options.ssl_key,
       }
 
-    for name, path in ssl_paths.iteritems():
+    for name, path in ssl_paths.items():
       if not os.path.isfile(path):
-        print >> sys.stderr, "SSL %s file '%s' was not found" % (name, path)
+        print("SSL %s file '%s' was not found" % (name, path), file=sys.stderr)
         sys.exit(constants.EXIT_FAILURE)
 
     # TODO: By initiating http.HttpSslParams here we would only read the files
@@ -806,7 +763,7 @@ def GenericMain(daemon_name, optionparser,
   if not result:
     msg = ("%s started using wrong user ID (%d), expected %d" %
            (daemon_name, running_uid, expected_uid))
-    print >> sys.stderr, msg
+    print(msg, file=sys.stderr)
     sys.exit(constants.EXIT_FAILURE)
 
   if check_fn is not None:
@@ -854,8 +811,8 @@ def GenericMain(daemon_name, optionparser,
 
   try:
     utils.WritePidFile(utils.DaemonPidFileName(daemon_name))
-  except errors.PidFileLockError, err:
-    print >> sys.stderr, "Error while locking PID file:\n%s" % err
+  except errors.PidFileLockError as err:
+    print("Error while locking PID file:\n%s" % err, file=sys.stderr)
     sys.exit(constants.EXIT_FAILURE)
 
   try:
@@ -865,7 +822,7 @@ def GenericMain(daemon_name, optionparser,
         prep_results = prepare_fn(options, args)
       else:
         prep_results = None
-    except Exception, err:
+    except Exception as err:
       utils.WriteErrorToFD(wpipe, _BeautifyError(err))
       raise
 
