@@ -242,6 +242,15 @@ class JobLost(GenericError):
   """
 
 
+class JobCanceled(GenericError):
+  """Submitted job was canceled.
+
+  The job that was submitted has transitioned to a canceling or canceled
+  state.
+
+  """
+
+
 class JobFileCorrupted(GenericError):
   """Job file could not be properly decoded/restored.
 
@@ -460,6 +469,17 @@ class SshUpdateError(GenericError):
   """
 
 
+class JobSubmittedException(Exception):
+  """Job was submitted, client should exit.
+
+  This exception has one argument, the ID of the job that was
+  submitted. The handler should print this ID.
+
+  This is not an error, just a structured way to exit from clients.
+
+  """
+
+
 # errors should be added above
 
 
@@ -527,5 +547,4 @@ def MaybeRaise(result):
   error = GetEncodedError(result)
   if error:
     (errcls, args) = error
-    # pylint: disable=W0142
     raise errcls(*args)
