@@ -58,11 +58,11 @@ def _CreateBlankFile(path, mode):
   """
   try:
     utils.WriteFile(path, data="", mode=mode)
-  except EnvironmentError, err:
+  except EnvironmentError as err:
     raise HypervisorError("Failed to create file %s: %s" % (path, err))
 
 
-class LXCVersion(tuple): # pylint: disable=R0924
+class LXCVersion(tuple):
   """LXC version class.
 
   """
@@ -93,6 +93,7 @@ class LXCVersion(tuple): # pylint: disable=R0924
     return version
 
   def __str__(self):
+    # pylint: disable=E1101
     return self.original_string
 
 
@@ -136,7 +137,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     "lxc-wait",
     ]
 
-  _DIR_MODE = 0755
+  _DIR_MODE = 0o755
   _STASH_KEY_ALLOCATED_LOOP_DEV = "allocated_loopdev"
 
   _MEMORY_PARAMETER = "memory.limit_in_bytes"
@@ -211,8 +212,8 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     utils.EnsureDirs([
       (self._ROOT_DIR, self._DIR_MODE),
-      (self._LOG_DIR, 0750),
-      (self._INSTANCE_DIR, 0750),
+      (self._LOG_DIR, 0o750),
+      (self._INSTANCE_DIR, 0o750),
       ])
 
   def _SaveInstanceStash(self, instance_name, data):
@@ -224,7 +225,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     try:
       utils.WriteFile(stash_file, data=serialized,
                       mode=constants.SECURE_FILE_MODE)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise HypervisorError("Failed to save instance stash file %s : %s" %
                             (stash_file, err))
 
@@ -236,7 +237,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     stash_file = self._InstanceStashFilePath(instance_name)
     try:
       return serializer.Load(utils.ReadFile(stash_file))
-    except (EnvironmentError, ValueError), err:
+    except (EnvironmentError, ValueError) as err:
       raise HypervisorError("Failed to load instance stash file %s : %s" %
                             (stash_file, err))
 
@@ -254,7 +255,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     if not os.path.isdir(subsys_dir):
       try:
         os.makedirs(subsys_dir)
-      except EnvironmentError, err:
+      except EnvironmentError as err:
         raise HypervisorError("Failed to create directory %s: %s" %
                               (subsys_dir, err))
 
@@ -280,7 +281,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       if self._STASH_KEY_ALLOCATED_LOOP_DEV in stash:
         loop_dev_path = stash[self._STASH_KEY_ALLOCATED_LOOP_DEV]
         utils.ReleaseBdevPartitionMapping(loop_dev_path)
-    except errors.CommandError, err:
+    except errors.CommandError as err:
       raise HypervisorError("Failed to cleanup partition mapping : %s" % err)
 
     utils.RemoveFile(self._InstanceStashFilePath(instance_name))
@@ -326,7 +327,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     try:
       cgroup_list = utils.ReadFile(cls._PROC_SELF_CGROUP_FILE)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise HypervisorError("Failed to read %s : %s" %
                             (cls._PROC_SELF_CGROUP_FILE, err))
 
@@ -409,7 +410,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     fd = -1
     try:
       fd = os.open(param_path, os.O_WRONLY)
-      os.write(fd, param_value)
+      os.write(fd, param_value.encode("ascii"))
     finally:
       if fd != -1:
         os.close(fd)
@@ -440,7 +441,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     try:
       cpumask = cls._GetCgroupInstanceValue(instance_name, "cpuset.cpus")
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise errors.HypervisorError("Getting CPU list for instance"
                                    " %s failed: %s" % (instance_name, err))
 
@@ -453,7 +454,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     try:
       cputime_ns = cls._GetCgroupInstanceValue(instance_name, "cpuacct.usage")
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise HypervisorError("Failed to get the cpu usage of %s: %s" %
                             (instance_name, err))
 
@@ -468,7 +469,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       mem_limit = cls._GetCgroupInstanceValue(instance_name,
                                               "memory.limit_in_bytes")
       return int(mem_limit)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise HypervisorError("Can't get instance memory limit of %s: %s" %
                             (instance_name, err))
 
@@ -476,19 +477,31 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """Get the list of running instances.
 
     """
-    return [iinfo[0] for iinfo in self.GetAllInstancesInfo()]
+    return self._ListAliveInstances()
 
   @classmethod
   def _IsInstanceAlive(cls, instance_name):
     """Return True if instance is alive.
 
+    """
+    result = utils.RunCmd(["lxc-ls", "--running", re.escape(instance_name)])
+    if result.failed:
+      raise HypervisorError("Failed to get running LXC containers list: %s" %
+                            result.output)
+
+    return instance_name in result.stdout.split()
+
+  @classmethod
+  def _ListAliveInstances(cls):
+    """Return list of alive instances.
+
     """
     result = utils.RunCmd(["lxc-ls", "--running"])
     if result.failed:
       raise HypervisorError("Failed to get running LXC containers list: %s" %
                             result.output)
 
-    return instance_name in result.stdout.split()
+    return result.stdout.split()
 
   def GetInstanceInfo(self, instance_name, hvparams=None):
     """Get instance properties.
@@ -504,8 +517,20 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     if not self._IsInstanceAlive(instance_name):
       return None
 
+    return self._GetInstanceInfoInner(instance_name)
+
+  def _GetInstanceInfoInner(self, instance_name):
+    """Get instance properties.
+
+    @type instance_name: string
+    @param instance_name: the instance name
+    @rtype: tuple of strings
+    @return: (name, id, memory, vcpus, stat, times)
+
+    """
+
     cpu_list = self._GetCgroupCpuList(instance_name)
-    memory = self._GetCgroupMemoryLimit(instance_name) / (1024 ** 2)
+    memory = self._GetCgroupMemoryLimit(instance_name) // (1024 ** 2)
     cputime = self._GetCgroupCpuUsage(instance_name)
     return (instance_name, 0, memory, len(cpu_list),
             hv_base.HvInstanceState.RUNNING, cputime)
@@ -519,10 +544,13 @@ class LXCHypervisor(hv_base.BaseHypervisor):
 
     """
     data = []
+    running_instances = self._ListAliveInstances()
     filter_fn = lambda x: os.path.isdir(utils.PathJoin(self._INSTANCE_DIR, x))
     for dirname in filter(filter_fn, os.listdir(self._INSTANCE_DIR)):
+      if dirname not in running_instances:
+        continue
       try:
-        info = self.GetInstanceInfo(dirname)
+        info = self._GetInstanceInfoInner(dirname)
       except errors.HypervisorError:
         continue
       if info:
@@ -644,7 +672,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     try:
       subsys_table = utils.ReadFile(cls._PROC_CGROUPS_FILE)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise HypervisorError("Failed to read cgroup info from %s: %s"
                             % (cls._PROC_CGROUPS_FILE, err))
     return [x.split(None, 1)[0] for x in subsys_table.split("\n")
@@ -681,7 +709,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     """
     try:
       ret = utils.CreateBdevPartitionMapping(storage_path)
-    except errors.CommandError, err:
+    except errors.CommandError as err:
       raise HypervisorError("Failed to create partition mapping for %s"
                             ": %s" % (storage_path, err))
 
@@ -773,7 +801,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     root_dir = self._InstanceDir(instance.name)
     try:
       utils.EnsureDirs([(root_dir, self._DIR_MODE)])
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       raise HypervisorError("Creating instance directory failed: %s", str(err))
 
     log_file = self._InstanceLogFilePath(instance)
@@ -801,10 +829,10 @@ class LXCHypervisor(hv_base.BaseHypervisor):
       exc_info = sys.exc_info()
       try:
         self._CleanupInstance(instance.name, stash)
-      except HypervisorError, err:
+      except HypervisorError as err:
         logging.warn("Cleanup for instance %s incomplete: %s",
                      instance.name, err)
-      raise exc_info[0], exc_info[1], exc_info[2]
+      raise exc_info[0](exc_info[1]).with_traceback(exc_info[2])
 
     self._SaveInstanceStash(instance.name, stash)
 
@@ -883,7 +911,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
     for i, cgparam in enumerate(cgparams):
       try:
         self._SetCgroupInstanceValue(instance.name, cgparam, str(mem_in_bytes))
-      except EnvironmentError, err:
+      except EnvironmentError as err:
         if shrinking and err.errno == errno.EBUSY:
           logging.warn("Unable to reclaim memory or swap usage from instance"
                        " %s", instance.name)
@@ -892,7 +920,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
           try:
             self._SetCgroupInstanceValue(instance.name, restore_param,
                                          str(current_mem_usage))
-          except EnvironmentError, restore_err:
+          except EnvironmentError as restore_err:
             logging.warn("Can't restore the cgroup parameter %s of %s: %s",
                          restore_param, instance.name, restore_err)
 
@@ -942,7 +970,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
 
     try:
       return LXCVersion(result.stdout.strip())
-    except ValueError, err:
+    except ValueError as err:
       raise HypervisorError("Can't parse LXC version from %s: %s" %
                             (from_cmd, err))
 
@@ -971,7 +999,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
         else:
           try:
             version = cls._GetLXCVersionFromCmd(cmd)
-          except HypervisorError, err:
+          except HypervisorError as err:
             msgs.append(str(err))
             continue
 
@@ -1003,7 +1031,7 @@ class LXCHypervisor(hv_base.BaseHypervisor):
 
     try:
       self._EnsureCgroupMounts(hvparams)
-    except errors.HypervisorError, err:
+    except errors.HypervisorError as err:
       msgs.append(str(err))
 
     msgs.extend(self._VerifyLXCCommands())
