@@ -41,7 +41,6 @@ import os
 import re
 import socket
 import struct
-import IN
 import logging
 
 from ganeti import constants
@@ -61,12 +60,6 @@ from ganeti import vcluster
 _STRUCT_UCRED = "iII"
 _STRUCT_UCRED_SIZE = struct.calcsize(_STRUCT_UCRED)
 
-# Workaround a bug in some linux distributions that don't define SO_PEERCRED
-try:
-  # pylint: disable=E1101
-  _SO_PEERCRED = IN.SO_PEERCRED
-except AttributeError:
-  _SO_PEERCRED = 17
 
 # Regexes used to find IP addresses in the output of ip.
 _IP_RE_TEXT = r"[.:a-z0-9]+"      # separate for testing purposes
@@ -109,7 +102,7 @@ def GetSocketCredentials(sock):
   @return: The PID, UID and GID of the connected foreign process.
 
   """
-  peercred = sock.getsockopt(socket.SOL_SOCKET, _SO_PEERCRED,
+  peercred = sock.getsockopt(socket.SOL_SOCKET, socket.SO_PEERCRED,
                              _STRUCT_UCRED_SIZE)
   return struct.unpack(_STRUCT_UCRED, peercred)
 
@@ -161,9 +154,10 @@ def GetHostname(name=None, family=None):
   """
   try:
     return Hostname(name=name, family=family)
-  except errors.ResolverError, err:
+  except errors.ResolverError as err:
     raise errors.OpPrereqError("The given name (%s) does not resolve: %s" %
-                               (err[0], err[2]), errors.ECODE_RESOLVER)
+                               (err.args[0], err.args[2]),
+                               errors.ECODE_RESOLVER)
 
 
 class Hostname(object):
@@ -236,7 +230,7 @@ class Hostname(object):
         result = socket.getaddrinfo(hostname, None, family)
       else:
         result = socket.getaddrinfo(hostname, None)
-    except (socket.gaierror, socket.herror, socket.error), err:
+    except (socket.gaierror, socket.herror, socket.error) as err:
       # hostname not found in DNS, or other socket exception in the
       # (code, description format)
       raise errors.ResolverError(hostname, err.args[0], err.args[1])
@@ -246,7 +240,7 @@ class Hostname(object):
     # sockaddr
     try:
       return result[0][4][0]
-    except IndexError, err:
+    except IndexError as err:
       # we don't have here an actual error code, it's just that the
       # data type returned by getaddrinfo is not what we expected;
       # let's keep the same format in the exception arguments with a
@@ -327,7 +321,7 @@ def TcpPing(target, port, timeout=10, live_port_needed=False, source=None):
 
   try:
     family = IPAddress.GetAddressFamily(target)
-  except errors.IPAddressError, err:
+  except errors.IPAddressError as err:
     raise errors.ProgrammerError("Family of IP address given in parameter"
                                  " 'target' can't be determined: %s" % err)
 
@@ -337,20 +331,21 @@ def TcpPing(target, port, timeout=10, live_port_needed=False, source=None):
   if source is not None:
     try:
       sock.bind((source, 0))
-    except socket.error, err:
-      if err[0] == errno.EADDRNOTAVAIL:
+    except socket.error as err:
+      if err.errno == errno.EADDRNOTAVAIL:
         success = False
 
   sock.settimeout(timeout)
 
   try:
     sock.connect((target, port))
-    sock.close()
     success = True
   except socket.timeout:
     success = False
-  except socket.error, err:
-    success = (not live_port_needed) and (err[0] == errno.ECONNREFUSED)
+  except socket.error as err:
+    success = (not live_port_needed) and (err.errno == errno.ECONNREFUSED)
+  finally:
+    sock.close()
 
   return success
 
@@ -428,7 +423,7 @@ class IPAddress(object):
     @return: True if valid, False otherwise
 
     """
-    assert isinstance(netmask, (int, long))
+    assert isinstance(netmask, int)
 
     return 0 < netmask <= cls.iplen
 
