@@ -35,21 +35,25 @@ pass to and from external parties.
 
 """
 
-# pylint: disable=E0203,W0201,R0902
+# pylint: disable=E0203,E0237,W0201,R0902
 
 # E0203: Access to member %r before its definition, since we use
 # objects.py which doesn't explicitly initialise its members
 
+# E0237: Assigning to attribute not defined in class slots. pylint doesn't
+# appear to notice many of the slots defined in __slots__ for several objects.
+
 # W0201: Attribute '%s' defined outside __init__
 
 # R0902: Allow instances of these objects to have more than 20 attributes
 
-import ConfigParser
+import configparser
 import re
 import copy
 import logging
 import time
-from cStringIO import StringIO
+from io import StringIO
+from socket import AF_INET
 
 from ganeti import errors
 from ganeti import constants
@@ -58,8 +62,6 @@ from ganeti import outils
 from ganeti import utils
 from ganeti import serializer
 
-from socket import AF_INET
-
 
 __all__ = ["ConfigObject", "ConfigData", "NIC", "Disk", "Instance",
            "OS", "Node", "NodeGroup", "Cluster", "FillDict", "Network",
@@ -95,7 +97,7 @@ def FillIPolicy(default_ipolicy, custom_ipolicy):
   """Fills an instance policy with defaults.
 
   """
-  assert frozenset(default_ipolicy.keys()) == constants.IPOLICY_ALL_KEYS
+  assert frozenset(default_ipolicy) == constants.IPOLICY_ALL_KEYS
   ret_dict = copy.deepcopy(custom_ipolicy)
   for key in default_ipolicy:
     if key not in ret_dict:
@@ -277,8 +279,8 @@ class ConfigObject(outils.ValidatedSlots):
     if not isinstance(val, dict):
       raise errors.ConfigurationError("Invalid object passed to FromDict:"
                                       " expected dict, got %s" % type(val))
-    val_str = dict([(str(k), v) for k, v in val.iteritems()])
-    obj = cls(**val_str) # pylint: disable=W0142
+    val_str = dict([(str(k), v) for k, v in val.items()])
+    obj = cls(**val_str)
     return obj
 
   def Copy(self):
@@ -293,10 +295,6 @@ class ConfigObject(outils.ValidatedSlots):
     """Implement __repr__ for ConfigObjects."""
     return repr(self.ToDict())
 
-  def __eq__(self, other):
-    """Implement __eq__ for ConfigObjects."""
-    return isinstance(other, self.__class__) and self.ToDict() == other.ToDict()
-
   def UpgradeConfig(self):
     """Fill defaults for missing configuration values.
 
@@ -322,7 +320,7 @@ class TaggableObject(ConfigObject):
     function has no return value.
 
     """
-    if not isinstance(tag, basestring):
+    if not isinstance(tag, str):
       raise errors.TagError("Invalid tag type (not a string)")
     if len(tag) > constants.MAX_TAG_LEN:
       raise errors.TagError("Tag too long (>%d characters)" %
@@ -522,7 +520,7 @@ class ConfigData(ConfigObject):
 class NIC(ConfigObject):
   """Config object representing a network card."""
   __slots__ = ["name", "mac", "ip", "network",
-               "nicparams", "netinfo", "pci"] + _UUID
+               "nicparams", "netinfo", "pci", "hvinfo"] + _UUID
 
   @classmethod
   def CheckParameterSyntax(cls, nicparams):
@@ -564,6 +562,7 @@ class Disk(ConfigObject):
     "params",
     "spindles",
     "pci",
+    "hvinfo",
     "serial_no",
     # dynamic_params is special. It depends on the node this instance
     # is sent to, and should not be persisted.
@@ -675,6 +674,17 @@ class Disk(ConfigObject):
       raise errors.ProgrammerError("Unhandled device type %s" % self.dev_type)
     return result
 
+  def GetPrimaryNode(self, node_uuid):
+    """This function returns the primary node of the device.
+
+    If the device is not a DRBD device, we still return the node the device
+    lives on.
+
+    """
+    if self.dev_type in constants.DTS_DRBD:
+      return self.logical_id[0]
+    return node_uuid
+
   def ComputeNodeTree(self, parent_node_uuid):
     """Compute the node/disk tree for this disk and its children.
 
@@ -1030,14 +1040,14 @@ class InstancePolicy(ConfigObject):
     for key in constants.IPOLICY_PARAMETERS:
       if key in ipolicy:
         InstancePolicy.CheckParameter(key, ipolicy[key])
-    wrong_keys = frozenset(ipolicy.keys()) - constants.IPOLICY_ALL_KEYS
+    wrong_keys = frozenset(ipolicy) - constants.IPOLICY_ALL_KEYS
     if wrong_keys:
       raise errors.ConfigurationError("Invalid keys in ipolicy: %s" %
                                       utils.CommaJoin(wrong_keys))
 
   @classmethod
   def _CheckIncompleteSpec(cls, spec, keyname):
-    missing_params = constants.ISPECS_PARAMETERS - frozenset(spec.keys())
+    missing_params = constants.ISPECS_PARAMETERS - frozenset(spec)
     if missing_params:
       msg = ("Missing instance specs parameters for %s: %s" %
              (keyname, utils.CommaJoin(missing_params)))
@@ -1069,7 +1079,7 @@ class InstancePolicy(ConfigObject):
       raise errors.ConfigurationError("Empty minmax specifications")
     std_is_good = False
     for minmaxspecs in ipolicy[constants.ISPECS_MINMAX]:
-      missing = constants.ISPECS_MINMAX_KEYS - frozenset(minmaxspecs.keys())
+      missing = constants.ISPECS_MINMAX_KEYS - frozenset(minmaxspecs)
       if missing:
         msg = "Missing instance specification: %s" % utils.CommaJoin(missing)
         raise errors.ConfigurationError(msg)
@@ -1144,7 +1154,7 @@ class InstancePolicy(ConfigObject):
     """
     try:
       float(value)
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       raise errors.ConfigurationError("Invalid value for key" " '%s':"
                                       " '%s', error: %s" % (key, value, err))
 
@@ -1223,7 +1233,7 @@ class Instance(TaggableObject):
     try:
       idx = int(idx)
       return self.disks[idx]
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       raise errors.OpPrereqError("Invalid disk index: '%s'" % str(err),
                                  errors.ECODE_INVAL)
     except IndexError:
@@ -1243,7 +1253,7 @@ class Instance(TaggableObject):
     if _with_private:
       bo["osparams_private"] = self.osparams_private.Unprivate()
 
-    for attr in "nics", :
+    for attr in ("nics",):
       alist = bo.get(attr, None)
       if alist:
         nlist = outils.ContainerToDicts(alist)
@@ -1663,6 +1673,8 @@ class Cluster(TaggableObject):
     "compression_tools",
     "enabled_user_shutdown",
     "data_collectors",
+    "ssh_key_type",
+    "ssh_key_bits",
     ] + _TIMESTAMPS + _UUID
 
   def UpgradeConfig(self):
@@ -1783,7 +1795,7 @@ class Cluster(TaggableObject):
       # we can either make sure to upgrade the ipolicy always, or only
       # do it in some corner cases (e.g. missing keys); note that this
       # will break any removal of keys from the ipolicy dict
-      wrongkeys = frozenset(self.ipolicy.keys()) - constants.IPOLICY_ALL_KEYS
+      wrongkeys = frozenset(self.ipolicy) - constants.IPOLICY_ALL_KEYS
       if wrongkeys:
         # These keys would be silently removed by FillIPolicy()
         msg = ("Cluster instance policy contains spurious keys: %s" %
@@ -1818,6 +1830,12 @@ class Cluster(TaggableObject):
     if self.enabled_user_shutdown is None:
       self.enabled_user_shutdown = False
 
+    if self.ssh_key_type is None:
+      self.ssh_key_type = constants.SSH_DEFAULT_KEY_TYPE
+
+    if self.ssh_key_bits is None:
+      self.ssh_key_bits = constants.SSH_DEFAULT_KEY_BITS
+
   @property
   def primary_hypervisor(self):
     """The first hypervisor is the primary.
@@ -2050,8 +2068,7 @@ class Cluster(TaggableObject):
       - at public visibility:  {public}
       - at private visibility: {private}
       - at secret visibility:  {secret}
-      """.format(dupes=formatter(duplicate_keys),
-                 public=formatter(params_public & duplicate_keys),
+      """.format(public=formatter(params_public & duplicate_keys),
                  private=formatter(params_private & duplicate_keys),
                  secret=formatter(params_secret & duplicate_keys))
       raise errors.OpPrereqError(msg)
@@ -2300,6 +2317,8 @@ class MigrationStatus(ConfigObject):
     "status",
     "transferred_ram",
     "total_ram",
+    # to signal, if migration has switched to postcopy
+    "postcopy_status",
     ]
 
 
@@ -2398,10 +2417,10 @@ class Network(TaggableObject):
 
 
 # need to inherit object in order to use super()
-class SerializableConfigParser(ConfigParser.SafeConfigParser, object):
+class SerializableConfigParser(configparser.ConfigParser, object):
   """Simple wrapper over ConfigParse that allows serialization.
 
-  This class is basically ConfigParser.SafeConfigParser with two
+  This class is basically configparser.ConfigParser with two
   additional methods that allow it to serialize/unserialize to/from a
   buffer.
 
@@ -2417,7 +2436,7 @@ class SerializableConfigParser(ConfigParser.SafeConfigParser, object):
     """Load data from a string."""
     buf = StringIO(data)
     cfp = cls()
-    cfp.readfp(buf)
+    cfp.read_file(buf)
     return cfp
 
   def get(self, section, option, **kwargs):
@@ -2427,7 +2446,7 @@ class SerializableConfigParser(ConfigParser.SafeConfigParser, object):
                                                         **kwargs)
       if value.lower() == constants.VALUE_NONE:
         value = None
-    except ConfigParser.NoOptionError:
+    except configparser.NoOptionError:
       r = re.compile(r"(disk|nic)\d+_name|nic\d+_(network|vlan)")
       match = r.match(option)
       if match:
