@@ -43,18 +43,19 @@
 # be standalone.
 
 import logging
-import simplejson
 import socket
-import urllib
 import threading
-import pycurl
 import time
 
 try:
-  from cStringIO import StringIO
+    from urllib import urlencode
 except ImportError:
-  from StringIO import StringIO
+    from urllib.parse import urlencode
 
+import pycurl
+import simplejson
+
+from io import StringIO, BytesIO
 
 GANETI_RAPI_PORT = 5080
 GANETI_RAPI_VERSION = 2
@@ -337,6 +338,8 @@ def GenericCurlConfig(verbose=False, use_signal=False,
       if capath:
         raise Error("cURL linked against GnuTLS has no support for a"
                     " CA path (%s)" % (pycurl.version, ))
+    elif lcsslver.startswith("boringssl"):
+      pass
     else:
       raise NotImplementedError("cURL uses unsupported SSL version '%s'" %
                                 sslver)
@@ -379,6 +382,36 @@ def GenericCurlConfig(verbose=False, use_signal=False,
   return _ConfigCurl
 
 
+class _CompatIO(object):
+  """ Stream that lazy-allocates its buffer based on the first write's type
+
+  This is a wrapper around BytesIO/StringIO that will allocate the
+  respective internal buffer based on whether the first write is bytes or
+  not. It is intended to be used with PycURL, which returns the response as
+  bytes in Python 3 and string in Python 2.
+
+  """
+  def __init__(self):
+    self.buffer = None
+
+  def write(self, data, *args, **kwargs):
+    if self.buffer is None:
+      self.buffer = BytesIO() if isinstance(data, bytes) else StringIO()
+    return self.buffer.write(data, *args, **kwargs)
+
+  def read(self, *args, **kwargs):
+    return self.buffer.read(*args, **kwargs)
+
+  def tell(self):
+    if self.buffer is None:
+      # We were never written to
+      return 0
+    return self.buffer.tell()
+
+  def seek(self, *args, **kwargs):
+    return self.buffer.seek(*args, **kwargs)
+
+
 class GanetiRapiClient(object): # pylint: disable=R0904
   """Ganeti RAPI client.
 
@@ -501,7 +534,7 @@ class GanetiRapiClient(object): # pylint: disable=R0904
     @type path: string
     @param path: HTTP URL path
     @type query: list of two-tuples
-    @param query: query arguments to pass to urllib.urlencode
+    @param query: query arguments to pass to urlencode
     @type content: str or None
     @param content: HTTP body content
 
@@ -525,7 +558,7 @@ class GanetiRapiClient(object): # pylint: disable=R0904
     urlparts = [self._base_url, path]
     if query:
       urlparts.append("?")
-      urlparts.append(urllib.urlencode(self._EncodeQuery(query)))
+      urlparts.append(urlencode(self._EncodeQuery(query)))
 
     url = "".join(urlparts)
 
@@ -533,7 +566,7 @@ class GanetiRapiClient(object): # pylint: disable=R0904
                        method, url, encoded_content)
 
     # Buffer for response
-    encoded_resp_body = StringIO()
+    encoded_resp_body = _CompatIO()
 
     # Configure cURL
     curl.setopt(pycurl.CUSTOMREQUEST, str(method))
@@ -545,7 +578,7 @@ class GanetiRapiClient(object): # pylint: disable=R0904
       # Send request and wait for response
       try:
         curl.perform()
-      except pycurl.error, err:
+      except pycurl.error as err:
         if err.args[0] in _CURL_SSL_CERT_ERRORS:
           raise CertificateError("SSL certificate error %s" % err,
                                  code=err.args[0])
@@ -562,7 +595,8 @@ class GanetiRapiClient(object): # pylint: disable=R0904
 
     # Was anything written to the response buffer?
     if encoded_resp_body.tell():
-      response_content = simplejson.loads(encoded_resp_body.getvalue())
+      encoded_resp_body.seek(0)
+      response_content = simplejson.load(encoded_resp_body)
     else:
       response_content = None
 
@@ -598,7 +632,7 @@ class GanetiRapiClient(object): # pylint: disable=R0904
     try:
       return self._SendRequest(HTTP_GET, "/%s/features" % GANETI_RAPI_VERSION,
                                None, None)
-    except GanetiApiError, err:
+    except GanetiApiError as err:
       # Older RAPI servers don't support this resource
       if err.code == HTTP_NOT_FOUND:
         return []
@@ -794,12 +828,12 @@ class GanetiRapiClient(object): # pylint: disable=R0904
     @note: This is an inplace update of base
 
     """
-    conflicts = set(kwargs.iterkeys()) & set(base.iterkeys())
+    conflicts = set(kwargs.keys()) & set(base.keys())
     if conflicts:
       raise GanetiApiError("Required fields can not be specified as"
                            " keywords: %s" % ", ".join(conflicts))
 
-    base.update((key, value) for key, value in kwargs.iteritems()
+    base.update((key, value) for key, value in kwargs.items()
                 if key != "dry_run")
 
   def InstanceAllocation(self, mode, name, disk_template, disks, nics,
