@@ -32,11 +32,13 @@
 
 """
 
+import base64
 import logging
 import re
-import base64
+
+from io import StringIO
+
 import pycurl
-from cStringIO import StringIO
 
 from ganeti import errors
 from ganeti import opcodes
@@ -49,8 +51,8 @@ import ganeti.rpc.client as rpccl
 from ganeti import rapi
 
 import ganeti.http.server # pylint: disable=W0611
-import ganeti.server.rapi
-import ganeti.rapi.client
+import ganeti.server.rapi # pylint: disable=W0611
+import ganeti.rapi.client # pylint: disable=W0611
 
 
 _URI_RE = re.compile(r"https://(?P<host>.*):(?P<port>\d+)(?P<path>/.*)")
@@ -83,7 +85,7 @@ def _HideInternalErrors(fn):
   def wrapper(*args, **kwargs):
     try:
       return fn(*args, **kwargs)
-    except (errors.GenericError, rapi.client.GanetiApiError), err:
+    except (errors.GenericError, rapi.client.GanetiApiError) as err:
       raise VerificationError("Unhandled Ganeti error: %s" % err)
 
   return wrapper
@@ -103,13 +105,13 @@ def VerifyOpInput(op_id, data):
   op_cls = _GetOpById(op_id)
 
   try:
-    op = op_cls(**data) # pylint: disable=W0142
-  except TypeError, err:
+    op = op_cls(**data)
+  except TypeError as err:
     raise VerificationError("Unable to create opcode instance: %s" % err)
 
   try:
     op.Validate(False)
-  except errors.OpPrereqError, err:
+  except errors.OpPrereqError as err:
     raise VerificationError("Parameter validation for opcode '%s' failed: %s" %
                             (op_id, err))
 
@@ -199,19 +201,22 @@ class FakeCurl(object):
 
     if self._opts.get(pycurl.HTTPAUTH, 0) & pycurl.HTTPAUTH_BASIC:
       try:
-        userpwd = self._opts[pycurl.USERPWD]
+        userpwd = self._opts[pycurl.USERPWD].encode("utf-8")
       except KeyError:
         raise errors.ProgrammerError("Basic authentication requires username"
                                      " and password")
 
       headers[http.HTTP_AUTHORIZATION] = \
-        "%s %s" % (http.auth.HTTP_BASIC_AUTH, base64.b64encode(userpwd))
+        "%s %s" % (http.auth.HTTP_BASIC_AUTH,
+                   base64.b64encode(userpwd).decode("ascii"))
 
     path = _GetPathFromUri(url)
     (code, _, resp_body) = \
       self._handler.FetchResponse(path, method, headers, request_body)
 
     self._info[pycurl.RESPONSE_CODE] = code
+    if isinstance(resp_body, bytes):
+      resp_body = resp_body.decode("utf-8")
     if resp_body is not None:
       writefn(resp_body)
 
@@ -242,7 +247,7 @@ class _RapiMock(object):
     @param method: HTTP method
     @type request_body: string
     @param request_body: Request body
-    @type headers: mimetools.Message
+    @type headers: email.message.Message
     @param headers: Request headers
     @return: Tuple containing status code, response headers and response body
 
@@ -252,9 +257,10 @@ class _RapiMock(object):
       http.HttpClientToServerStartLine(method, path, http.HTTP_1_0)
     req_msg.headers = headers
     req_msg.body = request_body
+    req_reader = type('TestReader', (object, ), {'sock': None})()
 
     (_, _, _, resp_msg) = \
-      http.server.HttpResponder(self.handler)(lambda: (req_msg, None))
+      http.server.HttpResponder(self.handler)(lambda: (req_msg, req_reader))
 
     return (resp_msg.start_line.code, resp_msg.headers, resp_msg.body)
 
