@@ -39,13 +39,14 @@
 # if they need to start using instance attributes
 # R0904: Too many public methods
 
-import logging
-import zlib
 import base64
-import pycurl
-import threading
 import copy
+import logging
 import os
+import threading
+import zlib
+
+import pycurl
 
 from ganeti import utils
 from ganeti import objects
@@ -109,8 +110,10 @@ def _ConfigRpcCurl(curl):
   noded_cert = pathutils.NODED_CERT_FILE
   noded_client_cert = pathutils.NODED_CLIENT_CERT_FILE
 
-  # FIXME: The next two lines are necessary to ensure upgradability from
-  # 2.10 to 2.11. Remove in 2.12, because this slows down RPC calls.
+  # This fallback is required for backwards compatibility with 2.10. Ganeti
+  # 2.11 introduced per-node client certificates, but when we restart after
+  # an upgrade from 2.10, the client certs are not in place yet, and we need
+  # to fall back to using the cluster-wide server cert.
   if not os.path.exists(noded_client_cert):
     logging.warn("Using server certificate as client certificate for RPC"
                  "call.")
@@ -250,7 +253,7 @@ class RpcResult(object):
       args = (msg, ecode)
     else:
       args = (msg, )
-    raise ec(*args) # pylint: disable=W0142
+    raise ec(*args)
 
   def Warn(self, msg, feedback_fn):
     """If the result has failed, call the feedback_fn.
@@ -302,7 +305,7 @@ def _SsconfResolver(ssconf_ips, node_list, _,
   return result
 
 
-class _StaticResolver:
+class _StaticResolver(object):
   def __init__(self, addresses):
     """Initializes this class.
 
@@ -314,7 +317,7 @@ class _StaticResolver:
 
     """
     assert len(hosts) == len(self._addresses)
-    return zip(hosts, self._addresses, hosts)
+    return list(zip(hosts, self._addresses, hosts))
 
 
 def _CheckConfigNode(node_uuid_or_name, node, accept_offline_node):
@@ -360,7 +363,7 @@ def _NodeConfigResolver(single_node_fn, all_nodes_fn, node_uuids, opts):
             for uuid in node_uuids]
 
 
-class _RpcProcessor:
+class _RpcProcessor(object):
   def __init__(self, resolver, port, lock_monitor_cb=None):
     """Initializes this class.
 
@@ -390,9 +393,9 @@ class _RpcProcessor:
 
     assert isinstance(body, dict)
     assert len(body) == len(hosts)
-    assert compat.all(isinstance(v, str) for v in body.values())
-    assert frozenset(map(lambda x: x[2], hosts)) == frozenset(body.keys()), \
-        "%s != %s" % (hosts, body.keys())
+    assert compat.all(isinstance(v, (str, bytes)) for v in body.values())
+    assert frozenset(h[2] for h in hosts) == frozenset(body), \
+        "%s != %s" % (hosts, list(body))
 
     for (name, ip, original_name) in hosts:
       if ip is _OFFLINE:
@@ -462,14 +465,15 @@ class _RpcProcessor:
       self._PrepareRequests(self._resolver(nodes, resolver_opts), self._port,
                             procedure, body, read_timeout)
 
-    _req_process_fn(requests.values(), lock_monitor_cb=self._lock_monitor_cb)
+    _req_process_fn(list(requests.values()),
+                    lock_monitor_cb=self._lock_monitor_cb)
 
     assert not frozenset(results).intersection(requests)
 
     return self._CombineResults(results, requests, procedure)
 
 
-class _RpcClientBase:
+class _RpcClientBase(object):
   def __init__(self, resolver, encoder_fn, lock_monitor_cb=None,
                _req_process_fn=None):
     """Initializes this class.
@@ -482,10 +486,11 @@ class _RpcClientBase:
     self._encoder = compat.partial(self._EncodeArg, encoder_fn)
 
   @staticmethod
-  def _EncodeArg(encoder_fn, node, (argkind, value)):
+  def _EncodeArg(encoder_fn, node, arg):
     """Encode argument.
 
     """
+    (argkind, value) = arg
     if argkind is None:
       return value
     else:
@@ -517,8 +522,8 @@ class _RpcClientBase:
 
     # encode the arguments for each node individually, pass them and the node
     # name to the prep_fn, and serialise its return value
-    encode_args_fn = lambda node: map(compat.partial(self._encoder, node),
-                                      zip(map(compat.snd, argdefs), args))
+    encode_args_fn = lambda node: [self._encoder(node, (argdef[1], val)) for
+                                      (argdef, val) in zip(argdefs, args)]
     pnbody = dict(
       (n,
        serializer.DumpJson(prep_fn(n, encode_args_fn(n)),
@@ -530,8 +535,7 @@ class _RpcClientBase:
                         req_resolver_opts)
 
     if postproc_fn:
-      return dict(map(lambda (key, value): (key, postproc_fn(value)),
-                      result.items()))
+      return dict((k, postproc_fn(v)) for (k, v) in result.items())
     else:
       return result
 
@@ -549,7 +553,7 @@ def _ObjectListToDict(node, value):
   """Converts a list of L{objects} to dictionaries.
 
   """
-  return map(compat.partial(_ObjectToDict, node), value)
+  return [_ObjectToDict(node, v) for v in value]
 
 
 def _PrepareFileUpload(getents_fn, node, filename):
@@ -557,7 +561,7 @@ def _PrepareFileUpload(getents_fn, node, filename):
 
   """
   statcb = utils.FileStatHelper()
-  data = _Compress(node, utils.ReadFile(filename, preread=statcb))
+  data = _Compress(node, utils.ReadBinaryFile(filename, preread=statcb))
   st = statcb.st
 
   if getents_fn is None:
@@ -653,10 +657,11 @@ def MakeLegacyNodeInfo(data, disk_template):
   return ret
 
 
-def _AnnotateDParamsDRBD(disk, (drbd_params, data_params, meta_params)):
+def _AnnotateDParamsDRBD(disk, params):
   """Annotates just DRBD disks layouts.
 
   """
+  (drbd_params, data_params, meta_params) = params
   assert disk.dev_type == constants.DT_DRBD8
 
   disk.params = objects.FillDict(drbd_params, disk.params)
@@ -667,13 +672,13 @@ def _AnnotateDParamsDRBD(disk, (drbd_params, data_params, meta_params)):
   return disk
 
 
-def _AnnotateDParamsGeneric(disk, (params, )):
+def _AnnotateDParamsGeneric(disk, params):
   """Generic disk parameter annotation routine.
 
   """
   assert disk.dev_type != constants.DT_DRBD8
 
-  disk.params = objects.FillDict(params, disk.params)
+  disk.params = objects.FillDict(params[0], disk.params)
 
   return disk
 
@@ -858,7 +863,8 @@ class RpcRunner(_RpcClientBase,
         n.netinfo = objects.Network.ToDict(nobj)
     return n.ToDict()
 
-  def _DeviceDict(self, _, (device, instance)):
+  def _DeviceDict(self, _, devinstance):
+    (device, instance) = devinstance
     if isinstance(device, objects.NIC):
       return self._NicDict(None, device)
     elif isinstance(device, objects.Disk):
@@ -910,22 +916,25 @@ class RpcRunner(_RpcClientBase,
           nic["netinfo"] = objects.Network.ToDict(nobj)
     return idict
 
-  def _InstDictHvpBepDp(self, node, (instance, hvp, bep)):
+  def _InstDictHvpBepDp(self, node, instance_params):
     """Wrapper for L{_InstDict}.
 
     """
+    (instance, hvp, bep) = instance_params
     return self._InstDict(node, instance, hvp=hvp, bep=bep)
 
-  def _InstDictOspDp(self, node, (instance, osparams)):
+  def _InstDictOspDp(self, node, instance_osparams):
     """Wrapper for L{_InstDict}.
 
     """
+    (instance, osparams) = instance_osparams
     return self._InstDict(node, instance, osp=osparams)
 
-  def _DisksDictDP(self, node, (disks, instance)):
+  def _DisksDictDP(self, node, instance_disks):
     """Wrapper for L{AnnotateDiskParams}.
 
     """
+    (disks, instance) = instance_disks
     diskparams = self._cfg.GetInstanceDiskParams(instance)
     ret = []
     for disk in AnnotateDiskParams(disks, diskparams):
@@ -947,11 +956,12 @@ class RpcRunner(_RpcClientBase,
     return [disk for disk_inst in disks_insts
             for disk in self._DisksDictDP(node, disk_inst)]
 
-  def _SingleDiskDictDP(self, node, (disk, instance)):
+  def _SingleDiskDictDP(self, node, instance_disk):
     """Wrapper for L{AnnotateDiskParams}.
 
     """
-    (anno_disk,) = self._DisksDictDP(node, ([disk], instance))
+    (disk, instance) = instance_disk
+    anno_disk = self._DisksDictDP(node, ([disk], instance))[0]
     return anno_disk
 
   def _EncodeNodeToDiskDictDP(self, node, value):
@@ -961,10 +971,11 @@ class RpcRunner(_RpcClientBase,
     return dict((name, [self._SingleDiskDictDP(node, disk) for disk in disks])
                 for name, disks in value.items())
 
-  def _EncodeImportExportIO(self, node, (ieio, ieioargs)):
+  def _EncodeImportExportIO(self, node, ieinfo):
     """Encodes import/export I/O information.
 
     """
+    (ieio, ieioargs) = ieinfo
     if ieio == constants.IEIO_RAW_DISK:
       assert len(ieioargs) == 2
       return (ieio, (self._SingleDiskDictDP(node, ieioargs), ))
