@@ -52,7 +52,7 @@ DEF_CTMO = constants.LUXI_DEF_CTMO
 DEF_RWTO = constants.LUXI_DEF_RWTO
 
 
-class Transport:
+class Transport(object):
   """Low-level transport class.
 
   This is used on the client side.
@@ -70,7 +70,8 @@ class Transport:
     There are two timeouts used since we might want to wait for a long
     time for a response, but the connect timeout should be lower.
 
-    If not passed, we use a default of 10 and respectively 60 seconds.
+    If not passed, we use the default luxi timeouts from the global
+    constants file.
 
     Note that on reading data, since the timeout applies to an
     invidual receive, it might be that the total duration is longer
@@ -92,7 +93,7 @@ class Transport:
       self._ctimeout, self._rwtimeout = timeouts
 
     self.socket = None
-    self._buffer = ""
+    self._buffer = b""
     self._msgs = collections.deque()
 
     try:
@@ -118,9 +119,9 @@ class Transport:
     sock.settimeout(timeout)
     try:
       sock.connect(address)
-    except socket.timeout, err:
+    except socket.timeout as err:
       raise errors.TimeoutError("Connect timed out: %s" % str(err))
-    except socket.error, err:
+    except socket.error as err:
       error_code = err.args[0]
       if error_code in (errno.ENOENT, errno.ECONNREFUSED):
         if not allow_non_master:
@@ -154,6 +155,9 @@ class Transport:
     This just sends a message and doesn't wait for the response.
 
     """
+    if isinstance(msg, str):
+      msg = msg.encode("utf-8")
+
     if constants.LUXI_EOM in msg:
       raise errors.ProtocolError("Message terminator found in payload")
 
@@ -161,7 +165,7 @@ class Transport:
     try:
       # TODO: sendall is not guaranteed to send everything
       self.socket.sendall(msg + constants.LUXI_EOM)
-    except socket.timeout, err:
+    except socket.timeout as err:
       raise errors.TimeoutError("Sending timeout: %s" % str(err))
 
   def Recv(self):
@@ -181,9 +185,9 @@ class Transport:
       while True:
         try:
           data = self.socket.recv(4096)
-        except socket.timeout, err:
+        except socket.timeout as err:
           raise errors.TimeoutError("Receive timeout: %s" % str(err))
-        except socket.error, err:
+        except socket.error as err:
           if err.args and err.args[0] == errno.EAGAIN:
             continue
           raise
@@ -193,7 +197,7 @@ class Transport:
       new_msgs = (self._buffer + data).split(constants.LUXI_EOM)
       self._buffer = new_msgs.pop()
       self._msgs.extend(new_msgs)
-    return self._msgs.popleft()
+    return self._msgs.popleft().decode("utf-8")
 
   def Call(self, msg):
     """Send a message and wait for the response.
@@ -228,10 +232,10 @@ class Transport:
         # we retry on a network error, unless it's the last try
         if try_no == retries - 1:
           raise
-        logging.error("Network error: %s, retring (retry attempt number %d)",
+        logging.error("Network error: %s, retrying (retry attempt number %d)",
                       ex, try_no + 1)
         time.sleep(wait_on_error * try_no)
-      except Exception, ex:
+      except Exception as ex:
         on_error(ex)
         raise
     assert False # we should never get here
@@ -243,7 +247,7 @@ class Transport:
       self.socket = None
 
 
-class FdTransport:
+class FdTransport(object):
   """Low-level transport class that works on arbitrary file descriptors.
 
   Unlike L{Transport}, this doesn't use timeouts.
@@ -265,7 +269,7 @@ class FdTransport:
     self._rstream = io.open(fds[0], 'rb', 0)
     self._wstream = io.open(fds[1], 'wb', 0)
 
-    self._buffer = ""
+    self._buffer = b""
     self._msgs = collections.deque()
 
   def _CheckSocket(self):
@@ -281,6 +285,9 @@ class FdTransport:
     This just sends a message and doesn't wait for the response.
 
     """
+    if isinstance(msg, str):
+      msg = msg.encode("utf-8")
+
     if constants.LUXI_EOM in msg:
       raise errors.ProtocolError("Message terminator found in payload")
 
@@ -303,7 +310,7 @@ class FdTransport:
       new_msgs = (self._buffer + data).split(constants.LUXI_EOM)
       self._buffer = new_msgs.pop()
       self._msgs.extend(new_msgs)
-    return self._msgs.popleft()
+    return self._msgs.popleft().decode("utf-8")
 
   def Call(self, msg):
     """Send a message and wait for the response.
