@@ -115,7 +115,8 @@ def _BlockdevGetMirrorStatusPostProc(result):
 
   """
   if not result.fail_msg:
-    result.payload = map(objects.BlockDevStatus.FromDict, result.payload)
+    result.payload = [objects.BlockDevStatus.FromDict(d)
+                      for d in result.payload]
   return result
 
 
@@ -147,7 +148,7 @@ def _NodeInfoPreProc(node, args):
   assert len(args) == 2
   # The storage_units argument is either a dictionary with one value for each
   # node, or a fixed value to be used for all the nodes
-  if type(args[0]) is dict:
+  if isinstance(args[0], dict):
     return [args[0][node], args[1]]
   else:
     return args
@@ -175,11 +176,12 @@ def _ImpExpStatusPostProc(result):
   return result
 
 
-def _TestDelayTimeout((duration, )):
+def _TestDelayTimeout(duration):
   """Calculate timeout for "test_delay" RPC.
 
   """
-  return int(duration + 5)
+  _duration = duration[0]
+  return int(_duration + 5)
 
 
 _FILE_STORAGE_CALLS = [
@@ -513,9 +515,6 @@ _NODE_CALLS = [
     ("checkdict", None, "What to verify"),
     ("cluster_name", None, "Cluster name"),
     ("all_hvparams", None, "Dictionary mapping hypervisor names to hvparams"),
-    ("node_groups", None, "node names mapped to their group uuids"),
-    ("groups_cfg", None,
-      "a dictionary mapping group uuids to their configuration"),
     ], None, None, "Request verification of given parameters"),
   ("node_volumes", MULTI, None, constants.RPC_TMO_FAST, [], None, None,
    "Gets all volumes on node(s)"),
@@ -537,7 +536,7 @@ _NODE_CALLS = [
     ("daemon", None, "Daemon name"),
     ("run", None, "Whether the daemon should be running or stopped"),
     ], None, None, "Ensure daemon is running on the node."),
-  ("node_ssh_key_add", MULTI, None, constants.RPC_TMO_URGENT, [
+  ("node_ssh_key_add", MULTI, None, constants.RPC_TMO_FAST, [
     ("node_uuid", None, "UUID of the node whose key is distributed"),
     ("node_name", None, "Name of the node whose key is distributed"),
     ("potential_master_candidates", None, "Potential master candidates"),
@@ -548,7 +547,7 @@ _NODE_CALLS = [
     ("get_public_keys", None, "Whether the node should get the other nodes'"
      " public keys")],
     None, None, "Distribute a new node's public SSH key on the cluster."),
-  ("node_ssh_key_remove", MULTI, None, constants.RPC_TMO_URGENT, [
+  ("node_ssh_key_remove", MULTI, None, constants.RPC_TMO_FAST, [
     ("node_uuid", None, "UUID of the node whose key is removed"),
     ("node_name", None, "Name of the node whose key is removed"),
     ("master_candidate_uuids", None, "List of UUIDs of master candidates."),
@@ -568,7 +567,10 @@ _NODE_CALLS = [
     ("node_uuids", None, "UUIDs of the nodes whose key is renewed"),
     ("node_names", None, "Names of the nodes whose key is renewed"),
     ("master_candidate_uuids", None, "List of UUIDs of master candidates."),
-    ("potential_master_candidates", None, "Potential master candidates")],
+    ("potential_master_candidates", None, "Potential master candidates"),
+    ("old_key_type", None, "The type of key previously used"),
+    ("new_key_type", None, "The type of key to generate"),
+    ("new_key_bits", None, "The length of the key to generate")],
     None, None, "Renew all SSH key pairs of all nodes nodes."),
   ]
 
@@ -683,9 +685,6 @@ CALLS = {
       ("checkdict", None, "What to verify"),
       ("cluster_name", None, "Cluster name"),
       ("hvparams", None, "Dictionary mapping hypervisor names to hvparams"),
-      ("node_groups", None, "node names mapped to their group uuids"),
-      ("groups_cfg", None,
-       "a dictionary mapping group uuids to their configuration"),
       ], None, None, "Request verification of given parameters"),
     ]),
   "RpcClientConfig": _Prepare([
