@@ -30,19 +30,17 @@
 
 """Ganeti node daemon"""
 
-# pylint: disable=C0103,W0142
+# pylint: disable=C0103
 
 # C0103: Functions in this module need to have a given name structure,
 # and the name of the daemon doesn't match
 
-# W0142: Used * or ** magic, since we do use it extensively in this
-# module
+from __future__ import print_function
 
 import os
 import sys
 import logging
 import signal
-import codecs
 
 from optparse import OptionParser
 
@@ -91,7 +89,7 @@ def _PrepareQueueLock():
   try:
     queue_lock = jstore.InitAndVerifyQueue(must_lock=False)
     return None
-  except EnvironmentError, err:
+  except EnvironmentError as err:
     return err
 
 
@@ -190,12 +188,12 @@ class NodeRequestHandler(http.server.HttpServerHandler):
     try:
       result = (True, method(serializer.LoadJson(req.request_body)))
 
-    except backend.RPCFail, err:
+    except backend.RPCFail as err:
       # our custom failure exception; str(err) works fine if the
       # exception was constructed with a single argument, and in
       # this case, err.message == err.args[0] == str(err)
       result = (False, str(err))
-    except errors.QuitGanetiException, err:
+    except errors.QuitGanetiException as err:
       # Tell parent to quit
       logging.info("Shutting down the node daemon, arguments: %s",
                    str(err.args))
@@ -203,7 +201,7 @@ class NodeRequestHandler(http.server.HttpServerHandler):
       # And return the error's arguments, which must be already in
       # correct tuple format
       result = err.args
-    except Exception, err:
+    except Exception as err: # pylint: disable=W0703
       logging.exception("Error in RPC call")
       result = (False, "Error while executing backend function: %s" % str(err))
 
@@ -821,9 +819,8 @@ class NodeRequestHandler(http.server.HttpServerHandler):
     """Run a verify sequence on this node.
 
     """
-    (what, cluster_name, hvparams, node_groups, groups_cfg) = params
-    return backend.VerifyNode(what, cluster_name, hvparams,
-                              node_groups, groups_cfg)
+    (what, cluster_name, hvparams) = params
+    return backend.VerifyNode(what, cluster_name, hvparams)
 
   @classmethod
   def perspective_node_verify_light(cls, params):
@@ -946,10 +943,11 @@ class NodeRequestHandler(http.server.HttpServerHandler):
 
     """
     (node_uuids, node_names, master_candidate_uuids,
-     potential_master_candidates) = params
-    return backend.RenewSshKeys(node_uuids, node_names,
-                                master_candidate_uuids,
-                                potential_master_candidates)
+     potential_master_candidates, old_key_type, new_key_type,
+     new_key_bits) = params
+    return backend.RenewSshKeys(node_uuids, node_names, master_candidate_uuids,
+                                potential_master_candidates, old_key_type,
+                                new_key_type, new_key_bits)
 
   @staticmethod
   def perspective_node_ssh_key_remove(params):
@@ -1283,20 +1281,18 @@ class NodeRequestHandler(http.server.HttpServerHandler):
     return backend.CleanupImportExport(params[0])
 
 
-def CheckNoded(_, args):
+def CheckNoded(options, args):
   """Initial checks whether to run or exit with a failure.
 
   """
   if args: # noded doesn't take any arguments
-    print >> sys.stderr, ("Usage: %s [-f] [-d] [-p port] [-b ADDRESS]" %
-                          sys.argv[0])
+    print("Usage: %s [-f] [-d] [-p port] [-b ADDRESS]" %
+                          sys.argv[0], file=sys.stderr)
     sys.exit(constants.EXIT_FAILURE)
-  try:
-    codecs.lookup("string-escape")
-  except LookupError:
-    print >> sys.stderr, ("Can't load the string-escape code which is part"
-                          " of the Python installation. Is your installation"
-                          " complete/correct? Aborting.")
+
+  if options.max_clients < 1:
+    print("%s --max-clients argument must be >= 1" %
+                          sys.argv[0], file=sys.stderr)
     sys.exit(constants.EXIT_FAILURE)
 
 
@@ -1338,15 +1334,16 @@ def SSLVerifyPeer(conn, cert, errnum, errdepth, ok):
   # If we receive a certificate from the certificate chain that is higher
   # than the lowest element of the chain, we have to check it against the
   # server certificate.
+  cert_digest = cert.digest("sha1").decode("ascii")
   if errdepth > 0:
     server_digest = utils.GetCertificateDigest(
         cert_filename=pathutils.NODED_CERT_FILE)
-    match = cert.digest("sha1") == server_digest
+    match = cert_digest == server_digest
     if not match:
       logging.debug("Received certificate from the certificate chain, which"
                     " does not match the server certficate. Digest of the"
                     " received certificate: %s. Digest of the server"
-                    " certificate: %s.", cert.digest("sha1"), server_digest)
+                    " certificate: %s.", cert_digest, server_digest)
     return match
   elif errdepth == 0:
     sstore = ssconf.SimpleStore()
@@ -1360,17 +1357,16 @@ def SSLVerifyPeer(conn, cert, errnum, errdepth, ok):
       candidate_certs = {
         constants.CRYPTO_BOOTSTRAP: utils.GetCertificateDigest(
           cert_filename=pathutils.NODED_CERT_FILE)}
-    match = cert.digest("sha1") in candidate_certs.values()
+    match = cert_digest in candidate_certs.values()
     if not match:
       logging.debug("Received certificate which is not a certificate of a"
                     " master candidate. Certificate digest: %s. List of master"
-                    " candidate certificate digests: %s.", cert.digest("sha1"),
+                    " candidate certificate digests: %s.", cert_digest,
                     str(candidate_certs))
     return match
   else:
     logging.error("Invalid errdepth value: %s.", errdepth)
     return False
-  # pylint: enable=W0613
 
 
 def PrepNoded(options, _):
@@ -1404,11 +1400,11 @@ def PrepNoded(options, _):
   handler = NodeRequestHandler()
 
   mainloop = daemon.Mainloop()
-  server = \
-    http.server.HttpServer(mainloop, options.bind_address, options.port,
-                           handler, ssl_params=ssl_params, ssl_verify_peer=True,
-                           request_executor_class=request_executor_class,
-                           ssl_verify_callback=SSLVerifyPeer)
+  server = http.server.HttpServer(
+      mainloop, options.bind_address, options.port, options.max_clients,
+      handler, ssl_params=ssl_params, ssl_verify_peer=True,
+      request_executor_class=request_executor_class,
+      ssl_verify_callback=SSLVerifyPeer)
   server.Start()
 
   return (mainloop, server)
@@ -1437,6 +1433,10 @@ def Main():
   parser.add_option("--no-mlock", dest="mlock",
                     help="Do not mlock the node memory in ram",
                     default=True, action="store_false")
+  parser.add_option("--max-clients", dest="max_clients",
+                    default=20, type="int",
+                    help="Number of simultaneous connections accepted"
+                    " by noded")
 
   daemon.GenericMain(constants.NODED, parser, CheckNoded, PrepNoded, ExecNoded,
                      default_ssl_cert=pathutils.NODED_CERT_FILE,
