@@ -31,10 +31,12 @@
 
 """
 
-# pylint: disable=C0103,W0142
+# pylint: disable=C0103
 
 # C0103: Invalid name ganeti-watcher
 
+from __future__ import print_function
+
 import logging
 import optparse
 import sys
@@ -61,7 +63,7 @@ from ganeti.rapi import connector
 from ganeti.rapi import baserlib
 
 import ganeti.http.auth   # pylint: disable=W0611
-import ganeti.http.server
+import ganeti.http.server # pylint: disable=W0611
 
 
 class RemoteApiRequestContext(object):
@@ -206,7 +208,7 @@ class RemoteApiHandler(http.auth.HttpServerRequestAuthentication,
       result = ctx.handler_fn()
     except rpcerr.TimeoutError:
       raise http.HttpGatewayTimeout()
-    except rpcerr.ProtocolError, err:
+    except rpcerr.ProtocolError as err:
       raise http.HttpBadGateway(str(err))
 
     req.resp_headers[http.HTTP_CONTENT_TYPE] = http.HTTP_APP_JSON
@@ -241,7 +243,7 @@ class RapiUsers(object):
     try:
       try:
         contents = utils.ReadFile(filename)
-      except EnvironmentError, err:
+      except EnvironmentError as err:
         self._users = None
         if err.errno == errno.ENOENT:
           logging.warning("No users file at %s", filename)
@@ -251,7 +253,7 @@ class RapiUsers(object):
 
       users = http.auth.ParsePasswordFile(contents)
 
-    except Exception, err: # pylint: disable=W0703
+    except Exception as err: # pylint: disable=W0703
       # We don't care about the type of exception
       logging.error("Error while parsing %s: %s", filename, err)
       return False
@@ -314,8 +316,13 @@ def CheckRapi(options, args):
 
   """
   if args: # rapi doesn't take any arguments
-    print >> sys.stderr, ("Usage: %s [-f] [-d] [-p port] [-b ADDRESS]" %
-                          sys.argv[0])
+    print("Usage: %s [-f] [-d] [-p port] [-b ADDRESS]" %
+                          sys.argv[0], file=sys.stderr)
+    sys.exit(constants.EXIT_FAILURE)
+
+  if options.max_clients < 1:
+    print("%s --max-clients argument must be >= 1" %
+                          sys.argv[0], file=sys.stderr)
     sys.exit(constants.EXIT_FAILURE)
 
   ssconf.CheckMaster(options.debug)
@@ -344,10 +351,9 @@ def PrepRapi(options, _):
 
   users.Load(pathutils.RAPI_USERS_FILE)
 
-  server = \
-    http.server.HttpServer(mainloop, options.bind_address, options.port,
-                           handler,
-                           ssl_params=options.ssl_params, ssl_verify_peer=False)
+  server = http.server.HttpServer(
+      mainloop, options.bind_address, options.port, options.max_clients,
+      handler, ssl_params=options.ssl_params, ssl_verify_peer=False)
   server.Start()
 
   return (mainloop, server)
@@ -377,6 +383,10 @@ def Main():
                     default=False, action="store_true",
                     help=("Disable anonymous HTTP requests and require"
                           " authentication"))
+  parser.add_option("--max-clients", dest="max_clients",
+                    default=20, type="int",
+                    help="Number of simultaneous connections accepted"
+                    " by ganeti-rapi")
 
   daemon.GenericMain(constants.RAPI, parser, CheckRapi, PrepRapi, ExecRapi,
                      default_ssl_cert=pathutils.RAPI_CERT_FILE,
