@@ -157,8 +157,9 @@ def AddAuthorizedKeys(file_obj, keys):
   """
   key_field_list = [(key, _SplitSshKey(key)) for key in keys]
 
-  if isinstance(file_obj, basestring):
+  if isinstance(file_obj, str):
     f = open(file_obj, "a+")
+    f.seek(0)
   else:
     f = file_obj
 
@@ -193,7 +194,7 @@ def HasAuthorizedKey(file_obj, key):
   """
   key_fields = _SplitSshKey(key)
 
-  if isinstance(file_obj, basestring):
+  if isinstance(file_obj, str):
     f = open(file_obj, "r")
   else:
     f = file_obj
@@ -221,7 +222,7 @@ def CheckForMultipleKeys(file_obj, node_names):
 
   """
 
-  if isinstance(file_obj, basestring):
+  if isinstance(file_obj, str):
     f = open(file_obj, "r")
   else:
     f = file_obj
@@ -620,7 +621,7 @@ def ReplaceNameByUuid(node_uuid, node_name, key_file=pathutils.SSH_PUB_KEYS,
                         error_fn=error_fn)
 
 
-def ClearPubKeyFile(key_file=pathutils.SSH_PUB_KEYS, mode=0600):
+def ClearPubKeyFile(key_file=pathutils.SSH_PUB_KEYS, mode=0o600):
   """Resets the content of the public key file.
 
   """
@@ -965,7 +966,7 @@ def WriteKnownHostsFile(cfg, file_name):
   if cfg.GetDsaHostKey():
     data += "%s ssh-dss %s\n" % (cfg.GetClusterName(), cfg.GetDsaHostKey())
 
-  utils.WriteFile(file_name, mode=0600, data=data)
+  utils.WriteFile(file_name, mode=0o600, data=data)
 
 
 def _EnsureCorrectGanetiVersion(cmd):
