@@ -82,7 +82,7 @@ class BlockDev(object):
 
   """
   # pylint: disable=W0613
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     self._children = children
     self.dev_path = None
     self.unique_id = unique_id
@@ -93,6 +93,19 @@ class BlockDev(object):
     self.params = params
     self.dyn_params = dyn_params
 
+  def __eq__(self, other):
+    if not isinstance(self, type(other)):
+      return False
+    return (self._children == other._children and # pylint: disable=W0212
+            self.dev_path == other.dev_path and
+            self.unique_id == other.unique_id and
+            self.major == other.major and
+            self.minor == other.minor and
+            self.attached == other.attached and
+            self.size == other.size and
+            self.params == other.params and
+            self.dyn_params == other.dyn_params)
+
   def Assemble(self):
     """Assemble the device from its components.
 
@@ -107,7 +120,7 @@ class BlockDev(object):
     """
     pass
 
-  def Attach(self):
+  def Attach(self, **kwargs):
     """Find a device which matches our config and attach to it.
 
     """
@@ -121,7 +134,7 @@ class BlockDev(object):
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *args):
+             dyn_params, **kwargs):
     """Create the device.
 
     If the device cannot be created, it will return None
@@ -409,7 +422,7 @@ class BlockDev(object):
                   result.fail_reason, result.output)
     try:
       sz = int(result.output.strip())
-    except (ValueError, TypeError), err:
+    except (ValueError, TypeError) as err:
       ThrowError("Failed to parse blockdev output: %s", str(err))
     return sz
 
@@ -480,6 +493,6 @@ def IgnoreError(fn, *args, **kwargs):
   try:
     fn(*args, **kwargs)
     return True
-  except errors.BlockDeviceError, err:
+  except errors.BlockDeviceError as err:
     logging.warning("Caught BlockDeviceError but ignoring: %s", str(err))
     return False
