@@ -73,7 +73,7 @@ class DRBD8(object):
     """
     try:
       helper = utils.ReadFile(filename).splitlines()[0]
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       if err.errno == errno.ENOENT:
         base.ThrowError("The file %s cannot be opened, check if the module"
                         " is loaded (%s)", filename, str(err))
@@ -102,8 +102,8 @@ class DRBD8(object):
 
     """
     info = DRBD8.GetProcInfo()
-    return filter(lambda m: not info.GetMinorStatus(m).is_unconfigured,
-                  info.GetMinors())
+    return [m for m in info.GetMinors()
+            if not info.GetMinorStatus(m).is_unconfigured]
 
   @staticmethod
   def FindUnusedMinor():
@@ -115,7 +115,7 @@ class DRBD8(object):
     @rtype: int
 
     """
-    highest = None
+    highest = -1
     info = DRBD8.GetProcInfo()
     for minor in info.GetMinors():
       status = info.GetMinorStatus(minor)
@@ -123,8 +123,6 @@ class DRBD8(object):
         return minor
       highest = max(highest, minor)
 
-    if highest is None: # there are no minors in use at all
-      return 0
     if highest >= DRBD8._MAX_MINORS:
       logging.error("Error: no free drbd minors!")
       raise errors.BlockDeviceError("Can't find a free DRBD minor")
@@ -183,7 +181,7 @@ class DRBD8Dev(base.BlockDev):
   # timeout constants
   _NET_RECONFIG_TIMEOUT = 60
 
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     if children and children.count(None) > 0:
       children = []
     if len(children) not in (0, 2):
@@ -210,7 +208,7 @@ class DRBD8Dev(base.BlockDev):
         logging.info("drbd%s: Ignoring unreadable meta device", self._aminor)
         children = []
     super(DRBD8Dev, self).__init__(unique_id, children, size, params,
-                                   dyn_params, *args)
+                                   dyn_params, **kwargs)
     self.major = self._DRBD_MAJOR
 
     info = DRBD8.GetProcInfo()
@@ -317,6 +315,13 @@ class DRBD8Dev(base.BlockDev):
     """
     return self._show_info_cls.GetDevInfo(self._GetShowData(minor))
 
+  @staticmethod
+  def _NeedsLocalSyncerParams():
+    # For DRBD >= 8.4, syncer init must be done after local, not in net.
+    info = DRBD8.GetProcInfo()
+    version = info.GetVersion()
+    return version["k_minor"] >= 4
+
   def _MatchesLocal(self, info):
     """Test if our local config matches with an existing device.
 
@@ -399,6 +404,20 @@ class DRBD8Dev(base.BlockDev):
         base.ThrowError("drbd%d: can't attach local disk: %s",
                         minor, result.output)
 
+    def _WaitForMinorSyncParams():
+      """Call _SetMinorSyncParams and raise RetryAgain on errors.
+      """
+      if self._SetMinorSyncParams(minor, self.params):
+        raise utils.RetryAgain()
+
+    if self._NeedsLocalSyncerParams():
+      # Retry because disk config for DRBD resource may be still uninitialized.
+      try:
+        utils.Retry(_WaitForMinorSyncParams, 1.0, 5.0)
+      except utils.RetryTimeout as e:
+        base.ThrowError("drbd%d: can't set the synchronization parameters: %s" %
+                        (minor, utils.CommaJoin(e.args[0])))
+
   def _AssembleNet(self, minor, net_info, dual_pri=False, hmac=None,
                    secret=None):
     """Configure the network part of the device.
@@ -434,21 +453,24 @@ class DRBD8Dev(base.BlockDev):
     # sync speed only after setting up both sides can race with DRBD
     # connecting, hence we set it here before telling DRBD anything
     # about its peer.
-    sync_errors = self._SetMinorSyncParams(minor, self.params)
-    if sync_errors:
-      base.ThrowError("drbd%d: can't set the synchronization parameters: %s" %
-                      (minor, utils.CommaJoin(sync_errors)))
+
+    if not self._NeedsLocalSyncerParams():
+      sync_errors = self._SetMinorSyncParams(minor, self.params)
+      if sync_errors:
+        base.ThrowError("drbd%d: can't set the synchronization parameters: %s" %
+                        (minor, utils.CommaJoin(sync_errors)))
 
     family = self._GetNetFamily(minor, lhost, rhost)
 
-    cmd = self._cmd_gen.GenNetInitCmd(minor, family, lhost, lport,
+    cmds = self._cmd_gen.GenNetInitCmds(minor, family, lhost, lport,
                                       rhost, rport, protocol,
                                       dual_pri, hmac, secret, self.params)
 
-    result = utils.RunCmd(cmd)
-    if result.failed:
-      base.ThrowError("drbd%d: can't setup network: %s - %s",
-                      minor, result.fail_reason, result.output)
+    for cmd in cmds:
+      result = utils.RunCmd(cmd)
+      if result.failed:
+        base.ThrowError("drbd%d: can't setup network: %s - %s",
+                         minor, result.fail_reason, result.output)
 
     def _CheckNetworkConfig():
       info = self._GetShowInfo(minor)
@@ -465,19 +487,20 @@ class DRBD8Dev(base.BlockDev):
       base.ThrowError("drbd%d: timeout while configuring network", minor)
 
     # Once the assembly is over, try to set the synchronization parameters
-    try:
-      # The minor may not have been set yet, requiring us to set it at least
-      # temporarily
-      old_minor = self.minor
-      self._SetFromMinor(minor)
-      sync_errors = self.SetSyncParams(self.params)
-      if sync_errors:
-        base.ThrowError("drbd%d: can't set the synchronization parameters: %s" %
-                        (self.minor, utils.CommaJoin(sync_errors)))
-    finally:
-      # Undo the change, regardless of whether it will have to be done again
-      # soon
-      self._SetFromMinor(old_minor)
+    if not self._NeedsLocalSyncerParams():
+      try:
+        # The minor may not have been set yet, requiring us to set it at least
+        # temporarily
+        old_minor = self.minor
+        self._SetFromMinor(minor)
+        sync_errors = self.SetSyncParams(self.params)
+        if sync_errors:
+          base.ThrowError("drbd%d: can't set the synchronization parameters: "
+                          "%s" % (self.minor, utils.CommaJoin(sync_errors)))
+      finally:
+        # Undo the change, regardless of whether it will have to be done again
+        # soon
+        self._SetFromMinor(old_minor)
 
   @staticmethod
   def _GetNetFamily(minor, lhost, rhost):
@@ -792,7 +815,7 @@ class DRBD8Dev(base.BlockDev):
                       dual_pri=multimaster, hmac=constants.DRBD_HMAC_ALG,
                       secret=self._secret)
 
-  def Attach(self):
+  def Attach(self, **kwargs):
     """Check if our minor is configured.
 
     This doesn't do any device configurations - it only checks if the
@@ -885,7 +908,7 @@ class DRBD8Dev(base.BlockDev):
         # reconnect to our correct one
         try:
           self._ShutdownNet(minor)
-        except errors.BlockDeviceError, err:
+        except errors.BlockDeviceError as err:
           base.ThrowError("drbd%d: device has correct local storage, wrong"
                           " remote peer and is unable to disconnect in order"
                           " to attach to the correct peer: %s", minor, str(err))
@@ -1021,7 +1044,7 @@ class DRBD8Dev(base.BlockDev):
     """
     # Zero the metadata first, in order to make sure drbdmeta doesn't
     # try to auto-detect existing filesystems or similar (see
-    # http://code.google.com/p/ganeti/issues/detail?id=182); we only
+    # https://github.com/ganeti/ganeti/issues/238); we only
     # care about the first 128MB of data in the device, even though it
     # can be bigger
     result = utils.RunCmd([constants.DD_CMD,
@@ -1041,7 +1064,7 @@ class DRBD8Dev(base.BlockDev):
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *_):
+             dyn_params, **kwargs):
     """Create a new DRBD8 device.
 
     Since DRBD devices are not created per se, just assembled, this
@@ -1087,7 +1110,7 @@ def _CanReadDevice(path):
 
   """
   try:
-    utils.ReadFile(path, size=_DEVICE_READ_SIZE)
+    utils.ReadBinaryFile(path, size=_DEVICE_READ_SIZE)
     return True
   except EnvironmentError:
     logging.warning("Can't read from device %s", path, exc_info=True)
