@@ -52,13 +52,14 @@ class ExtStorageDevice(base.BlockDev):
   handling of the externally provided block devices.
 
   """
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     """Attaches to an extstorage block device.
 
     """
     super(ExtStorageDevice, self).__init__(unique_id, children, size, params,
-                                           dyn_params, *args)
-    (self.name, self.uuid) = args
+                                           dyn_params, **kwargs)
+    self.name = kwargs["name"]
+    self.uuid = kwargs["uuid"]
 
     if not isinstance(unique_id, (tuple, list)) or len(unique_id) != 2:
       raise ValueError("Invalid configuration data %s" % str(unique_id))
@@ -72,14 +73,13 @@ class ExtStorageDevice(base.BlockDev):
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *args):
+             dyn_params, **kwargs):
     """Create a new extstorage device.
 
     Provision a new volume using an extstorage provider, which will
     then be mapped to a block device.
 
     """
-    (name, uuid) = args
 
     if not isinstance(unique_id, (tuple, list)) or len(unique_id) != 2:
       raise errors.ProgrammerError("Invalid configuration data %s" %
@@ -91,10 +91,11 @@ class ExtStorageDevice(base.BlockDev):
     # Call the External Storage's create script,
     # to provision a new Volume inside the External Storage
     _ExtStorageAction(constants.ES_ACTION_CREATE, unique_id,
-                      params, size=size, name=name, uuid=uuid)
+                      params, size=size, name=kwargs["name"],
+                      uuid=kwargs["uuid"])
 
     return ExtStorageDevice(unique_id, children, size, params, dyn_params,
-                            *args)
+                            **kwargs)
 
   def Remove(self):
     """Remove the extstorage device.
@@ -118,7 +119,7 @@ class ExtStorageDevice(base.BlockDev):
     """
     pass
 
-  def Attach(self):
+  def Attach(self, **kwargs):
     """Attach to an existing extstorage device.
 
     This method maps the extstorage volume that matches our name with
@@ -155,7 +156,7 @@ class ExtStorageDevice(base.BlockDev):
     # Verify that dev_path exists and is a block device
     try:
       st = os.stat(self.dev_path)
-    except OSError, err:
+    except OSError as err:
       logging.error("Error stat()'ing %s: %s", self.dev_path, str(err))
       return False
 
@@ -411,7 +412,7 @@ def _CheckExtStorageFile(base_dir, filename, required):
   file_path = utils.PathJoin(base_dir, filename)
   try:
     st = os.stat(file_path)
-  except EnvironmentError, err:
+  except EnvironmentError as err:
     if not required:
       logging.info("Optional file '%s' under path '%s' is missing",
                    filename, base_dir)
@@ -476,7 +477,7 @@ def ExtStorageFromDisk(name, base_dir=None):
       # script or None, depending on the corresponding checks. See the
       # function's docstrings for more on these checks.
       es_files[filename] = _CheckExtStorageFile(es_dir, filename, required)
-    except errors.BlockDeviceError, err:
+    except errors.BlockDeviceError as err:
       return False, str(err)
 
   parameters = []
@@ -484,7 +485,7 @@ def ExtStorageFromDisk(name, base_dir=None):
     parameters_file = es_files[constants.ES_PARAMETERS_FILE]
     try:
       parameters = utils.ReadFile(parameters_file).splitlines()
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       return False, ("Error while reading the EXT parameters file at %s: %s" %
                      (parameters_file, utils.ErrnoOrStr(err)))
     parameters = [v.split(None, 1) for v in parameters]
