@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Google Inc.
@@ -32,15 +32,18 @@
 
 """
 
+from __future__ import print_function
+
 import sys
 import optparse
 import time
 import socket
-import urllib
+import urllib.request, urllib.parse, urllib.error
 import random
 import string # pylint: disable=W0402
-from itertools import izip, islice, cycle
-from cStringIO import StringIO
+from functools import reduce
+from itertools import islice, cycle
+from io import StringIO
 from operator import or_
 
 from ganeti import opcodes
@@ -107,8 +110,8 @@ class BurninFailure(Exception):
 def Usage():
   """Shows program usage information and exits the program."""
 
-  print >> sys.stderr, "Usage:"
-  print >> sys.stderr, USAGE
+  print("Usage:", file=sys.stderr)
+  print(USAGE, file=sys.stderr)
   sys.exit(2)
 
 
@@ -137,7 +140,7 @@ def RandomString(size=8, chars=string.ascii_uppercase + string.digits):
   return ''.join(random.choice(chars) for x in range(size))
 
 
-class SimpleOpener(urllib.FancyURLopener):
+class SimpleOpener(urllib.request.FancyURLopener):
   """A simple url opener"""
   # pylint: disable=W0221
 
@@ -370,7 +373,7 @@ class JobHandler(FeedbackAccumulator):
         Log("Idempotent %s succeeded after %d retries",
             msg, MAX_RETRIES - retry_count)
       return val
-    except Exception, err: # pylint: disable=W0703
+    except Exception as err: # pylint: disable=W0703
       if retry_count == 0:
         Log("Non-idempotent %s failed, aborting", msg)
         raise
@@ -416,7 +419,7 @@ class JobHandler(FeedbackAccumulator):
       cli.SetGenericOpcodeOpts(ops, self.opts)
       self.queued_ops.append((ops, name, post_process))
     else:
-      val = self.ExecOp(self.queue_retry, *ops) # pylint: disable=W0142
+      val = self.ExecOp(self.queue_retry, *ops)
       if post_process is not None:
         post_process()
       return val
@@ -458,10 +461,10 @@ class JobHandler(FeedbackAccumulator):
     self.ClearFeedbackBuf()
     jex = cli.JobExecutor(cl=self.cl, feedback_fn=self.Feedback)
     for ops, name, _ in jobs:
-      jex.QueueJob(name, *ops) # pylint: disable=W0142
+      jex.QueueJob(name, *ops)
     try:
       results = jex.GetResults()
-    except Exception, err: # pylint: disable=W0703
+    except Exception as err: # pylint: disable=W0703
       Log("Jobs failed: %s", err)
       raise BurninFailure()
 
@@ -472,7 +475,7 @@ class JobHandler(FeedbackAccumulator):
         if post_process:
           try:
             post_process()
-          except Exception, err: # pylint: disable=W0703
+          except Exception as err: # pylint: disable=W0703
             Log("Post process call for job %s failed: %s", name, err)
             fail = True
         val.append(result)
@@ -584,7 +587,7 @@ class Burner(JobHandler):
     try:
       qcl = GetClient()
       result = qcl.QueryNodes(names, ["name", "offline", "drained"], False)
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       err_code, msg = cli.FormatError(err)
       Err(msg, exit_code=err_code)
     finally:
@@ -624,7 +627,7 @@ class Burner(JobHandler):
   def FindMatchingDisk(self, instance):
     """Find a disk whose nodes match the instance's disk nodes."""
     instance_nodes = self.instance_nodes[instance]
-    for disk, disk_nodes in self.disk_nodes.iteritems():
+    for disk, disk_nodes in self.disk_nodes.items():
       if instance_nodes == disk_nodes:
         # Erase that disk from the dictionary so that we don't pick it again.
         del self.disk_nodes[disk]
@@ -638,7 +641,7 @@ class Burner(JobHandler):
 
     """
     self.to_rem = []
-    mytor = izip(cycle(self.nodes),
+    mytor = zip(cycle(self.nodes),
                  islice(cycle(self.nodes), 1, None),
                  self.instances)
 
@@ -677,8 +680,10 @@ class Burner(JobHandler):
                                     hypervisor=self.hypervisor,
                                     osparams=self.opts.osparams,
                                     )
-      remove_instance = lambda name: lambda: self.to_rem.append(name)
-      self.ExecOrQueue(instance, [op], post_process=remove_instance(instance))
+      # NB the i=instance default param is needed here so the lambda captures
+      # the variable. See https://docs.python.org/2/faq/programming.html#id11
+      rm_inst = lambda i=instance: self.to_rem.append(i) # pylint: disable=C0322
+      self.ExecOrQueue(instance, [op], post_process=rm_inst)
 
   @_DoBatch(False)
   def BurnModifyRuntimeMemory(self):
@@ -729,7 +734,7 @@ class Burner(JobHandler):
     Log("Changing the secondary node")
     mode = constants.REPLACE_DISK_CHG
 
-    mytor = izip(islice(cycle(self.nodes), 2, None),
+    mytor = zip(islice(cycle(self.nodes), 2, None),
                  self.instances)
     for tnode, instance in mytor:
       Log("instance %s", instance, indent=1)
@@ -763,7 +768,7 @@ class Burner(JobHandler):
   def BurnMove(self):
     """Move the instances."""
     Log("Moving instances")
-    mytor = izip(islice(cycle(self.nodes), 1, None),
+    mytor = zip(islice(cycle(self.nodes), 1, None),
                  self.instances)
     for tnode, instance in mytor:
       Log("instance %s", instance, indent=1)
@@ -792,7 +797,7 @@ class Burner(JobHandler):
 
     """
     Log("Exporting and re-importing instances")
-    mytor = izip(cycle(self.nodes),
+    mytor = zip(cycle(self.nodes),
                  islice(cycle(self.nodes), 1, None),
                  islice(cycle(self.nodes), 2, None),
                  self.instances)
@@ -801,7 +806,7 @@ class Burner(JobHandler):
     for pnode, snode, enode, instance in mytor:
       Log("instance %s", instance, indent=1)
       # read the full name of the instance
-      ((full_name, ), ) = qcl.QueryInstances([instance], ["name"], False)
+      (full_name, ) = qcl.QueryInstances([instance], ["name"], False)[0]
 
       if self.opts.iallocator:
         pnode = snode = None
@@ -1018,7 +1023,7 @@ class Burner(JobHandler):
 
   def DoConfdRequestReply(self, req):
     self.confd_counting_callback.RegisterQuery(req.rsalt)
-    self.confd_client.SendRequest(req, async=False)
+    self.confd_client.SendRequest(req, async_=False)
     while not self.confd_counting_callback.AllAnswered():
       if not self.confd_client.ReceiveReply():
         Err("Did not receive all expected confd replies")
@@ -1253,7 +1258,7 @@ class Burner(JobHandler):
       if not self.opts.keep_instances:
         try:
           self.BurnRemove()
-        except Exception, err:  # pylint: disable=W0703
+        except Exception as err:  # pylint: disable=W0703
           if has_err: # already detected errors, so errors in removal
                       # are quite expected
             Log("Note: error detected during instance remove: %s", err)
