@@ -42,7 +42,7 @@ import logging
 import optparse
 import time
 import functools
-from cStringIO import StringIO
+from io import StringIO
 
 from ganeti import cli
 from ganeti import constants
@@ -57,13 +57,15 @@ from ganeti.utils import version
 
 
 #: Target major version we will upgrade to
-TARGET_MAJOR = 2
+TARGET_MAJOR = 3
 #: Target minor version we will upgrade to
-TARGET_MINOR = 15
+TARGET_MINOR = 0
+#: Last supported v2.x minor
+LAST_V2_MINOR = 16
 #: Target major version for downgrade
 DOWNGRADE_MAJOR = 2
 #: Target minor version for downgrade
-DOWNGRADE_MINOR = 14
+DOWNGRADE_MINOR = 16
 
 # map of legacy device types
 # (mapping differing old LD_* constants to new DT_* constants)
@@ -113,7 +115,7 @@ def OrFail(description=None):
       safety = copy.deepcopy(self.config_data)
       try:
         f(self)
-      except BaseException, e:
+      except BaseException as e:
         msg = "%s failed:\n%s" % (description or f.func_name, e)
         logging.exception(msg)
         self.config_data = safety
@@ -183,8 +185,9 @@ class CfgUpgrade(object):
       self._Downgrade(config_major, config_minor, config_version,
                       config_revision)
 
-    # Upgrade from 2.{0..14} to 2.15
-    elif config_major == 2 and config_minor in range(0, 15):
+    # Upgrade from 2.0-2.16 to 3.0
+    # TODO: handle upgrades from 2.17beta
+    elif config_major == 2 and config_minor in range(0, LAST_V2_MINOR + 1):
       if config_revision != 0:
         logging.warning("Config revision is %s, not 0", config_revision)
       if not self.UpgradeAll():
@@ -202,7 +205,7 @@ class CfgUpgrade(object):
                    self.opts.CONFIG_DATA_PATH)
       utils.WriteFile(file_name=self.opts.CONFIG_DATA_PATH,
                       data=serializer.DumpJson(self.config_data),
-                      mode=0600,
+                      mode=0o600,
                       dry_run=self.opts.dry_run,
                       backup=True)
 
@@ -307,24 +310,33 @@ class CfgUpgrade(object):
     cluster = self.config_data.get("cluster", None)
     if cluster is None:
       raise Error("Cannot find cluster")
+
     ipolicy = cluster.setdefault("ipolicy", None)
     if ipolicy:
       self.UpgradeIPolicy(ipolicy, constants.IPOLICY_DEFAULTS, False)
     ial_params = cluster.get("default_iallocator_params", None)
+
     if not ial_params:
       cluster["default_iallocator_params"] = {}
+
     if not "candidate_certs" in cluster:
       cluster["candidate_certs"] = {}
+
     cluster["instance_communication_network"] = \
       cluster.get("instance_communication_network", "")
+
     cluster["install_image"] = \
       cluster.get("install_image", "")
+
     cluster["zeroing_image"] = \
       cluster.get("zeroing_image", "")
+
     cluster["compression_tools"] = \
       cluster.get("compression_tools", constants.IEC_DEFAULT_TOOLS)
+
     if "enabled_user_shutdown" not in cluster:
       cluster["enabled_user_shutdown"] = False
+
     cluster["data_collectors"] = cluster.get("data_collectors", {})
     for name in constants.DATA_COLLECTOR_NAMES:
       cluster["data_collectors"][name] = \
@@ -332,6 +344,14 @@ class CfgUpgrade(object):
             name, dict(active=True,
                        interval=constants.MOND_TIME_INTERVAL * 1e6))
 
+    # These parameters are set to pre-2.16 default values, which
+    # differ from post-2.16 default values
+    if "ssh_key_type" not in cluster:
+      cluster["ssh_key_type"] = constants.SSHK_DSA
+
+    if "ssh_key_bits" not in cluster:
+      cluster["ssh_key_bits"] = 1024
+
   @OrFail("Upgrading groups")
   def UpgradeGroups(self):
     cl_ipolicy = self.config_data["cluster"].get("ipolicy")
@@ -469,7 +489,8 @@ class CfgUpgrade(object):
                    self.opts.RAPI_USERS_FILE_PRE24, self.opts.RAPI_USERS_FILE)
       if not self.opts.dry_run:
         utils.RenameFile(self.opts.RAPI_USERS_FILE_PRE24,
-                         self.opts.RAPI_USERS_FILE, mkdir=True, mkdir_mode=0750)
+                         self.opts.RAPI_USERS_FILE,
+                         mkdir=True, mkdir_mode=0o750)
 
     # Create a symlink for RAPI users file
     if (not (os.path.islink(self.opts.RAPI_USERS_FILE_PRE24) or
@@ -517,7 +538,7 @@ class CfgUpgrade(object):
         buf.write("%s\n" % shared_file_storage_dir)
       utils.WriteFile(file_name=self.opts.FILE_STORAGE_PATHS_FILE,
                       data=buf.getvalue(),
-                      mode=0600,
+                      mode=0o600,
                       dry_run=self.opts.dry_run,
                       backup=True)
 
@@ -657,7 +678,7 @@ class CfgUpgrade(object):
     consist only of the primary instance node.
     """
     disks = self.config_data["disks"]
-    for instance in self.config_data["instances"].itervalues():
+    for instance in self.config_data["instances"].values():
       # Get all disk nodes for an instance
       instance_node = set([instance["primary_node"]])
       disk_nodes = set()
@@ -701,19 +722,42 @@ class CfgUpgrade(object):
 
   # DOWNGRADE ------------------------------------------------------------
 
-  @classmethod
-  def DowngradeCollectors(cls, collectors):
-    if constants.DATA_COLLECTOR_XEN_CPU_LOAD in collectors:
-      del collectors[constants.DATA_COLLECTOR_XEN_CPU_LOAD]
+  @OrFail("Removing SSH parameters")
+  def DowngradeSshKeyParams(self):
+    """Removes the SSH key type and bits parameters from the config.
 
-  def DowngradeCluster(self, cluster):
-    self.DowngradeCollectors(cluster["data_collectors"])
+    Also fails if these have been changed from values appropriate in lower
+    Ganeti versions.
+
+    """
+    # pylint: disable=E1103
+    # Because config_data is a dictionary which has the get method.
+    cluster = self.config_data.get("cluster", None)
+    if cluster is None:
+      raise Error("Can't find the cluster entry in the configuration")
+
+    def _FetchAndDelete(key):
+      val = cluster.get(key, None)
+      if key in cluster:
+        del cluster[key]
+      return val
+
+    ssh_key_type = _FetchAndDelete("ssh_key_type")
+    _FetchAndDelete("ssh_key_bits")
+
+    if ssh_key_type is not None and ssh_key_type != "dsa":
+      raise Error("The current Ganeti setup is using non-DSA SSH keys, and"
+                  " versions below 2.16 do not support these. To downgrade,"
+                  " please perform a gnt-cluster renew-crypto using the "
+                  " --new-ssh-keys and --ssh-key-type=dsa options, generating"
+                  " DSA keys that older versions can also use.")
 
   def DowngradeAll(self):
-    self.DowngradeCluster(self.config_data["cluster"])
     self.config_data["version"] = version.BuildVersion(DOWNGRADE_MAJOR,
                                                        DOWNGRADE_MINOR, 0)
-    return True
+
+    self.DowngradeSshKeyParams()
+    return not self.errors
 
   def _ComposePaths(self):
     # We need to keep filenames locally because they might be renamed between
