@@ -143,26 +143,26 @@ def GetPaths():
   # listed before files (type C{FILE}) in that directory. Once the directory is
   # set, only files directly in that directory can be listed.
   paths = [
-    (pathutils.DATA_DIR, DIR, 0755, getent.masterd_uid, getent.masterd_gid),
-    (pathutils.CLUSTER_DOMAIN_SECRET_FILE, FILE, 0640,
+    (pathutils.DATA_DIR, DIR, 0o755, getent.masterd_uid, getent.masterd_gid),
+    (pathutils.CLUSTER_DOMAIN_SECRET_FILE, FILE, 0o640,
      getent.masterd_uid, getent.masterd_gid, False),
-    (pathutils.CLUSTER_CONF_FILE, FILE, 0640,
+    (pathutils.CLUSTER_CONF_FILE, FILE, 0o640,
      getent.masterd_uid, getent.confd_gid, False),
-    (pathutils.CONFD_HMAC_KEY, FILE, 0440,
+    (pathutils.CONFD_HMAC_KEY, FILE, 0o440,
      getent.confd_uid, getent.masterd_gid, False),
-    (pathutils.SSH_KNOWN_HOSTS_FILE, FILE, 0644,
+    (pathutils.SSH_KNOWN_HOSTS_FILE, FILE, 0o644,
      getent.masterd_uid, getent.masterd_gid, False),
-    (pathutils.RAPI_CERT_FILE, FILE, 0440,
+    (pathutils.RAPI_CERT_FILE, FILE, 0o440,
      getent.rapi_uid, getent.masterd_gid, False),
-    (pathutils.SPICE_CERT_FILE, FILE, 0440,
+    (pathutils.SPICE_CERT_FILE, FILE, 0o440,
      getent.noded_uid, getent.masterd_gid, False),
-    (pathutils.SPICE_CACERT_FILE, FILE, 0440,
+    (pathutils.SPICE_CACERT_FILE, FILE, 0o440,
      getent.noded_uid, getent.masterd_gid, False),
     (pathutils.NODED_CERT_FILE, FILE, pathutils.NODED_CERT_MODE,
      getent.masterd_uid, getent.masterd_gid, False),
     (pathutils.NODED_CLIENT_CERT_FILE, FILE, pathutils.NODED_CERT_MODE,
      getent.masterd_uid, getent.masterd_gid, False),
-    (pathutils.WATCHER_PAUSEFILE, FILE, 0644,
+    (pathutils.WATCHER_PAUSEFILE, FILE, 0o644,
      getent.masterd_uid, getent.masterd_gid, False),
     ]
 
@@ -172,10 +172,10 @@ def GetPaths():
                   getent.noded_uid, getent.noded_gid, False))
 
   paths.extend([
-    (pathutils.QUEUE_DIR, DIR, 0750, getent.masterd_uid, getent.daemons_gid),
+    (pathutils.QUEUE_DIR, DIR, 0o750, getent.masterd_uid, getent.daemons_gid),
     (pathutils.QUEUE_DIR, QUEUE_DIR, constants.JOB_QUEUE_FILES_PERMS,
      getent.masterd_uid, getent.daemons_gid),
-    (pathutils.JOB_QUEUE_DRAIN_FILE, FILE, 0644,
+    (pathutils.JOB_QUEUE_DRAIN_FILE, FILE, 0o644,
      getent.masterd_uid, getent.daemons_gid, False),
     (pathutils.JOB_QUEUE_LOCK_FILE, FILE, constants.JOB_QUEUE_FILES_PERMS,
      getent.masterd_uid, getent.daemons_gid, False),
@@ -183,43 +183,45 @@ def GetPaths():
      getent.masterd_uid, getent.daemons_gid, False),
     (pathutils.JOB_QUEUE_VERSION_FILE, FILE, constants.JOB_QUEUE_FILES_PERMS,
      getent.masterd_uid, getent.daemons_gid, False),
-    (pathutils.JOB_QUEUE_ARCHIVE_DIR, DIR, 0750,
+    (pathutils.JOB_QUEUE_ARCHIVE_DIR, DIR, 0o750,
      getent.masterd_uid, getent.daemons_gid),
-    (rapi_dir, DIR, 0750, getent.rapi_uid, getent.masterd_gid),
-    (pathutils.RAPI_USERS_FILE, FILE, 0640,
+    (rapi_dir, DIR, 0o750, getent.rapi_uid, getent.masterd_gid),
+    (pathutils.RAPI_USERS_FILE, FILE, 0o640,
      getent.rapi_uid, getent.masterd_gid, False),
-    (pathutils.RUN_DIR, DIR, 0775, getent.masterd_uid, getent.daemons_gid),
-    (pathutils.SOCKET_DIR, DIR, 0770, getent.masterd_uid, getent.daemons_gid),
-    (pathutils.MASTER_SOCKET, FILE, 0660,
+    (pathutils.RUN_DIR, DIR, 0o775, getent.masterd_uid, getent.daemons_gid),
+    (pathutils.SOCKET_DIR, DIR, 0o770, getent.masterd_uid, getent.daemons_gid),
+    (pathutils.MASTER_SOCKET, FILE, 0o660,
      getent.masterd_uid, getent.daemons_gid, False),
-    (pathutils.QUERY_SOCKET, FILE, 0660,
+    (pathutils.QUERY_SOCKET, FILE, 0o660,
      getent.luxid_uid, getent.daemons_gid, False),
-    (pathutils.BDEV_CACHE_DIR, DIR, 0755,
+    (pathutils.BDEV_CACHE_DIR, DIR, 0o755,
      getent.noded_uid, getent.masterd_gid),
-    (pathutils.UIDPOOL_LOCKDIR, DIR, 0750,
+    (pathutils.UIDPOOL_LOCKDIR, DIR, 0o750,
      getent.noded_uid, getent.masterd_gid),
-    (pathutils.DISK_LINKS_DIR, DIR, 0755,
+    (pathutils.DISK_LINKS_DIR, DIR, 0o755,
      getent.noded_uid, getent.masterd_gid),
-    (pathutils.CRYPTO_KEYS_DIR, DIR, 0700,
+    (pathutils.CRYPTO_KEYS_DIR, DIR, 0o700,
      getent.noded_uid, getent.masterd_gid),
-    (pathutils.IMPORT_EXPORT_DIR, DIR, 0755,
+    (pathutils.IMPORT_EXPORT_DIR, DIR, 0o755,
      getent.noded_uid, getent.masterd_gid),
-    (pathutils.LOG_DIR, DIR, 0770, getent.masterd_uid, getent.daemons_gid),
-    (masterd_log, FILE, 0600, getent.masterd_uid, getent.masterd_gid, False),
-    (confd_log, FILE, 0600, getent.confd_uid, getent.masterd_gid, False),
-    (luxid_log, FILE, 0600, getent.luxid_uid, getent.masterd_gid, False),
-    (noded_log, FILE, 0600, getent.noded_uid, getent.masterd_gid, False),
-    (rapi_log, FILE, 0600, getent.rapi_uid, getent.masterd_gid, False),
-    (mond_log, FILE, 0600, getent.mond_uid, getent.masterd_gid, False),
-    (pathutils.LOG_OS_DIR, DIR, 0750, getent.noded_uid, getent.daemons_gid),
-    (pathutils.LOG_XEN_DIR, DIR, 0750, getent.noded_uid, getent.daemons_gid),
-    (pathutils.LOG_KVM_DIR, DIR, 0750, getent.noded_uid, getent.daemons_gid),
-    (cleaner_log_dir, DIR, 0750, getent.noded_uid, getent.noded_gid),
-    (master_cleaner_log_dir, DIR, 0750, getent.masterd_uid, getent.masterd_gid),
-    (pathutils.INSTANCE_REASON_DIR, DIR, 0755, getent.noded_uid,
+    (pathutils.LOG_DIR, DIR, 0o770, getent.masterd_uid, getent.daemons_gid),
+    (masterd_log, FILE, 0o600, getent.masterd_uid, getent.masterd_gid, False),
+    (confd_log, FILE, 0o600, getent.confd_uid, getent.masterd_gid, False),
+    (luxid_log, FILE, 0o600, getent.luxid_uid, getent.masterd_gid, False),
+    (noded_log, FILE, 0o600, getent.noded_uid, getent.masterd_gid, False),
+    (rapi_log, FILE, 0o600, getent.rapi_uid, getent.masterd_gid, False),
+    (mond_log, FILE, 0o600, getent.mond_uid, getent.masterd_gid, False),
+    (pathutils.LOG_OS_DIR, DIR, 0o750, getent.noded_uid, getent.daemons_gid),
+    (pathutils.LOG_XEN_DIR, DIR, 0o750, getent.noded_uid, getent.daemons_gid),
+    (pathutils.LOG_KVM_DIR, DIR, 0o750, getent.noded_uid, getent.daemons_gid),
+    (cleaner_log_dir, DIR, 0o750, getent.noded_uid, getent.noded_gid),
+    (master_cleaner_log_dir, DIR, 0o750,
+     getent.masterd_uid, getent.masterd_gid),
+    (pathutils.INSTANCE_REASON_DIR, DIR, 0o755, getent.noded_uid,
      getent.noded_gid),
-    (pathutils.LIVELOCK_DIR, DIR, 0750, getent.masterd_uid, getent.daemons_gid),
-    (pathutils.LUXID_MESSAGE_DIR, DIR, 0750, getent.masterd_uid,
+    (pathutils.LIVELOCK_DIR, DIR, 0o750,
+     getent.masterd_uid, getent.daemons_gid),
+    (pathutils.LUXID_MESSAGE_DIR, DIR, 0o750, getent.masterd_uid,
      getent.daemons_gid),
     ])
 
@@ -268,8 +270,9 @@ def Main():
 
     if opts.full_run:
       RecursiveEnsure(pathutils.JOB_QUEUE_ARCHIVE_DIR, getent.masterd_uid,
-                      getent.daemons_gid, 0750, constants.JOB_QUEUE_FILES_PERMS)
-  except errors.GenericError, err:
+                      getent.daemons_gid, 0o750,
+                      constants.JOB_QUEUE_FILES_PERMS)
+  except errors.GenericError as err:
     logging.error("An error occurred while setting permissions: %s", err)
     return constants.EXIT_FAILURE
 
