@@ -170,9 +170,8 @@ def Retry(fn, delay, timeout, args=None, wait_fn=time.sleep,
   while True:
     retry_args = []
     try:
-      # pylint: disable=W0142
       return fn(*args)
-    except RetryAgain, err:
+    except RetryAgain as err:
       retry_args = err.args
     except RetryTimeout:
       raise errors.ProgrammerError("Nested retry loop detected that didn't"
@@ -181,7 +180,6 @@ def Retry(fn, delay, timeout, args=None, wait_fn=time.sleep,
     remaining_time = end_time - _time_fn()
 
     if remaining_time <= 0.0:
-      # pylint: disable=W0142
       raise RetryTimeout(*retry_args)
 
     assert remaining_time > 0.0
@@ -218,7 +216,6 @@ def SimpleRetry(expected, fn, delay, timeout, args=None, wait_fn=time.sleep,
   rdict = {}
 
   def helper(*innerargs):
-    # pylint: disable=W0142
     result = rdict["result"] = fn(*innerargs)
     if not ((callable(expected) and expected(result)) or result == expected):
       raise RetryAgain()
