@@ -39,9 +39,9 @@ by a node reboot.  Run from cron or similar.
 import os
 import os.path
 import sys
+import signal
 import time
 import logging
-import operator
 import errno
 from optparse import OptionParser
 
@@ -122,7 +122,7 @@ def RunWatcherHooks():
 
   try:
     results = utils.RunParts(hooks_dir)
-  except Exception, err: # pylint: disable=W0703
+  except Exception as err: # pylint: disable=W0703
     logging.exception("RunParts %s failed: %s", hooks_dir, err)
     return
 
@@ -148,13 +148,14 @@ class Instance(object):
 
   """
   def __init__(self, name, status, config_state, config_state_source,
-               disks_active, snodes):
+               disks_active, snodes, disk_template):
     self.name = name
     self.status = status
     self.config_state = config_state
     self.config_state_source = config_state_source
     self.disks_active = disks_active
     self.snodes = snodes
+    self.disk_template = disk_template
 
   def Restart(self, cl):
     """Encapsulates the start of an instance.
@@ -238,7 +239,7 @@ def _CheckInstances(cl, notepad, instances, locks):
   """Make a pass over the list of instances, restarting downed ones.
 
   """
-  notepad.MaintainInstanceList(instances.keys())
+  notepad.MaintainInstanceList(list(instances))
 
   started = set()
 
@@ -356,7 +357,10 @@ def _VerifyDisks(cl, uuid, nodes, instances):
   job_id = cl.SubmitJob([op])
   ((_, offline_disk_instances, _), ) = \
     cli.PollJob(job_id, cl=cl, feedback_fn=logging.debug)
-  cl.ArchiveJob(job_id)
+  try:
+    cl.ArchiveJob(job_id)
+  except Exception as err:
+    logging.exception("Error while archiving job %d" % job_id)
 
   if not offline_disk_instances:
     # nothing to do
@@ -417,10 +421,10 @@ def IsRapiResponding(hostname):
                                              curl_config_fn=curl_config)
   try:
     master_version = rapi_client.GetVersion()
-  except rapi.client.CertificateError, err:
+  except rapi.client.CertificateError as err:
     logging.warning("RAPI certificate error: %s", err)
     return False
-  except rapi.client.GanetiApiError, err:
+  except rapi.client.GanetiApiError as err:
     if err.code == 401:
       # Unauthorized, but RAPI is alive and responding
       return True
@@ -440,7 +444,7 @@ def IsWconfdResponding():
 
   try:
     result = wconfd.Client().Echo(probe_string)
-  except Exception, err: # pylint: disable=W0703
+  except Exception as err: # pylint: disable=W0703
     logging.warning("WConfd connection error: %s", err)
     return False
 
@@ -505,8 +509,7 @@ def _WriteInstanceStatus(filename, data):
                 filename, len(data))
 
   utils.WriteFile(filename,
-                  data="".join(map(compat.partial(operator.mod, "%s %s\n"),
-                                   sorted(data))))
+                  data="\n".join("%s %s" % (n, s) for (n, s) in sorted(data)))
 
 
 def _UpdateInstanceStatus(filename, instances):
@@ -537,7 +540,7 @@ def _ReadInstanceStatus(filename):
   statcb = utils.FileStatHelper()
   try:
     content = utils.ReadFile(filename, preread=statcb)
-  except EnvironmentError, err:
+  except EnvironmentError as err:
     if err.errno == errno.ENOENT:
       logging.error("Can't read '%s', does not exist (yet)", filename)
     else:
@@ -564,7 +567,7 @@ def _MergeInstanceStatus(filename, pergroup_filename, groups):
   lock = utils.FileLock.Open(filename)
   try:
     lock.Exclusive(blocking=True, timeout=INSTANCE_STATUS_LOCK_TIMEOUT)
-  except errors.LockError, err:
+  except errors.LockError as err:
     # All per-group processes will lock and update the file. None of them
     # should take longer than 10 seconds (the value of
     # INSTANCE_STATUS_LOCK_TIMEOUT).
@@ -602,11 +605,11 @@ def GetLuxiClient(try_restart):
   """
   try:
     return cli.GetClient()
-  except errors.OpPrereqError, err:
+  except errors.OpPrereqError as err:
     # this is, from cli.GetClient, a not-master case
     raise NotMasterError("Not on master node (%s)" % err)
 
-  except (rpcerr.NoMasterError, rpcerr.TimeoutError), err:
+  except (rpcerr.NoMasterError, rpcerr.TimeoutError) as err:
     if not try_restart:
       raise
 
@@ -632,34 +635,35 @@ def _StartGroupChildren(cl, wait):
   children = []
 
   for (idx, (name, uuid)) in enumerate(result):
-    args = sys.argv + [cli.NODEGROUP_OPT_NAME, uuid]
-
     if idx > 0:
       # Let's not kill the system
       time.sleep(CHILD_PROCESS_DELAY)
 
-    logging.debug("Spawning child for group '%s' (%s), arguments %s",
-                  name, uuid, args)
+    logging.debug("Spawning child for group %r (%s).", name, uuid)
 
+    signal.signal(signal.SIGCHLD, signal.SIG_IGN)
     try:
-      # TODO: Should utils.StartDaemon be used instead?
-      pid = os.spawnv(os.P_NOWAIT, args[0], args)
-    except Exception: # pylint: disable=W0703
-      logging.exception("Failed to start child for group '%s' (%s)",
-                        name, uuid)
+      pid = os.fork()
+    except OSError:
+      logging.exception("Failed to fork for group %r (%s)", name, uuid)
+
+    if pid == 0:
+      (options, _) = ParseOptions()
+      options.nodegroup = uuid
+      _GroupWatcher(options)
+      return
     else:
       logging.debug("Started with PID %s", pid)
       children.append(pid)
 
   if wait:
-    for pid in children:
-      logging.debug("Waiting for child PID %s", pid)
+    for child in children:
+      logging.debug("Waiting for child PID %s", child)
       try:
-        result = utils.RetryOnSignal(os.waitpid, pid, 0)
-      except EnvironmentError, err:
+        result = utils.RetryOnSignal(os.waitpid, child, 0)
+      except EnvironmentError as err:
         result = str(err)
-
-      logging.debug("Child PID %s exited with status %s", pid, result)
+      logging.debug("Child PID %s exited with status %s", child, result)
 
 
 def _ArchiveJobs(cl, age):
@@ -753,24 +757,24 @@ def _GetGroupData(qcl, uuid):
   queries = [
       (constants.QR_INSTANCE,
        ["name", "status", "admin_state", "admin_state_source", "disks_active",
-        "snodes", "pnode.group.uuid", "snodes.group.uuid"],
+        "snodes", "pnode.group.uuid", "snodes.group.uuid", "disk_template"],
        [qlang.OP_EQUAL, "pnode.group.uuid", uuid]),
       (constants.QR_NODE,
        ["name", "bootid", "offline"],
        [qlang.OP_EQUAL, "group.uuid", uuid]),
       ]
 
-  results = []
-  for what, fields, qfilter in queries:
-    results.append(qcl.Query(what, fields, qfilter))
-
-  results_data = map(operator.attrgetter("data"), results)
+  results_data = [
+    qcl.Query(what, field, qfilter).data
+    for (what, field, qfilter) in queries
+  ]
 
   # Ensure results are tuples with two values
-  assert compat.all(map(ht.TListOf(ht.TListOf(ht.TIsLength(2))), results_data))
+  assert compat.all(
+      ht.TListOf(ht.TListOf(ht.TIsLength(2)))(d) for d in results_data)
 
   # Extract values ignoring result status
-  (raw_instances, raw_nodes) = [[map(compat.snd, values)
+  (raw_instances, raw_nodes) = [[[v[1] for v in values]
                                  for values in res]
                                 for res in results_data]
 
@@ -779,14 +783,14 @@ def _GetGroupData(qcl, uuid):
 
   # Load all instances
   for (name, status, config_state, config_state_source, disks_active, snodes,
-       pnode_group_uuid, snodes_group_uuid) in raw_instances:
+       pnode_group_uuid, snodes_group_uuid, disk_template) in raw_instances:
     if snodes and set([pnode_group_uuid]) != set(snodes_group_uuid):
       logging.error("Ignoring split instance '%s', primary group %s, secondary"
                     " groups %s", name, pnode_group_uuid,
                     utils.CommaJoin(snodes_group_uuid))
     else:
       instances.append(Instance(name, status, config_state, config_state_source,
-                                disks_active, snodes))
+                                disks_active, snodes, disk_template))
 
       for node in snodes:
         secondaries.setdefault(node, set()).add(name)
@@ -809,7 +813,7 @@ def _LoadKnownGroups():
   result = list(line.split(None, 1)[0] for line in groups
                 if line.strip())
 
-  if not compat.all(map(utils.UUID_RE.match, result)):
+  if not compat.all(utils.UUID_RE.match(r) for r in result):
     raise errors.GenericError("Ssconf contains invalid group UUID")
 
   return result
@@ -857,7 +861,7 @@ def _GroupWatcher(opts):
     (nodes, instances, locks) = _GetGroupData(client, group_uuid)
 
     # Update per-group instance status file
-    _UpdateInstanceStatus(inst_status_path, instances.values())
+    _UpdateInstanceStatus(inst_status_path, list(instances.values()))
 
     _MergeInstanceStatus(pathutils.INSTANCE_STATUS_FILE,
                          pathutils.WATCHER_GROUP_INSTANCE_STATUS_FILE,
@@ -865,9 +869,21 @@ def _GroupWatcher(opts):
 
     started = _CheckInstances(client, notepad, instances, locks)
     _CheckDisks(client, notepad, nodes, instances, started)
-    if not opts.no_verify_disks:
+
+    # Check if the nodegroup only has ext storage type
+    only_ext = compat.all(i.disk_template == constants.DT_EXT
+                          for i in instances.values())
+
+    # We skip current NodeGroup verification if there are only external storage
+    # devices. Currently we provide an interface for external storage provider
+    # for disk verification implementations, however current ExtStorageDevice
+    # does not provide an API for this yet.
+    #
+    # This check needs to be revisited if ES_ACTION_VERIFY on ExtStorageDevice
+    # is implemented.
+    if not opts.no_verify_disks and not only_ext:
       _VerifyDisks(client, group_uuid, nodes, instances)
-  except Exception, err:
+  except Exception as err:
     logging.info("Not updating status file due to failure: %s", err)
     raise
   else:
@@ -890,15 +906,17 @@ def Main():
     logging.debug("Pause has been set, exiting")
     return constants.EXIT_SUCCESS
 
-  # Try to acquire global watcher lock in shared mode
+  # Try to acquire global watcher lock in shared mode.
+  # In case we are in the global watcher process, this lock will be held by all
+  # children processes (one for each nodegroup) and will only be released when
+  # all of them have finished running.
   lock = utils.FileLock.Open(pathutils.WATCHER_LOCK_FILE)
   try:
     lock.Shared(blocking=False)
-  except (EnvironmentError, errors.LockError), err:
+  except (EnvironmentError, errors.LockError) as err:
     logging.error("Can't acquire lock on %s: %s",
                   pathutils.WATCHER_LOCK_FILE, err)
     return constants.EXIT_SUCCESS
-
   if options.nodegroup is None:
     fn = _GlobalWatcher
   else:
@@ -912,14 +930,14 @@ def Main():
   except NotMasterError:
     logging.debug("Not master, exiting")
     return constants.EXIT_NOTMASTER
-  except errors.ResolverError, err:
+  except errors.ResolverError as err:
     logging.error("Cannot resolve hostname '%s', exiting", err.args[0])
     return constants.EXIT_NODESETUP_ERROR
   except errors.JobQueueFull:
     logging.error("Job queue is full, can't query cluster state")
   except errors.JobQueueDrainError:
     logging.error("Job queue is drained, can't maintain cluster state")
-  except Exception, err:
+  except Exception as err: # pylint: disable=W0703
     logging.exception(str(err))
     return constants.EXIT_FAILURE
 
