@@ -201,11 +201,11 @@ class BaseWorker(threading.Thread, object):
           logging.debug("Starting task %r, priority %s", args, priority)
           assert self.getName() == self._worker_id
           try:
-            self.RunTask(*args) # pylint: disable=W0142
+            self.RunTask(*args)
           finally:
             self.SetTaskName(None)
           logging.debug("Done with task %r, priority %s", args, priority)
-        except DeferTask, err:
+        except DeferTask as err:
           defer = err
 
           if defer.priority is None:
@@ -322,11 +322,11 @@ class WorkerPool(object):
 
     """
     assert isinstance(args, (tuple, list)), "Arguments must be a sequence"
-    assert isinstance(priority, (int, long)), "Priority must be numeric"
-    assert task_id is None or isinstance(task_id, (int, long)), \
+    assert isinstance(priority, int), "Priority must be numeric"
+    assert task_id is None or isinstance(task_id, int), \
       "Task ID must be numeric or None"
 
-    task = [priority, self._counter.next(), task_id, args]
+    task = [priority, next(self._counter), task_id, args]
 
     if task_id is not None:
       assert task_id not in self._taskdata
@@ -376,13 +376,13 @@ class WorkerPool(object):
     """
     assert compat.all(isinstance(task, (tuple, list)) for task in tasks), \
            "Each task must be a sequence"
-    assert (isinstance(priority, (int, long)) or
-            compat.all(isinstance(prio, (int, long)) for prio in priority)), \
+    assert (isinstance(priority, int) or
+            compat.all(isinstance(prio, int) for prio in priority)), \
            "Priority must be numeric or be a list of numeric values"
     assert task_id is None or isinstance(task_id, (tuple, list)), \
            "Task IDs must be in a sequence"
 
-    if isinstance(priority, (int, long)):
+    if isinstance(priority, int):
       priority = [priority] * len(tasks)
     elif len(priority) != len(tasks):
       raise errors.ProgrammerError("Number of priorities (%s) doesn't match"
@@ -400,7 +400,7 @@ class WorkerPool(object):
     try:
       self._WaitWhileQuiescingUnlocked()
 
-      assert compat.all(isinstance(prio, (int, long)) for prio in priority)
+      assert compat.all(isinstance(prio, int) for prio in priority)
       assert len(tasks) == len(priority)
       assert len(tasks) == len(task_id)
 
@@ -420,7 +420,7 @@ class WorkerPool(object):
       currently running)
 
     """
-    assert isinstance(priority, (int, long)), "Priority must be numeric"
+    assert isinstance(priority, int), "Priority must be numeric"
 
     self._lock.acquire()
     try:
@@ -587,12 +587,12 @@ class WorkerPool(object):
         self._lock.acquire()
 
       # Remove terminated threads. This could be done in a more efficient way
-      # (del self._termworkers[:]), but checking worker.isAlive() makes sure we
+      # (del self._termworkers[:]), but checking worker.is_alive() makes sure we
       # don't leave zombie threads around.
       for worker in termworkers:
         assert worker in self._termworkers, ("Worker not in list of"
                                              " terminating workers")
-        if not worker.isAlive():
+        if not worker.is_alive():
           self._termworkers.remove(worker)
 
       assert not self._termworkers, "Zombie worker detected"
