@@ -1,4 +1,4 @@
-#!/usr/bin/python -u
+#!/usr/bin/python3 -u
 #
 
 # Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Google Inc.
@@ -40,25 +40,25 @@ import datetime
 import optparse
 import sys
 
-import colors
-import qa_cluster
-import qa_config
-import qa_daemon
-import qa_env
-import qa_error
-import qa_filters
-import qa_group
-import qa_instance
-import qa_iptables
-import qa_monitoring
-import qa_network
-import qa_node
-import qa_os
-import qa_performance
-import qa_job
-import qa_rapi
-import qa_tags
-import qa_utils
+from qa import colors
+from qa import qa_cluster
+from qa import qa_config
+from qa import qa_daemon
+from qa import qa_env
+from qa import qa_error
+from qa import qa_filters
+from qa import qa_group
+from qa import qa_instance
+from qa import qa_iptables
+from qa import qa_monitoring
+from qa import qa_network
+from qa import qa_node
+from qa import qa_os
+from qa import qa_performance
+from qa import qa_job
+from qa import qa_rapi
+from qa import qa_tags
+from qa import qa_utils
 
 from ganeti import utils
 from ganeti import rapi # pylint: disable=W0611
@@ -104,22 +104,22 @@ def RunTest(fn, *args, **kwargs):
 
   desc = _DescriptionOf(fn)
 
-  print
-  print _FormatHeader("%s start %s" % (tstart, desc),
-                      color=colors.YELLOW, mark="<")
+  print()
+  print(_FormatHeader("%s start %s" % (tstart, desc),
+                      color=colors.YELLOW, mark="<"))
 
   try:
     retval = fn(*args, **kwargs)
-    print _FormatHeader("PASSED %s" % (desc, ), color=colors.GREEN)
+    print(_FormatHeader("PASSED %s" % (desc, ), color=colors.GREEN))
     return retval
-  except Exception, e:
-    print _FormatHeader("FAILED %s: %s" % (desc, e), color=colors.RED)
+  except Exception as e:
+    print(_FormatHeader("FAILED %s: %s" % (desc, e), color=colors.RED))
     raise
   finally:
     tstop = datetime.datetime.now()
     tdelta = tstop - tstart
-    print _FormatHeader("%s time=%s %s" % (tstop, tdelta, desc),
-                        color=colors.MAGENTA, mark=">")
+    print(_FormatHeader("%s time=%s %s" % (tstop, tdelta, desc),
+                        color=colors.MAGENTA, mark=">"))
 
 
 def ReportTestSkip(desc, testnames):
@@ -134,9 +134,9 @@ def ReportTestSkip(desc, testnames):
   """
   tstart = datetime.datetime.now()
   # TODO: Formatting test names when non-string names are involved
-  print _FormatHeader("%s skipping %s, test(s) %s disabled" %
+  print(_FormatHeader("%s skipping %s, test(s) %s disabled" %
                       (tstart, desc, testnames),
-                      color=colors.BLUE, mark="*")
+                      color=colors.BLUE, mark="*"))
 
 
 def RunTestIf(testnames, fn, *args, **kwargs):
@@ -161,21 +161,21 @@ def RunTestBlock(fn, *args, **kwargs):
 
   desc = _DescriptionOf(fn)
 
-  print
-  print _FormatHeader("BLOCK %s start %s" % (tstart, desc),
-                      color=[colors.YELLOW, colors.BOLD], mark="v")
+  print()
+  print(_FormatHeader("BLOCK %s start %s" % (tstart, desc),
+                      color=[colors.YELLOW, colors.BOLD], mark="v"))
 
   try:
     return fn(*args, **kwargs)
-  except Exception, e:
-    print _FormatHeader("BLOCK FAILED %s: %s" % (desc, e),
-                        color=[colors.RED, colors.BOLD])
+  except Exception as e:
+    print(_FormatHeader("BLOCK FAILED %s: %s" % (desc, e),
+                        color=[colors.RED, colors.BOLD]))
     raise
   finally:
     tstop = datetime.datetime.now()
     tdelta = tstop - tstart
-    print _FormatHeader("BLOCK %s time=%s %s" % (tstop, tdelta, desc),
-                        color=[colors.MAGENTA, colors.BOLD], mark="^")
+    print(_FormatHeader("BLOCK %s time=%s %s" % (tstop, tdelta, desc),
+                        color=[colors.MAGENTA, colors.BOLD], mark="^"))
 
 
 def RunEnvTests():
@@ -636,8 +636,8 @@ def RunCustomSshPortTests():
 
     for node in nodes:
       if qa_utils.UsesIPv6Connection(node.primary, std_port):
-        print ("Node %s is likely to be reached using IPv6,"
-               "skipping the test" % (node.primary, ))
+        print("Node %s is likely to be reached using IPv6,"
+              "skipping the test" % (node.primary, ))
         return
 
     for node in nodes:
@@ -704,7 +704,7 @@ def TestIPolicyPlainInstance():
   """Test instance policy interaction with instances"""
   params = ["memory-size", "cpu-count", "disk-count", "disk-size", "nic-count"]
   if not qa_config.IsTemplateSupported(constants.DT_PLAIN):
-    print "Template %s not supported" % constants.DT_PLAIN
+    print("Template %s not supported" % constants.DT_PLAIN)
     return
 
   # This test assumes that the group policy is empty
@@ -800,7 +800,7 @@ def IsExclusiveStorageInstanceTestEnabled():
     for node in nodes:
       try:
         pvnum = int(qa_utils.GetCommandOutput(node.primary, vgscmd))
-      except Exception, e:
+      except Exception as e:
         msg = ("Cannot get the number of PVs on %s, needed by '%s': %s" %
                (node.primary, test_name, e))
         raise qa_error.Error(msg)
@@ -931,7 +931,7 @@ def RunPerformanceTests():
 
   # Preparations need to be made only if some of these tests are enabled
   if qa_config.IsTemplateSupported(constants.DT_DRBD8) and \
-     qa_config.TestEnabled(qa_config.Either(PARALLEL_TEST_DICT.keys())):
+     qa_config.TestEnabled(qa_config.Either(list(PARALLEL_TEST_DICT))):
     inodes = qa_config.AcquireManyNodes(2)
     try:
       instance = qa_instance.TestInstanceAddWithDrbdDisk(inodes)
@@ -1063,6 +1063,9 @@ def RunQa():
     "instance-add-restricted-by-disktemplates",
     qa_instance.TestInstanceCreationRestrictedByDiskTemplates)
 
+  RunTestIf("instance-add-osparams", qa_instance.TestInstanceAddOsParams)
+  RunTestIf("instance-add-osparams", qa_instance.TestSecretOsParams)
+
   # Test removing instance with offline drbd secondary
   if qa_config.TestEnabled(["instance-remove-drbd-offline",
                             "instance-add-drbd-disk"]):
@@ -1119,10 +1122,10 @@ def main():
 
   primary = qa_config.GetMasterNode().primary
   qa_utils.StartMultiplexer(primary)
-  print ("SSH command for primary node: %s" %
-         utils.ShellQuoteArgs(qa_utils.GetSSHCommand(primary, "")))
-  print ("SSH command for other nodes: %s" %
-         utils.ShellQuoteArgs(qa_utils.GetSSHCommand("NODE", "")))
+  print("SSH command for primary node: %s" %
+        utils.ShellQuoteArgs(qa_utils.GetSSHCommand(primary, "")))
+  print("SSH command for other nodes: %s" %
+        utils.ShellQuoteArgs(qa_utils.GetSSHCommand("NODE", "")))
   try:
     RunQa()
   finally:
