@@ -43,14 +43,14 @@ from ganeti import compat
 from ganeti import utils
 from ganeti import pathutils
 
-import qa_config
-import qa_daemon
-import qa_error
-import qa_instance
-import qa_job_utils
-import qa_logging
-import qa_rapi
-import qa_utils
+from qa import qa_config
+from qa import qa_daemon
+from qa import qa_error
+from qa import qa_instance
+from qa import qa_job_utils
+from qa import qa_logging
+from qa import qa_rapi
+from qa import qa_utils
 
 from qa_utils import AssertEqual, AssertCommand, AssertRedirectedCommand, \
   GetCommandOutput, CheckFileUnmodified
@@ -155,7 +155,7 @@ def AssertClusterVerify(fail=False, errors=None,
     with CheckFileUnmodified(mnode.primary, pathutils.CLUSTER_CONF_FILE):
       cvout = GetCommandOutput(mnode.primary, cvcmd + " --error-codes",
                                fail=(fail or errors))
-    print cvout
+    print(cvout)
     (act_errs, act_warns) = _GetCVErrorCodes(cvout)
     if errors:
       _CheckVerifyErrors(act_errs, errors, "error")
@@ -318,7 +318,7 @@ def TestClusterRename():
   original_name = qa_config.get("name")
   rename_target = qa_config.get("rename", None)
   if rename_target is None:
-    print qa_logging.FormatError('"rename" entry is missing')
+    print(qa_logging.FormatError('"rename" entry is missing'))
     return
 
   for data in [
@@ -1139,8 +1139,8 @@ def TestClusterModifyUserShutdown():
       fn(nodes)
       qa_daemon.TestResumeWatcher()
     else:
-      print "%s hypervisor is not enabled, skipping test for this hypervisor" \
-          % hv
+      print("%s hypervisor is not enabled, skipping test for this hypervisor" \
+          % hv)
 
 
 def TestClusterInfo():
@@ -1169,14 +1169,14 @@ def _AssertSsconfCertFiles():
   """
   (vcluster_master, _) = qa_config.GetVclusterSettings()
   if vcluster_master:
-    print "Skipping asserting SsconfCertFiles for Vcluster"
+    print("Skipping asserting SsconfCertFiles for Vcluster")
     return
   nodes = qa_config.get("nodes")
   ssconf_file = "/var/lib/ganeti/ssconf_master_candidates_certs"
   ssconf_content = {}
   for node in nodes:
     cmd = ["cat", ssconf_file]
-    print "Ssconf Master Certificates of node '%s'." % node.primary
+    print("Ssconf Master Certificates of node '%s'." % node.primary)
     result_output = GetCommandOutput(node.primary, utils.ShellQuoteArgs(cmd))
     ssconf_content[node] = result_output
 
@@ -1195,6 +1195,63 @@ def _AssertSsconfCertFiles():
                       " '%s'." % (node, first_node))
 
 
+def _TestSSHKeyChanges(master_node):
+  """Tests a lot of SSH key type- and size- related functionality.
+
+  @type master_node: L{qa_config._QaNode}
+  @param master_node: The cluster master.
+
+  """
+  # Helper fn to avoid specifying base params too many times
+  def _RenewWithParams(new_params, verify=True, fail=False):
+    AssertCommand(["gnt-cluster", "renew-crypto", "--new-ssh-keys", "-f",
+                   "--no-ssh-key-check"] + new_params, fail=fail)
+    if not fail and verify:
+      AssertCommand(["gnt-cluster", "verify"])
+
+  # First test the simplest change
+  _RenewWithParams([])
+
+  # And stop here if vcluster
+  (vcluster_master, _) = qa_config.GetVclusterSettings()
+  if vcluster_master:
+    print("Skipping further SSH key replacement checks for vcluster")
+    return
+
+  # And the actual tests
+  with qa_config.AcquireManyNodesCtx(1, exclude=[master_node]) as nodes:
+    node_name = nodes[0].primary
+
+    # Another helper function for checking whether a specific key can log in
+    def _CheckLoginWithKey(key_path, fail=False):
+      AssertCommand(["ssh", "-oIdentityFile=%s" % key_path, "-oBatchMode=yes",
+                     "-oStrictHostKeyChecking=no", "-oIdentitiesOnly=yes",
+                     "-F/dev/null", node_name, "true"],
+                    fail=fail, forward_agent=False)
+
+    _RenewWithParams(["--ssh-key-type=dsa"])
+    _CheckLoginWithKey("/root/.ssh/id_dsa")
+    # Stash the key for now
+    old_key_backup = qa_utils.BackupFile(master_node.primary,
+                                         "/root/.ssh/id_dsa")
+
+    try:
+      _RenewWithParams(["--ssh-key-type=rsa"])
+      _CheckLoginWithKey("/root/.ssh/id_rsa")
+      # And check that we cannot log in with the old key
+      _CheckLoginWithKey(old_key_backup, fail=True)
+    finally:
+      AssertCommand(["rm", "-f", old_key_backup])
+
+    _RenewWithParams(["--ssh-key-bits=4096"])
+    _RenewWithParams(["--ssh-key-bits=521"], fail=True)
+
+    # Restore the cluster to its pristine state, skipping the verify as we did
+    # way too many already
+    _RenewWithParams(["--ssh-key-type=rsa", "--ssh-key-bits=2048"],
+                     verify=False)
+
+
 def TestClusterRenewCrypto():
   """gnt-cluster renew-crypto"""
   master = qa_config.GetMasterNode()
@@ -1218,7 +1275,7 @@ def TestClusterRenewCrypto():
                                          pathutils.RAPI_CERT_FILE)
   try:
     # Custom RAPI certificate
-    fh = tempfile.NamedTemporaryFile()
+    fh = tempfile.NamedTemporaryFile(mode="w")
 
     # Ensure certificate doesn't cause "gnt-cluster verify" to complain
     validity = constants.SSL_CERT_EXPIRATION_WARN * 3
@@ -1233,7 +1290,7 @@ def TestClusterRenewCrypto():
       AssertCommand(["rm", "-f", tmpcert])
 
     # Custom cluster domain secret
-    cds_fh = tempfile.NamedTemporaryFile()
+    cds_fh = tempfile.NamedTemporaryFile(mode="w")
     cds_fh.write(utils.GenerateSecret())
     cds_fh.write("\n")
     cds_fh.flush()
@@ -1266,9 +1323,8 @@ def TestClusterRenewCrypto():
     _AssertSsconfCertFiles()
     AssertCommand(["gnt-cluster", "verify"])
 
-    # Only renew SSH keys
-    AssertCommand(["gnt-cluster", "renew-crypto", "--force",
-                   "--new-ssh-keys", "--no-ssh-key-check"])
+    # Comprehensively test various types of SSH key changes
+    _TestSSHKeyChanges(master)
 
     # Restore RAPI certificate
     AssertCommand(["gnt-cluster", "renew-crypto", "--force",
@@ -1303,7 +1359,7 @@ def TestClusterBurnin():
       for _ in range(0, num):
         instances.append(qa_config.AcquireInstance())
     except qa_error.OutOfInstancesError:
-      print "Not enough instances, continuing anyway."
+      print("Not enough instances, continuing anyway.")
 
     if len(instances) < 1:
       raise qa_error.Error("Burnin needs at least one instance")
@@ -1351,7 +1407,7 @@ def TestClusterMasterFailover():
 
   # Flush the configuration to prevent race conditions when loading it
   # on another node
-  print qa_logging.FormatInfo("Flushing the configuration on the master node")
+  print(qa_logging.FormatInfo("Flushing the configuration on the master node"))
   AssertCommand(["gnt-debug", "wconfd", "flushconfig"])
 
   cmd = ["gnt-cluster", "master-failover"]
@@ -1371,14 +1427,14 @@ def TestUpgrade():
 
   This tests the 'gnt-cluster upgrade' command by flipping
   between the current and a different version of Ganeti.
-  To also recover subtile points in the configuration up/down
+  To also recover subtle points in the configuration up/down
   grades, instances are left over both upgrades.
 
   """
   this_version = qa_config.get("dir-version")
   other_version = qa_config.get("other-dir-version")
   if this_version is None or other_version is None:
-    print qa_utils.FormatInfo("Test not run, as versions not specified")
+    print(qa_utils.FormatInfo("Test not run, as versions not specified"))
     return
 
   inst_creates = []
@@ -1394,6 +1450,14 @@ def TestUpgrade():
     nodes = qa_config.AcquireManyNodes(n)
     live_instances.append(cf(nodes))
 
+  # 2.16 only - prior to performing a downgrade, we have to make sure that the
+  # SSH keys used are such that the lower version can still use them,
+  # regardless of cluster defaults.
+  if constants.VERSION_MINOR != 16:
+    raise qa_error.Error("Please remove the key type downgrade code in 2.17")
+  AssertCommand(["gnt-cluster", "renew-crypto", "--no-ssh-key-check", "-f",
+                 "--new-ssh-keys", "--ssh-key-type=dsa"])
+
   AssertRedirectedCommand(["gnt-cluster", "upgrade", "--to", other_version])
   AssertRedirectedCommand(["gnt-cluster", "verify"])
 
@@ -1464,7 +1528,7 @@ def TestClusterCopyfile():
   uniqueid = utils.NewUUID()
 
   # Create temporary file
-  f = tempfile.NamedTemporaryFile()
+  f = tempfile.NamedTemporaryFile(mode="w")
   f.write(uniqueid)
   f.flush()
   f.seek(0)
@@ -1566,7 +1630,7 @@ def TestInstanceCommunication():
 
   cmd = "gnt-cluster modify --instance-communication-network=%s" % network_name
   result_output = qa_utils.GetCommandOutput(master.primary, cmd)
-  print result_output
+  print(result_output)
 
   cmd = ["gnt-network", "list", "--no-headers", "-o", "name", network_name]
   result_output = qa_utils.GetCommandOutput(master.primary,
@@ -1620,7 +1684,7 @@ def TestInstanceCommunication():
   cmd = ["gnt-group", "add", group]
   result_output = qa_utils.GetCommandOutput(master.primary,
                                             utils.ShellQuoteArgs(cmd))
-  print result_output
+  print(result_output)
 
   cmd = ["gnt-network", "list", "--no-headers", "-o", "group_list",
          network_name]
@@ -1642,13 +1706,13 @@ def TestInstanceCommunication():
   # same value and check that nothing happens.
   cmd = "gnt-cluster modify --instance-communication-network=%s" % network_name
   result_output = qa_utils.GetCommandOutput(master.primary, cmd)
-  print result_output
+  print(result_output)
 
   # Disable instance communication network, disconnect the instance
   # communication network and remove it, and remove the group
   cmd = "gnt-cluster modify --instance-communication-network="
   result_output = qa_utils.GetCommandOutput(master.primary, cmd)
-  print result_output
+  print(result_output)
 
   cmd = ["gnt-network", "disconnect", network_name]
   AssertCommand(utils.ShellQuoteArgs(cmd))
