@@ -34,7 +34,7 @@
 
 from ganeti import utils
 
-import qa_config
+from qa import qa_config
 
 from qa_utils import AssertCommand
 
@@ -56,8 +56,11 @@ def TestGanetiCommands():
           ["gnt-debug", "--version"],
           ["gnt-instance", "--version"],
           ["gnt-job", "--version"],
+          ["gnt-network", "--version"],
           ["gnt-node", "--version"],
           ["gnt-os", "--version"],
+          ["gnt-storage", "--version"],
+          ["gnt-filter", "--version"],
           ["ganeti-noded", "--version"],
           ["ganeti-rapi", "--version"],
           ["ganeti-watcher", "--version"],
