@@ -42,11 +42,12 @@ from ganeti import pathutils
 from ganeti import query
 from ganeti.netutils import IP4Address
 
-import qa_config
-import qa_daemon
-import qa_utils
-import qa_error
+from qa import qa_config
+from qa import qa_daemon
+from qa import qa_utils
+from qa import qa_error
 
+from qa_filters import stdout_of
 from qa_utils import AssertCommand, AssertEqual, AssertIn
 from qa_utils import InstanceCheck, INST_DOWN, INST_UP, FIRST_ARG, RETURN_VALUE
 from qa_instance_utils import CheckSsconfInstanceList, \
@@ -93,7 +94,7 @@ def GetInstanceInfo(instance):
       if nodestr:
         m = re_nodelist.match(nodestr)
         if m:
-          nodes.extend(filter(None, m.groups()))
+          nodes.extend([n for n in m.groups() if n])
         else:
           nodes.append(nodestr)
 
@@ -382,14 +383,14 @@ def TestInstanceReboot(instance):
 def TestInstanceReinstall(instance):
   """gnt-instance reinstall"""
   if instance.disk_template == constants.DT_DISKLESS:
-    print qa_utils.FormatInfo("Test not supported for diskless instances")
+    print(qa_utils.FormatInfo("Test not supported for diskless instances"))
     return
 
   qa_storage = qa_config.get("qa-storage")
 
   if qa_storage is None:
-    print qa_utils.FormatInfo("Test not supported because the additional QA"
-                              " storage is not available")
+    print(qa_utils.FormatInfo("Test not supported because the additional QA"
+                              " storage is not available"))
   else:
     # Reinstall with OS image from QA storage
     url = "%s/busybox.img" % qa_storage
@@ -504,8 +505,8 @@ def TestInstanceRenameAndBack(rename_source, rename_target):
 def TestInstanceFailover(instance):
   """gnt-instance failover"""
   if not IsFailoverSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support failover, skipping"
-                              " test")
+    print(qa_utils.FormatInfo("Instance doesn't support failover, skipping"
+                              " test"))
     return
 
   cmd = ["gnt-instance", "failover", "--force", instance.name]
@@ -522,8 +523,8 @@ def TestInstanceFailover(instance):
 def TestInstanceMigrate(instance, toggle_always_failover=True):
   """gnt-instance migrate"""
   if not IsMigrationSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support migration, skipping"
-                              " test")
+    print(qa_utils.FormatInfo("Instance doesn't support migration, skipping"
+                              " test"))
     return
 
   cmd = ["gnt-instance", "migrate", "--force", instance.name]
@@ -691,7 +692,7 @@ def TestInstanceModifyPrimaryAndBack(instance, currentnode, othernode):
 
   """
   if instance.disk_template != constants.DT_FILE:
-    print qa_utils.FormatInfo("Test only supported for the file disk template")
+    print(qa_utils.FormatInfo("Test only supported for the file disk template"))
     return
 
   cluster_name = qa_config.get("name")
@@ -758,17 +759,17 @@ def TestInstanceConvertDiskTemplate(instance, requested_conversions):
     return cmd
 
   if len(requested_conversions) < 2:
-    print qa_utils.FormatInfo("You must specify more than one convertible"
+    print(qa_utils.FormatInfo("You must specify more than one convertible"
                               " disk templates in order to test the conversion"
-                              " feature")
+                              " feature"))
     return
 
   name = instance.name
 
   template = instance.disk_template
   if template in constants.DTS_NOT_CONVERTIBLE_FROM:
-    print qa_utils.FormatInfo("Unsupported template %s, skipping conversion"
-                              " test" % template)
+    print(qa_utils.FormatInfo("Unsupported template %s, skipping conversion"
+                              " test" % template))
     return
 
   inodes = qa_config.AcquireManyNodes(2)
@@ -795,7 +796,7 @@ def TestInstanceConvertDiskTemplate(instance, requested_conversions):
 def TestInstanceModifyDisks(instance):
   """gnt-instance modify --disk"""
   if not IsDiskSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support disks, skipping test")
+    print(qa_utils.FormatInfo("Instance doesn't support disks, skipping test"))
     return
 
   disk_conf = qa_config.GetDiskOptions()[-1]
@@ -821,7 +822,7 @@ def TestInstanceModifyDisks(instance):
 def TestInstanceGrowDisk(instance):
   """gnt-instance grow-disk"""
   if instance.disk_template == constants.DT_DISKLESS:
-    print qa_utils.FormatInfo("Test not supported for diskless instances")
+    print(qa_utils.FormatInfo("Test not supported for diskless instances"))
     return
 
   name = instance.name
@@ -850,7 +851,7 @@ def TestInstanceGrowDisk(instance):
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
 def TestInstanceDeviceNames(instance):
   if instance.disk_template == constants.DT_DISKLESS:
-    print qa_utils.FormatInfo("Test not supported for diskless instances")
+    print(qa_utils.FormatInfo("Test not supported for diskless instances"))
     return
 
   name = instance.name
@@ -900,12 +901,12 @@ def TestInstanceDeviceNames(instance):
 
 def TestInstanceList():
   """gnt-instance list"""
-  qa_utils.GenericQueryTest("gnt-instance", query.INSTANCE_FIELDS.keys())
+  qa_utils.GenericQueryTest("gnt-instance", list(query.INSTANCE_FIELDS))
 
 
 def TestInstanceListFields():
   """gnt-instance list-fields"""
-  qa_utils.GenericQueryFieldsTest("gnt-instance", query.INSTANCE_FIELDS.keys())
+  qa_utils.GenericQueryFieldsTest("gnt-instance", list(query.INSTANCE_FIELDS))
 
 
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
@@ -924,8 +925,8 @@ def TestReplaceDisks(instance, curr_nodes, other_nodes):
     return cmd
 
   if not IsDiskReplacingSupported(instance):
-    print qa_utils.FormatInfo("Instance doesn't support disk replacing,"
-                              " skipping test")
+    print(qa_utils.FormatInfo("Instance doesn't support disk replacing,"
+                              " skipping test"))
     return
 
   # Currently all supported templates have one primary and one secondary node
@@ -1005,10 +1006,10 @@ def _BuildRecreateDisksOpts(en_disks, with_spindles, with_growth,
                                    utils.ParseUnit(disk["growth"])))
   else:
     build_size_opt = (lambda disk: "size=%s" % disk["size"])
-  build_disk_opt = (lambda (idx, disk):
-                    "--disk=%s:%s%s" % (idx, build_size_opt(disk),
-                                        build_spindles_opt(disk)))
-  return map(build_disk_opt, en_disks)
+  build_disk_opt = (lambda idx_dsk:
+                    "--disk=%s:%s%s" % (idx_dsk[0], build_size_opt(idx_dsk[1]),
+                                        build_spindles_opt(idx_dsk[1])))
+  return [build_disk_opt(d) for d in en_disks]
 
 
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
@@ -1130,13 +1131,13 @@ def TestBackupList(expnode):
   """gnt-backup list"""
   AssertCommand(["gnt-backup", "list", "--node=%s" % expnode.primary])
 
-  qa_utils.GenericQueryTest("gnt-backup", query.EXPORT_FIELDS.keys(),
+  qa_utils.GenericQueryTest("gnt-backup", list(query.EXPORT_FIELDS),
                             namefield=None, test_unknown=False)
 
 
 def TestBackupListFields():
   """gnt-backup list-fields"""
-  qa_utils.GenericQueryFieldsTest("gnt-backup", query.EXPORT_FIELDS.keys())
+  qa_utils.GenericQueryFieldsTest("gnt-backup", list(query.EXPORT_FIELDS))
 
 
 def TestRemoveInstanceOfflineNode(instance, snode, set_offline, set_online):
@@ -1211,7 +1212,7 @@ def TestInstanceCreationRestrictedByDiskTemplates():
   if (len(enabled_disk_templates) > 1):
     # Partition the disk templates, enable them separately and check if the
     # disabled ones cannot be used by instances.
-    middle = len(enabled_disk_templates) / 2
+    middle = len(enabled_disk_templates) // 2
     templates1 = enabled_disk_templates[:middle]
     templates2 = enabled_disk_templates[middle:]
 
@@ -1408,8 +1409,8 @@ def TestInstanceUserDown(instance):
       fn(instance)
       qa_daemon.TestResumeWatcher()
     else:
-      print "%s hypervisor is not enabled, skipping test for this hypervisor" \
-          % hv
+      print("%s hypervisor is not enabled, skipping test for this hypervisor" \
+          % hv)
 
 
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
@@ -1423,7 +1424,7 @@ def TestInstanceCommunication(instance, master):
          "--instance-communication-network=%s" % network_name]
   result_output = qa_utils.GetCommandOutput(master.primary,
                                             utils.ShellQuoteArgs(cmd))
-  print result_output
+  print(result_output)
 
   # Enable instance communication mechanism for this instance
   AssertCommand(["gnt-instance", "modify", "-c", "yes", instance.name])
@@ -1438,7 +1439,7 @@ def TestInstanceCommunication(instance, master):
 
   ## Check the output of 'gnt-instance list'
   nic_names = _GetInstanceField(instance.name, "nic.names")
-  nic_names = map(lambda x: x.strip(" '"), nic_names.strip("[]").split(","))
+  nic_names = [x.strip(" '") for x in nic_names.strip("[]").split(",")]
 
   AssertIn(nic_name, nic_names,
            msg="Looking for instance communication TAP interface")
@@ -1516,7 +1517,71 @@ def TestInstanceCommunication(instance, master):
          "--instance-communication-network=%s" % network_name]
   result_output = qa_utils.GetCommandOutput(master.primary,
                                             utils.ShellQuoteArgs(cmd))
-  print result_output
+  print(result_output)
+
+
+def _TestRedactionOfSecretOsParams(node, cmd, secret_keys):
+  """Tests redaction of secret os parameters
+
+  """
+  AssertCommand(["gnt-cluster", "modify", "--max-running-jobs", "1"])
+  debug_delay_id = int(stdout_of(["gnt-debug", "delay", "--print-jobid",
+                       "--submit", "300"]))
+  cmd_jid = int(stdout_of(cmd))
+  job_file_abspath = "%s/job-%s" % (pathutils.QUEUE_DIR, cmd_jid)
+  job_file = qa_utils.MakeNodePath(node, job_file_abspath)
+
+  for k in secret_keys:
+    grep_cmd = ["grep", "\"%s\":\"<redacted>\"" % k, job_file]
+    AssertCommand(grep_cmd)
+
+  AssertCommand(["gnt-job", "cancel", "--kill", "--yes-do-it",
+                str(debug_delay_id)])
+  AssertCommand(["gnt-cluster", "modify", "--max-running-jobs", "20"])
+  AssertCommand(["gnt-job", "wait", str(cmd_jid)])
+
+
+def TestInstanceAddOsParams():
+  """Tests instance add with secret os parameters"""
+
+  if not qa_config.IsTemplateSupported(constants.DT_PLAIN):
+    return
+
+  master = qa_config.GetMasterNode()
+  instance = qa_config.AcquireInstance()
+
+  secret_keys = ["param1", "param2"]
+  cmd = (["gnt-instance", "add",
+          "--os-type=%s" % qa_config.get("os"),
+          "--disk-template=%s" % constants.DT_PLAIN,
+          "--os-parameters-secret",
+          "param1=secret1,param2=secret2",
+          "--node=%s" % master.primary] +
+          GetGenericAddParameters(instance, constants.DT_PLAIN))
+  cmd.append("--submit")
+  cmd.append("--print-jobid")
+  cmd.append(instance.name)
+
+  _TestRedactionOfSecretOsParams(master.primary, cmd, secret_keys)
+
+  TestInstanceRemove(instance)
+  instance.Release()
+
+
+def TestSecretOsParams():
+  """Tests secret os parameter transmission"""
+
+  master = qa_config.GetMasterNode()
+  secret_keys = ["param1", "param2"]
+  cmd = (["gnt-debug", "test-osparams", "--os-parameters-secret",
+         "param1=secret1,param2=secret2", "--submit", "--print-jobid"])
+  _TestRedactionOfSecretOsParams(master.primary, cmd, secret_keys)
+
+  cmd_output = stdout_of(["gnt-debug", "test-osparams",
+                         "--os-parameters-secret",
+                         "param1=secret1,param2=secret2"])
+  AssertIn("\'param1\': \'secret1\'", cmd_output)
+  AssertIn("\'param2\': \'secret2\'", cmd_output)
 
 
 available_instance_tests = [
