@@ -38,22 +38,22 @@ import re
 from ganeti.utils import retry
 from ganeti import constants
 from ganeti import query
-import qa_config
-import qa_error
-import qa_job_utils
-import qa_utils
+from qa import qa_config
+from qa import qa_error
+from qa import qa_job_utils
+from qa import qa_utils
 from qa_utils import AssertCommand, GetCommandOutput
 
 
 def TestJobList():
   """gnt-job list"""
-  qa_utils.GenericQueryTest("gnt-job", query.JOB_FIELDS.keys(),
+  qa_utils.GenericQueryTest("gnt-job", list(query.JOB_FIELDS),
                             namefield="id", test_unknown=False)
 
 
 def TestJobListFields():
   """gnt-node list-fields"""
-  qa_utils.GenericQueryFieldsTest("gnt-job", query.JOB_FIELDS.keys())
+  qa_utils.GenericQueryFieldsTest("gnt-job", list(query.JOB_FIELDS))
 
 
 def TestJobCancellation():
