@@ -37,9 +37,9 @@ from ganeti import constants
 from ganeti import query
 from ganeti import serializer
 
-import qa_config
-import qa_error
-import qa_utils
+from qa import qa_config
+from qa import qa_error
+from qa import qa_utils
 
 from qa_utils import AssertCommand, AssertRedirectedCommand, AssertEqual, \
   AssertIn, GetCommandOutput
@@ -284,6 +284,16 @@ def TestNodeModify(node):
   AssertCommand(["gnt-cluster", "modify",
                  "--candidate-pool-size=%s" % default_pool_size])
 
+  # For test clusters with more nodes than the default pool size,
+  # we now have too many master candidates. To readjust to the original
+  # size, manually demote all nodes and rely on auto-promotion to adjust.
+  if len(nodes) > default_pool_size:
+    master = qa_config.GetMasterNode()
+    for n in nodes:
+      if n.primary != master.primary:
+        AssertCommand(["gnt-node", "modify", "--master-candidate=no",
+                       "--auto-promote", n.primary])
+
 
 def _CreateOobScriptStructure():
   """Create a simple OOB handling script and its structure."""
@@ -299,7 +309,7 @@ def _CreateOobScriptStructure():
                  "exit $(< %s)\n") %
                 (utils.ShellQuote(verify_path), utils.ShellQuote(data_path),
                  utils.ShellQuote(exit_code_path)))
-  oob_path = qa_utils.UploadData(master.primary, oob_script, mode=0700)
+  oob_path = qa_utils.UploadData(master.primary, oob_script, mode=0o700)
 
   return [oob_path, verify_path, data_path, exit_code_path]
 
@@ -456,7 +466,7 @@ def TestOutOfBand():
     verify_path2 = qa_utils.UploadData(master.primary, "")
     oob_script = ("#!/bin/sh\n"
                   "echo \"$@\" > %s\n") % verify_path2
-    oob_path2 = qa_utils.UploadData(master.primary, oob_script, mode=0700)
+    oob_path2 = qa_utils.UploadData(master.primary, oob_script, mode=0o700)
 
     try:
       AssertCommand(["gnt-node", "modify", "--node-parameters",
@@ -476,12 +486,12 @@ def TestOutOfBand():
 
 def TestNodeList():
   """gnt-node list"""
-  qa_utils.GenericQueryTest("gnt-node", query.NODE_FIELDS.keys())
+  qa_utils.GenericQueryTest("gnt-node", list(query.NODE_FIELDS))
 
 
 def TestNodeListFields():
   """gnt-node list-fields"""
-  qa_utils.GenericQueryFieldsTest("gnt-node", query.NODE_FIELDS.keys())
+  qa_utils.GenericQueryFieldsTest("gnt-node", list(query.NODE_FIELDS))
 
 
 def TestNodeListDrbd(node, is_drbd):
