@@ -39,9 +39,9 @@ from ganeti import utils
 from ganeti import constants
 from ganeti import pathutils
 
-import qa_config
-import qa_utils
-import qa_error
+from qa import qa_config
+from qa import qa_utils
+from qa import qa_error
 
 from qa_utils import AssertCommand, AssertIn, AssertNotIn
 
@@ -117,9 +117,9 @@ def _SetupTempOs(node, dirname, variant, valid):
 
   cmd = " && ".join(parts)
 
-  print qa_utils.FormatInfo("Setting up %s with %s OS definition" %
+  print(qa_utils.FormatInfo("Setting up %s with %s OS definition" %
                             (node.primary,
-                             ["an invalid", "a valid"][int(valid)]))
+                             ["an invalid", "a valid"][int(valid)])))
 
   AssertCommand(cmd, node=node)
 
