@@ -33,7 +33,6 @@
 """
 
 import copy
-import functools
 import itertools
 import os.path
 import random
@@ -58,10 +57,10 @@ from ganeti.http.auth import ParsePasswordFile
 import ganeti.rapi.client        # pylint: disable=W0611
 import ganeti.rapi.client_utils
 
-import qa_config
-import qa_error
-import qa_logging
-import qa_utils
+from qa import qa_config
+from qa import qa_error
+from qa import qa_logging
+from qa import qa_utils
 
 from qa_instance import GetInstanceInfo
 from qa_instance import IsDiskReplacingSupported
@@ -95,9 +94,9 @@ def _EnsureRapiFilesPresence():
     # No files to be had
     return
 
-  print qa_logging.FormatWarning("Replacing the certificate and users file on"
+  print(qa_logging.FormatWarning("Replacing the certificate and users file on"
                                  " the node with the ones provided in %s"
-                                 % rapi_files_location)
+                                 % rapi_files_location))
 
   # The RAPI files
   AssertCommand(["mkdir", "-p", pathutils.RAPI_DATA_DIR])
@@ -139,7 +138,7 @@ def ReloadCertificates(ensure_presence=True):
          qa_utils.MakeNodePath(master, pathutils.RAPI_CERT_FILE)]
 
   # Write to temporary file
-  _rapi_ca = tempfile.NamedTemporaryFile()
+  _rapi_ca = tempfile.NamedTemporaryFile(mode="w")
   _rapi_ca.write(qa_utils.GetCommandOutput(master.primary,
                                            utils.ShellQuoteArgs(cmd)))
   _rapi_ca.flush()
@@ -150,8 +149,8 @@ def ReloadCertificates(ensure_presence=True):
 
   if qa_config.UseVirtualCluster():
     # TODO: Implement full support for RAPI on virtual clusters
-    print qa_logging.FormatWarning("RAPI tests are not yet supported on"
-                                   " virtual clusters and will be disabled")
+    print(qa_logging.FormatWarning("RAPI tests are not yet supported on"
+                                   " virtual clusters and will be disabled"))
 
     assert _rapi_client is None
   else:
@@ -160,7 +159,7 @@ def ReloadCertificates(ensure_presence=True):
                                                 password=_rapi_password,
                                                 curl_config_fn=cfg_curl)
 
-    print "RAPI protocol version: %s" % _rapi_client.GetVersion()
+    print("RAPI protocol version: %s" % _rapi_client.GetVersion())
 
 
 #TODO(riba): Remove in 2.13, used just by rapi-workload which disappears there
@@ -182,7 +181,7 @@ def _CreateRapiUser(rapi_user):
   rapi_users_path = qa_utils.MakeNodePath(master, pathutils.RAPI_USERS_FILE)
   rapi_dir = os.path.dirname(rapi_users_path)
 
-  fh = tempfile.NamedTemporaryFile()
+  fh = tempfile.NamedTemporaryFile(mode="w")
   try:
     fh.write("%s %s write\n" % (rapi_user, rapi_secret))
     fh.flush()
@@ -343,7 +342,7 @@ def _DoTests(uris):
   for uri, verify, method, body in uris:
     assert uri.startswith("/")
 
-    print "%s %s" % (method, uri)
+    print("%s %s" % (method, uri))
     data = _rapi_client._SendRequest(method, uri, None, body)
 
     if verify is not None:
@@ -393,10 +392,10 @@ def _DoGetPutTests(get_uri, modify_uri, opcode_params, rapi_only_aliases=None,
   if set_exceptions is None:
     set_exceptions = []
 
-  print "Testing get/modify symmetry of %s and %s" % (get_uri, modify_uri)
+  print("Testing get/modify symmetry of %s and %s" % (get_uri, modify_uri))
 
   # First we see if all parameters of the opcode are returned through RAPI
-  params_of_interest = map(lambda x: x[0], opcode_params)
+  params_of_interest = [x[0] for x in opcode_params]
 
   # The RAPI-specific aliases are to be checked as well
   if rapi_only_aliases is not None:
@@ -404,14 +403,14 @@ def _DoGetPutTests(get_uri, modify_uri, opcode_params, rapi_only_aliases=None,
 
   info = _rapi_client._SendRequest("GET", get_uri, None, {})
 
-  missing_params = filter(lambda x: x not in info and x not in exceptions,
-                          params_of_interest)
+  missing_params = [x for x in params_of_interest
+                    if x not in info and x not in exceptions]
   if missing_params:
     raise qa_error.Error("The parameters %s which can be set through the "
                          "appropriate opcode are not present in the response "
                          "from %s" % (','.join(missing_params), get_uri))
 
-  print "GET successful at %s" % get_uri
+  print("GET successful at %s" % get_uri)
 
   # Then if we can perform a set with the same values as received
   put_payload = {}
@@ -421,7 +420,7 @@ def _DoGetPutTests(get_uri, modify_uri, opcode_params, rapi_only_aliases=None,
 
   _rapi_client._SendRequest(modify_method, modify_uri, None, put_payload)
 
-  print "%s successful at %s" % (modify_method, modify_uri)
+  print("%s successful at %s" % (modify_method, modify_uri))
 # pylint: enable=W0212
 
 
@@ -499,7 +498,7 @@ def TestEmptyCluster():
   for method in ["GET", "PUT", "POST", "DELETE"]:
     try:
       _DoTests([("/99/resource/not/here/99", None, method, None)])
-    except rapi.client.GanetiApiError, err:
+    except rapi.client.GanetiApiError as err:
       AssertEqual(err.code, 404)
     else:
       raise qa_error.Error("Non-existent resource didn't return HTTP 404")
@@ -508,7 +507,7 @@ def TestEmptyCluster():
   for method in ["PUT", "POST", "DELETE"]:
     try:
       _DoTests([("/version", None, method, None)])
-    except rapi.client.GanetiApiError, err:
+    except rapi.client.GanetiApiError as err:
       AssertEqual(err.code, 501)
     else:
       raise qa_error.Error("Non-implemented method didn't fail")
@@ -551,7 +550,7 @@ def TestRapiQuery():
       constants.QR_FILTER: "uuid",
     }.get(what, "name")
 
-    all_fields = query.ALL_FIELDS[what].keys()
+    all_fields = list(query.ALL_FIELDS[what])
     rnd.shuffle(all_fields)
 
     # No fields, should return everything
@@ -590,7 +589,7 @@ def TestRapiQuery():
       _DoTests([
         ("/2/query/%s" % what, None, "GET", None),
         ])
-    except rapi.client.GanetiApiError, err:
+    except rapi.client.GanetiApiError as err:
       AssertEqual(err.code, 400)
     else:
       raise qa_error.Error("Request missing 'fields' parameter didn't fail")
@@ -746,7 +745,7 @@ def _FilterTags(seq):
   ignore_re = qa_config.get("ignore-tags-re", None)
 
   if ignore_re:
-    return itertools.ifilterfalse(re.compile(ignore_re).match, seq)
+    return itertools.filterfalse(re.compile(ignore_re).match, seq)
   else:
     return seq
 
@@ -980,8 +979,7 @@ def TestRapiInstanceMultiAlloc(node):
   instance_two = qa_config.AcquireInstance()
   instance_list = [instance_one, instance_two]
   try:
-    rapi_dicts = map(functools.partial(_GenInstanceAllocationDict, node),
-                     instance_list)
+    rapi_dicts = [_GenInstanceAllocationDict(node, i) for i in instance_list]
 
     job_id = _rapi_client.InstancesMultiAlloc(rapi_dicts)
 
@@ -1034,8 +1032,8 @@ def TestRapiInstanceRemove(instance, use_client):
 def TestRapiInstanceMigrate(instance):
   """Test migrating instance via RAPI"""
   if not IsMigrationSupported(instance):
-    print qa_logging.FormatInfo("Instance doesn't support migration, skipping"
-                                " test")
+    print(qa_logging.FormatInfo("Instance doesn't support migration, skipping"
+                                " test"))
     return
   # Move to secondary node
   _WaitForRapiJob(_rapi_client.MigrateInstance(instance.name))
@@ -1048,8 +1046,8 @@ def TestRapiInstanceMigrate(instance):
 def TestRapiInstanceFailover(instance):
   """Test failing over instance via RAPI"""
   if not IsFailoverSupported(instance):
-    print qa_logging.FormatInfo("Instance doesn't support failover, skipping"
-                                " test")
+    print(qa_logging.FormatInfo("Instance doesn't support failover, skipping"
+                                " test"))
     return
   # Move to secondary node
   _WaitForRapiJob(_rapi_client.FailoverInstance(instance.name))
@@ -1089,7 +1087,7 @@ def TestRapiInstanceRenameAndBack(rename_source, rename_target):
 def TestRapiInstanceReinstall(instance):
   """Test reinstalling an instance via RAPI"""
   if instance.disk_template == constants.DT_DISKLESS:
-    print qa_logging.FormatInfo("Test not supported for diskless instances")
+    print(qa_logging.FormatInfo("Test not supported for diskless instances"))
     return
 
   _WaitForRapiJob(_rapi_client.ReinstallInstance(instance.name))
@@ -1105,8 +1103,8 @@ def TestRapiInstanceReinstall(instance):
 def TestRapiInstanceReplaceDisks(instance):
   """Test replacing instance disks via RAPI"""
   if not IsDiskReplacingSupported(instance):
-    print qa_logging.FormatInfo("Instance doesn't support disk replacing,"
-                                " skipping test")
+    print(qa_logging.FormatInfo("Instance doesn't support disk replacing,"
+                                " skipping test"))
     return
   fn = _rapi_client.ReplaceInstanceDisks
   _WaitForRapiJob(fn(instance.name,
@@ -1161,7 +1159,7 @@ def TestRapiStoppedInstanceConsole(instance):
   """Test getting stopped instance's console information via RAPI"""
   try:
     _rapi_client.GetInstanceConsole(instance.name)
-  except rapi.client.GanetiApiError, err:
+  except rapi.client.GanetiApiError as err:
     AssertEqual(err.code, 503)
   else:
     raise qa_error.Error("Getting console for stopped instance didn't"
@@ -1225,7 +1223,7 @@ def TestInterClusterInstanceMove(src_instance, dest_instance,
   """Test tools/move-instance"""
   master = qa_config.GetMasterNode()
 
-  rapi_pw_file = tempfile.NamedTemporaryFile()
+  rapi_pw_file = tempfile.NamedTemporaryFile(mode="w")
   rapi_pw_file.write(_rapi_password)
   rapi_pw_file.flush()
 
@@ -1344,8 +1342,8 @@ def TestInstanceDataCensorship(instance, inodes):
   """Test protection of sensitive instance data."""
 
   if instance.disk_template != constants.DT_DRBD8:
-    print qa_utils.FormatInfo("Only the DRBD secret is a sensitive parameter"
-                              " right now, skipping for non-DRBD instance.")
+    print(qa_utils.FormatInfo("Only the DRBD secret is a sensitive parameter"
+                              " right now, skipping for non-DRBD instance."))
     return
 
   drbd_secret = _RetrieveSecret(instance, inodes[0])
@@ -1357,7 +1355,7 @@ def TestInstanceDataCensorship(instance, inodes):
   info_dict = _rapi_client.GetJobStatus(job_id)
 
   if drbd_secret in str(info_dict):
-    print qa_utils.FormatInfo("DRBD secret: %s" % drbd_secret)
-    print qa_utils.FormatInfo("Retrieved data\n%s" % str(info_dict))
+    print(qa_utils.FormatInfo("DRBD secret: %s" % drbd_secret))
+    print(qa_utils.FormatInfo("Retrieved data\n%s" % str(info_dict)))
     raise qa_error.Error("Found DRBD secret in contents of RAPI instance info"
                          " call; see above.")
