@@ -32,6 +32,8 @@
 
 """
 
+from __future__ import print_function
+
 import contextlib
 import copy
 import datetime
@@ -47,7 +49,7 @@ import yaml
 
 try:
   import functools
-except ImportError, err:
+except ImportError as err:
   raise ImportError("Python 2.5 or higher is required: %s" % err)
 
 from ganeti import utils
@@ -58,8 +60,8 @@ from ganeti import pathutils
 from ganeti import vcluster
 
 import colors
-import qa_config
-import qa_error
+from qa import qa_config
+from qa import qa_error
 
 from qa_logging import FormatInfo
 
@@ -133,7 +135,7 @@ def _GetName(entity, fn):
   @param fn: Function retrieving name from entity
 
   """
-  if isinstance(entity, basestring):
+  if isinstance(entity, str):
     result = entity
   else:
     result = fn(entity)
@@ -166,16 +168,17 @@ def _PrintCommandOutput(stdout, stderr):
   if stdout:
     stdout_clean = stdout.rstrip('\n')
     if stderr:
-      print "Stdout was:\n%s" % stdout_clean
+      print("Stdout was:\n%s" % stdout_clean)
     else:
-      print stdout_clean
+      print(stdout_clean)
 
   if stderr:
-    print "Stderr was:"
-    print >> sys.stderr, stderr.rstrip('\n')
+    print("Stderr was:")
+    print(stderr.rstrip('\n'), file=sys.stderr)
 
 
-def AssertCommand(cmd, fail=False, node=None, log_cmd=True, max_seconds=None):
+def AssertCommand(cmd, fail=False, node=None, log_cmd=True, forward_agent=True,
+                  max_seconds=None):
   """Checks that a remote command succeeds.
 
   @param cmd: either a string (the command to execute) or a list (to
@@ -188,6 +191,10 @@ def AssertCommand(cmd, fail=False, node=None, log_cmd=True, max_seconds=None):
       dict or a string)
   @param log_cmd: if False, the command won't be logged (simply passed to
       StartSSH)
+  @type forward_agent: boolean
+  @param forward_agent: whether to forward the agent when starting the SSH
+                        session or not, sometimes useful for crypto-related
+                        operations which can use a key they should not
   @type max_seconds: double
   @param max_seconds: fail if the command takes more than C{max_seconds}
       seconds
@@ -200,13 +207,14 @@ def AssertCommand(cmd, fail=False, node=None, log_cmd=True, max_seconds=None):
 
   nodename = _GetName(node, operator.attrgetter("primary"))
 
-  if isinstance(cmd, basestring):
+  if isinstance(cmd, str):
     cmdstr = cmd
   else:
     cmdstr = utils.ShellQuoteArgs(cmd)
 
   start = datetime.datetime.now()
-  popen = StartSSH(nodename, cmdstr, log_cmd=log_cmd)
+  popen = StartSSH(nodename, cmdstr, log_cmd=log_cmd,
+                   forward_agent=forward_agent)
   # Run the command
   stdout, stderr = popen.communicate()
   rcode = popen.returncode
@@ -263,7 +271,7 @@ def AssertRedirectedCommand(cmd, fail=False, node=None, log_cmd=True):
 
 
 def GetSSHCommand(node, cmd, strict=True, opts=None, tty=False,
-                  use_multiplexer=True):
+                  use_multiplexer=True, forward_agent=True):
   """Builds SSH command to be executed.
 
   @type node: string
@@ -279,6 +287,8 @@ def GetSSHCommand(node, cmd, strict=True, opts=None, tty=False,
   @param tty: if we should use tty; if None, will be auto-detected
   @type use_multiplexer: boolean
   @param use_multiplexer: if the multiplexer for the node should be used
+  @type forward_agent: boolean
+  @param forward_agent: whether to forward the ssh agent or not
 
   """
   args = ["ssh", "-oEscapeChar=none", "-oBatchMode=yes", "-lroot"]
@@ -289,9 +299,14 @@ def GetSSHCommand(node, cmd, strict=True, opts=None, tty=False,
   if tty:
     args.append("-t")
 
+  # Multiplexers we use right now forward agents, so even if we ought to be
+  # using one, ignore it if agent forwarding is disabled.
+  if not forward_agent:
+    use_multiplexer = False
+
   args.append("-oStrictHostKeyChecking=%s" % ("yes" if strict else "no", ))
   args.append("-oClearAllForwardings=yes")
-  args.append("-oForwardAgent=yes")
+  args.append("-oForwardAgent=%s" % ("yes" if forward_agent else "no", ))
   if opts:
     args.extend(opts)
   if node in _MULTIPLEXERS and use_multiplexer:
@@ -330,17 +345,18 @@ def StartLocalCommand(cmd, _nolog_opts=False, log_cmd=True, **kwargs):
       pcmd = [i for i in cmd if not i.startswith("-")]
     else:
       pcmd = cmd
-    print "%s %s" % (colors.colorize("Command:", colors.CYAN),
-                     utils.ShellQuoteArgs(pcmd))
-  return subprocess.Popen(cmd, shell=False, **kwargs)
+    print("%s %s" % (colors.colorize("Command:", colors.CYAN),
+                     utils.ShellQuoteArgs(pcmd)))
+  return subprocess.Popen(cmd, shell=False, encoding="utf-8", **kwargs)
 
 
-def StartSSH(node, cmd, strict=True, log_cmd=True):
+def StartSSH(node, cmd, strict=True, log_cmd=True, forward_agent=True):
   """Starts SSH.
 
   """
-  return StartLocalCommand(GetSSHCommand(node, cmd, strict=strict),
-                           _nolog_opts=True, log_cmd=log_cmd,
+  ssh_command = GetSSHCommand(node, cmd, strict=strict,
+                              forward_agent=forward_agent)
+  return StartLocalCommand(ssh_command, _nolog_opts=True, log_cmd=log_cmd,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
 
@@ -357,7 +373,7 @@ def StartMultiplexer(node):
   sname = tempfile.mktemp(prefix="ganeti-qa-multiplexer.")
   utils.RemoveFile(sname)
   opts = ["-N", "-oControlPath=%s" % sname, "-oControlMaster=yes"]
-  print "Created socket at %s" % sname
+  print("Created socket at %s" % sname)
   child = StartLocalCommand(GetSSHCommand(node, None, opts=opts))
   _MULTIPLEXERS[node] = (sname, child)
 
@@ -366,7 +382,7 @@ def CloseMultiplexers():
   """Closes all current multiplexers and cleans up.
 
   """
-  for node in _MULTIPLEXERS.keys():
+  for node in list(_MULTIPLEXERS):
     (sname, child) = _MULTIPLEXERS.pop(node)
     utils.KillProcess(child.pid, timeout=10, waitpid=True)
     utils.RemoveFile(sname)
@@ -445,7 +461,7 @@ def UploadFile(node, src):
 
   """
   # Make sure nobody else has access to it while preserving local permissions
-  mode = os.stat(src).st_mode & 0700
+  mode = os.stat(src).st_mode & 0o700
 
   cmd = ('tmp=$(mktemp --tmpdir gnt.XXXXXX) && '
          'chmod %o "${tmp}" && '
@@ -456,7 +472,7 @@ def UploadFile(node, src):
   f = open(src, "r")
   try:
     p = subprocess.Popen(GetSSHCommand(node, cmd), shell=False, stdin=f,
-                         stdout=subprocess.PIPE)
+                         stdout=subprocess.PIPE, encoding="utf-8")
     AssertEqual(p.wait(), 0)
 
     # Return temporary filename
@@ -465,7 +481,7 @@ def UploadFile(node, src):
     f.close()
 
 
-def UploadData(node, data, mode=0600, filename=None):
+def UploadData(node, data, mode=0o600, filename=None):
   """Uploads data to a node and returns the filename.
 
   Caller needs to remove the returned file on the node when it's not needed
@@ -483,7 +499,8 @@ def UploadData(node, data, mode=0600, filename=None):
          "echo \"${tmp}\"") % tmp
 
   p = subprocess.Popen(GetSSHCommand(node, cmd), shell=False,
-                       stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+                       encoding="utf-8", stdin=subprocess.PIPE,
+                       stdout=subprocess.PIPE)
   p.stdin.write(data)
   p.stdin.close()
   AssertEqual(p.wait(), 0)
@@ -509,7 +526,7 @@ def BackupFile(node, path):
   # Return temporary filename
   result = GetCommandOutput(node, cmd).strip()
 
-  print "Backup filename: %s" % result
+  print("Backup filename: %s" % result)
 
   return result
 
@@ -687,7 +704,7 @@ def AddToEtcHosts(hostnames):
 
   """
   master = qa_config.GetMasterNode()
-  tmp_hosts = UploadData(master.primary, "", mode=0644)
+  tmp_hosts = UploadData(master.primary, "", mode=0o644)
 
   data = []
   for localhost in ("::1", "127.0.0.1"):
@@ -712,7 +729,7 @@ def RemoveFromEtcHosts(hostnames):
 
   """
   master = qa_config.GetMasterNode()
-  tmp_hosts = UploadData(master.primary, "", mode=0644)
+  tmp_hosts = UploadData(master.primary, "", mode=0o644)
   quoted_tmp_hosts = utils.ShellQuote(tmp_hosts)
 
   sed_data = " ".join(hostnames)
@@ -749,8 +766,8 @@ def RunInstanceCheck(instance, running):
     running_shellval = ""
     running_text = "not "
 
-  print FormatInfo("Checking if instance '%s' is %srunning" %
-                   (instance_name, running_text))
+  print(FormatInfo("Checking if instance '%s' is %srunning" %
+                   (instance_name, running_text)))
 
   args = [script, instance_name]
   env = {
@@ -862,7 +879,7 @@ def MakeNodePath(node, path):
   """
   (_, basedir) = qa_config.GetVclusterSettings()
 
-  if isinstance(node, basestring):
+  if isinstance(node, str):
     name = node
   else:
     name = node.primary
