@@ -80,6 +80,7 @@ module Ganeti.BasicTypes
 import Control.Applicative
 import Control.Exception (try)
 import Control.Monad
+import Control.Monad.Fail (MonadFail)
 import Control.Monad.Base
 import Control.Monad.Error.Class
 import Control.Monad.Trans
@@ -87,11 +88,11 @@ import Control.Monad.Trans.Control
 import Data.Function
 import Data.List
 import Data.Maybe
-import Data.Monoid
 import Data.Set (Set)
 import qualified Data.Set as Set (empty)
 import Text.JSON (JSON)
 import qualified Text.JSON as JSON (readJSON, showJSON)
+import qualified Control.Monad.Fail as Fail
 
 -- Remove after we require >= 1.8.58
 -- See: https://github.com/ndmitchell/hlint/issues/24
@@ -117,7 +118,9 @@ instance (Error a) => Monad (GenericResult a) where
   (>>=) (Bad x) _ = Bad x
   (>>=) (Ok x) fn = fn x
   return = Ok
+#if !MIN_VERSION_base(4,13,0)
   fail   = Bad . strMsg
+#endif
 
 instance Functor (GenericResult a) where
   fmap _ (Bad msg) = Bad msg
@@ -147,6 +150,9 @@ instance Applicative (GenericResult a) where
   _       <*> (Bad x) = Bad x
   (Ok f)  <*> (Ok x)  = Ok $ f x
 
+instance (Error a) => Fail.MonadFail (GenericResult a) where
+  fail = Bad . strMsg
+
 -- | This is a monad transformation for Result. It's implementation is
 -- based on the implementations of MaybeT and ErrorT.
 --
@@ -173,9 +179,14 @@ instance (Applicative m, Monad m, Error a) => Applicative (ResultT a m) where
   (<*>) = ap
 
 instance (Monad m, Error a) => Monad (ResultT a m) where
-  fail err = ResultT (return . Bad $ strMsg err)
   return   = lift . return
   (>>=)    = flip (elimResultT throwError)
+#if !MIN_VERSION_base(4,13,0)
+  fail err = ResultT (return . Bad $ strMsg err)
+#endif
+
+instance (Monad m, Error a)=> MonadFail (ResultT a m) where
+  fail err = ResultT (return . Bad $ strMsg err)
 
 instance (Monad m, Error a) => MonadError a (ResultT a m) where
   throwError = ResultT . return . Bad
@@ -233,7 +244,7 @@ instance (Error a, MonadBaseControl IO m)
   {-# INLINE liftBaseWith #-}
   {-# INLINE restoreM #-}
 
-instance (Monad m, Applicative m, Error a, Monoid a)
+instance (Monad m, Error a, Monoid a)
          => Alternative (ResultT a m) where
   empty = ResultT $ return mzero
   -- Ensure that 'y' isn't run if 'x' contains a value. This makes it a bit
@@ -241,7 +252,7 @@ instance (Monad m, Applicative m, Error a, Monoid a)
   x <|> y = elimResultT combine return x
     where combine x' = ResultT $ liftM (mplus (Bad x')) (runResultT y)
 
-instance (Monad m, Applicative m, Error a, Monoid a)
+instance (Monad m, Error a, Monoid a)
          => MonadPlus (ResultT a m) where
   mzero = empty
   mplus = (<|>)
