@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP, FlexibleContexts #-}
+
 {-| Provides interface to the 'zlib' library.
 
 -}
@@ -37,24 +39,23 @@ module Ganeti.Codec
   , decompressZlib
   ) where
 
-import Codec.Compression.Zlib (compress)
+import Codec.Compression.Zlib
 import qualified Codec.Compression.Zlib.Internal as I
-import Control.Monad.Error
+import Control.Monad.Except
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString.Lazy.Internal as BL
-import Data.Monoid (mempty)
+
 
 -- | Compresses a lazy bytestring.
 compressZlib :: BL.ByteString -> BL.ByteString
-compressZlib = compress
+compressZlib = compressWith $
+  defaultCompressParams { compressLevel = CompressionLevel 3 }
 
 -- | Decompresses a lazy bytestring, throwing decoding errors using
 -- 'throwError'.
-decompressZlib :: (MonadError e m, Error e) => BL.ByteString -> m BL.ByteString
-decompressZlib = I.foldDecompressStream
-                     (liftM . BL.chunk)
-                     (return mempty)
-                     (const $ throwError . strMsg . ("Zlib: " ++))
-                 . I.decompressWithErrors
-                     I.zlibFormat
-                     I.defaultDecompressParams
+decompressZlib :: (MonadError String m) => BL.ByteString -> m BL.ByteString
+decompressZlib = I.foldDecompressStreamWithInput
+                   (liftM . BL.chunk)
+                   return
+                   (throwError . (++)"Zlib: " . show)
+                   $ I.decompressST I.zlibFormat I.defaultDecompressParams
