@@ -38,55 +38,24 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 -}
 
 module Ganeti.Compat
-  ( rwhnf
-  , Control.Parallel.Strategies.parMap
-  , finiteBitSize
-  , atomicModifyIORef'
-  , filePath'
+  ( filePath'
   , maybeFilePath'
   , toInotifyPath
+  , getPid'
   ) where
 
-import qualified Control.Parallel.Strategies
-import qualified Data.Bits
-import qualified Data.IORef
 import qualified Data.ByteString.UTF8 as UTF8
 import System.FilePath (FilePath)
 import System.Posix.ByteString.FilePath (RawFilePath)
 import qualified System.INotify
-
--- | Wrapper over the function exported from
--- "Control.Parallel.Strategies".
---
--- This wraps either the old or the new name of the function,
--- depending on the detected library version.
-rwhnf :: Control.Parallel.Strategies.Strategy a
-#if MIN_VERSION_parallel(3,0,0)
-rwhnf = Control.Parallel.Strategies.rseq
+import qualified Text.JSON
+import qualified Control.Monad.Fail as Fail
+import System.Process.Internals
+import System.Posix.Types (CPid (..))
+#if MIN_VERSION_process(1,6,3)
+import System.Process (getPid)
 #else
-rwhnf = Control.Parallel.Strategies.rwhnf
-#endif
-
-
-#if __GLASGOW_HASKELL__ < 707
-finiteBitSize :: (Data.Bits.Bits a) => a -> Int
-finiteBitSize = Data.Bits.bitSize
-#else
-finiteBitSize :: (Data.Bits.FiniteBits a) => a -> Int
-finiteBitSize = Data.Bits.finiteBitSize
-#endif
-{-# INLINE finiteBitSize #-}
-
--- FIXME: remove this when dropping support for GHC 7.4.
-atomicModifyIORef' :: Data.IORef.IORef a -> (a -> (a, b)) -> IO b
-#if MIN_VERSION_base(4,6,0)
-atomicModifyIORef' = Data.IORef.atomicModifyIORef'
-#else
-atomicModifyIORef' ref f = do
-    b <- Data.IORef.atomicModifyIORef ref $ \a ->
-            case f a of
-                v@(a',_) -> a' `seq` v
-    b `seq` return b
+import Control.Concurrent.Lifted (readMVar)
 #endif
 
 -- | Wrappers converting ByteString filepaths to Strings and vice versa
@@ -112,3 +81,28 @@ maybeFilePath' = System.INotify.maybeFilePath
 toInotifyPath :: FilePath -> FilePath
 toInotifyPath = id
 #endif
+
+#if !MIN_VERSION_json(0,10,0)
+-- | MonadFail.Fail instance definitions for JSON results
+--
+-- Required as of GHC 8.6 because MonadFailDesugaring is on by
+-- default:
+-- <https://gitlab.haskell.org/ghc/ghc/wikis/migration/8.6>. Added
+-- upstream in version 0.10.
+instance Fail.MonadFail Text.JSON.Result where
+  fail = Fail.fail
+#endif
+
+-- | Process 1.6.3. introduced the getPid function, for older versions
+-- provide an implemention here (https://github.com/haskell/process/pull/109)
+type Pid = CPid
+getPid' :: ProcessHandle -> IO (Maybe Pid)
+#if MIN_VERSION_process(1,6,3)
+getPid' = getPid
+#else
+getPid' (ProcessHandle mh _) = do
+  p_ <- readMVar mh
+  case p_ of
+    OpenHandle pid -> return $ Just pid
+    _ -> return Nothing
+#endif
