@@ -40,15 +40,16 @@ module Ganeti.Confd.Server
   , prepMain
   ) where
 
-import Control.Applicative((<$>))
 import Control.Concurrent
 import Control.Monad (forever, liftM)
 import Data.IORef
 import Data.List
+import qualified Data.ByteString.Char8 as Char8
 import qualified Data.Map as M
 import Data.Maybe (fromMaybe)
 import Network.BSD (getServicePortNumber)
 import qualified Network.Socket as S
+import Network.Socket.ByteString (recvFrom, sendTo)
 import System.Exit
 import System.IO
 import qualified Text.JSON as J
@@ -56,7 +57,7 @@ import qualified Text.JSON as J
 import Ganeti.BasicTypes
 import Ganeti.Errors
 import Ganeti.Daemon
-import Ganeti.JSON
+import Ganeti.JSON (containerFromList, fromContainer)
 import Ganeti.Objects
 import Ganeti.Confd.Types
 import Ganeti.Confd.Utils
@@ -104,13 +105,13 @@ nodeRole :: ConfigData -> String -> Result ConfdNodeRole
 nodeRole cfg name = do
   cmaster <- errToResult $ QCluster.clusterMasterNodeName cfg
   mnode <- errToResult $ getNode cfg name
-  let role = case mnode of
-               node | cmaster == name -> NodeRoleMaster
-                    | nodeDrained node -> NodeRoleDrained
-                    | nodeOffline node -> NodeRoleOffline
-                    | nodeMasterCandidate node -> NodeRoleCandidate
-               _ -> NodeRoleRegular
-  return role
+  let nrole = case mnode of
+                node | cmaster == name -> NodeRoleMaster
+                     | nodeDrained node -> NodeRoleDrained
+                     | nodeOffline node -> NodeRoleOffline
+                     | nodeMasterCandidate node -> NodeRoleCandidate
+                _ -> NodeRoleRegular
+  return nrole
 
 -- | Does an instance ip -> instance -> primary node -> primary ip
 -- transformation.
@@ -170,12 +171,12 @@ buildResponse cdata req@(ConfdRequest { confdRqType = ReqNodeRoleByName }) = do
   node_name <- case confdRqQuery req of
                  PlainQuery str -> return str
                  _ -> fail $ "Invalid query type " ++ show (confdRqQuery req)
-  role <- nodeRole (fst cdata) node_name
-  return (ReplyStatusOk, J.showJSON role,
+  nrole <- nodeRole (fst cdata) node_name
+  return (ReplyStatusOk, J.showJSON nrole,
           clusterSerial . configCluster $ fst cdata)
 
 buildResponse cdata (ConfdRequest { confdRqType = ReqNodePipList }) =
-  -- note: we use foldlWithKey because that's present accross more
+  -- note: we use foldlWithKey because that's present across more
   -- versions of the library
   return (ReplyStatusOk, J.showJSON $
           M.foldlWithKey (\accu _ n -> nodePrimaryIp n:accu) []
@@ -183,7 +184,7 @@ buildResponse cdata (ConfdRequest { confdRqType = ReqNodePipList }) =
           clusterSerial . configCluster $ fst cdata)
 
 buildResponse cdata (ConfdRequest { confdRqType = ReqMcPipList }) =
-  -- note: we use foldlWithKey because that's present accross more
+  -- note: we use foldlWithKey because that's present across more
   -- versions of the library
   return (ReplyStatusOk, J.showJSON $
           M.foldlWithKey (\accu _ n -> if nodeMasterCandidate n
@@ -303,7 +304,7 @@ responder cfgref socket hmac msg peer = do
               logDebug $ "Processing request: " ++ rStripSpace origmsg
               mcfg <- readIORef cfgref
               let response = respondInner mcfg hmac rq
-              _ <- S.sendTo socket response peer
+              _ <- sendTo socket (Char8.pack response) peer
               logDebug $ "Response sent: " ++ response
               return ()
     Bad err -> logInfo $ "Failed to parse incoming message: " ++ err
@@ -327,7 +328,7 @@ listener :: S.Socket -> HashKey
          -> (S.Socket -> HashKey -> String -> S.SockAddr -> IO ())
          -> IO ()
 listener s hmac resp = do
-  (msg, _, peer) <- S.recvFrom s 4096
+  (msg, peer) <- (\(m, p) -> (Char8.unpack m, p)) <$> recvFrom s 4096
   if C.confdMagicFourcc `isPrefixOf` msg
     then forkIO (resp s hmac (drop 4 msg) peer) >> return ()
     else logDebug "Invalid magic code!" >> return ()
@@ -354,7 +355,7 @@ prepMain :: PrepFn (S.Family, S.SockAddr) PrepResult
 prepMain _ (af_family, bindaddr) = do
   s <- S.socket af_family S.Datagram S.defaultProtocol
   S.setSocketOption s S.ReuseAddr 1
-  S.bindSocket s bindaddr
+  S.bind s bindaddr
   cref <- newIORef (Bad "Configuration not yet loaded")
   return (s, cref)
 
