@@ -47,7 +47,6 @@ module Ganeti.Confd.Utils
 
 import qualified Data.Attoparsec.Text as P
 
-import Control.Applicative ((*>))
 import qualified Data.ByteString as B
 import Data.Text (pack)
 import qualified Text.JSON as J
@@ -57,7 +56,7 @@ import Ganeti.Confd.Types
 import Ganeti.Hash
 import qualified Ganeti.Constants as C
 import qualified Ganeti.Path as Path
-import Ganeti.JSON
+import Ganeti.JSON (fromJResult)
 import Ganeti.Utils
 
 -- | Type-adjusted max clock skew constant.
