@@ -82,15 +82,14 @@ module Ganeti.Config
     , instNodes
     ) where
 
-import Control.Applicative
 import Control.Arrow ((&&&))
 import Control.Monad
 import Control.Monad.State
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.Foldable as F
-import Data.List (foldl', nub)
-import Data.Maybe (fromMaybe)
+import Data.List (foldl', nub, any)
+import Data.Maybe (fromMaybe, mapMaybe)
 import Data.Monoid
 import qualified Data.Map as M
 import qualified Data.Set as S
@@ -100,7 +99,7 @@ import System.IO
 import Ganeti.BasicTypes
 import qualified Ganeti.Constants as C
 import Ganeti.Errors
-import Ganeti.JSON
+import Ganeti.JSON (fromJResult, fromContainer, GenericContainer(..))
 import Ganeti.Objects
 import Ganeti.Types
 import qualified Ganeti.Utils.MultiMap as MM
@@ -158,21 +157,36 @@ instNodes :: ConfigData -> Instance -> S.Set String
 instNodes cfg inst = maybe id S.insert (instPrimaryNode inst)
                       $ instDiskNodes cfg inst
 
--- | Computes the secondary nodes of an instance. Since this is valid
--- only for DRBD, we call directly 'instDiskNodes', skipping over the
--- extra primary insert.
-instSecondaryNodes :: ConfigData -> Instance -> S.Set String
-instSecondaryNodes cfg inst =
-  maybe id S.delete (instPrimaryNode inst) $ instDiskNodes cfg inst
+-- | Computes the secondary node UUID for a DRBD disk
+computeDiskSecondaryNode :: Disk -> String -> Maybe String
+computeDiskSecondaryNode dsk primary =
+  case diskLogicalId dsk of
+    Just (LIDDrbd8 a b _ _ _ _) -> Just $ if primary == a then b else a
+    _ -> Nothing
 
 -- | Get instances of a given node.
 -- The node is specified through its UUID.
+-- The secondary calculation is expensive and frequently called, so optimise
+-- this to allocate fewer temporary values
 getNodeInstances :: ConfigData -> String -> ([Instance], [Instance])
 getNodeInstances cfg nname =
-    let all_inst = M.elems . fromContainer . configInstances $ cfg
-        pri_inst = filter ((== Just nname) . instPrimaryNode) all_inst
-        sec_inst = filter ((nname `S.member`) . instSecondaryNodes cfg) all_inst
-    in (pri_inst, sec_inst)
+    let all_insts = M.elems . fromContainer . configInstances $ cfg
+        all_disks = fromContainer . configDisks $ cfg
+
+        pri_inst = filter ((== Just nname) . instPrimaryNode) all_insts
+
+        find_disk :: String -> Maybe Disk
+        find_disk d_uuid = M.lookup (UTF8.fromString d_uuid) all_disks
+
+        inst_disks :: [(Instance, [Disk])]
+        inst_disks = [(i, mapMaybe find_disk $ instDisks i) | i <- all_insts]
+
+        sec_insts :: [Instance]
+        sec_insts = [inst |
+          (inst, disks) <- inst_disks,
+          any ((==) nname) $ mapMaybe (\d -> instPrimaryNode inst >>=
+                                       computeDiskSecondaryNode d) disks]
+    in (pri_inst, sec_insts)
 
 -- | Computes the role of a node.
 getNodeRole :: ConfigData -> Node -> NodeRole
@@ -285,6 +299,17 @@ getInstance cfg name =
                       $ instances
                 in getItem "Instance" name by_name
 
+-- | Looks up an instance by exact name match
+getInstanceByName :: ConfigData -> String -> ErrorResult Instance
+getInstanceByName cfg name =
+  let instances = M.elems . fromContainer . configInstances $ cfg
+      matching = F.find (maybe False (== name) . instName) instances
+  in case matching of
+      Just inst -> Ok inst
+      Nothing   -> Bad $ OpPrereqError
+                   ("Instance name " ++ name ++ " not found")
+                   ECodeNoEnt
+
 -- | Looks up a disk by uuid.
 getDisk :: ConfigData -> String -> ErrorResult Disk
 getDisk cfg name =
@@ -388,7 +413,7 @@ getFilledInstOsParams cfg inst =
 -- | Looks up an instance's primary node.
 getInstPrimaryNode :: ConfigData -> String -> ErrorResult Node
 getInstPrimaryNode cfg name =
-  getInstance cfg name
+  getInstanceByName cfg name
   >>= withMissingParam "Instance without primary node" return . instPrimaryNode
   >>= getNode cfg
 
@@ -407,7 +432,8 @@ getDrbdDiskNodes cfg disk =
 -- the primary node has to be appended to the results.
 getInstAllNodes :: ConfigData -> String -> ErrorResult [Node]
 getInstAllNodes cfg name = do
-  inst_disks <- getInstDisks cfg name
+  inst <- getInstanceByName cfg name
+  inst_disks <- getInstDisksFromObj cfg inst
   let disk_nodes = concatMap (getDrbdDiskNodes cfg) inst_disks
   pNode <- getInstPrimaryNode cfg name
   return . nub $ pNode:disk_nodes
@@ -479,9 +505,9 @@ getInstMinorsForNode :: ConfigData
                      -> Instance
                      -> [(String, Int, String, String, String, String)]
 getInstMinorsForNode cfg node inst =
-  let role = if Just node == instPrimaryNode inst
-               then rolePrimary
-               else roleSecondary
+  let nrole = if Just node == instPrimaryNode inst
+                then rolePrimary
+                else roleSecondary
       iname = fromMaybe "" $ instName inst
       inst_disks = case getInstDisksFromObj cfg inst of
                      Ok disks -> disks
@@ -490,7 +516,7 @@ getInstMinorsForNode cfg node inst =
   -- separate place, or reuse the iv_name (but that is deprecated on
   -- the Python side)
   in concatMap (\(idx, dsk) ->
-            [(node, minor, iname, "disk/" ++ show idx, role, peer)
+            [(node, minor, iname, "disk/" ++ show idx, nrole, peer)
                | (minor, peer) <- getDrbdMinorsForNode node dsk]) .
      zip [(0::Int)..] $ inst_disks
 
