@@ -46,6 +46,7 @@ import System.INotify
 
 import Ganeti.BasicTypes
 import Ganeti.Objects
+import Ganeti.Compat
 import Ganeti.Confd.Utils
 import Ganeti.Config
 import Ganeti.Logging
@@ -172,14 +173,13 @@ safeUpdateConfig path oldfstat save_fn =
 -- | Long-interval reload watcher.
 --
 -- This is on top of the inotify-based triggered reload.
-onWatcherTimer :: IO Bool -> FilePath -> (Result ConfigData -> IO ())
+onWatcherTimer :: FilePath -> (Result ConfigData -> IO ())
                -> MVar ServerState -> IO ()
-onWatcherTimer inotiaction path save_fn state = do
+onWatcherTimer path save_fn state = do
   threadDelay watchInterval
   logDebug "Config-reader watcher timer fired"
   modifyMVar_ state (onWatcherInner path save_fn)
-  _ <- inotiaction
-  onWatcherTimer inotiaction path save_fn state
+  onWatcherTimer path save_fn state
 
 -- | Inner onWatcher handler.
 --
@@ -201,7 +201,7 @@ onPollTimer :: IO Bool -> FilePath -> (Result ConfigData -> IO ())
             -> MVar ServerState -> IO ()
 onPollTimer inotiaction path save_fn state = do
   threadDelay pollInterval
-  logDebug "Poll timer fired"
+  logDebug $ "Poll timer fired for " ++ path
   continue <- modifyMVar state (onPollInner inotiaction path save_fn)
   if continue
     then onPollTimer inotiaction path save_fn state
@@ -247,7 +247,7 @@ addNotifier :: INotify -> FilePath -> (Result ConfigData -> IO ())
             -> MVar ServerState -> IO Bool
 addNotifier inotify path save_fn mstate =
   Control.Exception.catch
-        (addWatch inotify [CloseWrite] path
+        (addWatch inotify [CloseWrite] (toInotifyPath path)
             (onInotify inotify path save_fn mstate) >> return True)
         (\e -> const (return False) (e::IOError))
 
@@ -255,7 +255,7 @@ addNotifier inotify path save_fn mstate =
 onInotify :: INotify -> String -> (Result ConfigData -> IO ())
           -> MVar ServerState -> Event -> IO ()
 onInotify inotify path save_fn mstate Ignored = do
-  logDebug "File lost, trying to re-establish notifier"
+  logDebug $ "File lost, trying to re-establish notifier for " ++ path
   modifyMVar_ mstate $ \state -> do
     result <- addNotifier inotify path save_fn mstate
     (newfstat, _) <- safeUpdateConfig path (reloadFStat state) save_fn
@@ -267,7 +267,8 @@ onInotify inotify path save_fn mstate Ignored = do
                   path save_fn mstate
         return state' { reloadModel = mode }
 
-onInotify inotify path save_fn mstate _ =
+onInotify inotify path save_fn mstate _ = do
+  logDebug $ "onInotify fired for " ++ path
   modifyMVar_ mstate $ \state ->
     if reloadModel state == ReloadNotify
        then do
@@ -304,7 +305,7 @@ initConfigReader save_fn = do
       modifyMVar_ statemvar
         (\state -> return state { reloadModel = initialPoll })
   -- fork the timeout timer
-  _ <- forkIO $ onWatcherTimer inotiaction conf_file save_fn statemvar
+  _ <- forkIO $ onWatcherTimer conf_file save_fn statemvar
   -- fork the polling timer
   unless has_inotify $ do
     _ <- forkIO $ onPollTimer inotiaction conf_file save_fn statemvar
