@@ -1,4 +1,5 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell, CPP #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-| Implementation of the Ganeti data collector types.
 
@@ -47,23 +48,28 @@ module Ganeti.DataCollectors.Types
   , buildReport
   , mergeStatuses
   , getCategoryName
+  , ReportBuilder(..)
+  , DataCollector(..)
   ) where
 
 import Control.DeepSeq (NFData, rnf)
+#if !MIN_VERSION_containers(0,5,0)
 import Control.Seq (using, seqFoldable, rdeepseq)
+#endif
 import Data.Char
 import Data.Ratio
 import qualified Data.Map as Map
 import qualified Data.Sequence as Seq
+import System.Time (ClockTime(..))
 import Text.JSON
-
 import Ganeti.Constants as C
+import Ganeti.Objects (ConfigData)
 import Ganeti.THH
-import Ganeti.Utils (getCurrentTime)
+import Ganeti.Utils (getCurrentTimeUSec)
 
 -- | The possible classes a data collector can belong to.
 data DCCategory = DCInstance | DCStorage | DCDaemon | DCHypervisor
-  deriving (Show, Eq, Read)
+  deriving (Show, Eq, Read, Enum, Bounded)
 
 -- | Get the category name and return it as a string.
 getCategoryName :: DCCategory -> String
@@ -71,7 +77,7 @@ getCategoryName dcc = map toLower . drop 2 . show $ dcc
 
 categoryNames :: Map.Map String DCCategory
 categoryNames =
-  let l = [DCInstance, DCStorage, DCDaemon, DCHypervisor]
+  let l = [minBound ..]
   in Map.fromList $ zip (map getCategoryName l) l
 
 -- | The JSON instance for DCCategory.
@@ -81,8 +87,8 @@ instance JSON DCCategory where
     let s' = fromJSString s
     in case Map.lookup s' categoryNames of
          Just category -> Ok category
-         Nothing -> fail $ "Invalid category name " ++ s' ++ " for type\
-                           \ DCCategory"
+         Nothing -> fail $ "Invalid category name " ++ s' ++ " for type"
+                           ++ " DCCategory"
   readJSON v = fail $ "Invalid JSON value " ++ show v ++ " for type DCCategory"
 
 -- | The possible status codes of a data collector.
@@ -138,26 +144,35 @@ instance JSON DCVersion where
   readJSON v = fail $ "Invalid JSON value " ++ show v ++ " for type DCVersion"
 
 -- | Type for the value field of the `CollectorMap` below.
-data CollectorData = CPULoadData (Seq.Seq (Integer, [Int]))
+data CollectorData =
+  CPULoadData (Seq.Seq (ClockTime, [Int]))
+  | InstanceCpuLoad (Map.Map String (Seq.Seq (ClockTime, Double)))
 
-{-
+instance NFData ClockTime where
+  rnf (TOD x y) = rnf x `seq` rnf y
 
-Naturally, we want to make CollectorData an instance of NFData as
-follows.
+#if MIN_VERSION_containers(0,5,0)
 
 instance NFData CollectorData where
   rnf (CPULoadData x) = rnf x
+  rnf (InstanceCpuLoad x) = rnf x
 
-However, Seq.Seq only became an instance of NFData in version 0.5.0.0
-of containers (Released 2012). So, for the moment, we use a generic
-way to reduce to normal form. In later versions of Ganeti, where we
-have the infra structure to do so, we will choose implementation depending
-on the version of the containers library available.
+#else
+
+{-
+
+In older versions of the containers library, Seq is not an
+instance of NFData, so use a generic way to reduce to normal
+form
 
 -}
 
 instance NFData CollectorData where
   rnf (CPULoadData x) =  (x `using` seqFoldable rdeepseq) `seq` ()
+  rnf (InstanceCpuLoad x) = (x `using` seqFoldable (seqFoldable rdeepseq))
+                            `seq` ()
+
+#endif
 
 -- | Type for the map storing the data of the statefull DataCollectors.
 type CollectorMap = Map.Map String CollectorData
@@ -202,8 +217,30 @@ mergeStatuses (newStat, newStr) (storedStat, storedStrs) =
 buildReport :: String -> DCVersion -> Int -> Maybe DCCategory -> DCKind
             -> JSValue -> IO DCReport
 buildReport name version format_version category kind jsonData = do
-  now <- getCurrentTime
-  let timestamp = now * 1000000000 :: Integer
+  usecs <- getCurrentTimeUSec
+  let timestamp = usecs * 1000 :: Integer
   return $
     DCReport name version format_version timestamp category kind
       jsonData
+
+-- | A report of a data collector might be stateful or stateless.
+data ReportBuilder = StatelessR (IO DCReport)
+                   | StatefulR (Maybe CollectorData -> IO DCReport)
+
+type Name = String
+
+-- | Type describing a data collector basic information
+data DataCollector = DataCollector
+  { dName     :: Name           -- ^ Name of the data collector
+  , dCategory :: Maybe DCCategory -- ^ Category (storage, instance, ecc)
+                                 --   of the collector
+  , dKind     :: DCKind         -- ^ Kind (performance or status reporting) of
+                                 --   the data collector
+  , dReport   :: ReportBuilder  -- ^ Report produced by the collector
+  , dUpdate   :: Maybe (Maybe CollectorData -> IO CollectorData)
+                                 -- ^ Update operation for stateful collectors.
+  , dActive   :: Name -> ConfigData -> Bool
+                    -- ^ Checks if the collector applies for the cluster.
+  , dInterval :: Name -> ConfigData -> Integer
+                    -- ^ Interval between collection in microseconds
+  }
