@@ -52,6 +52,8 @@ import Text.JSON (JSObject, JSValue(JSArray),
 
 import Ganeti.BasicTypes
 import qualified Ganeti.HTools.Cluster as Cluster
+import qualified Ganeti.HTools.Cluster.AllocationSolution as AllocSol
+import qualified Ganeti.HTools.Cluster.AllocateSecondary as AllocSecondary
 import qualified Ganeti.HTools.Cluster.Evacuate as Evacuate
 import qualified Ganeti.HTools.Container as Container
 import qualified Ganeti.HTools.Group as Group
@@ -59,11 +61,13 @@ import qualified Ganeti.HTools.Node as Node
 import qualified Ganeti.HTools.Instance as Instance
 import qualified Ganeti.HTools.Nic as Nic
 import qualified Ganeti.Constants as C
-import Ganeti.HTools.AlgorithmParams (AlgorithmOptions)
+import Ganeti.HTools.AlgorithmParams (AlgorithmOptions(algRestrictToNodes))
 import Ganeti.HTools.CLI
 import Ganeti.HTools.Loader
 import Ganeti.HTools.Types
-import Ganeti.JSON
+import Ganeti.JSON (maybeFromObj, JSRecord, tryFromObj, toArray, asObjectList,
+                    readEitherString, fromJResult, fromObj, fromObjWithDefault,
+                    asJSObject)
 import Ganeti.Types ( EvacMode(ChangePrimary, ChangeSecondary)
                     , adminStateFromRaw, AdminState(..))
 import Ganeti.Utils
@@ -201,8 +205,9 @@ parseGroup u a = do
 -- value.
 parseData :: ClockTime -- ^ The current time
           -> String -- ^ The JSON message as received from Ganeti
+          -> Int -- ^ Static node memory size, see optStaticKvmNodeMemory
           -> Result ([String], Request) -- ^ Result tuple
-parseData now body = do
+parseData now body static_n_mem = do
   decoded <- fromJResult "Parsing input IAllocator message" (decodeStrict body)
   let obj = fromJSObject decoded
       extrObj x = tryFromObj "invalid iallocator message" obj x
@@ -227,8 +232,25 @@ parseData now body = do
   let (kti, il) = assignIndices iobj
   -- cluster tags
   ctags <- extrObj "cluster_tags"
-  cdata1 <- mergeData [] [] [] [] now (ClusterData gl nl il ctags defIPolicy)
-  let (msgs, fix_nl) = checkData (cdNodes cdata1) (cdInstances cdata1)
+  let ex_tags = extractExTags ctags
+      dsrd_loc_tags = extractDesiredLocations ctags
+      updateTags = updateExclTags ex_tags .
+                   updateDesiredLocationTags dsrd_loc_tags
+  -- hypervisor
+  enabled_hypervisors <- extrObj "enabled_hypervisors"
+  -- This is ugly, unfortunately the  IAllocator "API" (which is a CLI not an
+  -- API to be precise) is different from other htools backends that receive
+  -- the 'default_hypervisor' field. Normally there is no reason to have
+  -- more than one hypervisor so in practice this should be fine. Ganeti 2.17
+  -- exports the hv_stat per node field with the missing information.
+  -- TODO: Remove, once the caller can get the correct nodes size on KVM.
+  let nl2 = case enabled_hypervisors of
+              hv : _ -> Container.map (`Node.setHypervisor` hv) nl
+              _ -> nl
+  cdata1 <- mergeData [] [] [] [] now (ClusterData gl nl2 il ctags defIPolicy)
+  let (msgs, fix_nl) = updateMissing (cdNodes cdata1)
+                                     (cdInstances cdata1)
+                                     static_n_mem
       cdata = cdata1 { cdNodes = fix_nl }
       map_n = cdNodes cdata
       map_i = cdInstances cdata
@@ -240,10 +262,12 @@ parseData now body = do
             do
               rname     <- extrReq "name"
               rgn       <- maybeFromObj request "group_name"
+              rest_nodes <- maybeFromObj request "restrict-to-nodes"
               req_nodes <- extrReq "required_nodes"
               inew      <- parseBaseInstance rname request
-              let io = updateExclTags (extractExTags ctags) $ snd inew
+              let io = updateTags $ snd inew
               return $ Allocate io (Cluster.AllocDetails req_nodes rgn)
+                                rest_nodes
         | optype == C.iallocatorModeReloc ->
             do
               rname     <- extrReq "name"
@@ -278,11 +302,15 @@ parseData now body = do
                                  rgn       <- maybeFromObj request "group_name"
                                  req_nodes <- extrFromReq r "required_nodes"
                                  inew      <- parseBaseInstance rname r
-                                 let io = updateExclTags (extractExTags ctags)
-                                            $ snd inew
+                                 let io = updateTags $ snd inew
                                  return (io, Cluster.AllocDetails
                                                req_nodes rgn))
               return $ MultiAllocate prqs
+        | optype == C.iallocatorModeAllocateSecondary ->
+            do
+              rname <- extrReq "name"
+              ridx  <- lookupInstance kti rname
+              return $ AllocateSecondary ridx
         | otherwise -> fail ("Invalid request type '" ++ optype ++ "'")
   return (msgs, Request rqtype cdata)
 
@@ -298,31 +326,45 @@ formatResponse success info result =
   in encodeStrict $ makeObj [e_success, e_info, e_result]
 
 -- | Flatten the log of a solution into a string.
-describeSolution :: Cluster.GenericAllocSolution a -> String
-describeSolution = intercalate ", " . Cluster.asLog
+describeSolution :: AllocSol.GenericAllocSolution a -> String
+describeSolution = intercalate ", " . AllocSol.asLog
 
 -- | Convert allocation/relocation results into the result format.
 formatAllocate :: Instance.List
-               -> Cluster.GenericAllocSolution a
+               -> AllocSol.GenericAllocSolution a
                -> Result IAllocResult
 formatAllocate il as = do
   let info = describeSolution as
-  case Cluster.asSolution as of
+  case AllocSol.asSolution as of
     Nothing -> fail info
     Just (nl, inst, nodes, _) ->
       do
         let il' = Container.add (Instance.idx inst) inst il
         return (info, showJSON $ map Node.name nodes, nl, il')
 
+-- | Convert allocation/relocation results into the result format.
+formatAllocateSecondary :: Instance.List
+                        -> AllocSol.GenericAllocSolution a
+                        -> Result IAllocResult
+formatAllocateSecondary il as = do
+  let info = describeSolution as
+  case AllocSol.asSolution as of
+    Nothing -> fail info
+    Just (nl, inst, [_, snode], _) ->
+      do
+        let il' = Container.add (Instance.idx inst) inst il
+        return (info, showJSON $ Node.name snode, nl, il')
+    _ -> fail $ "Internal error (not a DRBD allocation); info was: " ++ info
+
 -- | Convert multi allocation results into the result format.
 formatMultiAlloc :: ( Node.List, Instance.List
                     , Cluster.GenericAllocSolutionList a)
                  -> Result IAllocResult
 formatMultiAlloc (fin_nl, fin_il, ars) =
   let rars = reverse ars
-      (allocated, failed) = partition (isJust . Cluster.asSolution . snd) rars
+      (allocated, failed) = partition (isJust . AllocSol.asSolution . snd) rars
       aars = map (\(_, ar) ->
-                     let (_, inst, nodes, _) = fromJust $ Cluster.asSolution ar
+                     let (_, inst, nodes, _) = fromJust $ AllocSol.asSolution ar
                          iname = Instance.name inst
                          nnames = map Node.name nodes
                      in (iname, nnames)) allocated
@@ -434,10 +476,15 @@ processRequest :: AlgorithmOptions -> Request -> Result IAllocResult
 processRequest opts request =
   let Request rqtype (ClusterData gl nl il _ _) = request
   in case rqtype of
-       Allocate xi (Cluster.AllocDetails reqn Nothing) ->
-         Cluster.tryMGAlloc opts gl nl il xi reqn >>= formatAllocate il
-       Allocate xi (Cluster.AllocDetails reqn (Just gn)) ->
-         Cluster.tryGroupAlloc opts gl nl il gn xi reqn >>= formatAllocate il
+       Allocate xi (Cluster.AllocDetails reqn Nothing) rest_nodes ->
+         let opts' = opts { algRestrictToNodes = algRestrictToNodes opts
+                                                 `mplus` rest_nodes }
+         in Cluster.tryMGAlloc opts' gl nl il xi reqn >>= formatAllocate il
+       Allocate xi (Cluster.AllocDetails reqn (Just gn)) rest_nodes ->
+         let opts' = opts { algRestrictToNodes = algRestrictToNodes opts
+                                                 `mplus` rest_nodes }
+         in Cluster.tryGroupAlloc opts' gl nl il gn xi reqn
+            >>= formatAllocate il
        Relocate idx reqn exnodes ->
          processRelocate opts gl nl il idx reqn exnodes >>= formatRelocate
        ChangeGroup gdxs idxs ->
@@ -448,15 +495,20 @@ processRequest opts request =
                 formatNodeEvac gl nl il
        MultiAllocate xies ->
          Cluster.allocList opts gl nl il xies [] >>= formatMultiAlloc
+       AllocateSecondary xi ->
+         AllocSecondary.tryAllocateSecondary opts gl nl il xi
+           >>= formatAllocateSecondary il
 
 -- | Reads the request from the data file(s).
-readRequest :: FilePath -> IO Request
-readRequest fp = do
+readRequest :: FilePath -- ^ Path to IAllocator input file
+            -> Int -- ^ Static node memory size, see optStaticKvmNodeMemory
+            -> IO Request
+readRequest fp static_n_mem = do
   now <- getClockTime
   input_data <- case fp of
                   "-" -> getContents
                   _   -> readFile fp
-  case parseData now input_data of
+  case parseData now input_data static_n_mem of
     Bad err -> exitErr err
     Ok (fix_msgs, rq) -> maybeShowWarnings fix_msgs >> return rq
 
@@ -478,8 +530,9 @@ runIAllocator :: AlgorithmOptions
 runIAllocator opts request = formatIAllocResult $ processRequest opts request
 
 -- | Load the data from an iallocation request file
-loadData :: FilePath -- ^ The path to the file
+loadData :: FilePath -- ^ Path to IAllocator input file
+         -> Int -- ^ Static node memory size, see optStaticKvmNodeMemory
          -> IO (Result ClusterData)
-loadData fp = do
-  Request _ cdata <- readRequest fp
+loadData fp static_n_mem = do
+  Request _ cdata <- readRequest fp static_n_mem
   return $ Ok cdata
