@@ -56,6 +56,7 @@ module Ganeti.HTools.CLI
   , oDataFile
   , oDiskMoves
   , oDiskTemplate
+  , oDryRun
   , oSpindleUse
   , oDynuFile
   , oMonD
@@ -104,12 +105,14 @@ module Ganeti.HTools.CLI
   , oQuiet
   , oRapiMaster
   , oReason
+  , oRestrictToNodes
   , oSaveCluster
   , oSelInst
   , oShowHelp
   , oShowVer
   , oShowComp
   , oSkipNonRedundant
+  , oStaticKvmNodeMemory
   , oStdSpec
   , oTargetResources
   , oTieredSpec
@@ -161,6 +164,7 @@ data Options = Options
   , optExInst      :: [String]       -- ^ Instances to be excluded
   , optExTags      :: Maybe [String] -- ^ Tags to use for exclusion
   , optExecJobs    :: Bool           -- ^ Execute the commands via Luxi
+  , optDryRun      :: Bool           -- ^ Only do a dry run
   , optFirstJobGroup :: Bool         -- ^ Only execute the first group of jobs
   , optForce       :: Bool           -- ^ Force the execution
   , optFullEvacuation :: Bool        -- ^ Fully evacuate nodes to be rebooted
@@ -182,8 +186,10 @@ data Options = Options
   , optNoHeaders   :: Bool           -- ^ Do not show a header line
   , optNoSimulation :: Bool          -- ^ Skip the rebalancing dry-run
   , optNodeSim     :: [String]       -- ^ Cluster simulation mode
-  , optNodeTags    :: Maybe [String] -- ^ List of node tags to restrict to 
+  , optNodeTags    :: Maybe [String] -- ^ List of node tags to restrict to
   , optOffline     :: [String]       -- ^ Names of offline nodes
+  , optRestrictToNodes :: Maybe [String] -- ^ if not Nothing, restrict
+                                     -- allocation to those nodes
   , optOfflineMaintenance :: Bool    -- ^ Pretend all instances are offline
   , optOneStepOnly :: Bool           -- ^ Only do the first step
   , optOutPath     :: FilePath       -- ^ Path to the output directory
@@ -198,6 +204,8 @@ data Options = Options
   , optShowNodes   :: Maybe [String] -- ^ Whether to show node status
   , optShowVer     :: Bool           -- ^ Just show the program version
   , optSkipNonRedundant :: Bool      -- ^ Skip nodes with non-redundant instance
+  , optStaticKvmNodeMemory :: Int    -- ^ Use static value for node memory
+                                     -- ^ on KVM
   , optStdSpec     :: Maybe RSpec    -- ^ Requested standard specs
   , optTargetResources :: Double     -- ^ Target resources for squeezing
   , optTestCount   :: Maybe Int      -- ^ Optional test count override
@@ -230,6 +238,7 @@ defaultOptions  = Options
   , optExInst      = []
   , optExTags      = Nothing
   , optExecJobs    = False
+  , optDryRun      = False
   , optFirstJobGroup = False
   , optForce       = False
   , optFullEvacuation = False
@@ -253,7 +262,9 @@ defaultOptions  = Options
   , optNodeSim     = []
   , optNodeTags    = Nothing
   , optSkipNonRedundant = False
+  , optStaticKvmNodeMemory = 4096
   , optOffline     = []
+  , optRestrictToNodes = Nothing
   , optOfflineMaintenance = False
   , optOneStepOnly = False
   , optOutPath     = "."
@@ -472,6 +483,13 @@ oExecJobs =
    \ it for data gathering)",
    OptComplNone)
 
+oDryRun :: OptType
+oDryRun =
+  (Option "" ["dry-run"]
+   (NoArg (\ opts -> Ok opts { optDryRun = True}))
+   "do not execute any commands and just report what would be done",
+   OptComplNone)
+
 oReason :: OptType
 oReason =
   (Option "" ["reason"]
@@ -605,7 +623,7 @@ oMinResources =
   (Option "" ["minimal-resources"]
    (reqWithConversion (tryRead "minimal resources")
     (\d opts -> Ok opts { optMinResources = d}) "FACTOR")
-   "minimal resources to be present on each in multiples of\ 
+   "minimal resources to be present on each in multiples of\
    \ the standard allocation for not onlining standby nodes",
    OptComplFloat)
 
@@ -645,7 +663,7 @@ oNodeTags =
    (ReqArg (\ f opts -> Ok opts { optNodeTags = Just $ sepSplit ',' f })
     "TAG,...") "Restrict to nodes with the given tags",
    OptComplString)
-     
+
 oOfflineMaintenance :: OptType
 oOfflineMaintenance =
   (Option "" ["offline-maintenance"]
@@ -661,6 +679,13 @@ oOfflineNode =
    "set node as offline",
    OptComplOneNode)
 
+oRestrictToNodes :: OptType
+oRestrictToNodes =
+  (Option "" ["restrict-allocation-to"]
+    (ReqArg (\ ns o -> Ok o { optRestrictToNodes = Just $ sepSplit ',' ns })
+     "NODE,...") "Restrict allocations to the given set of nodes",
+   OptComplManyNodes)
+
 oOneStepOnly :: OptType
 oOneStepOnly =
   (Option "" ["one-step-only"]
@@ -741,6 +766,15 @@ oSkipNonRedundant =
     "Skip nodes that host a non-redundant instance",
     OptComplNone)
 
+oStaticKvmNodeMemory :: OptType
+oStaticKvmNodeMemory =
+  (Option "" ["static-kvm-node-memory"]
+   (reqWithConversion (tryRead "static node memory")
+    (\i opts -> Ok opts { optStaticKvmNodeMemory = i }) "N")
+   "use static node memory [in MB] on KVM instead of value reported by \
+   \hypervisor. Use 0 to take value reported from hypervisor.",
+   OptComplInteger)
+
 oStdSpec :: OptType
 oStdSpec =
   (Option "" ["standard-alloc"]
