@@ -38,7 +38,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 module Ganeti.HTools.Loader
   ( mergeData
   , clearDynU
-  , checkData
+  , updateMissing
+  , updateMemStat
   , assignIndices
   , setMaster
   , lookupNode
@@ -53,12 +54,16 @@ module Ganeti.HTools.Loader
   , ClusterData(..)
   , isAllocationRequest
   , emptyCluster
+  , extractDesiredLocations
+  , updateDesiredLocationTags
   ) where
 
 import Control.Monad
+import Control.Monad.Fail (MonadFail)
 import Data.List
 import qualified Data.Map as M
 import Data.Maybe
+import qualified Data.Set as Set
 import Text.Printf (printf)
 import System.Time (ClockTime(..))
 
@@ -71,9 +76,10 @@ import qualified Ganeti.HTools.Cluster.Moves as Moves
 
 import Ganeti.BasicTypes
 import qualified Ganeti.HTools.Tags as Tags
+import qualified Ganeti.HTools.Tags.Constants as TagsC
 import Ganeti.HTools.Types
 import Ganeti.Utils
-import Ganeti.Types (EvacMode)
+import Ganeti.Types (EvacMode, Hypervisor(..))
 
 -- * Types
 
@@ -84,8 +90,11 @@ request-specific fields.
 
 -}
 data RqType
-  = Allocate Instance.Instance Cluster.AllocDetails   -- ^ A new instance
-                                                      --   allocation
+  = Allocate Instance.Instance Cluster.AllocDetails (Maybe [String])
+    -- ^ A new instance allocation, maybe with allocation restrictions
+  | AllocateSecondary Idx                             -- ^ Find a suitable
+                                                      -- secondary node for disk
+                                                      -- conversion
   | Relocate Idx Int [Ndx]                            -- ^ Choose a new
                                                       --   secondary node
   | NodeEvacuate [Idx] EvacMode                       -- ^ node-evacuate mode
@@ -105,7 +114,7 @@ data Request = Request RqType ClusterData
 -- group specified, and return `Just (Just g)` if it is an allocation request
 -- uniquely requesting Group `g`.
 isAllocationRequest :: RqType -> Maybe (Maybe String)
-isAllocationRequest (Allocate _ (Cluster.AllocDetails _ grp)) = Just grp
+isAllocationRequest (Allocate _ (Cluster.AllocDetails _ grp) _) = Just grp
 isAllocationRequest (MultiAllocate reqs) = Just $
   case ordNub . catMaybes
        $ map (\(_, Cluster.AllocDetails _ grp) -> grp) reqs of
@@ -130,18 +139,18 @@ emptyCluster = ClusterData Container.empty Container.empty Container.empty []
 -- * Functions
 
 -- | Lookups a node into an assoc list.
-lookupNode :: (Monad m) => NameAssoc -> String -> String -> m Ndx
+lookupNode :: (MonadFail m) => NameAssoc -> String -> String -> m Ndx
 lookupNode ktn inst node =
   maybe (fail $ "Unknown node '" ++ node ++ "' for instance " ++ inst) return $
     M.lookup node ktn
 
 -- | Lookups an instance into an assoc list.
-lookupInstance :: (Monad m) => NameAssoc -> String -> m Idx
+lookupInstance :: (MonadFail m) => NameAssoc -> String -> m Idx
 lookupInstance kti inst =
   maybe (fail $ "Unknown instance '" ++ inst ++ "'") return $ M.lookup inst kti
 
 -- | Lookups a group into an assoc list.
-lookupGroup :: (Monad m) => NameAssoc -> String -> String -> m Gdx
+lookupGroup :: (MonadFail m) => NameAssoc -> String -> String -> m Gdx
 lookupGroup ktg nname gname =
   maybe (fail $ "Unknown group '" ++ gname ++ "' for node " ++ nname) return $
     M.lookup gname ktg
@@ -157,7 +166,7 @@ assignIndices name_element =
   in (M.fromList name_idx, Container.fromList idx_element)
 
 -- | Given am indexed node list, and the name of the master, mark it as such.
-setMaster :: (Monad m) => NameAssoc -> Node.List -> String -> m Node.List
+setMaster :: (MonadFail m) => NameAssoc -> Node.List -> String -> m Node.List
 setMaster node_names node_idx master = do
   kmaster <- maybe (fail $ "Master node " ++ master ++ " unknown") return $
              M.lookup master node_names
@@ -169,7 +178,7 @@ setMaster node_names node_idx master = do
 setLocationScore :: Node.List -> Instance.Instance -> Instance.Instance
 setLocationScore nl inst =
   let pnode = Container.find (Instance.pNode inst) nl
-      snode = Container.find (Instance.sNode inst) nl
+      snode = Container.lookup (Instance.sNode inst) nl
   in Moves.setInstanceLocationScore inst pnode snode
 
 -- | For each instance, add its index to its primary and secondary nodes.
@@ -204,6 +213,14 @@ updateExclTags tl inst =
       exclTags = filter (\tag -> any (`isPrefixOf` tag) tl) allTags
   in inst { Instance.exclTags = exclTags }
 
+-- | Update instance with desired location tags list.
+updateDesiredLocationTags :: [String] -> Instance.Instance -> Instance.Instance
+updateDesiredLocationTags tl inst =
+  let allTags = Instance.allTags inst
+      dsrdLocTags = filter (\tag -> any (`isPrefixOf` tag) tl) allTags
+  in inst { Instance.dsrdLocTags = Set.fromList dsrdLocTags }
+
+
 -- | Update the movable attribute.
 updateMovable :: [String]           -- ^ Selected instances (if not empty)
               -> [String]           -- ^ Excluded instances
@@ -249,8 +266,8 @@ setArPolicy ctags gl nl il time =
 getArPolicy :: [String] -> ClockTime -> Maybe AutoRepairPolicy
 getArPolicy tags time =
   let enabled = mapMaybe (autoRepairTypeFromRaw <=<
-                          chompPrefix Tags.autoRepairTagEnabled) tags
-      suspended = mapMaybe (chompPrefix Tags.autoRepairTagSuspended) tags
+                          chompPrefix TagsC.autoRepairTagEnabled) tags
+      suspended = mapMaybe (chompPrefix TagsC.autoRepairTagSuspended) tags
       futureTs = filter (> time) . map (flip TOD 0) $
                    mapMaybe (tryRead "auto-repair suspend time") suspended
   in
@@ -275,7 +292,12 @@ longestDomain (x:xs) =
 
 -- | Extracts the exclusion tags from the cluster configuration.
 extractExTags :: [String] -> [String]
-extractExTags = filter (not . null) . mapMaybe (chompPrefix Tags.exTagsPrefix)
+extractExTags = filter (not . null) . mapMaybe (chompPrefix TagsC.exTagsPrefix)
+
+-- | Extracts the desired locations from the instance tags.
+extractDesiredLocations :: [String] -> [String]
+extractDesiredLocations =
+  filter (not . null) . mapMaybe (chompPrefix TagsC.desiredLocationPrefix)
 
 -- | Extracts the common suffix from node\/instance names.
 commonSuffix :: Node.List -> Instance.List -> String
@@ -321,6 +343,7 @@ mergeData um extags selinsts exinsts time cdata@(ClusterData gl nl il ctags _) =
                               in Container.add (Instance.idx inst) new_i im
                    ) il2 um
       allextags = extags ++ extractExTags ctags
+      dsrdLocTags = extractDesiredLocations ctags
       inst_names = map Instance.name $ Container.elems il3
       selinst_lkp = map (lookupName inst_names) selinsts
       exinst_lkp = map (lookupName inst_names) exinsts
@@ -331,6 +354,7 @@ mergeData um extags selinsts exinsts time cdata@(ClusterData gl nl il ctags _) =
       common_suffix = longestDomain (node_names ++ inst_names)
       il4 = Container.map (computeAlias common_suffix .
                            updateExclTags allextags .
+                           updateDesiredLocationTags dsrdLocTags .
                            updateMovable selinst_names exinst_names) il3
       nl2 = Container.map (addLocationTags ctags) nl
       il5 = Container.map (setLocationScore nl2) il4
@@ -350,29 +374,49 @@ clearDynU cdata@(ClusterData _ _ il _ _) =
   let il2 = Container.map (\ inst -> inst {Instance.util = zeroUtil }) il
   in Ok cdata { cdInstances = il2 }
 
--- | Checks the cluster data for consistency.
-checkData :: Node.List -> Instance.List
-          -> ([String], Node.List)
-checkData nl il =
-    Container.mapAccum
-        (\ msgs node ->
-             let nname = Node.name node
-                 delta_mem = truncate (Node.tMem node)
-                             - Node.nMem node
-                             - Node.fMem node
-                             - nodeImem node il
-                 delta_dsk = truncate (Node.tDsk node)
-                             - Node.fDsk node
-                             - nodeIdsk node il
-                 newn = node `Node.setXmem` delta_mem
-                 umsg1 =
-                   if delta_mem > 512 || delta_dsk > 1024
-                      then printf "node %s is missing %d MB ram \
-                                  \and %d GB disk"
-                                  nname delta_mem (delta_dsk `div` 1024):msgs
-                      else msgs
-             in (umsg1, newn)
-        ) [] nl
+-- | Update cluster data to use static node memory on KVM.
+setStaticKvmNodeMem :: Node.List -- ^ Nodes to update
+                       -> Int -- ^ Static node size
+                       -> Node.List -- ^ Updated nodes
+setStaticKvmNodeMem nl static_node_mem =
+  let updateNM n
+          | Node.hypervisor n == Just Kvm = n { Node.nMem = static_node_mem }
+          | otherwise = n
+  in if static_node_mem > 0
+     then Container.map updateNM nl
+     else nl
+
+-- | Update node memory stat based on instance list.
+updateMemStat :: Node.Node -> Instance.List -> Node.Node
+updateMemStat node il =
+  let node2 = node { Node.iMem = nodeImem node il }
+      node3 = node2 { Node.xMem = Node.missingMem node2 }
+  in node3 { Node.pMem = fromIntegral (Node.unallocatedMem node3)
+                         / Node.tMem node3 }
+
+-- | Check the cluster for memory/disk allocation consistency and update stats.
+updateMissing :: Node.List -- ^ All nodes in the cluster
+              -> Instance.List  -- ^ All instances in the cluster
+              -> Int -- ^ Static node memory for KVM
+              -> ([String], Node.List) -- ^ Pair of errors, update node list
+updateMissing nl il static_node_mem =
+    -- This overrides node mem on KVM as loaded from backend. Ganeti 2.17
+    -- handles this using obtainNodeMemory.
+    let nl2 = setStaticKvmNodeMem nl static_node_mem
+        updateSingle msgs node =
+            let nname = Node.name node
+                newn = updateMemStat node il
+                delta_mem = Node.xMem newn
+                delta_dsk = truncate (Node.tDsk node)
+                            - Node.fDsk node
+                            - nodeIdsk node il
+                umsg1 = if delta_mem > 512 || delta_dsk > 1024
+                        then printf
+                             "node %s is missing %d MB ram and %d GB disk"
+                             nname delta_mem (delta_dsk `div` 1024):msgs
+                        else msgs
+            in (umsg1, newn)
+    in Container.mapAccum updateSingle [] nl2
 
 -- | Compute the amount of memory used by primary instances on a node.
 nodeImem :: Node.Node -> Instance.List -> Int
@@ -392,6 +436,6 @@ nodeIdsk node il =
        $ Node.pList node ++ Node.sList node
 
 -- | Get live information or a default value
-eitherLive :: (Monad m) => Bool -> a -> m a -> m a
+eitherLive :: (MonadFail m) => Bool -> a -> m a -> m a
 eitherLive True _ live_data = live_data
 eitherLive False def_data _ = return def_data
