@@ -73,6 +73,7 @@ module Ganeti.JQueue
     , cancelJob
     , tellJobPriority
     , notifyJob
+    , waitUntilJobExited
     , queueDirPermissions
     , archiveJobs
     -- re-export
@@ -82,16 +83,17 @@ module Ganeti.JQueue
     , QueuedJob(..)
     ) where
 
-import Control.Applicative (liftA2, (<|>), (<$>))
+import Control.Applicative (liftA2, (<|>))
 import Control.Arrow (first, second)
 import Control.Concurrent (forkIO, threadDelay)
 import Control.Exception
 import Control.Lens (over)
 import Control.Monad
+import Control.Monad.Fail (MonadFail)
+import Control.Monad.Fix
 import Control.Monad.IO.Class
 import Control.Monad.Trans (lift)
 import Control.Monad.Trans.Maybe
-import Data.Functor ((<$))
 import Data.List
 import Data.Maybe
 import Data.Ord (comparing)
@@ -189,7 +191,7 @@ queuedOpCodeFromMetaOpCode op =
 -- | From a job-id and a list of op-codes create a job. This is
 -- the pure part of job creation, as allocating a new job id
 -- lives in IO.
-queuedJobFromOpCodes :: (Monad m) => JobId -> [MetaOpCode] -> m QueuedJob
+queuedJobFromOpCodes :: (MonadFail m) => JobId -> [MetaOpCode] -> m QueuedJob
 queuedJobFromOpCodes jobid ops = do
   ops' <- mapM (`resolveDependencies` jobid) ops
   return QueuedJob { qjId = jobid
@@ -307,7 +309,7 @@ jobFileName :: JobId -> FilePath
 jobFileName jid = jobFilePrefix ++ show (fromJobId jid)
 
 -- | Parses a job ID from a file name.
-parseJobFileId :: (Monad m) => FilePath -> m JobId
+parseJobFileId :: (MonadFail m) => FilePath -> m JobId
 parseJobFileId path =
   case stripPrefix jobFilePrefix path of
     Nothing -> fail $ "Job file '" ++ path ++
@@ -563,6 +565,34 @@ isQueuedJobDead ownlivelock =
   . mfilter (/= ownlivelock)
   . qjLivelock
 
+-- | Waits for a job's process to exit
+waitUntilJobExited :: Livelock -- ^ LuxiD's own livelock
+                   -> QueuedJob -- ^ the job to wait for
+                   -> Int -- ^ timeout in milliseconds
+                   -> ResultG (Bool, String)
+waitUntilJobExited ownlivelock job tmout = do
+  let sleepDelay = 100 :: Int -- ms
+      jName = ("Job " ++) . show . fromJobId . qjId $ job
+  logDebug $ "Waiting for " ++ jName ++ " to exit"
+
+  start <- liftIO getClockTime
+  let tmoutPicosec :: Integer
+      tmoutPicosec = fromIntegral $ tmout * 1000 * 1000 * 1000
+      wait = noTimeDiff { tdPicosec = tmoutPicosec }
+      deadline = addToClockTime wait start
+
+  liftIO . fix $ \loop -> do
+    -- fail if the job is in the startup phase or has no livelock
+    dead <- maybe (fail $ jName ++ " has not yet started up")
+      (liftIO . isDead) . mfilter (/= ownlivelock) . qjLivelock $ job
+    curtime <- getClockTime
+    let elapsed = timeDiffToString $ System.Time.diffClockTimes
+                  curtime start
+    case dead of
+      True -> return (True, jName ++ " process exited after " ++ elapsed)
+      _ | curtime < deadline -> threadDelay (sleepDelay * 1000) >> loop
+      _ -> fail $ jName ++ " still running after " ++ elapsed
+
 -- | Waits for a job ordered to cancel to react, and returns whether it was
 -- canceled, and a user-intended description of the reason.
 waitForJobCancelation :: JobId -> Int -> ResultG (Bool, String)
