@@ -86,6 +86,7 @@ module Ganeti.JSON
 import Control.Applicative
 import Control.DeepSeq
 import Control.Monad.Error.Class
+import Control.Monad.Fail (MonadFail)
 import Control.Monad.Writer
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.UTF8 as UTF8
@@ -104,10 +105,11 @@ import qualified Text.JSON.Types as JT
 import Text.JSON.Pretty (pp_value)
 
 -- Note: this module should not import any Ganeti-specific modules
--- beside BasicTypes, since it's used in THH which is used itself to
--- build many other modules.
+-- beside BasicTypes and Compact, since it's used in THH which is used
+-- itself to build many other modules.
 
 import Ganeti.BasicTypes
+import Ganeti.Compat
 
 -- Remove after we require >= 1.8.58
 -- See: https://github.com/ndmitchell/hlint/issues/24
@@ -144,7 +146,7 @@ readJSONWithDesc name input =
     J.Error e -> J.Error $ "Can't parse value for '" ++ name ++ "': " ++ e
 
 -- | Converts a JSON Result into a monadic value.
-fromJResult :: Monad m => String -> J.Result a -> m a
+fromJResult :: (Monad m, MonadFail m) => String -> J.Result a -> m a
 fromJResult s (J.Error x) = fail (s ++ ": " ++ x)
 fromJResult _ (J.Ok x) = return x
 
@@ -157,14 +159,14 @@ fromJResultE _ (J.Ok x) = return x
 --
 -- In case the value was not a string, we fail the read (in the
 -- context of the current monad.
-readEitherString :: (Monad m) => J.JSValue -> m String
+readEitherString :: (MonadFail m) => J.JSValue -> m String
 readEitherString v =
   case v of
     J.JSString s -> return $ J.fromJSString s
     _ -> fail "Wrong JSON type"
 
 -- | Converts a JSON message into an array of JSON objects.
-loadJSArray :: (Monad m)
+loadJSArray :: MonadFail m
                => String -- ^ Operation description (for error reporting)
                -> String -- ^ Input message
                -> m [J.JSObject J.JSValue]
@@ -176,7 +178,7 @@ buildNoKeyError o k =
   printf "key '%s' not found, object contains only %s" k (show (map fst o))
 
 -- | Reads the value of a key in a JSON object.
-fromObj :: (J.JSON a, Monad m) => JSRecord -> String -> m a
+fromObj :: (J.JSON a, MonadFail m) => JSRecord -> String -> m a
 fromObj o k =
   case lookup k o of
     Nothing -> fail $ buildNoKeyError o k
@@ -186,7 +188,7 @@ fromObj o k =
 -- keys, or keys that have a \'null\' value, will be returned as
 -- 'Nothing', otherwise we attempt deserialisation and return a 'Just'
 -- value.
-maybeFromObj :: (J.JSON a, Monad m) =>
+maybeFromObj :: (J.JSON a, MonadFail m) =>
                 JSRecord -> String -> m (Maybe a)
 maybeFromObj o k =
   case lookup k o of
@@ -202,11 +204,12 @@ maybeFromObj o k =
 -- | Reads the value of a key in a JSON object with a default if
 -- missing. Note that both missing keys and keys with value \'null\'
 -- will cause the default value to be returned.
-fromObjWithDefault :: (J.JSON a, Monad m) =>
+fromObjWithDefault :: (J.JSON a, MonadFail m) =>
                       JSRecord -> String -> a -> m a
 fromObjWithDefault o k d = liftM (fromMaybe d) $ maybeFromObj o k
 
-arrayMaybeFromJVal :: (J.JSON a, Monad m) => J.JSValue -> m [Maybe a]
+arrayMaybeFromJVal :: (J.JSON a, Monad m, MonadFail m) =>
+  J.JSValue -> m [Maybe a]
 arrayMaybeFromJVal (J.JSArray xs) =
   mapM parse xs
     where
@@ -216,7 +219,7 @@ arrayMaybeFromJVal v =
   fail $ "Expecting array, got '" ++ show (pp_value v) ++ "'"
 
 -- | Reads an array of optional items
-arrayMaybeFromObj :: (J.JSON a, Monad m) =>
+arrayMaybeFromObj :: (J.JSON a, MonadFail m) =>
                      JSRecord -> String -> m [Maybe a]
 arrayMaybeFromObj o k =
   case lookup k o of
@@ -232,7 +235,7 @@ tryArrayMaybeFromObj :: (J.JSON a)
 tryArrayMaybeFromObj t o = annotateResult t . arrayMaybeFromObj o
 
 -- | Reads a JValue, that originated from an object key.
-fromKeyValue :: (J.JSON a, Monad m)
+fromKeyValue :: (J.JSON a, MonadFail m)
               => String     -- ^ The key name
               -> J.JSValue  -- ^ The value to read
               -> m a
@@ -240,7 +243,7 @@ fromKeyValue k val =
   fromJResult (printf "key '%s'" k) (J.readJSON val)
 
 -- | Small wrapper over readJSON.
-fromJVal :: (Monad m, J.JSON a) => J.JSValue -> m a
+fromJVal :: (Monad m, MonadFail m, J.JSON a) => J.JSValue -> m a
 fromJVal v =
   case J.readJSON v of
     J.Error s -> fail ("Cannot convert value '" ++ show (pp_value v) ++
@@ -275,12 +278,13 @@ getMaybeJsonElem (_:xs) n f
   | otherwise = getMaybeJsonElem xs (n - 1) f
 
 -- | Converts a JSON value into a JSON object.
-asJSObject :: (Monad m) => J.JSValue -> m (J.JSObject J.JSValue)
+asJSObject :: (Monad m, MonadFail m) => J.JSValue -> m (J.JSObject J.JSValue)
 asJSObject (J.JSObject a) = return a
 asJSObject _ = fail "not an object"
 
 -- | Coneverts a list of JSON values into a list of JSON objects.
-asObjectList :: (Monad m) => [J.JSValue] -> m [J.JSObject J.JSValue]
+asObjectList :: (Monad m, MonadFail m) =>
+  [J.JSValue] -> m [J.JSObject J.JSValue]
 asObjectList = mapM asJSObject
 
 -- | Try to extract a key from an object with better error reporting
@@ -293,7 +297,7 @@ tryFromObj :: (J.JSON a) =>
 tryFromObj t o = annotateResult t . fromObj o
 
 -- | Ensure a given JSValue is actually a JSArray.
-toArray :: (Monad m) => J.JSValue -> m [J.JSValue]
+toArray :: (Monad m, MonadFail m) => J.JSValue -> m [J.JSValue]
 toArray (J.JSArray arr) = return arr
 toArray o =
   fail $ "Invalid input, expected array but got " ++ show (pp_value o)
@@ -316,7 +320,7 @@ optFieldsToObj = J.makeObj . catMaybes
 -- useful for custom key types in JSON dictionaries, which have to be
 -- backed by strings.
 class HasStringRepr a where
-  fromStringRepr :: (Monad m) => String -> m a
+  fromStringRepr :: (MonadFail m) => String -> m a
   toStringRepr :: a -> String
 
 -- | Trivial instance 'HasStringRepr' for 'String'.
@@ -368,7 +372,7 @@ alterContainerL key f (GenericContainer m) =
        (f $ Map.lookup key m)
 
 -- | Container loader.
-readContainer :: (Monad m, HasStringRepr a, Ord a, J.JSON b) =>
+readContainer :: (MonadFail m, HasStringRepr a, Ord a, J.JSON b) =>
                  J.JSObject J.JSValue -> m (GenericContainer a b)
 readContainer obj = do
   let kjvlist = J.fromJSObject obj
