@@ -1,4 +1,4 @@
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE TupleSections, TemplateHaskell #-}
 {-| Configuration server for the metadata daemon.
 
 -}
@@ -34,71 +34,34 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 -}
 module Ganeti.Metad.ConfigServer where
 
-import Control.Concurrent
-import Control.Exception (try, finally)
-import Control.Monad (unless)
-import Text.JSON
-import System.IO.Error (isEOFError)
+import Control.Exception (finally)
+import Control.Monad.Reader
 
 import Ganeti.Path as Path
 import Ganeti.Daemon (DaemonOptions, cleanupSocket, describeError)
-import qualified Ganeti.Logging as Logging
 import Ganeti.Runtime (GanetiDaemon(..), GanetiGroup(..), MiscGroup(..))
-import Ganeti.UDSServer (Client, ConnectConfig(..), Server, ServerConfig(..))
+import Ganeti.THH.RPC
+import Ganeti.UDSServer (ConnectConfig(..), ServerConfig(..))
 import qualified Ganeti.UDSServer as UDSServer
 import Ganeti.Utils (FilePermissions(..))
 
-import Ganeti.Metad.Config as Config
-import Ganeti.Metad.Types (InstanceParams)
+import Ganeti.Metad.ConfigCore
 
--- | Update the configuration with the received instance parameters.
-updateConfig :: MVar InstanceParams -> String -> IO ()
-updateConfig config str =
-  case decode str of
-    Error err ->
-      Logging.logDebug $ show err
-    Ok x ->
-      case Config.getInstanceParams x of
-        Error err ->
-          Logging.logError $ "Could not get instance parameters: " ++ err
-        Ok (name, instanceParams) -> do
-          cfg <- takeMVar config
-          let cfg' = mergeConfig cfg instanceParams
-          putMVar config cfg'
-          Logging.logInfo $
-            "Updated instance " ++ show name ++ " configuration"
-          Logging.logDebug $ "Instance configuration: " ++ show cfg'
+-- * The handler that converts RPCs to calls to the above functions
 
--- | Reads messages from clients and update the configuration
--- according to these messages.
-acceptConfig :: MVar InstanceParams -> Client -> IO ()
-acceptConfig config client =
-  do res <- try $ UDSServer.recvMsg client
-     case res of
-       Left err -> do
-         unless (isEOFError err) .
-           Logging.logDebug $ show err
-         return ()
-       Right str -> do
-         Logging.logDebug $ "Received: " ++ str
-         updateConfig config str
+handler :: RpcServer MetadMonadInt
+handler = $( mkRpcM exportedFunctions )
 
--- | Loop that accepts clients and dispatches them to an isolated
--- thread that will handle the client's requests.
-acceptClients :: MVar InstanceParams -> Server -> IO ()
-acceptClients config server =
-  do client <- UDSServer.acceptClient server
-     _ <- forkIO $ acceptConfig config client
-     acceptClients config server
+-- * The main server code
 
-start :: DaemonOptions -> MVar InstanceParams -> IO ()
+start :: DaemonOptions -> MetadHandle -> IO ()
 start _ config = do
      socket_path <- Path.defaultMetadSocket
      cleanupSocket socket_path
      server <- describeError "binding to the socket" Nothing (Just socket_path)
                $ UDSServer.connectServer metadConfig True socket_path
      finally
-       (acceptClients config server)
+       (forever $ runMetadMonadInt (UDSServer.listener handler server) config)
        (UDSServer.closeServer server)
   where
     metadConfig =
