@@ -36,7 +36,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 module Ganeti.Objects.Disk where
 
-import Control.Applicative ((<*>), (<$>))
 import qualified Data.ByteString.UTF8 as UTF8
 import Data.Char (isAsciiLower, isAsciiUpper, isDigit)
 import Data.List (isPrefixOf, isInfixOf)
@@ -81,7 +80,7 @@ instance Show LogicalVolume where
   showsPrec _ (LogicalVolume g v) =
     showString g . showString "/" . showString v
 
--- | Check the constraints for a VG/LV names (except the @\/dev\/@ check).
+-- | Check the constraints for VG\/LV names (except the @\/dev\/@ check).
 instance Validatable LogicalVolume where
   validate (LogicalVolume g v) = do
       let vgn = "Volume group name"
@@ -257,9 +256,16 @@ $(buildObjectWithForthcoming "Disk" "disk" $
   ++ serialFields
   ++ timeStampFields)
 
+instance TimeStampObject Disk where
+  cTimeOf = diskCtime
+  mTimeOf = diskMtime
+
 instance UuidObject Disk where
   uuidOf = UTF8.toString . diskUuid
 
+instance SerialNoObject Disk where
+  serialOf = diskSerial
+
 instance ForthcomingObject Disk where
   isForthcoming = diskForthcoming
 
