@@ -58,7 +58,7 @@ module Ganeti.OpCodes
   , setOpPriority
   ) where
 
-import Control.Applicative
+import Control.Monad.Fail (MonadFail)
 import Data.List (intercalate)
 import Data.Map (Map)
 import qualified Text.JSON
@@ -188,7 +188,8 @@ $(genOpCode "OpCode"
   , ("OpClusterVerifyDisks",
      [t| JobIdListOnly |],
      OpDoc.opClusterVerifyDisks,
-     [],
+     [ pOptGroupName
+     ],
      [])
   , ("OpGroupVerifyDisks",
      [t| (Map String String, [String], Map String [[String]]) |],
@@ -282,7 +283,9 @@ $(genOpCode "OpCode"
      [t| () |],
      OpDoc.opClusterRenewCrypto,
      [ pNodeSslCerts
-     , pSshKeys
+     , pRenewSshKeys
+     , pSshKeyType
+     , pSshKeyBits
      , pVerbose
      , pDebug
      ],
@@ -691,6 +694,7 @@ $(genOpCode "OpCode"
      , withDoc
        "Secondary node UUID (used when changing disk template)"
        pRemoteNodeUuid
+     , pIallocator
      , pOsNameChange
      , pInstOsParams
      , pInstOsParamsPrivate
@@ -905,6 +909,12 @@ $(genOpCode "OpCode"
      , pJQueueFail
      ],
      [])
+  , ("OpTestOsParams",
+     [t| () |],
+     OpDoc.opTestOsParams,
+     [ pInstOsParamsSecret
+     ],
+     [])
   , ("OpTestDummy",
      [t| () |],
      OpDoc.opTestDummy,
@@ -1070,7 +1080,8 @@ defOpParams =
                  }
 
 -- | Resolve relative dependencies to absolute ones, given the job ID.
-resolveDependsCommon :: (Monad m) => CommonOpParams -> JobId -> m CommonOpParams
+resolveDependsCommon :: (MonadFail m) =>
+  CommonOpParams -> JobId -> m CommonOpParams
 resolveDependsCommon p@(CommonOpParams { opDepends = Just deps}) jid = do
   deps' <- mapM (`absoluteJobDependency` jid) deps
   return p { opDepends = Just deps' }
@@ -1082,7 +1093,7 @@ data MetaOpCode = MetaOpCode { metaParams :: CommonOpParams
                              } deriving (Show, Eq, Ord)
 
 -- | Resolve relative dependencies to absolute ones, given the job Id.
-resolveDependencies :: (Monad m) => MetaOpCode -> JobId -> m MetaOpCode
+resolveDependencies :: (MonadFail m) => MetaOpCode -> JobId -> m MetaOpCode
 resolveDependencies mopc jid = do
   mpar <- resolveDependsCommon (metaParams mopc) jid
   return (mopc { metaParams = mpar })
