@@ -37,30 +37,61 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 module Ganeti.Query.Locks
   ( fieldsMap
+  , RuntimeData
   ) where
 
-import qualified Data.Map as Map
+import qualified Text.JSON as J
 
+import Control.Arrow (first)
+import Data.Tuple (swap)
+
+import Ganeti.Locking.Allocation (OwnerState(..))
+import Ganeti.Locking.Locks (ClientId, ciIdentifier)
 import Ganeti.Query.Common
 import Ganeti.Query.Language
 import Ganeti.Query.Types
 
+-- | The runtime information for locks. As all information about locks
+-- is handled by WConfD, the actual information is obtained as live data.
+-- The type represents the information for a single lock, even though all
+-- locks are queried simultaneously, ahead of time.
+type RuntimeData = ( [(ClientId, OwnerState)] -- current state
+                   , [(ClientId, OwnerState)] -- pending requests
+                   )
+
+-- | Obtain the owners of a lock from the runtime data.
+getOwners :: RuntimeData -> a -> ResultEntry
+getOwners (ownerinfo, _) _ =
+  rsNormal . map (J.encode . ciIdentifier . fst)
+    $ ownerinfo
+
+-- | Obtain the mode of a lock from the runtime data.
+getMode :: RuntimeData -> a -> ResultEntry
+getMode (ownerinfo, _) _
+  | null ownerinfo = rsNormal J.JSNull
+  | any ((==) OwnExclusive . snd) ownerinfo = rsNormal "exclusive"
+  | otherwise = rsNormal "shared"
+
+-- | Obtain the pending requests from the runtime data.
+getPending :: RuntimeData -> a -> ResultEntry
+getPending (_, pending) _ =
+  rsNormal . map (swap . first ((:[]) . J.encode . ciIdentifier)) $ pending
+
 -- | List of all lock fields.
-lockFields :: FieldList String ()
+lockFields :: FieldList String RuntimeData
 lockFields =
   [ (FieldDefinition "name" "Name" QFTOther "Lock name",
      FieldSimple rsNormal, QffNormal)
   , (FieldDefinition "mode" "Mode" QFTOther "Mode in which the lock is\
                                              \ currently acquired\
                                              \ (exclusive or shared)",
-     FieldSimple rsNormal, QffNormal)
+     FieldRuntime getMode, QffNormal)
   , (FieldDefinition "owner" "Owner" QFTOther "Current lock owner(s)",
-     FieldSimple rsNormal, QffNormal)
+     FieldRuntime getOwners, QffNormal)
   , (FieldDefinition "pending" "Pending" QFTOther "Jobs waiting for the lock",
-     FieldSimple rsNormal, QffNormal)
+     FieldRuntime getPending, QffNormal)
   ]
 
 -- | The lock fields map.
-fieldsMap :: FieldMap String ()
-fieldsMap =
-  Map.fromList $ map (\v@(f, _, _) -> (fdefName f, v)) lockFields
+fieldsMap :: FieldMap String RuntimeData
+fieldsMap = fieldListToFieldMap lockFields
