@@ -40,7 +40,6 @@ module Ganeti.Query.Server
   , prepMain
   ) where
 
-import Control.Applicative
 import Control.Concurrent
 import Control.Exception
 import Control.Lens ((.~))
@@ -52,6 +51,7 @@ import Control.Monad.Trans (lift)
 import Control.Monad.Trans.Maybe
 import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.Set as Set (toList)
+import Data.Bits (finiteBitSize)
 import Data.IORef
 import Data.List (intersperse)
 import Data.Maybe (fromMaybe)
@@ -71,7 +71,6 @@ import Ganeti.Daemon.Utils (handleMasterVerificationOptions)
 import Ganeti.Objects
 import Ganeti.Objects.Lens (configFiltersL)
 import qualified Ganeti.Config as Config
-import qualified Ganeti.Compat as Compat
 import Ganeti.ConfigReader
 import Ganeti.BasicTypes
 import Ganeti.JQueue
@@ -193,7 +192,7 @@ handleCall _ _ cdata QueryClusterInfo =
       def_hv = case hypervisors of
                  x:_ -> showJSON x
                  [] -> JSNull
-      bits = show (Compat.finiteBitSize (0::Int)) ++ "bits"
+      bits = show (finiteBitSize (0::Int)) ++ "bits"
       arch_tuple = [bits, arch]
       obj = [ ("software_version", showJSON C.releaseVersion)
             , ("protocol_version", showJSON C.protocolVersion)
@@ -508,7 +507,7 @@ handleCall _ qstat  cfg (CancelJob jid kill) = do
       return result
     Bad s -> return . Ok . showJSON $ (False, s)
 
-handleCall qlock _ cfg (ArchiveJob jid) =
+handleCall qlock qstat cfg (ArchiveJob jid) =
   -- By adding a layer of MaybeT, we can prematurely end a computation
   -- using 'mzero' or other 'MonadPlus' primitive and return 'Ok False'.
   runResultT . liftM (showJSON . fromMaybe False) . runMaybeT $ do
@@ -516,10 +515,18 @@ handleCall qlock _ cfg (ArchiveJob jid) =
     let mcs = Config.getMasterCandidates cfg
         live = liveJobFile qDir jid
         archive = archivedJobFile qDir jid
+        -- We want to wait for the Job process to actually finish executing to
+        -- make sure its file has been properly replicated to all master
+        -- candidates (GH #1266). We wait up to 2 * (RPC connect timeout) + 1
+        -- second to allow for up to two master candidates to be unreachable,
+        -- without the archival failing. For more than that we let the job
+        -- archival fail; it will eventually be autoarchived by the watcher.
+        exitTimeout = (2 * C.rpcConnectTimeout + 1) * 1000 -- milliseconds
+    (job, _) <- (lift . mkResultT $ loadJobFromDisk qDir False jid)
+                `orElse` mzero
+    guard $ jobFinalized job
+    _ <- lift $ waitUntilJobExited (jqLivelock qstat) job exitTimeout
     withLock qlock $ do
-      (job, _) <- (lift . mkResultT $ loadJobFromDisk qDir False jid)
-                  `orElse` mzero
-      guard $ jobFinalized job
       lift . withErrorT JobQueueError
            . annotateError "Archiving failed in an unexpected way"
            . mkResultT $ safeRenameFile queueDirPermissions live archive
@@ -664,9 +671,6 @@ checkMain = handleMasterVerificationOptions
 -- | Prepare function for luxid.
 prepMain :: PrepFn () PrepResult
 prepMain _ _ = do
-  Exec.isForkSupported
-    >>= flip exitUnless "The daemon must be compiled without -threaded"
-
   socket_path <- Path.defaultQuerySocket
   cleanupSocket socket_path
   s <- describeError "binding to the Luxi socket"
