@@ -42,7 +42,6 @@ module Ganeti.THH.RPC
   , mkRpcM
   ) where
 
-import Control.Applicative
 import Control.Arrow ((&&&))
 import Control.Monad
 import Control.Monad.Error.Class
@@ -53,7 +52,7 @@ import qualified Text.JSON as J
 
 import Ganeti.BasicTypes
 import Ganeti.Errors
-import Ganeti.JSON
+import Ganeti.JSON (fromJResultE, fromJVal)
 import Ganeti.THH.Types
 import qualified Ganeti.UDSServer as US
 
