@@ -56,6 +56,7 @@ module Ganeti.UDSServer
   , acceptClient
   , closeClient
   , clientToFd
+  , clientToHandle
   , closeServer
   , buildResponse
   , parseResponse
@@ -70,7 +71,6 @@ module Ganeti.UDSServer
   , listener
   ) where
 
-import Control.Applicative
 import Control.Concurrent.Lifted (fork, yield)
 import Control.Monad.Base
 import Control.Monad.Trans.Control
@@ -83,7 +83,8 @@ import Data.List
 import Data.Word (Word8)
 import qualified Network.Socket as S
 import System.Directory (removeFile)
-import System.IO (hClose, hFlush, hWaitForInput, Handle, IOMode(..))
+import System.IO ( hClose, hFlush, hPutStr, hWaitForInput, Handle, IOMode(..)
+                 , hSetBuffering, BufferMode(..))
 import System.IO.Error (isEOFError)
 import System.Posix.Types (Fd)
 import System.Posix.IO (createPipe, fdToHandle, handleToFd)
@@ -94,7 +95,7 @@ import Text.JSON.Types
 
 import Ganeti.BasicTypes
 import Ganeti.Errors (GanetiException(..), ErrorResult)
-import Ganeti.JSON
+import Ganeti.JSON (fromJResult, fromJVal, fromJResultE, fromObj)
 import Ganeti.Logging
 import Ganeti.THH
 import Ganeti.Utils
@@ -192,12 +193,12 @@ closeClientSocket = hClose
 openServerSocket :: FilePath -> IO S.Socket
 openServerSocket path = do
   sock <- S.socket S.AF_UNIX S.Stream S.defaultProtocol
-  S.bindSocket sock (S.SockAddrUnix path)
+  S.bind sock (S.SockAddrUnix path)
   return sock
 
 closeServerSocket :: S.Socket -> FilePath -> IO ()
 closeServerSocket sock path = do
-  S.sClose sock
+  S.close sock
   removeFile path
 
 acceptSocket :: S.Socket -> IO Handle
@@ -283,14 +284,26 @@ clientToFd client | rh == wh  = join (,) <$> handleToFd rh
     rh = rsocket client
     wh = wsocket client
 
+-- | Extracts the read (first) and the write (second) handles of a client.
+-- The purpose of this function is to allow using a client's handles as
+-- input/output streams elsewhere.
+clientToHandle :: Client -> (Handle, Handle)
+clientToHandle client = (rsocket client, wsocket client)
+
 -- | Sends a message over a transport.
 sendMsg :: Client -> String -> IO ()
 sendMsg s buf = withTimeout (sendTmo $ clientConfig s) "sending a message" $ do
-  let encoded = UTF8.fromString buf
-      handle = wsocket s
-  B.hPut handle encoded
+  t1 <- getCurrentTimeUSec
+  let handle = wsocket s
+  -- Allow buffering (up to 1MiB) when writing to the socket. Note that
+  -- otherwise we get the default of sending each byte in a separate
+  -- system call, resulting in very poor performance.
+  hSetBuffering handle (BlockBuffering . Just $ 1024 * 1024)
+  hPutStr handle buf
   B.hPut handle bEOM
   hFlush handle
+  t2 <- getCurrentTimeUSec
+  logDebug $ "sendMsg: " ++ show ((t2 - t1) `div` 1000) ++ "ms"
 
 -- | Given a current buffer and the handle, it will read from the
 -- network until we get a full message, and it will return that
