@@ -52,7 +52,6 @@ module Test.Ganeti.Objects
 import Test.QuickCheck
 import qualified Test.HUnit as HUnit
 
-import Control.Applicative
 import Control.Monad
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.UTF8 as UTF8
@@ -92,7 +91,7 @@ instance Arbitrary (Container DataCollectorConfig) where
       fromContainer = Map.fromList $ zip names configs }
 
 instance Arbitrary BS.ByteString where
-  arbitrary = fmap UTF8.fromString arbitrary
+  arbitrary = genPrintableByteString
 
 $(genArbitrary ''PartialNDParams)
 
@@ -393,9 +392,11 @@ genValidNetwork :: Gen Objects.Network
 genValidNetwork = do
   -- generate netmask for the IPv4 network
   netmask <- fromIntegral <$> choose (24::Int, 30)
+  let hostLen = 32-netmask
   name <- genName >>= mkNonEmpty
   mac_prefix <- genMaybe genName
-  net <- arbitrary
+  net <- Objects.ip4AddressFromNumber . (* 2^hostLen) . fromIntegral <$>
+    choose (1::Int, 2^netmask-1)
   net6 <- genMaybe genIp6Net
   gateway <- genMaybe arbitrary
   gateway6 <- genMaybe genIp6Addr
@@ -570,7 +571,7 @@ casePyCompatNetworks = do
               \  a = network.AddressPool(net)\n\
               \  encoded.append((a.GetFreeCount(), a.GetReservedCount(), \\\n\
               \    net.ToDict()))\n\
-              \print serializer.Dump(encoded)" serialized
+              \sys.stdout.buffer.write(serializer.Dump(encoded))" serialized
     >>= checkPythonResult
   let deserialised = J.decode py_stdout::J.Result [(Int, Int, Network)]
   decoded <- case deserialised of
@@ -610,7 +611,7 @@ casePyCompatNodegroups = do
               \group_data = serializer.Load(sys.stdin.read())\n\
               \decoded = [objects.NodeGroup.FromDict(g) for g in group_data]\n\
               \encoded = [g.ToDict() for g in decoded]\n\
-              \print serializer.Dump(encoded)" serialized
+              \sys.stdout.buffer.write(serializer.Dump(encoded))" serialized
     >>= checkPythonResult
   let deserialised = J.decode py_stdout::J.Result [NodeGroup]
   decoded <- case deserialised of
@@ -714,7 +715,7 @@ casePyCompatInstances = do
               \inst_data = serializer.Load(sys.stdin.read())\n\
               \decoded = [objects.Instance.FromDict(i) for i in inst_data]\n\
               \encoded = [i.ToDict() for i in decoded]\n\
-              \print serializer.Dump(encoded)" serialized
+              \sys.stdout.buffer.write(serializer.Dump(encoded))" serialized
     >>= checkPythonResult
   let deserialised = J.decode py_stdout::J.Result [Instance]
   decoded <- case deserialised of
