@@ -43,7 +43,6 @@ module Test.Ganeti.OpCodes
 import Test.HUnit as HUnit
 import Test.QuickCheck as QuickCheck
 
-import Control.Applicative
 import Control.Monad
 import Data.Char
 import Data.List
@@ -612,7 +611,8 @@ case_AllDefined = do
      runPython "from ganeti import opcodes\n\
                \from ganeti import serializer\n\
                \import sys\n\
-               \print serializer.Dump([opid for opid in opcodes.OP_MAPPING])\n"
+               \sys.stdout.buffer.write(\
+               \  serializer.Dump([opid for opid in opcodes.OP_MAPPING]))"
                ""
      >>= checkPythonResult
   py_ops <- case J.decode py_stdout::J.Result [String] of
@@ -669,9 +669,9 @@ case_py_compat_types = do
                \  op.Validate(True)\n\
                \encoded = [(op.Summary(), op.__getstate__())\n\
                \           for op in decoded]\n\
-               \print serializer.Dump(\
+               \sys.stdout.buffer.write(serializer.Dump(\
                \  encoded,\
-               \  private_encoder=serializer.EncodeWithPrivateFields)"
+               \  private_encoder=serializer.EncodeWithPrivateFields))"
                serialized
      >>= checkPythonResult
   let deserialised =
@@ -701,7 +701,7 @@ case_py_compat_fields = do
                \from ganeti import serializer\n\
                \fields = [(k, sorted([p[0] for p in v.OP_PARAMS]))\n\
                \           for k, v in opcodes.OP_MAPPING.items()]\n\
-               \print serializer.Dump(fields)" ""
+               \sys.stdout.buffer.write(serializer.Dump(fields))" ""
      >>= checkPythonResult
   let deserialised = J.decode py_stdout::J.Result [(String, [String])]
   py_fields <- case deserialised of
