@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, FlexibleInstances #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-| Common helper functions and instances for all Ganeti tests.
@@ -56,6 +56,7 @@ module Test.Ganeti.TestCommon
   , genPrintableAsciiChar
   , genPrintableAsciiString
   , genPrintableAsciiStringNE
+  , genPrintableByteString
   , genName
   , genFQDN
   , genUUID
@@ -90,16 +91,22 @@ module Test.Ganeti.TestCommon
   , getTempFileName
   , listOfUniqueBy
   , counterexample
+  , cover'
   ) where
 
-import Control.Applicative
 import Control.Exception (catchJust)
 import Control.Monad
+import Control.Monad.Fail (MonadFail, fail)
 import Data.Attoparsec.Text (Parser, parseOnly)
 import Data.List
 import qualified Data.Map as M
 import Data.Text (pack)
 import Data.Word
+import qualified Data.ByteString as BS
+import qualified Data.ByteString.UTF8 as UTF8
+#if !MIN_VERSION_QuickCheck(2,10,0)
+import Data.Char (isPrint)
+#endif
 import qualified Data.Set as Set
 import System.Directory (getTemporaryDirectory, removeFile)
 import System.Environment (getEnv)
@@ -109,9 +116,6 @@ import System.IO.Error (isDoesNotExistError)
 import System.Process (readProcessWithExitCode)
 import qualified Test.HUnit as HUnit
 import Test.QuickCheck
-#if !MIN_VERSION_QuickCheck(2,7,0)
-import qualified Test.QuickCheck as QC
-#endif
 import Test.QuickCheck.Monadic
 import qualified Text.JSON as J
 import Numeric
@@ -122,17 +126,19 @@ import Ganeti.Objects (TagSet(..))
 import Ganeti.Types
 import Ganeti.Utils.Monad (unfoldrM)
 
+-- * Compatibility instances
+
+instance MonadFail Gen where
+  fail = error "No monadfail instance"
+
+instance MonadFail (Either String) where
+  fail x = Left x
+
 -- * Arbitrary orphan instances
 
 instance Arbitrary TagSet where
   arbitrary = (TagSet . Set.fromList) <$> genTags
 
-#if !MIN_VERSION_QuickCheck(2,8,0)
-instance (Ord k, Arbitrary k, Arbitrary a) => Arbitrary (M.Map k a) where
-  arbitrary = M.fromList <$> arbitrary
-  shrink m = M.fromList <$> shrink (M.toList m)
-#endif
-
 -- * Constants
 
 -- | Maximum memory (1TiB, somewhat random value).
@@ -195,23 +201,33 @@ failTest msg = counterexample msg False
 passTest :: Property
 passTest = property True
 
+-- | QuickCheck 2.12 swapped the order of the first two arguments, so provide a
+-- compatibility function here
+
+cover' :: Testable prop => Double -> Bool -> String -> prop -> Property
+#if MIN_VERSION_QuickCheck(2, 12, 0)
+cover' = cover
+#else
+cover' p x = cover x (round p)
+#endif
+
 -- | A stable version of QuickCheck's `cover`. In its current implementation,
 -- cover will not detect insufficient coverage if the actual coverage in the
 -- sample is 0. Work around this by lifting the probability to at least
 -- 10 percent.
 -- The underlying issue is tracked at
 -- https://github.com/nick8325/quickcheck/issues/26
-stableCover :: Testable prop => Bool -> Int -> String -> prop -> Property
+stableCover :: Testable prop => Bool -> Double -> String -> prop -> Property
 stableCover p percent s prop =
   let newlabel = "(stabilized to at least 10%) " ++ s
   in forAll (frequency [(1, return True), (9, return False)]) $ \ basechance ->
-     cover (basechance || p) (10 + (percent * 9 `div` 10)) newlabel prop
+     cover' (10 + (percent * 9 / 10)) (basechance || p) newlabel prop
 
 -- | Return the python binary to use. If the PYTHON environment
--- variable is defined, use its value, otherwise use just \"python\".
+-- variable is defined, use its value, otherwise use just \"python3\".
 pythonCmd :: IO String
 pythonCmd = catchJust (guard . isDoesNotExistError)
-            (getEnv "PYTHON") (const (return "python"))
+            (getEnv "PYTHON") (const (return "python3"))
 
 -- | Run Python with an expression, returning the exit code, standard
 -- output and error.
@@ -257,6 +273,15 @@ genPrintableAsciiStringNE = do
   n <- choose (1, 40)
   vectorOf n genPrintableAsciiChar >>= mkNonEmpty
 
+-- | Generates a printable Unicode ByteString
+genPrintableByteString :: Gen BS.ByteString
+#if MIN_VERSION_QuickCheck(2, 10, 0)
+genPrintableByteString = fmap (UTF8.fromString . getPrintableString) arbitrary
+#else
+genPrintableByteString =
+  fmap UTF8.fromString $ listOf (arbitrary `suchThat` isPrint)
+#endif
+
 -- | Generates a single name component.
 genName :: Gen String
 genName = do
@@ -413,9 +438,15 @@ genIPv4Address = mkIPv4Address =<< genIPv4
 -- | Generate an arbitrary IPv4 network in textual form.
 genIPv4AddrRange :: Gen String
 genIPv4AddrRange = do
-  ip <- genIPv4
-  netmask <- choose (8::Int, 30)
-  return $ ip ++ "/" ++ show netmask
+  pfxLen <- choose (8::Int, 30)
+  -- Generate a number that fits in pfxLen bits
+  addr <- choose(1::Int, 2^pfxLen-1)
+  let hostLen = 32 - pfxLen
+      -- ...and shift it left until it becomes a 32-bit number
+      -- with the low hostLen bits unset
+      net = addr * 2^hostLen
+      netBytes = [(net `div` 2^x) `mod` 256 | x <- [24::Int, 16, 8, 0]]
+  return $ intercalate "." (map show netBytes) ++ "/" ++ show pfxLen
 
 genIPv4Network :: Gen IPv4Network
 genIPv4Network = mkIPv4Network =<< genIPv4AddrRange
@@ -434,12 +465,31 @@ genIp6Addr = do
   rawIp <- vectorOf 8 $ choose (0::Integer, 65535)
   return $ intercalate ":" (map (`showHex` "") rawIp)
 
+-- | Helper function to convert an integer in to an IPv6 address in textual
+-- form
+ip6AddressFromNumber :: Integer -> [Char]
+ip6AddressFromNumber ipInt =
+  -- chunksOf splits a sequence in chunks of n elements length each
+  -- chunksOf 4 "20010db80000" = ["2001", "0db8", "0000"]
+  let chunksOf :: Int -> [a] -> [[a]]
+      chunksOf _ [] = []
+      chunksOf n lst = take n lst : (chunksOf n $ drop n lst)
+      -- Left-pad a sequence with a fixed element if it's shorter than n
+      -- elements, or trim it to the first n elements otherwise
+      -- e.g. lPadTrim 6 '0' "abcd" = "00abcd"
+      lPadTrim :: Int -> a -> [a] -> [a]
+      lPadTrim n p lst
+        | length lst < n = lPadTrim n p $ p : lst
+        | otherwise = take n lst
+      rawIp = lPadTrim 32 '0' $ (`showHex` "") ipInt
+  in intercalate ":" $ chunksOf 4 rawIp
+
 -- | Generates an arbitrary IPv6 network in textual form.
 genIp6Net :: Gen String
 genIp6Net = do
   netmask <- choose (8::Int, 126)
-  ip <- genIp6Addr
-  return $ ip ++ "/" ++ show netmask
+  raw_ip <- (* 2^(128-netmask)) <$> choose(1::Integer, 2^netmask-1)
+  return $ ip6AddressFromNumber raw_ip ++ "/" ++ show netmask
 
 -- | Generates a valid, arbitrary tag name with respect to the given
 -- 'TagKind' for opcodes.
@@ -589,9 +639,3 @@ listOfUniqueBy gen keyFun forbidden = do
       else do
         x <- gen `suchThat` ((`Set.notMember` usedKeys) . keyFun)
         return $ Just (x, (i + 1, Set.insert (keyFun x) usedKeys))
-
-
-#if !MIN_VERSION_QuickCheck(2,7,0)
-counterexample :: Testable prop => String -> prop -> Property
-counterexample = QC.printTestCase
-#endif
