@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell, CPP #-}
+{-# LANGUAGE TemplateHaskell #-}
 
 {-| Unittest helpers for TemplateHaskell components.
 
@@ -39,7 +39,6 @@ module Test.Ganeti.TestHelper
   , genArbitrary
   ) where
 
-import Control.Applicative
 import Data.List (stripPrefix, isPrefixOf)
 import Data.Maybe (fromMaybe)
 import Test.Framework
@@ -141,17 +140,10 @@ genArbitrary :: Name -> Q [Dec]
 genArbitrary name = do
   r <- reify name
   case r of
-#if MIN_VERSION_template_haskell(2,11,0)
     TyConI (DataD _ _ _ _ cons _) ->
       mkRegularArbitrary name cons
     TyConI (NewtypeD _ _ _ _ con _) ->
       mkRegularArbitrary name [con]
-#else
-    TyConI (DataD _ _ _ cons _) ->
-      mkRegularArbitrary name cons
-    TyConI (NewtypeD _ _ _ con _) ->
-      mkRegularArbitrary name [con]
-#endif
     TyConI (TySynD _ _ (ConT tn)) -> genArbitrary tn
     _ -> fail $ "Invalid type in call to genArbitrary for " ++ show name
          ++ ", type " ++ show r
