@@ -75,6 +75,7 @@ import Test.Ganeti.Network
 import Test.Ganeti.Objects
 import Test.Ganeti.Objects.BitArray
 import Test.Ganeti.OpCodes
+import Test.Ganeti.PyValue
 import Test.Ganeti.Query.Aliases
 import Test.Ganeti.Query.Filter
 import Test.Ganeti.Query.Instance
@@ -95,6 +96,7 @@ import Test.Ganeti.Types
 import Test.Ganeti.Utils
 import Test.Ganeti.Utils.MultiMap
 import Test.Ganeti.Utils.Statistics
+import Test.Ganeti.WConfd.Ssconf
 import Test.Ganeti.WConfd.TempRes
 
 -- | Our default test options, overring the built-in test-framework
@@ -150,6 +152,7 @@ allTests =
   , testObjects
   , testObjects_BitArray
   , testOpCodes
+  , testPyValue
   , testQuery_Aliases
   , testQuery_Filter
   , testQuery_Instance
@@ -167,6 +170,7 @@ allTests =
   , testUtils
   , testUtils_MultiMap
   , testUtils_Statistics
+  , testWConfd_Ssconf
   , testWConfd_TempRes
   ]
 
