@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2010, 2012, 2013, 2014, 2015 Google Inc.
@@ -74,6 +74,8 @@ def GetMinimalConfig():
         "cpu-avg-load": { "active": True, "interval": 5000000 },
         "xen-cpu-avg-load": { "active": True, "interval": 5000000 },
       },
+      "ssh_key_type": "dsa",
+      "ssh_key_bits": 1024,
     },
     "instances": {},
     "disks": {},
@@ -116,6 +118,10 @@ def _RunUpgrade(path, dry_run, no_verify, ignore_hostname=True,
 
 class TestCfgupgrade(unittest.TestCase):
   def setUp(self):
+    # Since we are comparing large dictionaries here, this is vital to getting
+    # useful feedback about differences in config content using assertEquals.
+    self.maxDiff = None
+
     self.tmpdir = tempfile.mkdtemp()
 
     self.config_path = utils.PathJoin(self.tmpdir, "config.data")
@@ -224,9 +230,9 @@ class TestCfgupgrade(unittest.TestCase):
       expversion = constants.CONFIG_VERSION
       checkfn = operator.truth
 
-    self.assert_(checkfn(os.path.isfile(self.rapi_cert_path)))
-    self.assert_(checkfn(os.path.isfile(self.confd_hmac_path)))
-    self.assert_(checkfn(os.path.isfile(self.cds_path)))
+    self.assertTrue(checkfn(os.path.isfile(self.rapi_cert_path)))
+    self.assertTrue(checkfn(os.path.isfile(self.confd_hmac_path)))
+    self.assertTrue(checkfn(os.path.isfile(self.cds_path)))
 
     newcfg = self._LoadConfig()
     self.assertEqual(newcfg["version"], expversion)
@@ -240,8 +246,8 @@ class TestCfgupgrade(unittest.TestCase):
     self._TestSimpleUpgrade(version.BuildVersion(2, 3, 0), False)
 
     self.assertTrue(os.path.isdir(os.path.dirname(self.rapi_users_path)))
-    self.assert_(os.path.islink(self.rapi_users_path_pre24))
-    self.assert_(os.path.isfile(self.rapi_users_path))
+    self.assertTrue(os.path.islink(self.rapi_users_path_pre24))
+    self.assertTrue(os.path.isfile(self.rapi_users_path))
     self.assertEqual(os.readlink(self.rapi_users_path_pre24),
                      self.rapi_users_path)
     for path in [self.rapi_users_path, self.rapi_users_path_pre24]:
@@ -255,8 +261,8 @@ class TestCfgupgrade(unittest.TestCase):
     utils.WriteFile(self.rapi_users_path, data="other user\n")
     self._TestSimpleUpgrade(version.BuildVersion(2, 3, 0), False)
 
-    self.assert_(os.path.islink(self.rapi_users_path_pre24))
-    self.assert_(os.path.isfile(self.rapi_users_path))
+    self.assertTrue(os.path.islink(self.rapi_users_path_pre24))
+    self.assertTrue(os.path.isfile(self.rapi_users_path))
     self.assertEqual(os.readlink(self.rapi_users_path_pre24),
                      self.rapi_users_path)
     for path in [self.rapi_users_path, self.rapi_users_path_pre24]:
@@ -272,9 +278,9 @@ class TestCfgupgrade(unittest.TestCase):
 
     self._TestSimpleUpgrade(version.BuildVersion(2, 2, 0), False)
 
-    self.assert_(os.path.isfile(self.rapi_users_path) and
+    self.assertTrue(os.path.isfile(self.rapi_users_path) and
                  not os.path.islink(self.rapi_users_path))
-    self.assert_(os.path.islink(self.rapi_users_path_pre24))
+    self.assertTrue(os.path.islink(self.rapi_users_path_pre24))
     self.assertEqual(os.readlink(self.rapi_users_path_pre24),
                      self.rapi_users_path)
     for path in [self.rapi_users_path, self.rapi_users_path_pre24]:
@@ -292,7 +298,7 @@ class TestCfgupgrade(unittest.TestCase):
                       version.BuildVersion(2, 2, 0), False)
 
     for path in [self.rapi_users_path, self.rapi_users_path_pre24]:
-      self.assert_(os.path.isfile(path) and not os.path.islink(path))
+      self.assertTrue(os.path.isfile(path) and not os.path.islink(path))
     self.assertEqual(utils.ReadFile(self.rapi_users_path), "other user\n")
     self.assertEqual(utils.ReadFile(self.rapi_users_path_pre24),
                      "hello world\n")
@@ -362,8 +368,8 @@ class TestCfgupgrade(unittest.TestCase):
     self.assertEqual(lines.pop(0), self.tmpdir)
     self.assertEqual(lines.pop(0), "/tmp")
     self.assertFalse(lines)
-    self.assertEqual(os.stat(self.file_storage_paths).st_mode & 0777,
-                     0600, msg="Wrong permissions")
+    self.assertEqual(os.stat(self.file_storage_paths).st_mode & 0o777,
+                     0o600, msg="Wrong permissions")
 
   def testFileStoragePathsSharedOnly(self):
     self.assertFalse(os.path.exists(self.file_storage_paths))
@@ -426,6 +432,12 @@ class TestCfgupgrade(unittest.TestCase):
   def testUpgradeFullConfigFrom_2_14(self):
     self._TestUpgradeFromFile("cluster_config_2.14.json", False)
 
+  def testUpgradeFullConfigFrom_2_15(self):
+    self._TestUpgradeFromFile("cluster_config_2.15.json", False)
+
+  def testUpgradeFullConfigFrom_2_16(self):
+    self._TestUpgradeFromFile("cluster_config_2.16.json", False)
+
   def testUpgradeCurrent(self):
     self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
 
@@ -443,7 +455,7 @@ class TestCfgupgrade(unittest.TestCase):
   def testDowngradeFullConfig(self):
     """Test for upgrade + downgrade combination."""
     # This test can work only with the previous version of a configuration!
-    oldconfname = "cluster_config_2.14.json"
+    oldconfname = "cluster_config_2.16.json"
     self._TestUpgradeFromFile(oldconfname, False)
     _RunUpgrade(self.tmpdir, False, True, downgrade=True)
     oldconf = self._LoadTestDataConfig(oldconfname)
