@@ -38,7 +38,7 @@ import traceback
 import functools
 import sys
 
-from testutils.config_mock import ConfigMock
+from testutils.config_mock import ConfigMock, ConfigObjectMatcher
 from cmdlib.testsupport.iallocator_mock import patchIAllocator
 from cmdlib.testsupport.livelock_mock import LiveLockMock
 from cmdlib.testsupport.netutils_mock import patchNetutils, \
@@ -80,9 +80,6 @@ class GanetiContextMock(object):
   def AddNode(self, cfg, node, ec_id):
     cfg.AddNode(node, ec_id)
 
-  def ReaddNode(self, node):
-    pass
-
   def RemoveNode(self, cfg, node):
     cfg.RemoveNode(node.uuid)
 
@@ -170,9 +167,8 @@ class CmdlibTestCase(testutils.GanetiTestCase):
     while self._cleanups:
       f, args, kwargs = self._cleanups.pop(-1)
       try:
-        # pylint: disable=W0142
         f(*args, **kwargs)
-      except BaseException, e:
+      except BaseException as e:
         sys.stderr.write('Error in cleanup: %s\n' % e)
 
   def _GetTestModule(self):
@@ -234,7 +230,7 @@ class CmdlibTestCase(testutils.GanetiTestCase):
     @return: A mock LU
 
     """
-    return MockLU(self.mcpu, mock.MagicMock(), self.ctx, self.cfg, self.rpc,
+    return MockLU(self.mcpu, mock.MagicMock(), self.cfg, self.rpc,
                   (1234, "/tmp/mock/livelock"), self.wconfd)
 
   def RpcResultsBuilder(self, use_node_names=False):
@@ -272,12 +268,12 @@ class CmdlibTestCase(testutils.GanetiTestCase):
     """
     try:
       self.ExecOpCode(opcode)
-    except expected_exception, e:
+    except expected_exception as e:
       if expected_regex is not None:
         assert re.search(expected_regex, str(e)) is not None, \
                 "Caught exception '%s' did not match '%s'" % \
                   (str(e), expected_regex)
-    except Exception, e:
+    except Exception as e:
       tb = traceback.format_exc()
       raise AssertionError("%s\n(See original exception above)\n"
                            "Expected exception '%s' was not raised,"
@@ -395,6 +391,9 @@ class CmdlibTestCase(testutils.GanetiTestCase):
         return group
     assert False
 
+  def _MatchMasterParams(self):
+    return ConfigObjectMatcher(self.cfg.GetMasterNetworkParameters())
+
   def MockOut(self, *args, **kwargs):
     """Immediately start mock.patch.object."""
     patcher = mock.patch.object(*args, **kwargs)
@@ -442,7 +441,6 @@ def withLockedLU(func):
       prepare_fn = getattr(test, "PrepareLU")
       assert callable(prepare_fn)
 
-    # pylint: disable=W0142
     def callWithLU(lu):
       if prepare_fn:
         prepare_fn(lu)
