@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2008, 2011, 2012, 2013 Google Inc.
@@ -35,7 +35,7 @@ import testutils
 import time
 import unittest
 import yaml
-from cStringIO import StringIO
+from io import StringIO
 
 from ganeti import constants
 from ganeti import cli
@@ -62,7 +62,7 @@ class TestParseTimespec(unittest.TestCase):
       ("61m", 61 * 60),
       ]
     for value, expected_result in test_data:
-      self.failUnlessEqual(cli.ParseTimespec(value), expected_result)
+      self.assertEqual(cli.ParseTimespec(value), expected_result)
 
   def testInvalidTime(self):
     """Test invalid timespecs"""
@@ -73,7 +73,7 @@ class TestParseTimespec(unittest.TestCase):
       "s",
       ]
     for value in test_data:
-      self.failUnlessRaises(OpPrereqError, cli.ParseTimespec, value)
+      self.assertRaises(OpPrereqError, cli.ParseTimespec, value)
 
 
 class TestToStream(unittest.TestCase):
@@ -88,18 +88,18 @@ class TestToStream(unittest.TestCase):
                  ]:
       buf = StringIO()
       cli._ToStream(buf, data)
-      self.failUnlessEqual(buf.getvalue(), data + "\n")
+      self.assertEqual(buf.getvalue(), data + "\n")
 
   def testParams(self):
       buf = StringIO()
       cli._ToStream(buf, "foo %s", 1)
-      self.failUnlessEqual(buf.getvalue(), "foo 1\n")
+      self.assertEqual(buf.getvalue(), "foo 1\n")
       buf = StringIO()
       cli._ToStream(buf, "foo %s", (15,16))
-      self.failUnlessEqual(buf.getvalue(), "foo (15, 16)\n")
+      self.assertEqual(buf.getvalue(), "foo (15, 16)\n")
       buf = StringIO()
       cli._ToStream(buf, "foo %s %s", "a", "b")
-      self.failUnlessEqual(buf.getvalue(), "foo a b\n")
+      self.assertEqual(buf.getvalue(), "foo a b\n")
 
 
 class TestGenerateTable(unittest.TestCase):
@@ -502,7 +502,8 @@ class _MockJobPollCb(cli.JobPollCbBase, cli.JobPollReportCbBase):
     self._jobstatus.append(args)
 
   def WaitForJobChangeOnce(self, job_id, fields,
-                           prev_job_info, prev_log_serial):
+                           prev_job_info, prev_log_serial,
+                           timeout=constants.DEFAULT_WFJC_TIMEOUT):
     self.tc.assertEqual(job_id, self.job_id)
     self.tc.assertEqualValues(fields, ["status"])
     self.tc.assertFalse(self._expect_notchanged)
@@ -531,6 +532,9 @@ class _MockJobPollCb(cli.JobPollCbBase, cli.JobPollReportCbBase):
     self.tc.assertEqual(len(fields), len(result))
     return [result]
 
+  def CancelJob(self, job_id):
+    self.tc.assertEqual(job_id, self.job_id)
+
   def ReportLogMessage(self, job_id, serial, timestamp, log_type, log_msg):
     self.tc.assertEqual(job_id, self.job_id)
     self.tc.assertEqualValues((serial, timestamp, log_type, log_msg),
@@ -538,7 +542,7 @@ class _MockJobPollCb(cli.JobPollCbBase, cli.JobPollReportCbBase):
 
   def ReportNotChanged(self, job_id, status):
     self.tc.assertEqual(job_id, self.job_id)
-    self.tc.assert_(self._expect_notchanged)
+    self.tc.assertTrue(self._expect_notchanged)
     self._expect_notchanged = False
 
 
@@ -646,9 +650,43 @@ class TestGenericPollJob(testutils.GanetiTestCase):
                            [constants.OP_STATUS_CANCELING,
                             constants.OP_STATUS_CANCELING],
                            [None, None])
-    self.assertRaises(errors.OpExecError, cli.GenericPollJob, job_id, cbs, cbs)
+    self.assertRaises(errors.JobCanceled, cli.GenericPollJob, job_id, cbs, cbs)
     cbs.CheckEmpty()
 
+  def testNegativeUpdateFreqParameter(self):
+    job_id = 12345
+
+    cbs = _MockJobPollCb(self, job_id)
+    self.assertRaises(errors.ParameterError, cli.GenericPollJob, job_id, cbs,
+                      cbs, update_freq=-30)
+
+  def testZeroUpdateFreqParameter(self):
+    job_id = 12345
+
+    cbs = _MockJobPollCb(self, job_id)
+    self.assertRaises(errors.ParameterError, cli.GenericPollJob, job_id, cbs,
+                      cbs, update_freq=0)
+
+  def testShouldCancel(self):
+    job_id = 12345
+
+    cbs = _MockJobPollCb(self, job_id)
+    cbs.AddWfjcResult(None, None, constants.JOB_NOTCHANGED)
+    self.assertRaises(errors.JobCanceled, cli.GenericPollJob, job_id, cbs, cbs,
+                      cancel_fn=(lambda: True))
+
+  def testIgnoreCancel(self):
+    job_id = 12345
+    cbs = _MockJobPollCb(self, job_id)
+    cbs.AddWfjcResult(None, None, ((constants.JOB_STATUS_SUCCESS, ), None))
+    cbs.AddQueryJobsResult(constants.JOB_STATUS_SUCCESS,
+                           [constants.OP_STATUS_SUCCESS,
+                            constants.OP_STATUS_SUCCESS],
+                           ["Hello World", "Foo man bar"])
+    self.assertEqual(["Hello World", "Foo man bar"],
+                     cli.GenericPollJob(
+                         job_id, cbs, cbs, cancel_fn=(lambda: False)))
+    cbs.CheckEmpty()
 
 class TestFormatLogMessage(unittest.TestCase):
   def test(self):
@@ -658,7 +696,7 @@ class TestFormatLogMessage(unittest.TestCase):
     self.assertRaises(TypeError, cli.FormatLogMessage,
                       constants.ELOG_MESSAGE, [1, 2, 3])
 
-    self.assert_(cli.FormatLogMessage("some other type", (1, 2, 3)))
+    self.assertTrue(cli.FormatLogMessage("some other type", (1, 2, 3)))
 
 
 class TestParseFields(unittest.TestCase):
@@ -908,7 +946,7 @@ class TestParseArgs(unittest.TestCase):
     for argv in [[], ["gnt-unittest"]]:
       try:
         cli._ParseArgs("gnt-unittest", argv, {}, {}, set())
-      except cli._ShowUsage, err:
+      except cli._ShowUsage as err:
         self.assertTrue(err.exit_error)
       else:
         self.fail("Did not raise exception")
@@ -926,7 +964,7 @@ class TestParseArgs(unittest.TestCase):
     for argv in [["test", "--help"], ["test", "--help", "somethingelse"]]:
       try:
         cli._ParseArgs("test", argv, {}, {}, set())
-      except cli._ShowUsage, err:
+      except cli._ShowUsage as err:
         self.assertFalse(err.exit_error)
       else:
         self.fail("Did not raise exception")
@@ -935,7 +973,7 @@ class TestParseArgs(unittest.TestCase):
     for argv in [["test", "list"], ["test", "somethingelse", "--help"]]:
       try:
         cli._ParseArgs("test", argv, {}, {}, set())
-      except cli._ShowUsage, err:
+      except cli._ShowUsage as err:
         self.assertTrue(err.exit_error)
       else:
         self.fail("Did not raise exception")
@@ -967,7 +1005,7 @@ class TestQftNames(unittest.TestCase):
     self.assertEqual(frozenset(cli._QFT_NAMES), constants.QFT_ALL)
 
   def testUnique(self):
-    lcnames = map(lambda s: s.lower(), cli._QFT_NAMES.values())
+    lcnames = [s.lower() for s in cli._QFT_NAMES.values()]
     self.assertFalse(utils.FindDuplicates(lcnames))
 
   def testUppercase(self):
@@ -1097,7 +1135,7 @@ class TestFormatPolicyInfo(unittest.TestCase):
       ]:
       self._RenameDictItem(parsed, pretty, raw)
     for minmax in parsed[constants.ISPECS_MINMAX]:
-      for key in minmax:
+      for key in set(minmax.keys()):
         keyparts = key.split("/", 1)
         if len(keyparts) > 1:
           self._RenameDictItem(minmax, key, keyparts[0])
@@ -1122,8 +1160,8 @@ class TestFormatPolicyInfo(unittest.TestCase):
       self.assertTrue(isinstance(cluster, dict))
       if skip is None:
         skip = frozenset()
-      self.assertEqual(frozenset(cluster.keys()).difference(skip),
-                       frozenset(group.keys()))
+      self.assertEqual(frozenset(cluster).difference(skip),
+                       frozenset(group))
       for key in group:
         self._CompareClusterGroupItems(cluster[key], group[key])
     elif isinstance(group, list):
@@ -1132,7 +1170,7 @@ class TestFormatPolicyInfo(unittest.TestCase):
       for (cval, gval) in zip(cluster, group):
         self._CompareClusterGroupItems(cval, gval)
     else:
-      self.assertTrue(isinstance(group, basestring))
+      self.assertTrue(isinstance(group, str))
       self.assertEqual("default (%s)" % cluster, group)
 
   def _TestClusterVsGroup(self, policy):
@@ -1156,8 +1194,8 @@ class TestCreateIPolicyFromOpts(unittest.TestCase):
     self.assertTrue(type(default_pol) is dict)
     self.assertTrue(type(diff_pol) is dict)
     self.assertTrue(type(merged_pol) is dict)
-    self.assertEqual(frozenset(default_pol.keys()),
-                     frozenset(merged_pol.keys()))
+    self.assertEqual(frozenset(default_pol),
+                     frozenset(merged_pol))
     for (key, val) in merged_pol.items():
       if key in diff_pol:
         if type(val) is dict:
