@@ -1,22 +1,31 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2009 Google Inc.
+# All rights reserved.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# 1. Redistributions of source code must retain the above copyright notice,
+# this list of conditions and the following disclaimer.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in the
+# documentation and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 """Script for unittesting the confd client module"""
@@ -131,13 +140,13 @@ class _BaseClientTest:
     # Coverage is too big
     self.assertRaises(errors.ConfdClientError, self.client.SendRequest,
                       req2, coverage=15)
-    self.assertEquals(self.client._socket.send_count,
+    self.assertEqual(self.client._socket.send_count,
                       constants.CONFD_DEFAULT_REQ_COVERAGE)
     # Send with max coverage
     self.client.SendRequest(req2, coverage=-1)
-    self.assertEquals(self.client._socket.send_count,
+    self.assertEqual(self.client._socket.send_count,
                       constants.CONFD_DEFAULT_REQ_COVERAGE + len(self.mc_list))
-    self.assert_(self.client._socket.last_address in self.mc_list)
+    self.assertTrue(self.client._socket.last_address in self.mc_list)
 
 
   def testClientExpire(self):
@@ -151,17 +160,17 @@ class _BaseClientTest:
     self.mock_time.increase(constants.CONFD_CLIENT_EXPIRE_TIMEOUT - 1)
     # First request should be expired, second one should not
     self.client.ExpireRequests()
-    self.assertEquals(self.callback.call_count, 1)
-    self.assertEquals(self.callback.last_up.type, confd.client.UPCALL_EXPIRE)
-    self.assertEquals(self.callback.last_up.salt, req.rsalt)
-    self.assertEquals(self.callback.last_up.orig_request, req)
+    self.assertEqual(self.callback.call_count, 1)
+    self.assertEqual(self.callback.last_up.type, confd.client.UPCALL_EXPIRE)
+    self.assertEqual(self.callback.last_up.salt, req.rsalt)
+    self.assertEqual(self.callback.last_up.orig_request, req)
     self.mock_time.increase(3)
-    self.assertEquals(self.callback.call_count, 1)
+    self.assertEqual(self.callback.call_count, 1)
     self.client.ExpireRequests()
-    self.assertEquals(self.callback.call_count, 2)
-    self.assertEquals(self.callback.last_up.type, confd.client.UPCALL_EXPIRE)
-    self.assertEquals(self.callback.last_up.salt, req2.rsalt)
-    self.assertEquals(self.callback.last_up.orig_request, req2)
+    self.assertEqual(self.callback.call_count, 2)
+    self.assertEqual(self.callback.last_up.type, confd.client.UPCALL_EXPIRE)
+    self.assertEqual(self.callback.last_up.salt, req2.rsalt)
+    self.assertEqual(self.callback.last_up.orig_request, req2)
 
   def testClientCascadeExpire(self):
     req = confd.client.ConfdClientRequest(type=constants.CONFD_REQ_PING)
@@ -169,19 +178,19 @@ class _BaseClientTest:
     self.mock_time.increase(constants.CONFD_CLIENT_EXPIRE_TIMEOUT +1)
     req2 = confd.client.ConfdClientRequest(type=constants.CONFD_REQ_PING)
     self.client.SendRequest(req2)
-    self.assertEquals(self.callback.call_count, 1)
+    self.assertEqual(self.callback.call_count, 1)
 
   def testUpdatePeerList(self):
     self.client.UpdatePeerList(self.new_peers)
-    self.assertEquals(self.client._peers, self.new_peers)
+    self.assertEqual(self.client._peers, self.new_peers)
     req = confd.client.ConfdClientRequest(type=constants.CONFD_REQ_PING)
     self.client.SendRequest(req)
-    self.assertEquals(self.client._socket.send_count, len(self.new_peers))
-    self.assert_(self.client._socket.last_address in self.new_peers)
+    self.assertEqual(self.client._socket.send_count, len(self.new_peers))
+    self.assertTrue(self.client._socket.last_address in self.new_peers)
 
   def testSetPeersFamily(self):
     self.client._SetPeersAddressFamily()
-    self.assertEquals(self.client._family, self.family)
+    self.assertEqual(self.client._family, self.family)
     mixed_peers = ["192.0.2.99", "2001:db8:beef::13"]
     self.client.UpdatePeerList(mixed_peers)
     self.assertRaises(errors.ConfdClientError,
