@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2006, 2007, 2010, 2011, 2012, 2013 Google Inc.
@@ -51,7 +51,7 @@ from ganeti.config import TemporaryReservationManager
 import testutils
 import mocks
 import mock
-from testutils.config_mock import ConfigMock
+from testutils.config_mock import ConfigMock, _UpdateIvNames
 
 
 def _StubGetEntResolver():
@@ -136,8 +136,8 @@ class TestConfigRunner(unittest.TestCase):
   def testInit(self):
     """Test initialize the config file"""
     cfg = self._get_object()
-    self.failUnlessEqual(1, len(cfg.GetNodeList()))
-    self.failUnlessEqual(0, len(cfg.GetInstanceList()))
+    self.assertEqual(1, len(cfg.GetNodeList()))
+    self.assertEqual(0, len(cfg.GetInstanceList()))
 
   def _GenericNodesCheck(self, iobj, all_nodes, secondary_nodes):
     for i in [all_nodes, secondary_nodes]:
@@ -303,60 +303,6 @@ class TestConfigRunner(unittest.TestCase):
       node2.uuid: ["myxenvg/disk0", "myxenvg/meta0"],
       })
 
-  def testUpdateCluster(self):
-    """Test updates on the cluster object"""
-    cfg = self._get_object()
-    # construct a fake cluster object
-    fake_cl = objects.Cluster()
-    # fail if we didn't read the config
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_cl, None)
-
-    cl = cfg.GetClusterInfo()
-    # first pass, must not fail
-    cfg.Update(cl, None)
-    # second pass, also must not fail (after the config has been written)
-    cfg.Update(cl, None)
-    # but the fake_cl update should still fail
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_cl, None)
-
-  def testUpdateNode(self):
-    """Test updates on one node object"""
-    cfg = self._get_object()
-    # construct a fake node
-    fake_node = objects.Node()
-    # fail if we didn't read the config
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_node,
-                          None)
-
-    node = cfg.GetNodeInfo(cfg.GetNodeList()[0])
-    # first pass, must not fail
-    cfg.Update(node, None)
-    # second pass, also must not fail (after the config has been written)
-    cfg.Update(node, None)
-    # but the fake_node update should still fail
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_node,
-                          None)
-
-  def testUpdateInstance(self):
-    """Test updates on one instance object"""
-    cfg = self._get_object_mock()
-    # construct a fake instance
-    inst = self._create_instance(cfg)
-    fake_instance = objects.Instance()
-    # fail if we didn't read the config
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_instance,
-                          None)
-
-    cfg.AddInstance(inst, "my-job")
-    instance = cfg.GetInstanceInfo(cfg.GetInstanceList()[0])
-    # first pass, must not fail
-    cfg.Update(instance, None)
-    # second pass, also must not fail (after the config has been written)
-    cfg.Update(instance, None)
-    # but the fake_instance update should still fail
-    self.failUnlessRaises(errors.ConfigurationError, cfg.Update, fake_instance,
-                          None)
-
   def testUpgradeSave(self):
     """Test that any modification done during upgrading is saved back"""
     cfg = self._get_object()
@@ -430,7 +376,7 @@ class TestConfigRunner(unittest.TestCase):
         constants.ND_CPU_SPEED: 1.0,
         }
 
-    cfg = self._get_object()
+    cfg = self._get_object_mock()
     node = cfg.GetNodeInfo(cfg.GetNodeList()[0])
     node.ndparams = my_ndparams
     cfg.Update(node, None)
@@ -458,7 +404,7 @@ class TestConfigRunner(unittest.TestCase):
       constants.ND_SSH_PORT: 222,
       constants.ND_CPU_SPEED: 1.0,
       }
-    cfg = self._get_object()
+    cfg = self._get_object_mock()
     node = cfg.GetNodeInfo(cfg.GetNodeList()[0])
     node.ndparams = node_ndparams
     cfg.Update(node, None)
@@ -471,7 +417,7 @@ class TestConfigRunner(unittest.TestCase):
     cfg = self._get_object()
     group = objects.NodeGroup(name="test", members=[])
     cfg.AddNodeGroup(group, "my-job")
-    self.assert_(utils.UUID_RE.match(group.uuid))
+    self.assertTrue(utils.UUID_RE.match(group.uuid))
     self.assertEqual(constants.ALLOC_POLICY_PREFERRED, group.alloc_policy)
 
   def testAddGroupPreservesFields(self):
@@ -487,8 +433,8 @@ class TestConfigRunner(unittest.TestCase):
                               serial_no=17, ctime=123, mtime=456)
     cfg.AddNodeGroup(group, "my-job")
     self.assertEqual(1, group.serial_no)
-    self.assert_(group.ctime > 1200000000)
-    self.assert_(group.mtime > 1200000000)
+    self.assertTrue(group.ctime > 1200000000)
+    self.assertTrue(group.mtime > 1200000000)
 
   def testAddGroupCanSkipUUIDCheck(self):
     cfg = self._get_object()
@@ -516,8 +462,7 @@ class TestConfigRunner(unittest.TestCase):
                              uuid="798d0de3-680f-4a0e-b29a-0f54f693b3f1")
     grp2_serial = 1
     cfg.AddNodeGroup(grp2, "job")
-    self.assertEqual(set(map(operator.attrgetter("name"),
-                             cfg.GetAllNodeGroupsInfo().values())),
+    self.assertEqual(set(ng.name for ng in cfg.GetAllNodeGroupsInfo().values()),
                      set(["grp1", "grp2", constants.INITIAL_NODE_GROUP_NAME]))
 
     # No-op
@@ -632,7 +577,7 @@ class TestConfigRunner(unittest.TestCase):
 
     self.assertTrue("a=A" in result)
     lines = [line for line in result.split('\n') if line != '']
-    self.assertEqual(len(hvparams.keys()), len(lines))
+    self.assertEqual(len(hvparams), len(lines))
 
   def testExtendByAllHvparamsStrings(self):
     all_hvparams = {constants.HT_XEN_PVM: "foo"}
@@ -707,26 +652,13 @@ class TestConfigRunner(unittest.TestCase):
     self.assertRaises(errors.ProgrammerError, cfg.DetachInstanceDisk,
                       "test-uuid", "disk0")
 
-    # Attach disk to non-existent instance
-    self.assertRaises(errors.ConfigurationError, cfg.AttachInstanceDisk,
-                      "1134", "disk0")
-
-    # Attach non-existent disk
-    self.assertRaises(errors.ConfigurationError, cfg.AttachInstanceDisk,
-                      "test-uuid", "disk1")
-
     # Attach disk
     cfg.AttachInstanceDisk("test-uuid", "disk0")
     instance_disks = cfg.GetInstanceDisks("test-uuid")
     self.assertEqual(instance_disks, [disk])
 
-    # Attach disk again
-    self.assertRaises(errors.ReservationError, cfg.AttachInstanceDisk,
-                      "test-uuid", "disk0")
-
-
 def _IsErrorInList(err_str, err_list):
-  return any(map(lambda e: err_str in e, err_list))
+  return any((err_str in e) for e in err_list)
 
 
 class TestTRM(unittest.TestCase):
@@ -755,7 +687,7 @@ class TestCheckInstanceDiskIvNames(unittest.TestCase):
   def testNoError(self):
     disks = self._MakeDisks(["disk/0", "disk/1"])
     self.assertEqual(config._CheckInstanceDiskIvNames(disks), [])
-    config._UpdateIvNames(0, disks)
+    _UpdateIvNames(0, disks)
     self.assertEqual(config._CheckInstanceDiskIvNames(disks), [])
 
   def testWrongNames(self):
@@ -766,7 +698,7 @@ class TestCheckInstanceDiskIvNames(unittest.TestCase):
       ])
 
     # Fix names
-    config._UpdateIvNames(0, disks)
+    _UpdateIvNames(0, disks)
     self.assertEqual(config._CheckInstanceDiskIvNames(disks), [])
 
 
