@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2010 Google Inc.
@@ -72,13 +72,28 @@ def CheckCmdWord(cmd, word):
 
 class TestCommandBuilder(unittest.TestCase):
   def test(self):
+
+    # The commands various compressions should use
+    compress_import = {
+      constants.IEC_GZIP: "gzip -d",
+      constants.IEC_GZIP_FAST: "gzip -d",
+      constants.IEC_GZIP_SLOW: "gzip -d",
+      constants.IEC_LZOP: "lzop -d",
+      }
+    compress_export = {
+      constants.IEC_GZIP: "gzip -1",
+      constants.IEC_GZIP_FAST: "gzip -1",
+      constants.IEC_GZIP_SLOW: "gzip",
+      constants.IEC_LZOP: "lzop",
+      }
+
     for mode in [constants.IEM_IMPORT, constants.IEM_EXPORT]:
       if mode == constants.IEM_IMPORT:
-        comprcmd = "gunzip"
+        compress_dict = compress_import
       elif mode == constants.IEM_EXPORT:
-        comprcmd = "gzip"
+        compress_dict = compress_export
 
-      for compress in [constants.IEC_NONE, constants.IEC_GZIP]:
+      for compress in constants.IEC_ALL:
         for magic in [None, 10 * "-", "HelloWorld", "J9plh4nFo2",
                       "24A02A81-2264-4B51-A882-A2AB9D85B420"]:
           opts = CmdBuilderConfig(magic=magic, compress=compress)
@@ -88,8 +103,8 @@ class TestCommandBuilder(unittest.TestCase):
           dd_cmd = builder._GetDdCommand()
 
           if magic:
-            self.assert_(("M=%s" % magic) in magic_cmd)
-            self.assert_(("M=%s" % magic) in dd_cmd)
+            self.assertTrue(("M=%s" % magic) in magic_cmd)
+            self.assertTrue(("M=%s" % magic) in dd_cmd)
           else:
             self.assertFalse(magic_cmd)
 
@@ -107,50 +122,57 @@ class TestCommandBuilder(unittest.TestCase):
 
                 # Check complete command
                 cmd = builder.GetCommand()
-                self.assert_(isinstance(cmd, list))
+                self.assertTrue(isinstance(cmd, list))
 
-                if compress == constants.IEC_GZIP:
-                  self.assert_(CheckCmdWord(cmd, comprcmd))
+                if compress != constants.IEC_NONE:
+                  self.assertTrue(CheckCmdWord(cmd, compress_dict[compress]))
 
                 if cmd_prefix is not None:
-                  self.assert_(compat.any(cmd_prefix in i for i in cmd))
+                  self.assertTrue(compat.any(cmd_prefix in i for i in cmd))
 
                 if cmd_suffix is not None:
-                  self.assert_(compat.any(cmd_suffix in i for i in cmd))
+                  self.assertTrue(compat.any(cmd_suffix in i for i in cmd))
 
                 # Check socat command
                 socat_cmd = builder._GetSocatCommand()
 
                 if mode == constants.IEM_IMPORT:
                   ssl_addr = socat_cmd[-2].split(",")
-                  self.assert_(("OPENSSL-LISTEN:%s" % port) in ssl_addr)
+                  self.assertTrue(("OPENSSL-LISTEN:%s" % port) in ssl_addr)
                 elif mode == constants.IEM_EXPORT:
                   ssl_addr = socat_cmd[-1].split(",")
-                  self.assert_(("OPENSSL:%s:%s" % (host, port)) in ssl_addr)
+                  self.assertTrue(("OPENSSL:%s:%s" % (host, port)) in ssl_addr)
+                  if impexpd.CommandBuilder._GetSocatVersion() >= (1, 7, 3):
+                    self.assertTrue("openssl-commonname=%s" %
+                                 constants.X509_CERT_CN in ssl_addr)
+                  else:
+                    self.assertTrue("openssl-commonname=%s" %
+                                 constants.X509_CERT_CN not in ssl_addr)
 
-                self.assert_("verify=1" in ssl_addr)
+                self.assertTrue("verify=1" in ssl_addr)
 
+  @testutils.RequiresIPv6()
   def testIPv6(self):
     for mode in [constants.IEM_IMPORT, constants.IEM_EXPORT]:
       opts = CmdBuilderConfig(host="localhost", port=6789,
                               ipv4=False, ipv6=False)
       builder = impexpd.CommandBuilder(mode, opts, 1, 2, 3)
       cmd = builder._GetSocatCommand()
-      self.assert_(compat.all("pf=" not in i for i in cmd))
+      self.assertTrue(compat.all("pf=" not in i for i in cmd))
 
       # IPv4
       opts = CmdBuilderConfig(host="localhost", port=6789,
                               ipv4=True, ipv6=False)
       builder = impexpd.CommandBuilder(mode, opts, 1, 2, 3)
       cmd = builder._GetSocatCommand()
-      self.assert_(compat.any(",pf=ipv4" in i for i in cmd))
+      self.assertTrue(compat.any(",pf=ipv4" in i for i in cmd))
 
       # IPv6
       opts = CmdBuilderConfig(host="localhost", port=6789,
                               ipv4=False, ipv6=True)
       builder = impexpd.CommandBuilder(mode, opts, 1, 2, 3)
       cmd = builder._GetSocatCommand()
-      self.assert_(compat.any(",pf=ipv6" in i for i in cmd))
+      self.assertTrue(compat.any(",pf=ipv6" in i for i in cmd))
 
       # IPv4 and IPv6
       opts = CmdBuilderConfig(host="localhost", port=6789,
