@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2011 Google Inc.
@@ -73,11 +73,11 @@ class TestFillOpcode(unittest.TestCase):
                       self.OpTest, {}, {"test": [1, 2, 3]})
 
   def testUnicode(self):
-    op = baserlib.FillOpcode(self.OpTest, {u"test": "abc"}, {})
+    op = baserlib.FillOpcode(self.OpTest, {"test": "abc"}, {})
     self.assertTrue(isinstance(op, self.OpTest))
     self.assertEqual(op.test, "abc")
 
-    op = baserlib.FillOpcode(self.OpTest, {}, {u"test": "abc"})
+    op = baserlib.FillOpcode(self.OpTest, {}, {"test": "abc"})
     self.assertTrue(isinstance(op, self.OpTest))
     self.assertEqual(op.test, "abc")
 
@@ -116,8 +116,9 @@ class TestOpcodeResource(unittest.TestCase):
   @staticmethod
   def _GetMethodAttributes(method):
     attrs = ["%s_OPCODE" % method, "%s_RENAME" % method,
-             "%s_ALIASES" % method, "Get%sOpInput" % method.capitalize()]
-    assert attrs == dict((opattrs[0], list(opattrs[1:]))
+             "%s_ALIASES" % method, "%s_FORBIDDEN" % method,
+             "Get%sOpInput" % method.capitalize()]
+    assert attrs == dict((opattrs.method, opattrs.GetModifiers())
                          for opattrs in baserlib.OPCODE_ATTRS)[method]
     return attrs
 
@@ -125,8 +126,9 @@ class TestOpcodeResource(unittest.TestCase):
     for method in baserlib._SUPPORTED_METHODS:
       # Empty handler
       obj = self._MakeClass(method, {})(None, {}, None)
-      for attr in itertools.chain(*baserlib.OPCODE_ATTRS):
-        self.assertFalse(hasattr(obj, attr))
+      for m_attr in baserlib.OPCODE_ATTRS:
+        for attr in m_attr.GetAll():
+          self.assertFalse(hasattr(obj, attr))
 
       # Direct handler function
       obj = self._MakeClass(method, {
@@ -162,8 +164,9 @@ class TestOpcodeResource(unittest.TestCase):
 
     obj = _Empty(None, {}, None)
 
-    for attr in itertools.chain(*baserlib.OPCODE_ATTRS):
-      self.assertFalse(hasattr(obj, attr))
+    for m_attr in baserlib.OPCODE_ATTRS:
+        for attr in m_attr.GetAll():
+          self.assertFalse(hasattr(obj, attr))
 
 
 if __name__ == "__main__":
