@@ -1,22 +1,31 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2012 Google Inc.
+# All rights reserved.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# 1. Redistributions of source code must retain the above copyright notice,
+# this list of conditions and the following disclaimer.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in the
+# documentation and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 """Script for testing ganeti.server.rapi"""
@@ -24,9 +33,7 @@
 import re
 import unittest
 import random
-import mimetools
-import base64
-from cStringIO import StringIO
+from io import StringIO
 
 from ganeti import constants
 from ganeti import utils
@@ -139,8 +146,8 @@ class TestRemoteApiHandler(unittest.TestCase):
                      "Basic authentication requires credentials")
 
   def testInvalidBasicAuth(self):
-    for auth in ["!invalid=base!64.", base64.b64encode(" "),
-                 base64.b64encode("missingcolonchar")]:
+    for auth in ["!invalid=base!64.", testutils.b64encode_string(" "),
+                 testutils.b64encode_string("missingcolonchar")]:
       headers = rapi.testutils._FormatHeaders([
         "%s: Basic %s" % (http.HTTP_AUTHORIZATION, auth),
         ])
@@ -155,9 +162,10 @@ class TestRemoteApiHandler(unittest.TestCase):
     else:
       pw = "wrongpass"
 
+    authtok = "%s:%s" % (username, pw)
     return rapi.testutils._FormatHeaders([
       "%s: Basic %s" % (http.HTTP_AUTHORIZATION,
-                        base64.b64encode("%s:%s" % (username, pw))),
+                        testutils.b64encode_string(authtok)),
       "%s: %s" % (http.HTTP_CONTENT_TYPE, http.HTTP_APP_JSON),
       ])
 
