@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2012 Google Inc.
@@ -164,6 +164,8 @@ class TestUpdateSshDaemon(unittest.TestCase):
         (constants.SSHK_ECDSA, "ecdsapriv", "ecdsapub"),
         (constants.SSHK_RSA, "rsapriv", "rsapub"),
         ],
+      constants.SSHS_SSH_KEY_TYPE: "dsa",
+      constants.SSHS_SSH_KEY_BITS: 1024,
       }
     runcmd_fn = compat.partial(self._RunCmd, failcmd)
     if failcmd:
@@ -228,7 +230,9 @@ class TestUpdateSshRoot(unittest.TestCase):
     data = {
       constants.SSHS_SSH_ROOT_KEY: [
         (constants.SSHK_DSA, "privatedsa", "ssh-dss pubdsa"),
-        ]
+        ],
+      constants.SSHS_SSH_KEY_TYPE: "dsa",
+      constants.SSHS_SSH_KEY_BITS: 1024,
       }
 
     prepare_node_join.UpdateSshRoot(data, False,
@@ -240,7 +244,7 @@ class TestUpdateSshRoot(unittest.TestCase):
                     is not None)
     pub_key = utils.ReadFile(utils.PathJoin(self.sshdir, "id_dsa.pub"))
     self.assertTrue(pub_key is not None)
-    self.assertEquals(utils.ReadFile(utils.PathJoin(self.sshdir,
+    self.assertEqual(utils.ReadFile(utils.PathJoin(self.sshdir,
                                                     "authorized_keys")),
                       pub_key)
 
