@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2006, 2007, 2010, 2011 Google Inc.
@@ -40,6 +40,8 @@ import signal
 import stat
 import errno
 
+from hashlib import md5
+
 from ganeti import constants
 from ganeti import utils
 from ganeti import compat
@@ -48,24 +50,45 @@ from ganeti import errors
 import testutils
 
 
+class TestReadBinaryFile(testutils.GanetiTestCase):
+  def testReadAll(self):
+    data = utils.ReadBinaryFile(testutils.TestDataFilename("cert1.pem"))
+    self.assertEqual(len(data), 1229)
+
+    h = md5()
+    h.update(data)
+    self.assertEqual(h.hexdigest(), "a02be485db0d82b70c0ae7913b26894e")
+
+  def testReadSize(self):
+    data = utils.ReadBinaryFile(testutils.TestDataFilename("cert1.pem"),
+                                size=100)
+    self.assertEqual(len(data), 100)
+
+    h = md5()
+    h.update(data)
+    self.assertEqual(h.hexdigest(), "256d28505448898d4741b10c5f5dbc12")
+
+  def testCallback(self):
+    def _Cb(fh):
+      self.assertEqual(fh.tell(), 0)
+    data = utils.ReadBinaryFile(testutils.TestDataFilename("cert1.pem"),
+                                preread=_Cb)
+    self.assertEqual(len(data), 1229)
+
+  def testError(self):
+    self.assertRaises(EnvironmentError, utils.ReadBinaryFile,
+                      "/dev/null/does-not-exist")
+
+
 class TestReadFile(testutils.GanetiTestCase):
   def testReadAll(self):
     data = utils.ReadFile(testutils.TestDataFilename("cert1.pem"))
     self.assertEqual(len(data), 1229)
 
-    h = compat.md5_hash()
-    h.update(data)
-    self.assertEqual(h.hexdigest(), "a02be485db0d82b70c0ae7913b26894e")
-
   def testReadSize(self):
-    data = utils.ReadFile(testutils.TestDataFilename("cert1.pem"),
-                          size=100)
+    data = utils.ReadFile(testutils.TestDataFilename("cert1.pem"), size=100)
     self.assertEqual(len(data), 100)
 
-    h = compat.md5_hash()
-    h.update(data)
-    self.assertEqual(h.hexdigest(), "256d28505448898d4741b10c5f5dbc12")
-
   def testCallback(self):
     def _Cb(fh):
       self.assertEqual(fh.tell(), 0)
@@ -124,7 +147,7 @@ class TestReadOneLineFile(testutils.GanetiTestCase):
         utils.WriteFile(myfile, data=dummydata)
         datalax = utils.ReadOneLineFile(myfile, strict=False)
         if nl:
-          self.assert_(set("\r\n") & set(dummydata))
+          self.assertTrue(set("\r\n") & set(dummydata))
           self.assertRaises(errors.GenericError, utils.ReadOneLineFile,
                             myfile, strict=True)
           explen = len("Foo bar baz ") + len(ws)
@@ -143,7 +166,7 @@ class TestReadOneLineFile(testutils.GanetiTestCase):
       for ol in ["", "otherline"]:
         dummydata = "%s%s%s%s%s%s" % (nl, nl, myline, nl, ol, nl)
         utils.WriteFile(myfile, data=dummydata)
-        self.assert_(set("\r\n") & set(dummydata))
+        self.assertTrue(set("\r\n") & set(dummydata))
         datalax = utils.ReadOneLineFile(myfile, strict=False)
         self.assertEqual(myline, datalax)
         if ol:
@@ -160,8 +183,8 @@ class TestReadOneLineFile(testutils.GanetiTestCase):
 
 class TestTimestampForFilename(unittest.TestCase):
   def test(self):
-    self.assert_("." not in utils.TimestampForFilename())
-    self.assert_(":" not in utils.TimestampForFilename())
+    self.assertTrue("." not in utils.TimestampForFilename())
+    self.assertTrue(":" not in utils.TimestampForFilename())
 
 
 class TestCreateBackup(testutils.GanetiTestCase):
@@ -241,11 +264,11 @@ class TestListVisibleFiles(unittest.TestCase):
     self._test(files, expected)
 
   def testNonAbsolutePath(self):
-    self.failUnlessRaises(errors.ProgrammerError, utils.ListVisibleFiles,
+    self.assertRaises(errors.ProgrammerError, utils.ListVisibleFiles,
                           "abc")
 
   def testNonNormalizedPath(self):
-    self.failUnlessRaises(errors.ProgrammerError, utils.ListVisibleFiles,
+    self.assertRaises(errors.ProgrammerError, utils.ListVisibleFiles,
                           "/bin/../tmp")
 
   def testMountpoint(self):
@@ -299,7 +322,7 @@ class TestWriteFile(testutils.GanetiTestCase):
     self.assertEqual(utils.ReadFile(self.tfile.name), data)
 
   def testWriteSimpleUnicode(self):
-    data = u"abc"
+    data = "αβγ"
     utils.WriteFile(self.tfile.name, data=data)
     self.assertEqual(utils.ReadFile(self.tfile.name), data)
 
@@ -310,13 +333,13 @@ class TestWriteFile(testutils.GanetiTestCase):
     self.assertRaises(errors.ProgrammerError, utils.WriteFile,
                       self.tfile.name, data="test", atime=0)
     self.assertRaises(errors.ProgrammerError, utils.WriteFile, self.tfile.name,
-                      mode=0400, keep_perms=utils.KP_ALWAYS)
+                      mode=0o400, keep_perms=utils.KP_ALWAYS)
     self.assertRaises(errors.ProgrammerError, utils.WriteFile, self.tfile.name,
                       uid=0, keep_perms=utils.KP_ALWAYS)
     self.assertRaises(errors.ProgrammerError, utils.WriteFile, self.tfile.name,
                       gid=0, keep_perms=utils.KP_ALWAYS)
     self.assertRaises(errors.ProgrammerError, utils.WriteFile, self.tfile.name,
-                      mode=0400, uid=0, keep_perms=utils.KP_ALWAYS)
+                      mode=0o400, uid=0, keep_perms=utils.KP_ALWAYS)
 
   def testPreWrite(self):
     utils.WriteFile(self.tfile.name, data="", prewrite=self.markPre)
@@ -338,7 +361,7 @@ class TestWriteFile(testutils.GanetiTestCase):
 
   def testDryRun(self):
     orig = "abc"
-    self.tfile.write(orig)
+    self.tfile.write(orig.encode())
     self.tfile.flush()
     utils.WriteFile(self.tfile.name, data="hello", dry_run=True)
     self.assertEqual(utils.ReadFile(self.tfile.name), orig)
@@ -353,7 +376,7 @@ class TestWriteFile(testutils.GanetiTestCase):
       self.assertEqual(st.st_mtime, mt)
 
   def testNoClose(self):
-    data = "hello"
+    data = b"hello"
     self.assertEqual(utils.WriteFile(self.tfile.name, data="abc"), None)
     fd = utils.WriteFile(self.tfile.name, data=data, close=False)
     try:
@@ -419,28 +442,28 @@ class TestWriteFile(testutils.GanetiTestCase):
     self.assertRaises(OSError, utils.WriteFile, target, data="data",
                       keep_perms=utils.KP_ALWAYS)
     # All masks have only user bits set, to avoid interactions with umask
-    utils.WriteFile(target, data="data", mode=0200)
-    self.assertFileMode(target, 0200)
-    utils.WriteFile(target, data="data", mode=0400,
+    utils.WriteFile(target, data="data", mode=0o200)
+    self.assertFileMode(target, 0o200)
+    utils.WriteFile(target, data="data", mode=0o400,
                     keep_perms=utils.KP_IF_EXISTS)
-    self.assertFileMode(target, 0200)
+    self.assertFileMode(target, 0o200)
     utils.WriteFile(target, data="data", keep_perms=utils.KP_ALWAYS)
-    self.assertFileMode(target, 0200)
-    utils.WriteFile(target, data="data", mode=0700)
-    self.assertFileMode(target, 0700)
+    self.assertFileMode(target, 0o200)
+    utils.WriteFile(target, data="data", mode=0o700)
+    self.assertFileMode(target, 0o700)
 
   def testNewFileMode(self):
     self.tmpdir = tempfile.mkdtemp()
     target = utils.PathJoin(self.tmpdir, "target")
-    utils.WriteFile(target, data="data", mode=0400,
+    utils.WriteFile(target, data="data", mode=0o400,
                     keep_perms=utils.KP_IF_EXISTS)
-    self.assertFileMode(target, 0400)
+    self.assertFileMode(target, 0o400)
 
 class TestFileID(testutils.GanetiTestCase):
   def testEquality(self):
     name = self._CreateTempFile()
     oldi = utils.GetFileID(path=name)
-    self.failUnless(utils.VerifyFileID(oldi, oldi))
+    self.assertTrue(utils.VerifyFileID(oldi, oldi))
 
   def testUpdate(self):
     name = self._CreateTempFile()
@@ -448,8 +471,8 @@ class TestFileID(testutils.GanetiTestCase):
     fd = os.open(name, os.O_RDWR)
     try:
       newi = utils.GetFileID(fd=fd)
-      self.failUnless(utils.VerifyFileID(oldi, newi))
-      self.failUnless(utils.VerifyFileID(newi, oldi))
+      self.assertTrue(utils.VerifyFileID(oldi, newi))
+      self.assertTrue(utils.VerifyFileID(newi, oldi))
     finally:
       os.close(fd)
 
@@ -554,20 +577,20 @@ class TestRename(unittest.TestCase):
   def testSimpleRename1(self):
     """Simple rename 1"""
     utils.RenameFile(self.tmpfile, os.path.join(self.tmpdir, "xyz"))
-    self.assert_(os.path.isfile(os.path.join(self.tmpdir, "xyz")))
+    self.assertTrue(os.path.isfile(os.path.join(self.tmpdir, "xyz")))
 
   def testSimpleRename2(self):
     """Simple rename 2"""
     utils.RenameFile(self.tmpfile, os.path.join(self.tmpdir, "xyz"),
                      mkdir=True)
-    self.assert_(os.path.isfile(os.path.join(self.tmpdir, "xyz")))
+    self.assertTrue(os.path.isfile(os.path.join(self.tmpdir, "xyz")))
 
   def testRenameMkdir(self):
     """Rename with mkdir"""
     utils.RenameFile(self.tmpfile, os.path.join(self.tmpdir, "test/xyz"),
                      mkdir=True)
-    self.assert_(os.path.isdir(os.path.join(self.tmpdir, "test")))
-    self.assert_(os.path.isfile(os.path.join(self.tmpdir, "test/xyz")))
+    self.assertTrue(os.path.isdir(os.path.join(self.tmpdir, "test")))
+    self.assertTrue(os.path.isfile(os.path.join(self.tmpdir, "test/xyz")))
 
     self.assertRaises(EnvironmentError, utils.RenameFile,
                       os.path.join(self.tmpdir, "test/xyz"),
@@ -590,25 +613,25 @@ class TestMakedirs(unittest.TestCase):
   def testNonExisting(self):
     path = utils.PathJoin(self.tmpdir, "foo")
     utils.Makedirs(path)
-    self.assert_(os.path.isdir(path))
+    self.assertTrue(os.path.isdir(path))
 
   def testExisting(self):
     path = utils.PathJoin(self.tmpdir, "foo")
     os.mkdir(path)
     utils.Makedirs(path)
-    self.assert_(os.path.isdir(path))
+    self.assertTrue(os.path.isdir(path))
 
   def testRecursiveNonExisting(self):
     path = utils.PathJoin(self.tmpdir, "foo/bar/baz")
     utils.Makedirs(path)
-    self.assert_(os.path.isdir(path))
+    self.assertTrue(os.path.isdir(path))
 
   def testRecursiveExisting(self):
     path = utils.PathJoin(self.tmpdir, "B/moo/xyz")
     self.assertFalse(os.path.exists(path))
     os.mkdir(utils.PathJoin(self.tmpdir, "B"))
     utils.Makedirs(path)
-    self.assert_(os.path.isdir(path))
+    self.assertTrue(os.path.isdir(path))
 
 
 class TestEnsureDirs(unittest.TestCase):
@@ -616,15 +639,16 @@ class TestEnsureDirs(unittest.TestCase):
 
   def setUp(self):
     self.dir = tempfile.mkdtemp()
-    self.old_umask = os.umask(0777)
+    self.old_umask = os.umask(0o777)
 
   def testEnsureDirs(self):
     utils.EnsureDirs([
-        (utils.PathJoin(self.dir, "foo"), 0777),
+        (utils.PathJoin(self.dir, "foo"), 0o777),
         (utils.PathJoin(self.dir, "bar"), 0000),
         ])
-    self.assertEquals(os.stat(utils.PathJoin(self.dir, "foo"))[0] & 0777, 0777)
-    self.assertEquals(os.stat(utils.PathJoin(self.dir, "bar"))[0] & 0777, 0000)
+    self.assertEqual(os.stat(utils.PathJoin(self.dir, "foo"))[0] & 0o777,
+                      0o777)
+    self.assertEqual(os.stat(utils.PathJoin(self.dir, "bar"))[0] & 0o777, 0000)
 
   def tearDown(self):
     os.rmdir(utils.PathJoin(self.dir, "foo"))
@@ -638,7 +662,7 @@ class TestIsNormAbsPath(unittest.TestCase):
 
   def _pathTestHelper(self, path, result):
     if result:
-      self.assert_(utils.IsNormAbsPath(path),
+      self.assertTrue(utils.IsNormAbsPath(path),
           msg="Path %s should result absolute and normalized" % path)
     else:
       self.assertFalse(utils.IsNormAbsPath(path),
@@ -719,16 +743,16 @@ class TestPathJoin(unittest.TestCase):
 
   def testBasicItems(self):
     mlist = ["/a", "b", "c"]
-    self.failUnlessEqual(utils.PathJoin(*mlist), "/".join(mlist))
+    self.assertEqual(utils.PathJoin(*mlist), "/".join(mlist))
 
   def testNonAbsPrefix(self):
-    self.failUnlessRaises(ValueError, utils.PathJoin, "a", "b")
+    self.assertRaises(ValueError, utils.PathJoin, "a", "b")
 
   def testBackTrack(self):
-    self.failUnlessRaises(ValueError, utils.PathJoin, "/a", "b/../c")
+    self.assertRaises(ValueError, utils.PathJoin, "/a", "b/../c")
 
   def testMultiAbs(self):
-    self.failUnlessRaises(ValueError, utils.PathJoin, "/a", "/b")
+    self.assertRaises(ValueError, utils.PathJoin, "/a", "/b")
 
 
 class TestTailFile(testutils.GanetiTestCase):
@@ -736,8 +760,8 @@ class TestTailFile(testutils.GanetiTestCase):
 
   def testEmpty(self):
     fname = self._CreateTempFile()
-    self.failUnlessEqual(utils.TailFile(fname), [])
-    self.failUnlessEqual(utils.TailFile(fname, lines=25), [])
+    self.assertEqual(utils.TailFile(fname), [])
+    self.assertEqual(utils.TailFile(fname, lines=25), [])
 
   def testAllLines(self):
     data = ["test %d" % i for i in range(30)]
@@ -748,7 +772,7 @@ class TestTailFile(testutils.GanetiTestCase):
       if i > 0:
         fd.write("\n")
       fd.close()
-      self.failUnlessEqual(utils.TailFile(fname, lines=i), data[:i])
+      self.assertEqual(utils.TailFile(fname, lines=i), data[:i])
 
   def testPartialLines(self):
     data = ["test %d" % i for i in range(30)]
@@ -758,7 +782,7 @@ class TestTailFile(testutils.GanetiTestCase):
     fd.write("\n")
     fd.close()
     for i in range(1, 30):
-      self.failUnlessEqual(utils.TailFile(fname, lines=i), data[-i:])
+      self.assertEqual(utils.TailFile(fname, lines=i), data[-i:])
 
   def testBigFile(self):
     data = ["test %d" % i for i in range(30)]
@@ -770,7 +794,7 @@ class TestTailFile(testutils.GanetiTestCase):
     fd.write("\n")
     fd.close()
     for i in range(1, 30):
-      self.failUnlessEqual(utils.TailFile(fname, lines=i), data[-i:])
+      self.assertEqual(utils.TailFile(fname, lines=i), data[-i:])
 
 
 class TestPidFileFunctions(unittest.TestCase):
@@ -783,29 +807,29 @@ class TestPidFileFunctions(unittest.TestCase):
   def testPidFileFunctions(self):
     pid_file = self.f_dpn("test")
     fd = utils.WritePidFile(self.f_dpn("test"))
-    self.failUnless(os.path.exists(pid_file),
+    self.assertTrue(os.path.exists(pid_file),
                     "PID file should have been created")
     read_pid = utils.ReadPidFile(pid_file)
-    self.failUnlessEqual(read_pid, os.getpid())
-    self.failUnless(utils.IsProcessAlive(read_pid))
-    self.failUnlessRaises(errors.PidFileLockError, utils.WritePidFile,
+    self.assertEqual(read_pid, os.getpid())
+    self.assertTrue(utils.IsProcessAlive(read_pid))
+    self.assertRaises(errors.PidFileLockError, utils.WritePidFile,
                           self.f_dpn("test"))
     os.close(fd)
     utils.RemoveFile(self.f_dpn("test"))
-    self.failIf(os.path.exists(pid_file),
+    self.assertFalse(os.path.exists(pid_file),
                 "PID file should not exist anymore")
-    self.failUnlessEqual(utils.ReadPidFile(pid_file), 0,
+    self.assertEqual(utils.ReadPidFile(pid_file), 0,
                          "ReadPidFile should return 0 for missing pid file")
     fh = open(pid_file, "w")
     fh.write("blah\n")
     fh.close()
-    self.failUnlessEqual(utils.ReadPidFile(pid_file), 0,
+    self.assertEqual(utils.ReadPidFile(pid_file), 0,
                          "ReadPidFile should return 0 for invalid pid file")
     # but now, even with the file existing, we should be able to lock it
     fd = utils.WritePidFile(self.f_dpn("test"))
     os.close(fd)
     utils.RemoveFile(self.f_dpn("test"))
-    self.failIf(os.path.exists(pid_file),
+    self.assertFalse(os.path.exists(pid_file),
                 "PID file should not exist anymore")
 
   def testKill(self):
@@ -814,7 +838,7 @@ class TestPidFileFunctions(unittest.TestCase):
     new_pid = os.fork()
     if new_pid == 0: #child
       utils.WritePidFile(self.f_dpn("child"))
-      os.write(w_fd, "a")
+      os.write(w_fd, b"a")
       signal.pause()
       os._exit(0)
       return
@@ -822,13 +846,13 @@ class TestPidFileFunctions(unittest.TestCase):
     # wait until the child has written the pid file
     os.read(r_fd, 1)
     read_pid = utils.ReadPidFile(pid_file)
-    self.failUnlessEqual(read_pid, new_pid)
-    self.failUnless(utils.IsProcessAlive(new_pid))
+    self.assertEqual(read_pid, new_pid)
+    self.assertTrue(utils.IsProcessAlive(new_pid))
 
     # Try writing to locked file
     try:
       utils.WritePidFile(pid_file)
-    except errors.PidFileLockError, err:
+    except errors.PidFileLockError as err:
       errmsg = str(err)
       self.assertTrue(errmsg.endswith(" %s" % new_pid),
                       msg=("Error message ('%s') didn't contain correct"
@@ -837,9 +861,9 @@ class TestPidFileFunctions(unittest.TestCase):
       self.fail("Writing to locked file didn't fail")
 
     utils.KillProcess(new_pid, waitpid=True)
-    self.failIf(utils.IsProcessAlive(new_pid))
+    self.assertFalse(utils.IsProcessAlive(new_pid))
     utils.RemoveFile(self.f_dpn("child"))
-    self.failUnlessRaises(errors.ProgrammerError, utils.KillProcess, 0)
+    self.assertRaises(errors.ProgrammerError, utils.KillProcess, 0)
 
   def testExceptionType(self):
     # Make sure the PID lock error is a subclass of LockError in case some code
@@ -854,7 +878,7 @@ class TestNewUUID(unittest.TestCase):
   """Test case for NewUUID"""
 
   def runTest(self):
-    self.failUnless(utils.UUID_RE.match(utils.NewUUID()))
+    self.assertTrue(utils.UUID_RE.match(utils.NewUUID()))
 
 
 def _MockStatResult(cb, mode, uid, gid):
@@ -908,41 +932,41 @@ class TestPermissionEnforcements(unittest.TestCase):
 
   def _VerifyPerm(self, path, mode, uid=-1, gid=-1):
     self.assertEqual(path, "/ganeti-qa-non-test")
-    self.assertEqual(mode, 0700)
+    self.assertEqual(mode, 0o700)
     self.assertEqual(uid, self.UID_A)
     self.assertEqual(gid, self.GID_A)
 
   def testMakeDirWithPerm(self):
     is_dir_stat = _MockStatResult(None, stat.S_IFDIR, 0, 0)
-    utils.MakeDirWithPerm("/ganeti-qa-non-test", 0700, self.UID_A, self.GID_A,
+    utils.MakeDirWithPerm("/ganeti-qa-non-test", 0o700, self.UID_A, self.GID_A,
                           _lstat_fn=is_dir_stat, _perm_fn=self._VerifyPerm)
 
   def testDirErrors(self):
     self.assertRaises(errors.GenericError, utils.MakeDirWithPerm,
-                      "/ganeti-qa-non-test", 0700, 0, 0,
+                      "/ganeti-qa-non-test", 0o700, 0, 0,
                       _lstat_fn=_MockStatResult(None, 0, 0, 0))
     self.assertRaises(IndexError, self._mkdir_calls.pop)
 
     other_stat_raise = _MockStatResult(_OtherStatRaise, stat.S_IFDIR, 0, 0)
     self.assertRaises(errors.GenericError, utils.MakeDirWithPerm,
-                      "/ganeti-qa-non-test", 0700, 0, 0,
+                      "/ganeti-qa-non-test", 0o700, 0, 0,
                       _lstat_fn=other_stat_raise)
     self.assertRaises(IndexError, self._mkdir_calls.pop)
 
     non_exist_stat = _MockStatResult(_RaiseNoEntError, stat.S_IFDIR, 0, 0)
-    utils.MakeDirWithPerm("/ganeti-qa-non-test", 0700, self.UID_A, self.GID_A,
+    utils.MakeDirWithPerm("/ganeti-qa-non-test", 0o700, self.UID_A, self.GID_A,
                           _lstat_fn=non_exist_stat, _mkdir_fn=self._FakeMkdir,
                           _perm_fn=self._VerifyPerm)
     self.assertEqual(self._mkdir_calls.pop(0), "/ganeti-qa-non-test")
 
   def testEnforcePermissionNoEnt(self):
     self.assertRaises(errors.GenericError, utils.EnforcePermission,
-                      "/ganeti-qa-non-test", 0600,
+                      "/ganeti-qa-non-test", 0o600,
                       _chmod_fn=NotImplemented, _chown_fn=NotImplemented,
                       _stat_fn=_MockStatResult(_RaiseNoEntError, 0, 0, 0))
 
   def testEnforcePermissionNoEntMustNotExist(self):
-    utils.EnforcePermission("/ganeti-qa-non-test", 0600, must_exist=False,
+    utils.EnforcePermission("/ganeti-qa-non-test", 0o600, must_exist=False,
                             _chmod_fn=NotImplemented,
                             _chown_fn=NotImplemented,
                             _stat_fn=_MockStatResult(_RaiseNoEntError,
@@ -950,27 +974,27 @@ class TestPermissionEnforcements(unittest.TestCase):
 
   def testEnforcePermissionOtherErrorMustNotExist(self):
     self.assertRaises(errors.GenericError, utils.EnforcePermission,
-                      "/ganeti-qa-non-test", 0600, must_exist=False,
+                      "/ganeti-qa-non-test", 0o600, must_exist=False,
                       _chmod_fn=NotImplemented, _chown_fn=NotImplemented,
                       _stat_fn=_MockStatResult(_OtherStatRaise, 0, 0, 0))
 
   def testEnforcePermissionNoChanges(self):
-    utils.EnforcePermission("/ganeti-qa-non-test", 0600,
-                            _stat_fn=_MockStatResult(None, 0600, 0, 0),
+    utils.EnforcePermission("/ganeti-qa-non-test", 0o600,
+                            _stat_fn=_MockStatResult(None, 0o600, 0, 0),
                             _chmod_fn=self._ChmodWrapper(None),
                             _chown_fn=self._FakeChown)
 
   def testEnforcePermissionChangeMode(self):
-    utils.EnforcePermission("/ganeti-qa-non-test", 0444,
-                            _stat_fn=_MockStatResult(None, 0600, 0, 0),
+    utils.EnforcePermission("/ganeti-qa-non-test", 0o444,
+                            _stat_fn=_MockStatResult(None, 0o600, 0, 0),
                             _chmod_fn=self._ChmodWrapper(None),
                             _chown_fn=self._FakeChown)
-    self.assertEqual(self._chmod_calls.pop(0), ("/ganeti-qa-non-test", 0444))
+    self.assertEqual(self._chmod_calls.pop(0), ("/ganeti-qa-non-test", 0o444))
 
   def testEnforcePermissionSetUidGid(self):
-    utils.EnforcePermission("/ganeti-qa-non-test", 0600,
+    utils.EnforcePermission("/ganeti-qa-non-test", 0o600,
                             uid=self.UID_B, gid=self.GID_B,
-                            _stat_fn=_MockStatResult(None, 0600,
+                            _stat_fn=_MockStatResult(None, 0o600,
                                                      self.UID_A,
                                                      self.GID_A),
                             _chmod_fn=self._ChmodWrapper(None),
