@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2013 Google Inc.
@@ -38,33 +38,39 @@ import testutils
 
 class ParseVersionTest(unittest.TestCase):
     def testParseVersion(self):
-        self.assertEquals(version.ParseVersion("2.10"), (2, 10, 0))
-        self.assertEquals(version.ParseVersion("2.10.1"), (2, 10, 1))
-        self.assertEquals(version.ParseVersion("2.10.1~beta2"), (2, 10, 1))
-        self.assertEquals(version.ParseVersion("2.10.1-3"), (2, 10, 1))
-        self.assertEquals(version.ParseVersion("2"), None)
-        self.assertEquals(version.ParseVersion("pink bunny"), None)
+        self.assertEqual(version.ParseVersion("2.10"), (2, 10, 0))
+        self.assertEqual(version.ParseVersion("2.10.1"), (2, 10, 1))
+        self.assertEqual(version.ParseVersion("2.10.1~beta2"), (2, 10, 1))
+        self.assertEqual(version.ParseVersion("2.10.1-3"), (2, 10, 1))
+        self.assertEqual(version.ParseVersion("2"), None)
+        self.assertEqual(version.ParseVersion("pink bunny"), None)
 
 class UpgradeRangeTest(unittest.TestCase):
     def testUpgradeRange(self):
-        self.assertEquals(version.UpgradeRange((2,11,0), current=(2,10,0)),
+        self.assertEqual(version.UpgradeRange((2,11,0), current=(2,10,0)),
                           None)
-        self.assertEquals(version.UpgradeRange((2,10,0), current=(2,10,0)),
+        self.assertEqual(version.UpgradeRange((2,10,0), current=(2,10,0)),
                           None)
-        self.assertEquals(version.UpgradeRange((2,11,3), current=(2,12,0)),
+        self.assertEqual(version.UpgradeRange((2,11,3), current=(2,12,0)),
                           None)
-        self.assertEquals(version.UpgradeRange((2,11,3), current=(2,12,99)),
+        self.assertEqual(version.UpgradeRange((2,11,3), current=(2,12,99)),
                           None)
-        self.assertEquals(version.UpgradeRange((3,0,0), current=(2,12,0)),
-                          "different major versions")
-        self.assertEquals(version.UpgradeRange((2,12,0), current=(3,0,0)),
-                          "different major versions")
-        self.assertEquals(version.UpgradeRange((2,10,0), current=(2,12,0)),
+        self.assertEqual(version.UpgradeRange((3,0,0), current=(2,12,0)),
+                         "major version up- or downgrades are only supported"
+                         " between 2.16 and 3.0")
+        self.assertEqual(version.UpgradeRange((2,12,0), current=(3,0,0)),
+                         "major version up- or downgrades are only supported"
+                         " between 2.16 and 3.0")
+        self.assertEqual(version.UpgradeRange((2,10,0), current=(2,12,0)),
                           "can only downgrade one minor version at a time")
-        self.assertEquals(version.UpgradeRange((2,9,0), current=(2,10,0)),
+        self.assertEqual(version.UpgradeRange((2,9,0), current=(2,10,0)),
                           "automatic upgrades only supported from 2.10 onwards")
-        self.assertEquals(version.UpgradeRange((2,10,0), current=(2,9,0)),
+        self.assertEqual(version.UpgradeRange((2,10,0), current=(2,9,0)),
                           "automatic upgrades only supported from 2.10 onwards")
+        self.assertEqual(version.UpgradeRange((3,0,0), current=(2,16,1)),
+                          None)
+        self.assertEqual(version.UpgradeRange((2,16,1), current=(3,0,0)),
+                          None)
 
 class ShouldCfgdowngradeTest(unittest.TestCase):
     def testShouldCfgDowngrade(self):
@@ -91,6 +97,16 @@ class IsBeforeTest(unittest.TestCase):
         self.assertTrue(version.IsBefore((2, 10, 1), 2, 11, 0))
         self.assertFalse(version.IsBefore((2, 11, 0), 2, 10, 3))
 
+class IsEqualTest(unittest.TestCase):
+    def testIsEqual(self):
+        self.assertTrue(version.IsEqual((2, 10, 0), 2, 10, 0))
+        self.assertFalse(version.IsEqual((2, 10, 0), 2, 10, 2))
+        self.assertFalse(version.IsEqual((2, 10, 0), 2, 12, 0))
+        self.assertFalse(version.IsEqual((2, 10, 0), 3, 10, 0))
+        self.assertTrue(version.IsEqual((2, 10, 0), 2, 10, None))
+        self.assertTrue(version.IsEqual((2, 10, 5), 2, 10, None))
+        self.assertFalse(version.IsEqual((2, 11, 5), 2, 10, None))
+        self.assertFalse(version.IsEqual((3, 10, 5), 2, 10, None))
 
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
