@@ -1,22 +1,31 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 
 # Copyright (C) 2006, 2007, 2010, 2011 Google Inc.
+# All rights reserved.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# 1. Redistributions of source code must retain the above copyright notice,
+# this list of conditions and the following disclaimer.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in the
+# documentation and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 """Script for testing ganeti.utils.wrapper"""
@@ -43,34 +52,40 @@ class TestSetCloseOnExecFlag(unittest.TestCase):
 
   def testEnable(self):
     utils.SetCloseOnExecFlag(self.tmpfile.fileno(), True)
-    self.failUnless(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFD) &
+    self.assertTrue(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFD) &
                     fcntl.FD_CLOEXEC)
 
   def testDisable(self):
     utils.SetCloseOnExecFlag(self.tmpfile.fileno(), False)
-    self.failIf(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFD) &
+    self.assertFalse(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFD) &
                 fcntl.FD_CLOEXEC)
 
+  def tearDown(self):
+    self.tmpfile.close()
+
 
 class TestSetNonblockFlag(unittest.TestCase):
   def setUp(self):
     self.tmpfile = tempfile.TemporaryFile()
 
+  def tearDown(self):
+    self.tmpfile.close()
+
   def testEnable(self):
     utils.SetNonblockFlag(self.tmpfile.fileno(), True)
-    self.failUnless(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFL) &
+    self.assertTrue(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFL) &
                     os.O_NONBLOCK)
 
   def testDisable(self):
     utils.SetNonblockFlag(self.tmpfile.fileno(), False)
-    self.failIf(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFL) &
+    self.assertFalse(fcntl.fcntl(self.tmpfile.fileno(), fcntl.F_GETFL) &
                 os.O_NONBLOCK)
 
 
 class TestIgnoreProcessNotFound(unittest.TestCase):
   @staticmethod
   def _WritePid(fd):
-    os.write(fd, str(os.getpid()))
+    os.write(fd, b"%d" % os.getpid())
     os.close(fd)
     return True
 
@@ -78,7 +93,7 @@ class TestIgnoreProcessNotFound(unittest.TestCase):
     (pid_read_fd, pid_write_fd) = os.pipe()
 
     # Start short-lived process which writes its PID to pipe
-    self.assert_(utils.RunInSeparateProcess(self._WritePid, pid_write_fd))
+    self.assertTrue(utils.RunInSeparateProcess(self._WritePid, pid_write_fd))
     os.close(pid_write_fd)
 
     # Read PID from pipe
@@ -112,15 +127,15 @@ class TestIgnoreSignals(unittest.TestCase):
     self.assertRaises(EnvironmentError, self._Raise, env_err_intr)
     self.assertRaises(EnvironmentError, self._Raise, env_err_inval)
 
-    self.assertEquals(utils.IgnoreSignals(self._Raise, sock_err_intr), None)
-    self.assertEquals(utils.IgnoreSignals(self._Raise, env_err_intr), None)
+    self.assertEqual(utils.IgnoreSignals(self._Raise, sock_err_intr), None)
+    self.assertEqual(utils.IgnoreSignals(self._Raise, env_err_intr), None)
     self.assertRaises(socket.error, utils.IgnoreSignals, self._Raise,
                       sock_err_inval)
     self.assertRaises(EnvironmentError, utils.IgnoreSignals, self._Raise,
                       env_err_inval)
 
-    self.assertEquals(utils.IgnoreSignals(self._Return, True), True)
-    self.assertEquals(utils.IgnoreSignals(self._Return, 33), 33)
+    self.assertEqual(utils.IgnoreSignals(self._Return, True), True)
+    self.assertEqual(utils.IgnoreSignals(self._Return, 33), 33)
 
 
 class TestIsExecutable(unittest.TestCase):
@@ -143,7 +158,7 @@ class TestIsExecutable(unittest.TestCase):
 
   def testExecutable(self):
     fname = utils.PathJoin(self.tmpdir, "file")
-    utils.WriteFile(fname, data="#!/bin/bash", mode=0700)
+    utils.WriteFile(fname, data="#!/bin/bash", mode=0o700)
     assert os.path.exists(fname)
     self.assertTrue(utils.IsExecutable(fname))
 
@@ -151,7 +166,7 @@ class TestIsExecutable(unittest.TestCase):
 
   def testFileNotExecutable(self):
     fname = utils.PathJoin(self.tmpdir, "file")
-    utils.WriteFile(fname, data="#!/bin/bash", mode=0600)
+    utils.WriteFile(fname, data="#!/bin/bash", mode=0o600)
     assert os.path.exists(fname)
     self.assertFalse(utils.IsExecutable(fname))
 
