@@ -33,6 +33,7 @@
 
 import os
 
+from ganeti import locking
 from ganeti import netutils
 
 
@@ -47,6 +48,11 @@ FAKE_CLUSTER_KEY = ("AAAAB3NzaC1yc2EAAAABIwAAAQEAsuGLw70et3eApJ/ZEJkAVZogIrm"
 
 class FakeConfig(object):
   """Fake configuration object"""
+  def __init__(self):
+    self.write_count = 0
+
+  def OutDate(self):
+    pass
 
   def IsCluster(self):
     return True
@@ -79,7 +85,7 @@ class FakeConfig(object):
       return None
 
   def GetNodeNames(self, node_uuids):
-    return map(self.GetNodeName, node_uuids)
+    return [self.GetNodeName(uuid) for uuid in node_uuids]
 
 
 class FakeProc(object):
@@ -107,11 +113,21 @@ class FakeGLM(object):
 
 class FakeContext(object):
   """Fake context object"""
+  # pylint: disable=W0613
 
   def __init__(self):
     self.cfg = FakeConfig()
     self.glm = FakeGLM()
 
+  def GetConfig(self, ec_id):
+    return self.cfg
+
+  def GetRpc(self, cfg):
+    return None
+
+  def GetWConfdContext(self, ec_id):
+    return (None, None, None)
+
 
 class FakeGetentResolver(object):
   """Fake runtime.GetentResolver"""
@@ -139,3 +155,52 @@ class FakeGetentResolver(object):
 
   def LookupGid(self, gid):
     return "group%s" % gid
+
+
+class FakeLU(object):
+  HPATH = "fake-lu"
+  HTYPE = None
+
+  def __init__(self, processor, op, cfg, rpc_runner, prereq_err):
+    self.proc = processor
+    self.cfg = cfg
+    self.op = op
+    self.rpc = rpc_runner
+    self.prereq_err = prereq_err
+
+    self.needed_locks = {}
+    self.opportunistic_locks = dict.fromkeys(locking.LEVELS, False)
+    self.dont_collate_locks = dict.fromkeys(locking.LEVELS, False)
+    self.add_locks = {}
+
+    self.LogWarning = processor.LogWarning # pylint: disable=C0103
+
+  def CheckArguments(self):
+    pass
+
+  def ExpandNames(self):
+    pass
+
+  def DeclareLocks(self, level):
+    pass
+
+  def CheckPrereq(self):
+    if self.prereq_err:
+      raise self.prereq_err
+
+  def Exec(self, feedback_fn):
+    pass
+
+  def BuildHooksNodes(self):
+    return ([], [])
+
+  def BuildHooksEnv(self):
+    return {}
+
+  def PreparePostHookNodes(self, post_hook_node_uuids):
+    # pylint: disable=W0613
+    return []
+
+  def HooksCallBack(self, phase, hook_results, feedback_fn, lu_result):
+    # pylint: disable=W0613
+    return lu_result
