@@ -33,6 +33,9 @@
 import os
 import sys
 import stat
+import errno
+import base64
+import socket
 import tempfile
 import unittest
 import logging
@@ -105,6 +108,24 @@ def _SetupLogging(verbose):
   root_logger.addHandler(handler)
 
 
+def RequiresIPv6():
+  """Decorator for tests requiring IPv6 support
+
+  Decorated tests will be skipped if no IPv6 networking
+  support is available on the host system.
+
+  """
+  try:
+    # Try to bind a DGRAM socket on IPv6 localhost
+    sock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
+    sock.bind(('::1', 0))
+    sock.close()
+  except socket.error as err:
+    if err.errno in (errno.EADDRNOTAVAIL, errno.EAFNOSUPPORT):
+      return unittest.skip("IPv6 not available")
+  return lambda thing: thing
+
+
 class GanetiTestProgram(unittest.TestProgram):
   def runTests(self):
     """Runs all tests.
@@ -267,7 +288,7 @@ def UnifyValueType(data):
 
   elif isinstance(data, dict):
     return dict([(UnifyValueType(key), UnifyValueType(value))
-                 for (key, value) in data.iteritems()])
+                 for (key, value) in data.items()])
 
   return data
 
@@ -299,3 +320,14 @@ class CallCounter(object):
 
     """
     return self._count
+
+def b64encode_string(text, encoding="utf-8"):
+  """Utility to base64-encode a string
+
+  This exposes a string interface for Python3's b64encode
+
+  @type text: string
+  @param text: input string
+  """
+
+  return base64.b64encode(text.encode(encoding)).decode("ascii").strip()
